SystemOrganization addCategory: #'Widgets-Text'!
SystemOrganization addCategory: #'Widgets-Core'!
SystemOrganization addCategory: #'Widgets-Support'!
SystemOrganization addCategory: #'Widgets-Views'!
SystemOrganization addCategory: #'Widgets-Views-Support'!
SystemOrganization addCategory: #'Widgets-Views-Examples'!
SystemOrganization addCategory: #'Widgets-Events'!
SystemOrganization addCategory: #'Widgets-Documentation'!
SystemOrganization addCategory: #'Widgets-Painter'!
SystemOrganization addCategory: #'Widgets-Layouts'!
SystemOrganization addCategory: #'Widgets-Platforms'!

Trait named: #TUiItemModel
	uses: {}
	category: 'Widgets-Views'!

!TUiItemModel commentStamp: 'mt 1/25/2013 13:11:15.041' prior: 0!
This trait can be used for models/nodes that should be able to communicate with item views.

Dynamic contents
----------------
? special signals can trigger selected updates in item views
* add/remove/change child nodes (w/ all properties)
* change selected properties

Static contents
---------------
? some contents may only be specified on initial view creation
* number of properties (use #modelReset if changed)
* number of groups (use #modelReset if changed)!

!TUiItemModel methodsFor: 'item model - hierarchy' stamp: 'mt 4/22/2012 15:48:17.262'!
childAt: index inGroup: groupName
	"Return the child at the given index in the given group. Override this if you need index-based child access."
	
	^ (self perform: groupName) at: index! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 5/2/2012 10:37:00.346'!
childChanged: index

	self childrenChangedFrom: index to: index inGroup: self groups first.! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 4/22/2012 16:05:08.614'!
childChanged: index inGroup: groupName

	self childrenChangedFrom: index to: index inGroup: groupName.! !

!TUiItemModel methodsFor: 'item model - hierarchy' stamp: 'mt 4/26/2012 13:37:40.081'!
childCount

	^ self groups
		inject: 0
		into: [:sum :group | sum + (self childCountInGroup: group)]! !

!TUiItemModel methodsFor: 'item model - hierarchy' stamp: 'mt 4/22/2012 15:50:03.599'!
childCountInGroup: groupName
	"Override this if you can return the group size in a faster way ;-)"

	^ (self perform: groupName) size! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 5/2/2012 10:37:10.347'!
childInserted: index

	self childrenInsertedFrom: index to: index inGroup: self groups first.! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 4/22/2012 16:05:19.22'!
childInserted: index inGroup: groupName

	self childrenInsertedFrom: index to: index inGroup: groupName.! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 5/2/2012 10:37:32.383'!
childRemoved: index

	self childrenRemovedFrom: index to: index inGroup: self groups first.! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 5/2/2012 10:37:38.338'!
childRemoved: index inGroup: groupName

	self childrenRemovedFrom: index to: index inGroup: groupName.! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 4/22/2012 16:04:57.207'!
childrenChangedFrom: startIndex to: endIndex inGroup: groupName

	self emit.! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 4/22/2012 16:04:50.038'!
childrenInsertedFrom: startIndex to: endIndex inGroup: groupName

	self emit.! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 4/22/2012 16:04:38.76'!
childrenRemovedFrom: startIndex to: endIndex inGroup: groupName

	self emit.! !

!TUiItemModel methodsFor: 'item model - hierarchy' stamp: 'mt 4/22/2012 15:46:27.653'!
groups
	"Return a list of all child groups. Each group can be implemented as #groupName returning a list of all child nodes OR handled in #childAt:inGroup:."
	
	^ #()! !

!TUiItemModel methodsFor: 'item model - hierarchy' stamp: 'mt 7/31/2013 18:42:58.835'!
hasChildren

	^ self childCount > 0! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 4/27/2012 11:01:43.402'!
modelReset
	"The internal state has been invalidated. A view needs to refetch all indices."
	
	self emit.! !

!TUiItemModel methodsFor: 'item model - properties' stamp: 'mt 2/12/2012 15:27:56.656'!
properties
	"Return an ordered list of all known properties. This list can then be used to configure column specifications of item views more conveniently."
	
	^ #()! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 2/3/2012 14:20:14.294'!
propertiesChanged
	"Emit this whenever you want all your properties to be refetched. If you are the invisible root of a model, all visible nodes will be updated."

	self emit.! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 4/26/2012 13:42:30.714'!
propertiesChanged: someProperties

	self emit.! !

!TUiItemModel methodsFor: 'item model - signals' stamp: 'mt 2/3/2012 14:21:00.332'!
propertyChanged: aSymbol
	"A property that may be used in an item view via a colum spec has changed and needs to be updated. If you are the invisible root, all corresponding colums (resp. visible nodes) will be updated and not just single nodes."

	self propertiesChanged: {aSymbol}.! !

MouseEvent subclass: #UiMouseWheelEvent
	instanceVariableNames: 'direction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Events'!

!UiMouseWheelEvent commentStamp: 'mt 3/11/2011 16:34:25.021' prior: 0!
A mouse event generated by intercepting the keyboard events (ctrl+up/down arrow) generated by the VM in response to mouse wheel activity.!

!UiMouseWheelEvent methodsFor: 'accessing' stamp: 'mt 4/16/2010 13:52:21.616'!
direction
	^ direction! !

!UiMouseWheelEvent methodsFor: 'accessing' stamp: 'mt 4/16/2010 13:52:21.706'!
direction: anObject
	direction := anObject! !

!UiMouseWheelEvent methodsFor: 'testing' stamp: 'mt 4/16/2010 13:52:40.662'!
isMouseWheel

	^ true! !

!UiMouseWheelEvent methodsFor: 'dispatching' stamp: 'mt 4/16/2010 13:54:16.355'!
sentTo: anObject
	"Dispatch the receiver into anObject"
	
	type == #mouseWheel ifTrue:[^anObject handleMouseWheel: self].
	^super sentTo: anObject.! !

!UiMouseWheelEvent methodsFor: 'dispatching' stamp: 'mt 4/16/2010 13:54:24.836'!
setType: evtType position: evtPos direction: dirSymbol buttons: evtButtons hand: evtHand stamp: stamp
	"Set the state for the receiver."
	
	type := evtType.
	position := evtPos.
	buttons := evtButtons.
	source := evtHand.
	wasHandled := false.
	direction := dirSymbol.
	timeStamp := stamp.! !

Dictionary subclass: #UiPropertyItemNode
	uses: TUiItemModel
	instanceVariableNames: 'groups properties children deferAdd deferThreshold deferredAdditions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiPropertyItemNode commentStamp: 'mt 1/25/2013 13:33:01.248' prior: 0!
This class can be used to build a lightweight adaptor for item views.

Keys are ordered and used as properties. The key #children is ignored in this order.!

UiPropertyItemNode class
	uses: TUiItemModel classTrait
	instanceVariableNames: ''!

!UiPropertyItemNode class methodsFor: 'instance creation' stamp: 'mt 1/25/2013 14:34:31.683'!
newTreeFrom: someAssociations
	"Allow to create a tree structure by following the path of association values."

	| node |
	node := self new addDefaultGroup.
	someAssociations do: [:assoc |
		| child |
		child := self newTreeFrom: assoc value.
		child data: assoc key.
		node addChild: child].
	
	^ node

"	UiPropertyItemNode newFrom: {#apple -> {#green -> {}. #yellow -> {}. #brown -> {#noWorm -> {}} }}.
"! !

!UiPropertyItemNode class methodsFor: 'instance creation' stamp: 'mt 1/25/2013 14:46:03.447'!
newTreeFromPaths: somePaths
	"Create a tree structure from given paths which are lists of nodes."

	^ self new
		addDefaultGroup;
		addTreeFromPaths: somePaths;
		yourself  ! !

!UiPropertyItemNode class methodsFor: 'instance creation' stamp: 'mt 1/25/2013 14:46:08.621'!
newTreeFromPaths: somePaths withProperties: someBlocks
	"Create a tree structure from given paths which are lists of nodes."

	^ self new
		addDefaultGroup;
		addTreeFromPaths: somePaths withProperties: someBlocks;
		yourself  ! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 1/25/2013 15:15:39.167'!
addAllChildren: someNodes
		
	^ self addAllChildren: someNodes inGroup: self defaultGroup! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 1/25/2013 15:15:34.062'!
addAllChildren: someNodes inGroup: aGroup

	(self children at: aGroup) addAll: someNodes.
	
	self
		childrenInsertedFrom: (self childCountInGroup: aGroup) - someNodes size + 1
		to: (self childCountInGroup: aGroup)
		inGroup: aGroup.
		
	^ someNodes! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 1/25/2013 15:15:44.393'!
addChild: aNode

	^ self addChild: aNode inGroup: self defaultGroup! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 1/25/2013 15:15:48.939'!
addChild: aNode beforeIndex: anIndex

	^ self addChild: aNode beforeIndex: anIndex inGroup: self defaultGroup! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 1/25/2013 15:15:54.969'!
addChild: aNode beforeIndex: anIndex inGroup: aGroup

	(self children at: aGroup) add: aNode beforeIndex: anIndex.
	self childInserted: anIndex inGroup: aGroup.
	^ aNode! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 1/25/2013 16:00:51.568'!
addChild: aNode inGroup: aGroup

	deferAdd == true ifTrue: [
		self addChildDeferred: aNode inGroup: aGroup.
		^ #deferred].

	(self children at: aGroup) add: aNode.
	self childInserted: (self children at: aGroup) size inGroup: aGroup.
	^ aNode! !

!UiPropertyItemNode methodsFor: 'adding - deferred' stamp: 'mt 1/25/2013 16:01:24.132'!
addChildDeferred: aNode inGroup: aGroup
	"Private. Use #addChild:inGroup: with a prior call to #beginDeferAdd."

	deferredAdditions add: aNode -> aGroup.
	deferredAdditions size >= self deferThreshold
		ifTrue: [self processDeferredAdditions].! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 1/25/2013 14:32:59.318'!
addDefaultGroup

	self addGroup: self defaultGroup.! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 1/25/2013 14:09:07.126'!
addGroup: aGroup

	(self children includesKey: aGroup)
		ifTrue: [^ self].
		  
	self groups add: aGroup.
	self children at: aGroup put: OrderedCollection new.
	
	self modelReset.! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 7/21/2012 17:18:16.822'!
addTreeFromPath: path

	self addTreeFromPath: path withProperties: {}.! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 7/18/2013 08:09:57.517'!
addTreeFromPath: path withProperties: someBlocks

	| currentChild |
	path ifEmpty: [^ self].

	currentChild := (self children at: self defaultGroup)
		detect: [:child | child data == path first]  
		ifNone: [
			| newChild |
			newChild := self class new
				addDefaultGroup;
				data: path first;
				setProperties: someBlocks withObject: path first.
			self addChild: newChild].
		
	currentChild addTreeFromPath: path allButFirst withProperties: someBlocks.! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 7/21/2012 17:18:33.446'!
addTreeFromPaths: somePaths

	self addTreeFromPaths: somePaths withProperties: {}! !

!UiPropertyItemNode methodsFor: 'adding' stamp: 'mt 7/21/2012 17:11:36.877'!
addTreeFromPaths: somePaths withProperties: someBlocks

	somePaths do: [:path | self addTreeFromPath: path withProperties: someBlocks].! !

!UiPropertyItemNode methodsFor: 'as yet unclassified' stamp: 'mt 8/8/2013 16:12:30.849'!
allNodes

	^ self allNodesInto: OrderedCollection new! !

!UiPropertyItemNode methodsFor: 'as yet unclassified' stamp: 'mt 8/23/2013 13:25:51.623'!
allNodesInto: result

	result
		add: self
		ifPresent: [^ result "avoid cycles"].
	
	self children collect: [:nodes |
		nodes do: [:node |
			node allNodesInto: result]].
	
	^ result! !

!UiPropertyItemNode methodsFor: 'dictionary' stamp: 'mt 1/25/2013 14:26:32.341'!
at: aProperty put: anObject

	| result wasChanged |
	(self properties includes: aProperty)
		ifFalse: [self properties add: aProperty].
	
	wasChanged := (self at: aProperty ifAbsent: []) ~= anObject.
	
	result := super at: aProperty put: anObject.
	wasChanged ifTrue: [self propertyChanged: aProperty].
	^ result! !

!UiPropertyItemNode methodsFor: 'adding - deferred' stamp: 'mt 1/25/2013 15:58:48.732'!
beginDeferAdd

	deferAdd := true.
	deferredAdditions := OrderedCollection new.! !

!UiPropertyItemNode methodsFor: 'item model - hierarchy' stamp: 'mt 1/25/2013 14:04:13.961'!
childAt: index
	
	^ self childAt: index inGroup: self defaultGroup! !

!UiPropertyItemNode methodsFor: 'item model - hierarchy' stamp: 'mt 1/25/2013 14:02:25.767'!
childAt: index inGroup: groupName
	
	^ (self children at: groupName) at: index! !

!UiPropertyItemNode methodsFor: 'item model - hierarchy' stamp: 'mt 1/25/2013 14:01:58.502'!
childCountInGroup: groupName

	^ (self children at: groupName) size! !

!UiPropertyItemNode methodsFor: 'private' stamp: 'mt 1/25/2013 13:57:42.192'!
children

	^ children ifNil: [children := Dictionary new]! !

!UiPropertyItemNode methodsFor: 'item model - hierarchy' stamp: 'mt 1/29/2013 17:55:46.71'!
childrenInDefaultGroup

	^ self children at: self defaultGroup! !

!UiPropertyItemNode methodsFor: 'accessing' stamp: 'mt 2/9/2012 15:14:45.991'!
data
	"Universal data entry."
	
	^ self at: #data ifAbsent: []! !

!UiPropertyItemNode methodsFor: 'accessing' stamp: 'mt 2/9/2012 15:14:53.385'!
data: anObject
	"Universal data entry."
	
	self at: #data put: anObject.! !

!UiPropertyItemNode methodsFor: 'accessing' stamp: 'mt 1/25/2013 14:03:33.646'!
defaultGroup

	^ #children ! !

!UiPropertyItemNode methodsFor: 'adding - deferred' stamp: 'mt 1/25/2013 15:46:48.607'!
deferThreshold

	^ deferThreshold ifNil: [deferThreshold := 10]! !

!UiPropertyItemNode methodsFor: 'adding - deferred' stamp: 'mt 1/25/2013 15:46:58.892'!
deferThreshold: anInteger

	deferThreshold := anInteger.! !

!UiPropertyItemNode methodsFor: 'removing' stamp: 'mt 1/27/2013 18:59:14.204'!
delete
	"Try to self-destruct this node."
	
	self deleted.! !

!UiPropertyItemNode methodsFor: 'signals' stamp: 'mt 1/27/2013 18:57:57.412'!
deleted

	self emit.! !

!UiPropertyItemNode methodsFor: 'error handling' stamp: 'mt 8/14/2013 10:56:09.495'!
doesNotUnderstand: aMessage

	aMessage arguments size > 1 ifTrue: [
		^ super doesNotUnderstand: aMessage].
	
	aMessage arguments ifEmpty: [
		^ self
			at: aMessage selector
			ifAbsent: [super doesNotUnderstand: aMessage]].
	
	^ self
		at: aMessage selector allButLast
		put: aMessage arguments first! !

!UiPropertyItemNode methodsFor: 'adding - deferred' stamp: 'mt 1/25/2013 17:22:49.093'!
endDeferAdd

	deferAdd := false.
	deferredAdditions ifNotNil: [
		self processDeferredAdditions.
		deferredAdditions := nil].! !

!UiPropertyItemNode methodsFor: 'item model - hierarchy' stamp: 'mt 1/25/2013 14:01:22.114'!
groups
	
	^  groups ifNil: [groups := OrderedCollection new]! !

!UiPropertyItemNode methodsFor: 'item model - hierarchy' stamp: 'mt 1/25/2013 14:24:44.023'!
groups: someGroups
	
	groups := someGroups asOrderedCollection.
	
	self children removeAll.
	someGroups do: [:group | self children at: group put: OrderedCollection new].
	
	self modelReset.! !

!UiPropertyItemNode methodsFor: 'item model - hierarchy' stamp: 'mt 8/1/2013 11:35:56.472'!
hasChildren

	^ self children anySatisfy: [:nodes | nodes notEmpty]! !

!UiPropertyItemNode methodsFor: 'adding - deferred' stamp: 'mt 1/25/2013 17:01:29.133'!
processDeferredAdditions
	"Private. Use #beginDeferAdd, #addChild:inGroup:, and #endDeferAdd."
	
	| changedGroups oldSizes |
	changedGroups := OrderedCollection new.
	oldSizes := OrderedCollection new.
	deferredAdditions do: [:nodeToGroup | | group |
		group := nodeToGroup value.
		(changedGroups includes: group) ifFalse: [
			oldSizes add: (self childCountInGroup: group).
			changedGroups add: group].
		
		(self children at: group) add: nodeToGroup key].
	
	changedGroups with: oldSizes do: [:group :oldSize |
		self childrenInsertedFrom: oldSize + 1 to: (self childCountInGroup: group)  inGroup: group].
	
	deferredAdditions removeAll.! !

!UiPropertyItemNode methodsFor: 'item model - properties' stamp: 'mt 1/25/2013 14:01:05.103'!
properties
	"Return an ordered list of all known properties. This list can then be used to configure column specifications of item views more conveniently."
	
	^ properties ifNil: [properties := OrderedCollection new]! !

!UiPropertyItemNode methodsFor: 'removing' stamp: 'mt 1/25/2013 15:40:13.443'!
removeAllGroups

	self children removeAll.
	groups := nil.
	
	self modelReset.! !

!UiPropertyItemNode methodsFor: 'removing' stamp: 'mt 1/27/2013 19:15:42.294'!
removeChild: aNode 
	
	^ self removeChild: aNode inGroup: self defaultGroup! !

!UiPropertyItemNode methodsFor: 'removing' stamp: 'mt 1/25/2013 15:16:57.696'!
removeChild: aNode inGroup: aGroup

	^ self removeChildAt: ((self children at: aGroup) indexOf: aNode) inGroup: aGroup! !

!UiPropertyItemNode methodsFor: 'removing' stamp: 'mt 1/27/2013 19:15:55.206'!
removeChildAt: anIndex
	
	^ self removeChildAt: anIndex inGroup: self defaultGroup! !

!UiPropertyItemNode methodsFor: 'removing' stamp: 'mt 1/25/2013 15:17:37.273'!
removeChildAt: anIndex inGroup: aGroup

	| removedNode |
	removedNode := (self children at: aGroup) removeAt: anIndex.
	self childRemoved: anIndex inGroup: aGroup.
	^ removedNode! !

!UiPropertyItemNode methodsFor: 'removing' stamp: 'mt 1/25/2013 14:20:52.19'!
removeGroup: aGroup

	self groups remove: aGroup.
	self children removeKey: aGroup.
	
	self modelReset.! !

!UiPropertyItemNode methodsFor: 'dictionary' stamp: 'mt 8/29/2013 13:37:51.131'!
removeKey: key ifAbsent: aBlock

	| result wasRemoved |
	wasRemoved := true.
	self properties remove: key ifAbsent: [wasRemoved := false].
	
	result := super removeKey: key ifAbsent: aBlock.
	wasRemoved ifTrue: [self propertyChanged: key].
	^ result! !

!UiPropertyItemNode methodsFor: 'removing' stamp: 'mt 1/27/2013 19:17:13.622'!
removeLast
	
	^ self removeLastInGroup: self defaultGroup! !

!UiPropertyItemNode methodsFor: 'removing' stamp: 'mt 1/27/2013 19:16:56.604'!
removeLastInGroup: aGroup
	
	^ self removeChildAt: (self childCountInGroup: aGroup) inGroup: aGroup! !

!UiPropertyItemNode methodsFor: 'testing' stamp: 'mt 7/5/2012 15:37:41.371'!
respondsTo: aSelector
	"Map keys to messages/accessors."

	^ (super respondsTo: aSelector) or: [(aSelector endsWith: ':')
		ifTrue: [self includesKey: aSelector allButLast]
		ifFalse: [self includesKey: aSelector]]! !

!UiPropertyItemNode methodsFor: 'item model - properties' stamp: 'mt 8/22/2013 13:04:31.865'!
setProperties: somePropertySpecs withObject: anObject

	self flag: #obsolete.
	somePropertySpecs do: [:spec | self setProperty: spec withObject: anObject].! !

!UiPropertyItemNode methodsFor: 'item model - properties' stamp: 'mt 8/22/2013 13:04:23.39'!
setProperty: aPropertySpec withObject: anObject
	"The specification of a property can be differently shaped:
	a) propertyName -> [:obj | #text -> obj asString].
	b) propertyName -> [:obj | {#text -> obj asString. #icon -> obj asMorph imageForm}].
	c) propertyName -> [:obj | obj asString].
	d) [:obj | obj asString].
	e) [:obj | #text -> obj asString].
	..."

	| propKey propValue |
	self flag: #obsolete.
	propKey := ViQueryModel propertyFromSpec: aPropertySpec key.
	propValue := aPropertySpec value "block" value: anObject.
	propValue isCollection
		"list of associations"
		ifTrue: [propValue do: [:typeToValue |
			self at: propKey, typeToValue key put: typeToValue value]]
		"single association"
		ifFalse: [self at: propKey, propValue key put: propValue value].! !

!OrderedCollection methodsFor: '*widgets-support' stamp: 'mt 10/9/2010 15:41:33.496'!
reverseDetect: aBlock

	^ self reverseDetect: aBlock ifNone: [self errorNotFound: aBlock]! !

!OrderedCollection methodsFor: '*widgets-support' stamp: 'mt 10/9/2010 15:41:12.167'!
reverseDetect: aBlock ifNone: exceptionBlock

	self reverseDo: [:each | (aBlock value: each) ifTrue: [^ each]].
	^ exceptionBlock value	! !

LayoutPolicy subclass: #UiComboBoxLayout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Layouts'!

!UiComboBoxLayout methodsFor: 'layout' stamp: 'mt 1/6/2012 10:46:28.532'!
layout: aComboBox in: newBounds

	aComboBox submorphsDo: [:s | s fullBounds].

	aComboBox button topRight: newBounds topRight.
	aComboBox arrowButton topRight: newBounds topRight.
	
	aComboBox lineEdit topLeft: newBounds topLeft.
	aComboBox readOnlyButton topLeft: newBounds topLeft.
	
	aComboBox readOnlyButton width: newBounds width - aComboBox button width - 1.
	aComboBox lineEdit width: newBounds width - aComboBox button width - 1.
	
	"vertical shrink wrap"
	aComboBox height: aComboBox readOnlyButton height.

	aComboBox button height: aComboBox readOnlyButton height.
	aComboBox arrowButton height: aComboBox readOnlyButton height.
	aComboBox lineEdit height: aComboBox readOnlyButton height.! !

!Morph class methodsFor: '*widgets-designer' stamp: 'mt 2/12/2011 12:26:11.435'!
isExperimentalWidgetClass
	"Indicate whether this widget CAN NOT be expected as feature-complete and stable and therefore SHOULD NOT BE USED."
	
	^ false! !

!Morph class methodsFor: '*widgets-designer' stamp: 'mt 6/28/2010 21:31:27.001'!
isWidgetClass

	^ false! !

!Morph class methodsFor: '*widgets-designer' stamp: 'mt 10/19/2010 13:31:11.159'!
widgetCategory
	
	^ 'Classic Morphs'! !

!Morph class methodsFor: '*widgets-designer' stamp: 'mt 10/19/2010 11:46:11.254'!
widgetClassIcon

	^ nil! !

!Morph class methodsFor: '*widgets-designer' stamp: 'mt 10/19/2010 13:38:38.799'!
widgetClassName

	^ self name! !

!Morph class methodsFor: '*widgets-designer' stamp: 'mt 10/28/2010 16:54:01.158'!
widgetProperties
	"Describe custom properties for this type of widget."
	
	^ {{'Morph'. #yourself} -> {
		#balloonText -> String.
		#balloonColor -> Object}}! !

!Morph class methodsFor: '*widgets-designer' stamp: 'mt 7/16/2010 11:22:50.821'!
widgetPropertiesToIgnore
	"Describe properties that should not appear in the designer."
	
	^ #()! !

!Morph methodsFor: '*widgets' stamp: 'mt 4/29/2011 15:07:55.523'!
balloonText: stringOrText

	^ self setBalloonText: (stringOrText
		ifNotNil: [:s | s ifEmpty: []])! !

!Morph methodsFor: '*widgets-signals-processing' stamp: 'mt 6/14/2012 09:09:15.465'!
disconnectAllSignals

	self removeProperty: #signalConnections.! !

!Morph methodsFor: '*widgets-events' stamp: 'mt 1/2/2012 17:24:01.244'!
enabled
	"Check whether this morph is capable of proecssing user input events."
	
	^ self
		valueOfProperty: #enabled
		ifAbsentPut: [true]! !

!Morph methodsFor: '*widgets-events' stamp: 'mt 1/2/2012 17:23:37.826'!
enabled: aBoolean

	self setProperty: #enabled toValue: aBoolean.
	self redrawLater.! !

!Morph methodsFor: '*widgets-geometry-override' stamp: 'mt 6/14/2012 09:18:57.546'!
extent: aPoint

	(bounds extent closeTo: aPoint) ifTrue: [^ self].
	self changed.
	bounds := bounds topLeft extent: aPoint.
	self layoutChanged.
	self changed.

	"Widgets: Emit extent change."
	self extentChanged: aPoint.! !

!Morph methodsFor: '*widgets-signals' stamp: 'mt 6/14/2012 09:18:29.676'!
extentChanged: aPoint

	self emit.! !

!Morph methodsFor: '*widgets' stamp: 'mt 10/27/2010 20:21:50.604'!
frameFractions

	self layoutFrame ifNil: [^ 0@0 corner: 1@1].
	^ self layoutFrame leftFraction @ self layoutFrame topFraction
		corner:
		self layoutFrame rightFraction @ self layoutFrame bottomFraction! !

!Morph methodsFor: '*widgets' stamp: 'mt 10/27/2010 20:29:01.842'!
frameFractions: aRectangle

	| frame |
	frame := self layoutFrame ifNil: [LayoutFrame new].
	frame
		topFraction: aRectangle top;
		bottomFraction: aRectangle bottom;
		leftFraction: aRectangle left;
		rightFraction: aRectangle right.
	self layoutFrame: frame.
	self layoutChanged.! !

!Morph methodsFor: '*widgets' stamp: 'mt 10/28/2010 10:48:13.066'!
frameOffsets

	self layoutFrame ifNil: [^ 0@0 corner: 0@0].
	^ (self layoutFrame leftOffset ifNil: [0]) @ (self layoutFrame topOffset ifNil: [0])
		corner:
		(self layoutFrame rightOffset ifNil: [0]) @ (self layoutFrame bottomOffset ifNil: [0])! !

!Morph methodsFor: '*widgets' stamp: 'mt 10/27/2010 20:28:56.456'!
frameOffsets: aRectangle

	| frame |
	frame := self layoutFrame ifNil: [LayoutFrame new].
	frame
		topOffset: aRectangle top;
		bottomOffset: aRectangle bottom;
		leftOffset: aRectangle left;
		rightOffset: aRectangle right.
	self layoutFrame: frame.
	self layoutChanged.! !

!Morph methodsFor: '*widgets' stamp: 'mt 4/20/2011 13:41:23.07'!
fullDrawCacheOn: aCanvas

	^ self fullDrawOn: aCanvas  ! !

!Morph methodsFor: '*widgets-events' stamp: 'mt 5/11/2011 11:45:11.168'!
grabKeyboard

	self hasKeyboard ifTrue: [^ self].
	
	self activeHand newKeyboardFocus: self.  ! !

!Morph methodsFor: '*widgets-events' stamp: 'mt 2/26/2011 14:10:23.137'!
grabMouse

	self activeHand newMouseFocus: self.! !

!Morph methodsFor: '*widgets-events-testing' stamp: 'mt 6/22/2011 14:17:34.896'!
hasAnyKeyboard

	^ self hasKeyboard or: [self hasSubmorphKeyboard]! !

!Morph methodsFor: '*widgets-events-testing' stamp: 'mt 2/26/2011 14:14:31.513'!
hasKeyboard

	^ self activeHand keyboardFocus = self! !

!Morph methodsFor: '*widgets-events-testing' stamp: 'mt 2/26/2011 14:14:44.687'!
hasMouse

	^ self activeHand mouseFocus = self! !

!Morph methodsFor: '*widgets-signals-processing' stamp: 'mt 6/14/2012 09:08:56.887'!
hasSignalConnections

	^ true! !

!Morph methodsFor: '*widgets-events-testing' stamp: 'mt 6/22/2011 14:17:56.099'!
hasSubmorphKeyboard

	^ self submorphs anySatisfy: [:morph | morph hasAnyKeyboard]! !

!Morph methodsFor: '*widgets' stamp: 'mt 7/22/2012 09:38:21.432'!
hideFull
	"The hidden morph will not be drawn AND does not consume space in table layouts."
	
	self
		hide;
		disableTableLayout: true.
	
	"Workaround if #fullBounds are nil. See Morph>>#layoutChanged. Owner MUST BE notified."
	self owner ifNotNil: [:o |
		o layoutChanged].! !

!Morph methodsFor: '*widgets' stamp: 'mt 6/13/2012 17:15:16.643'!
isTransformMorph

	^ false! !

!Morph methodsFor: '*widgets' stamp: 'mt 10/27/2010 20:38:40.152'!
isUiContainer

	^ false! !

!Morph methodsFor: '*widgets-events-override' stamp: 'mt 2/26/2011 13:51:41.519'!
keyDown: anEvent
	"Handle a key down event. "
	
	self eventHandler isEventHandler
		ifTrue: [self eventHandler keyDown: anEvent fromMorph: self].! !

!Morph methodsFor: '*widgets-events-override' stamp: 'mt 2/26/2011 13:51:26.514'!
keyUp: anEvent
	"Handle a key up event. "
	
	self eventHandler isEventHandler
		ifTrue: [self eventHandler keyUp: anEvent fromMorph: self].! !

!Morph methodsFor: '*widgets-events-override' stamp: 'mt 2/26/2011 14:03:16.929'!
keyboardFocusChange: aBoolean
	"The message is sent to a morph when its keyboard focus change. The given argument indicates that the receiver is gaining keyboard focus (versus losing) the keyboard focus. Morphs that accept keystrokes should change their appearance in some way when they are the current keyboard focus."
	
	self eventHandler isEventHandler
		ifTrue: [self eventHandler keyboardFocusChange: aBoolean fromMorph: self].! !

!Morph methodsFor: '*widgets-events' stamp: 'mt 5/6/2011 10:29:18.815'!
keyboardFocusChange: aBoolean inSubmorph: aMorph
	"Give the owner a chance to update appearance if child changes focus."
	
	self eventHandler isEventHandler
		ifTrue: [self eventHandler
			keyboardFocusChange: aBoolean
			inSubmorph: aMorph
			fromMorph: self]
		ifFalse: [self owner ifNotNil: [:owner |
			owner
				keyboardFocusChange: aBoolean
				inSubmorph: aMorph]].! !

!Morph methodsFor: '*widgets-events-override' stamp: 'mt 2/26/2011 13:46:30.171'!
mouseDown: evt 
	"Overridden to suppress #yellowButtonActivity for specific event handlers."

	self eventHandler isEventHandler
		ifTrue: [^ self eventHandler mouseDown: evt fromMorph: self].

	evt yellowButtonPressed
		ifTrue: ["First check for option (menu) click"
			^ self yellowButtonActivity: evt shiftPressed].
	self eventHandler
		ifNotNil: [self eventHandler mouseDown: evt fromMorph: self]
! !

!Morph methodsFor: '*widgets-events-override' stamp: 'mt 9/28/2012 11:19:06.231'!
mouseDownPriority
	"Return the default mouse down priority for the receiver"

	self eventHandler isEventHandler
		ifTrue: [^ self eventHandler mouseDownPriority].

	^ 0
	"(self isPartsDonor or: [self isPartsBin])
		ifTrue:	[50]
		ifFalse:	[0]
"
	"The above is a workaround for the complete confusion between parts donors and parts bins. Morphs residing in a parts bin may or may not have the parts donor property set; if they have they may or may not actually handle events. To work around this, parts bins get an equal priority to parts donors so that when a morph in the parts bin does have the property set but does not handle the event we still get a copy from picking it up through the parts bin. Argh. This just *cries* for a cleanup."
	"And the above comment is Andreas's from 10/2000, which was formerly retrievable by a #flag: call which however caused a problem when trying to recompile the method from decompiled source."! !

!Morph methodsFor: '*widgets-signals' stamp: 'mt 6/14/2012 09:14:46.869'!
positionChanged: aPoint

	self emit.! !

!Morph methodsFor: '*widgets-geometry-override' stamp: 'mt 9/28/2012 11:12:50.838'!
privateMoveBy: delta 
	"Private!! Use 'position:' instead."
	| fill |
	self flag: #pharo. "mt: Check for #player to be compatible with Pharo."
	((self respondsTo: #player) and: [self player notNil]) ifTrue: ["Most cases eliminated fast by above test"
		self getPenDown ifTrue: [
			"If this is a costume for a player with its 
			pen down, draw a line."
			self moveWithPenDownBy: delta]].
	bounds := bounds translateBy: delta.
	fullBounds ifNotNil: [fullBounds := fullBounds translateBy: delta].
	fill := self fillStyle.
	fill isOrientedFill ifTrue: [fill origin: fill origin + delta].
	
	"Widgets: Emit position change."
	self positionChanged: self position.! !

!Morph methodsFor: '*widgets-events' stamp: 'mt 2/26/2011 14:11:25.922'!
releaseKeyboard

	self activeHand newKeyboardFocus: nil.! !

!Morph methodsFor: '*widgets-events' stamp: 'mt 2/26/2011 14:11:13.349'!
releaseMouse

	self activeHand newMouseFocus: nil.! !

!Morph methodsFor: '*widgets' stamp: 'mt 7/22/2012 09:38:31.54'!
showFull
	"The morph is visible again and considered in table layouts."
	
	self
		show;
		disableTableLayout: false.
	
	"Workaround if #fullBounds are nil. See Morph>>#layoutChanged. Owner MUST BE notified."
	self owner ifNotNil: [:o |
		o layoutChanged].! !

!Morph methodsFor: '*widgets-signals-processing' stamp: 'mt 6/15/2012 10:26:49.45'!
signalConnections

	^ self
		valueOfProperty: #signalConnections
		ifAbsentPut: [Dictionary new]! !

!Morph methodsFor: '*widgets-signals-processing' stamp: 'mt 6/14/2012 09:07:46.521'!
signalMutex

	^ UiNullMutex! !

!Morph methodsFor: '*widgets-geometry' stamp: 'mt 9/19/2012 10:20:42.756'!
visibleBounds
	"Computes the rectangle that is not occluded and actually visible on the screen."

	| globalBounds |
	globalBounds := self boundsInWorld.
	  
	^ self owner
		ifNotNil: [self owner visibleBounds intersect: globalBounds]
		ifNil: [globalBounds]! !

Morph subclass: #UiAbstractSlider
	instanceVariableNames: 'currentValue minimum maximum orientation pageStep singleStep sliderDown sliderPosition tracking truncate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiAbstractSlider commentStamp: 'mt 2/11/2011 15:21:00.539' prior: 0!
This is a Squeak port of QAbstractSlider from the Nokia/Qt framework.!

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 10:44:02.309'!
currentValue

	^ currentValue ifNil: [0]! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 11:09:53.675'!
currentValue: aNumber

	| guardedValue |
	guardedValue := self guardedValue: aNumber.
	
	guardedValue = self currentValue ifTrue: [^ self].

	currentValue := guardedValue.
	
	self
		sliderChange: #valueChange;
		valueChanged: self currentValue.! !

!UiAbstractSlider methodsFor: 'actions' stamp: 'mt 2/26/2011 14:37:43.316'!
decreaseValueByPageStep

	self currentValue: self currentValue - self pageStep.  ! !

!UiAbstractSlider methodsFor: 'actions' stamp: 'mt 2/26/2011 14:37:32.985'!
decreaseValueBySingleStep

	self currentValue: self currentValue - self singleStep.  ! !

!UiAbstractSlider methodsFor: 'guarding' stamp: 'mt 2/25/2011 11:15:26.423'!
guardedValue: aNumber

	^ (self truncate
		ifTrue: [aNumber ceiling asInteger]
		ifFalse: [aNumber])
		min: self maximum
		max: self minimum! !

!UiAbstractSlider methodsFor: 'testing' stamp: 'mt 2/11/2011 16:30:22.079'!
hasTracking
	"Is #valueChanged: being emitted while the slider is dragged?"
	
	^ tracking! !

!UiAbstractSlider methodsFor: 'actions' stamp: 'mt 2/26/2011 14:37:48.965'!
increaseValueByPageStep

	self currentValue: self currentValue + self pageStep.  ! !

!UiAbstractSlider methodsFor: 'actions' stamp: 'mt 2/26/2011 14:37:26.474'!
increaseValueBySingleStep

	self currentValue: self currentValue + self singleStep.  ! !

!UiAbstractSlider methodsFor: 'initialization' stamp: 'mt 3/25/2011 14:21:27.102'!
initialize

	super initialize.
	
	tracking := true.
	orientation := #horizontal.
	minimum := 0.
	maximum := 100.
	pageStep := 10.
	currentValue := 0.
	sliderPosition := 0.
	singleStep := 1.
	
	self
		color: Color transparent;
		extent: 100@14.! !

!UiAbstractSlider methodsFor: 'testing' stamp: 'mt 2/11/2011 16:19:29.364'!
isSliderDown

	^ sliderDown! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 10:44:12.921'!
maximum

	^ maximum ifNil: [0]! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 11:14:42.57'!
maximum: aNumber

	| guardedValue |
	guardedValue := self truncate
		ifTrue: [aNumber ceiling asInteger]
		ifFalse: [aNumber].
	
	guardedValue = self maximum ifTrue: [^ self].
	
	maximum := guardedValue.
	
	self
		sliderChange: #rangeChange;
		rangeChanged: self range.! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 10:44:17.153'!
minimum

	^ minimum ifNil: [0]! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 11:14:39.382'!
minimum: aNumber

	| guardedValue |
	guardedValue := self truncate
		ifTrue: [aNumber ceiling asInteger]
		ifFalse: [aNumber].
		
	guardedValue = self minimum ifTrue: [^ self].
	
	minimum := guardedValue.
	
	self
		sliderChange: #rangeChange;
		rangeChanged: self range.! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/11/2011 15:30:01.97'!
orientation

	^ orientation! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/11/2011 15:59:09.449'!
orientation: aSymbol
	"#horizontal, #vertical"

	aSymbol = self orientation ifTrue: [^ self].

	orientation := aSymbol.
	
	self sliderChange: #orientationChange.! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 3/1/2011 14:22:00.471'!
pageStep

	^ pageStep ifNil: [0]! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 4/12/2011 11:32:46.858'!
pageStep: anInteger

	| guardedValue |
	guardedValue := anInteger max: 0.
	
	guardedValue = self pageStep ifTrue: [^ self].

	pageStep := guardedValue.
	
	self sliderChange: #stepsChange.! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/11/2011 15:47:10.587'!
range
	
	^ self minimum to: self maximum! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 11:14:31.742'!
range: anInterval

	| guardedValue |
	guardedValue := self truncate
		ifTrue: [anInterval first ceiling asInteger to: anInterval last ceiling asInteger]
		ifFalse: [anInterval].
		
	guardedValue = self range ifTrue: [^ self].
	
	minimum := guardedValue first.
	maximum := guardedValue last.
	
	self
		sliderChange: #rangeChange;
		rangeChanged: self range.! !

!UiAbstractSlider methodsFor: 'signals' stamp: 'mt 2/11/2011 15:42:27.109'!
rangeChanged: anInterval

	self emit.! !

!UiAbstractSlider methodsFor: 'actions' stamp: 'mt 2/26/2011 17:17:26.272'!
setValueToMaximum
	
	self currentValue: self maximum.  ! !

!UiAbstractSlider methodsFor: 'actions' stamp: 'mt 2/26/2011 17:17:18.761'!
setValueToMinimum
	
	self currentValue: self minimum.  ! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/11/2011 15:30:02.221'!
singleStep

	^ singleStep! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/11/2011 16:18:02.19'!
singleStep: anInteger

	anInteger = self singleStep ifTrue: [^ self].

	singleStep := anInteger.
	
	self sliderChange: #stepsChange.! !

!UiAbstractSlider methodsFor: 'updating' stamp: 'mt 3/5/2011 12:14:02.902'!
sliderChange: aSliderChange
	"This is a local callback. Possible values: #rangeChange, #orientationChange, #stepsChange, #valueChange"
	
	aSliderChange = #valueChange ifTrue: [
		self sliderPosition: self currentValue].
	
	aSliderChange = #rangeChange ifTrue: [
		self sliderPosition: self sliderPosition.
		self currentValue: self currentValue].
	
	aSliderChange = #orientationChange ifTrue: [
		self extent: self height @ self width].! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/19/2011 14:13:04.16'!
sliderDown: aBoolean

	aBoolean = self isSliderDown ifTrue: [^ self].

	sliderDown := aBoolean.
	
	"Synchronize on slider release."
	aBoolean
		ifFalse: [self currentValue: self sliderPosition].
	
	aBoolean
		ifTrue: [self sliderPressed]
		ifFalse: [self sliderReleased].! !

!UiAbstractSlider methodsFor: 'signals' stamp: 'mt 2/11/2011 15:43:18.329'!
sliderMoved: anInteger

	self emit.! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 10:44:22.745'!
sliderPosition

	^ sliderPosition ifNil: [0]! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 11:10:16.198'!
sliderPosition: aNumber

	| guardedValue |
	guardedValue := self guardedValue: aNumber.
	
	guardedValue = self sliderPosition ifTrue: [^ self].
	
	sliderPosition := guardedValue.
	
	self hasTracking ifTrue: [
		self currentValue: self sliderPosition].
	
	self sliderMoved: self sliderPosition.! !

!UiAbstractSlider methodsFor: 'signals' stamp: 'mt 2/11/2011 15:43:27.704'!
sliderPressed

	self emit.! !

!UiAbstractSlider methodsFor: 'signals' stamp: 'mt 2/11/2011 15:43:33.933'!
sliderReleased

	self emit.! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:24:46.16'!
tracking
	
	^ tracking! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/19/2011 11:11:08.734'!
tracking: aBoolean
	"Lock the #sliderPosition to the #currentValue."
	
	tracking := aBoolean.! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 11:05:58.445'!
truncate
	"Are only integer values allowed?"
	
	^ truncate ifNil: [true]! !

!UiAbstractSlider methodsFor: 'accessing' stamp: 'mt 2/25/2011 11:07:12.284'!
truncate: aBoolean

	aBoolean = truncate ifTrue: [^ self].

	truncate := aBoolean.
	
	self truncate ifTrue: [
		self
			range: self range;
			currentValue: self currentValue].! !

!UiAbstractSlider methodsFor: 'signals' stamp: 'mt 2/11/2011 15:43:43.887'!
valueChanged: anInteger

	self emit.! !

UiAbstractSlider subclass: #UiSlider
	instanceVariableNames: 'pagingMorph sliderButton mouseDownSliderPosition mouseDownEventPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiSlider commentStamp: 'mt 2/27/2011 16:32:16.852' prior: 0!
pagingMorph ... a simple morph in background that indicates the range of this slider
sliderButton ... a button that can be dragged to change the slider position and the current slider value

Used to compute the new slider position during the dragging operation:

mouseDownSliderPosition ... value of the slider when the mouseDown event occured
mouseDownEventPosition ... pixel position of the mouseDown event when it occured

Morphic stepping is used for automatic page-step moving.

[Hold Left Click] ... scroll by page step up to click position
[Shift] + [Left Click] ... jump to specific value!

UiSlider subclass: #UiScrollBar
	instanceVariableNames: 'decreaseValueButton1 decreaseValueButton2 increaseValueButton outerBorderWidth buttonMode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiScrollBar commentStamp: 'mt 2/27/2011 16:31:15.05' prior: 0!
!!!!!! Rename to UiScrollBar !!!!!!

A scroll bar is like a slider that as additional buttons to modify the current value. In addition, the slider extent is changed with respect to the current scroll bar #extent, #orientation, and #pageStep.!

!UiScrollBar class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:48.967'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiScrollBarHorizontalIcon! !

!UiScrollBar class methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:18:27.526'!
widgetClassName

	^ 'Scroll Bar'! !

!UiScrollBar class methodsFor: 'accessing' stamp: 'mt 4/5/2011 15:58:43.309'!
widgetProperties

	^ super widgetProperties, {{'Scroll Bar'. #yourself} -> {
		#buttonMode -> #(none distant adjacent both).
		}}! !

!UiScrollBar methodsFor: 'accessing' stamp: 'mt 5/16/2012 10:28:50.47'!
buttonMode

	^ buttonMode! !

!UiScrollBar methodsFor: 'accessing' stamp: 'mt 5/16/2012 09:52:27.669'!
buttonMode: aSymbol
	"Specifiies how many arrow buttons are visible to modify the value with mouse clicks
	#distant ... one arrow button at each end of the scroll bar
	#adjacent ... both arrow buttons at the higher end of the scroll bar (like on Mac OS X)
	#both ... all three arrow buttons (like Cleanlooks in Gnome/Linux)
	#none ... hide all buttons"

	buttonMode = aSymbol ifTrue: [^ self].

	buttonMode := aSymbol.
	
	self
		initializeButtons;
		updateSupportMorphs.! !

!UiScrollBar methodsFor: 'geometry' stamp: 'mt 5/16/2012 10:07:21.643'!
defaultThickness

	^ 14! !

!UiScrollBar methodsFor: 'actions' stamp: 'mt 3/2/2011 09:17:57.288'!
doSliderMovement: aMouseMoveEvent
	"Only allow integer steps with mouse dragging."
	
	| value |
	self isStepping
		ifTrue: [^ mouseDownEventPosition := aMouseMoveEvent position].
	
	value := self orientation = #horizontal
		ifTrue: [self pixelToValue: aMouseMoveEvent position x - mouseDownEventPosition x]
		ifFalse: [self pixelToValue: aMouseMoveEvent position y - mouseDownEventPosition y].

	self sliderPosition: mouseDownSliderPosition + (self truncate
		ifTrue: [value reduce asInteger]
		ifFalse: [value]).
		
	self raiseButtons.! !

!UiScrollBar methodsFor: 'drawing' stamp: 'mt 3/22/2011 17:18:11.583'!
drawOn: aCanvas

	super drawOn: aCanvas.
	
	self outerBorderWidth > 0 ifTrue: [
		aCanvas
			frameRectangle: self outerBounds
			width: self outerBorderWidth
			color: self color darker].! !

!UiScrollBar methodsFor: 'actions' stamp: 'mt 3/2/2011 09:21:30.987'!
finishSliderMovement: aMouseEvent

	super finishSliderMovement: aMouseEvent.  
	
	(self innerBounds containsPoint: aMouseEvent position)
		ifFalse: [self flattenButtons].! !

!UiScrollBar methodsFor: 'actions' stamp: 'mt 5/16/2012 10:24:13.597'!
flattenButtons

	mouseDownEventPosition ifNotNil: [^ self].

	{decreaseValueButton1. decreaseValueButton2. increaseValueButton}
		do: [:btn | btn ifNotNil: [btn fadeOut]].! !

!UiScrollBar methodsFor: 'initialization' stamp: 'mt 5/16/2012 10:29:00.174'!
initialize

	buttonMode := #none.
	
	super initialize.
	self color: (Color gray: 0.85).
! !

!UiScrollBar methodsFor: 'initialization' stamp: 'mt 5/16/2012 09:51:35.228'!
initializeButtons

	self buttonMode == #none
		ifTrue: [^ self].

	self initializeIncreaseValueButton.

	{#(adjacent both) -> [self initializeDecreaseValueButton2].
	#(distant both) -> [self initializeDecreaseValueButton1]} do: [:assoc |
		(assoc key includes: self buttonMode)
			ifTrue: [assoc value value]].! !

!UiScrollBar methodsFor: 'initialization' stamp: 'mt 5/16/2012 10:17:48.318'!
initializeDecreaseValueButton1

	decreaseValueButton1 ifNil: [
		decreaseValueButton1 := UiScrollBarArrowButton new.
		self connect: decreaseValueButton1 signal: #pressed toSelector: #decreaseValueBySingleStep.
		self addMorphFront: decreaseValueButton1.
		decreaseValueButton1]! !

!UiScrollBar methodsFor: 'initialization' stamp: 'mt 5/16/2012 10:18:06.922'!
initializeDecreaseValueButton2

	decreaseValueButton2 ifNil: [
		decreaseValueButton2 := UiScrollBarArrowButton new.
		self connect: decreaseValueButton2 signal: #pressed toSelector: #decreaseValueBySingleStep.
		self addMorph: decreaseValueButton2 after: pagingMorph.
		decreaseValueButton2]! !

!UiScrollBar methodsFor: 'initialization' stamp: 'mt 4/7/2011 14:25:49.362'!
initializeEventHandler

	super initializeEventHandler.
	
	self connect: self eventHandler signal: #mouseEntered: toSelector: #raiseButtons.
	self connect: self eventHandler signal: #mouseLeft: toSelector: #flattenButtons.

	"Keyboard interaction."
	self eventHandler on: Character arrowDown send: #increaseValueBySingleStep to: self.
	self eventHandler on: Character arrowUp send: #decreaseValueBySingleStep to: self.
	
	self eventHandler on: Character pageUp send: #decreaseValueByPageStep to: self.
	self eventHandler on: Character pageDown send: #increaseValueByPageStep to: self.! !

!UiScrollBar methodsFor: 'initialization' stamp: 'mt 5/16/2012 10:17:35.919'!
initializeIncreaseValueButton

	increaseValueButton ifNil: [
		increaseValueButton := UiScrollBarArrowButton new.
		self connect: increaseValueButton signal: #pressed toSelector: #increaseValueBySingleStep.
		self addMorphBack: increaseValueButton.
		increaseValueButton]! !

!UiScrollBar methodsFor: 'initialization' stamp: 'mt 5/16/2012 09:51:53.683'!
initializeSupportMorphs

	pagingMorph := Morph new
		color: Color transparent;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
			
	sliderButton := UiScrollBarSliderButton new.! !

!UiScrollBar methodsFor: 'actions' stamp: 'mt 2/26/2011 20:50:37.799'!
jumpToValue: aMouseEvent
	"Jumps to the value under the mouse cursor if shift was pressed."
	
	| value |
	aMouseEvent shiftPressed ifFalse: [^ self].
	
	self orientation = #horizontal
		ifTrue: [value := self pixelToValue: aMouseEvent position x - pagingMorph left - (sliderButton width / 2)]
		ifFalse: [value := self pixelToValue: aMouseEvent position y - pagingMorph top - (sliderButton height / 2)].
		
	self sliderPosition: value.! !

!UiScrollBar methodsFor: 'accessing' stamp: 'mt 3/22/2011 17:17:23.628'!
outerBorderWidth

	^ outerBorderWidth ifNil: [outerBorderWidth := 1]! !

!UiScrollBar methodsFor: 'accessing' stamp: 'mt 3/22/2011 17:17:47.691'!
outerBorderWidth: anInteger

	anInteger = outerBorderWidth ifTrue: [^ self].

	outerBorderWidth := anInteger.
	
	self redrawLater.! !

!UiScrollBar methodsFor: 'geometry' stamp: 'mt 1/4/2012 14:08:13.631'!
outerBounds

	^ self bounds outsetBy: self outerBorderWidth! !

!UiScrollBar methodsFor: 'converting' stamp: 'mt 3/7/2011 15:39:49.883'!
preferredSliderSize
	"Like #valueToPixel. Ignores the extent of the slider button."

	| abs |
	abs := self pageStep = 0
		ifFalse: [self pageStep / (self range extent + self pageStep)]
		ifTrue: [1].
	
	self orientation = #horizontal
		ifTrue: [^ abs * pagingMorph width].
		
	self orientation = #vertical
		ifTrue: [^ abs * pagingMorph height].
		
	^ 0! !

!UiScrollBar methodsFor: 'actions' stamp: 'mt 5/16/2012 10:24:23.568'!
raiseButtons

	{decreaseValueButton1. decreaseValueButton2. increaseValueButton}
		do: [:btn | btn ifNotNil: [btn fadeIn]].! !

!UiScrollBar methodsFor: 'updating' stamp: 'mt 2/27/2011 15:32:23.619'!
sliderChange: aSliderChange

	super sliderChange: aSliderChange.
	  
	"Slider button extent indicates pageStep."
	aSliderChange = #stepsChange
		ifTrue: [self updateSliderButton].! !

!UiScrollBar methodsFor: 'actions' stamp: 'mt 2/26/2011 19:59:15.607'!
startSliderMovement: aMouseEvent

	mouseDownEventPosition := aMouseEvent position.
	
	(sliderButton innerBounds containsPoint: aMouseEvent position) ifTrue: [
		"Start slider dragging."
		mouseDownSliderPosition := self sliderPosition.
		^ self sliderDown: true].
		
	(pagingMorph innerBounds containsPoint: aMouseEvent position)	 ifTrue: [
		"Start slider stepping."
		^ self startStepping].! !

!UiScrollBar methodsFor: 'stepping' stamp: 'mt 2/26/2011 20:51:22.19'!
step
	"Automatically move the slider page-step-wise."
	
	| value |
	mouseDownEventPosition ifNil: [^ self stopStepping].
	
	self orientation = #horizontal
		ifTrue: [value := self pixelToValue: mouseDownEventPosition x - pagingMorph left - (sliderButton width / 2)]
		ifFalse: [value := self pixelToValue: mouseDownEventPosition y - pagingMorph top - (sliderButton height / 2)].
		
	value < self sliderPosition
		ifTrue: [self sliderPosition: (self sliderPosition - self pageStep max: value)]
		ifFalse: [self sliderPosition: (self sliderPosition + self pageStep min: value)].! !

!UiScrollBar methodsFor: 'updating' stamp: 'mt 4/12/2011 09:46:59.45'!
updateSliderButton
	"Reposition the support morph for the slider to #sliderPosition."
	
	| pt |
	self sliderPosition ifNil: [^ self].
	
	sliderButton orientation: self orientation.
	
	self orientation = #horizontal	ifTrue: [
		sliderButton
			width: self preferredSliderSize;
			height: self height.

		pt := ((self valueToPixel: self sliderPosition)
			"Add the offset considering the current position."
			+ pagingMorph left)
			@ self top.
			
		sliderButton topLeft: pt].
		
	self orientation = #vertical ifTrue: [
		sliderButton
			width: self width;
			height: self preferredSliderSize.
		
		pt := self left @(
			(self valueToPixel: self sliderPosition)
			+ pagingMorph top).
			
		sliderButton topLeft: pt].! !

!UiScrollBar methodsFor: 'updating' stamp: 'mt 5/16/2012 10:15:32.02'!
updateSupportMorphs

	"General layout."
	self orientation = #horizontal ifTrue: [
		self listDirection: #leftToRight.
		(#(distant both) includes: self buttonMode) ifTrue: [self layoutInset: (-1@0 corner: -1@0)].
		self buttonMode == #adjacent ifTrue: [self layoutInset: (0@0 corner: -1@0)].
		self buttonMode == #none ifTrue: [self layoutInset: 0]]. 
	self orientation = #vertical ifTrue: [
		self listDirection: #topToBottom.
		(#(distant both) includes: self buttonMode) ifTrue: [self layoutInset: (0@ -1 corner: 0@ -1)].
		self buttonMode == #adjacent ifTrue: [self layoutInset: (0@0 corner: 0@ -1)].
		self buttonMode == #none ifTrue: [self layoutInset: 0]].

	"Button visibility."
	{#(distant both) -> decreaseValueButton1.
	#(adjacent both) -> decreaseValueButton2} do: [:assoc |
		(assoc key includes: self buttonMode)
			ifTrue: [assoc value ifNotNil: [:btn | btn showFull]]
			ifFalse: [assoc value ifNotNil: [:btn | btn hideFull]]].

	self buttonMode = #none
		ifTrue: [increaseValueButton ifNotNil: [:btn | btn hideFull]]
		ifFalse: [increaseValueButton ifNotNil: [:btn | btn showFull]].

	"Other geometry."
	self
		updateValueButtons;
		updateSliderButton.! !

!UiScrollBar methodsFor: 'updating' stamp: 'mt 5/16/2012 10:23:24.248'!
updateValueButtons

	self orientation = #horizontal
		ifTrue: [
			decreaseValueButton1 ifNotNil: [:btn | btn direction: #left; width: self defaultThickness].
			decreaseValueButton2 ifNotNil: [:btn | btn direction: #left; width: self defaultThickness].
			increaseValueButton ifNotNil: [:btn | btn direction: #right; width: self defaultThickness]]
		ifFalse: [
			decreaseValueButton1 ifNotNil: [:btn | btn direction: #up; height: self defaultThickness].
			decreaseValueButton2 ifNotNil: [:btn | btn direction: #up; height: self defaultThickness].
			increaseValueButton ifNotNil: [:btn | btn direction: #down; height: self defaultThickness]].
! !

!UiSlider class methodsFor: 'testing' stamp: 'mt 2/27/2011 16:17:38.592'!
isWidgetClass

	^ true! !

!UiSlider class methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:18:00.811'!
widgetCategory

	^ 'Input Widgets'! !

!UiSlider class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:14:31.207'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiSlider050Icon! !

!UiSlider class methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:18:19.955'!
widgetClassName

	^ 'Slider'! !

!UiSlider class methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:25:51.081'!
widgetProperties

	^ super widgetProperties, {{'Slider'. #yourself} -> {
		#orientation -> #(horizontal vertical).
		#minimum -> Integer.
		#maximum -> Integer.
		#currentValue -> Integer.
		#sliderPosition -> Integer.
		#tracking -> Boolean.
		#singleStep -> Integer.
		#pageStep -> Integer.
		#truncate -> Boolean.
		}}! !

!UiSlider class methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:20:57.046'!
widgetPropertiesToIgnore

	^ #('TableLayout' 'ProportionalLayout' cellPositioning cellSpacing listCentering listSpacing minCellSize maxCellSize wrapCentering wrapDirection)! !

!UiSlider methodsFor: 'layout' stamp: 'mt 2/27/2011 15:55:22.353'!
adjustLayoutBounds

	super adjustLayoutBounds.
	self updateSliderButton. "Excluded from automatic layouting, so do it manually here."! !

!UiSlider methodsFor: 'actions' stamp: 'mt 2/26/2011 17:14:05.103'!
doSliderMovement: aMouseMoveEvent
	"Only allow integer steps with mouse dragging."
	
	| value |
	self isStepping
		ifTrue: [^ mouseDownEventPosition := aMouseMoveEvent position].
	
	value := self orientation = #horizontal
		ifTrue: [self pixelToValue: aMouseMoveEvent position x - mouseDownEventPosition x]
		ifFalse: [self pixelToValue: mouseDownEventPosition y - aMouseMoveEvent position y].

	self sliderPosition: mouseDownSliderPosition + (self truncate
		ifTrue: [value reduce asInteger]
		ifFalse: [value]).! !

!UiSlider methodsFor: 'geometry' stamp: 'mt 2/19/2011 11:59:01.107'!
extent: aPoint

	super extent: aPoint.	
	self updateSliderButton.! !

!UiSlider methodsFor: 'actions' stamp: 'mt 3/1/2011 14:13:25.91'!
finishSliderMovement: aMouseEvent

	self sliderDown: false.
	self stopStepping.
	
	mouseDownEventPosition := nil.! !

!UiSlider methodsFor: 'initialization' stamp: 'mt 2/26/2011 17:01:11.001'!
initialize

	self
		initializeSupportMorphs;
		initializeEventHandler.

	super initialize.

	self
		changeTableLayout;
		listCentering: #center.
		
	self
		addMorphBack: pagingMorph;
		addMorphFront: sliderButton.! !

!UiSlider methodsFor: 'initialization' stamp: 'mt 3/1/2011 14:07:28.021'!
initializeEventHandler

	self eventHandler: UiEventHandler new.

	self connect: self eventHandler signal: #mousePressed: toSelector: #startSliderMovement:.
	self connect: self eventHandler signal: #mouseMoved: toSelector: #doSliderMovement:.
	self connect: self eventHandler signal: #mouseReleased: toSelector: #finishSliderMovement:.

	self connect: self eventHandler signal: #mousePressed: toSelector: #jumpToValue:.

	"Keyboard interaction."
	self eventHandler on: Character arrowLeft send: #decreaseValueBySingleStep to: self.
	self eventHandler on: Character arrowRight send: #increaseValueBySingleStep to: self.
	self eventHandler on: Character arrowDown send: #decreaseValueBySingleStep to: self.
	self eventHandler on: Character arrowUp send: #increaseValueBySingleStep to: self.
	
	self eventHandler on: Character pageUp send: #increaseValueByPageStep to: self.
	self eventHandler on: Character pageDown send: #decreaseValueByPageStep to: self.
	
	self eventHandler on: Character home send: #setValueToMinimum to: self.
	self eventHandler on: Character end send: #setValueToMaximum to: self.! !

!UiSlider methodsFor: 'initialization' stamp: 'mt 2/27/2011 15:50:36.457'!
initializeSupportMorphs

	pagingMorph := Morph new
		borderStyle: (BorderStyle inset width: 1);
		color: (Color gray: 0.85);
		hResizing: #spaceFill;
		height: 5.  
			
	sliderButton := UiSliderButton new.! !

!UiSlider methodsFor: 'actions' stamp: 'mt 2/26/2011 17:09:05.644'!
jumpToValue: aMouseEvent
	"Jumps to the value under the mouse cursor if shift was pressed."
	
	| value |
	aMouseEvent shiftPressed ifFalse: [^ self].
	
	self orientation = #horizontal
		ifTrue: [value := self pixelToValue: aMouseEvent position x - self left - (sliderButton width / 2)]
		ifFalse: [value := self pixelToValue: self bottom - aMouseEvent position y - (sliderButton height / 2)].
		
	self sliderPosition: value.! !

!UiSlider methodsFor: 'converting' stamp: 'mt 2/27/2011 15:27:16.273'!
pixelToValue: anInteger

	self range extent = 0 ifTrue: [^ 0].

	self orientation = #horizontal
		ifTrue: [^ anInteger / ((pagingMorph width - sliderButton width) max: 1) * self range extent].
		
	self orientation = #vertical
		ifTrue: [^ anInteger / ((pagingMorph height - sliderButton height) max: 1) * self range extent].
		
	^ 0! !

!UiSlider methodsFor: 'updating' stamp: 'mt 2/19/2011 14:19:35.439'!
sliderChange: aSliderChange

	super sliderChange: aSliderChange.
	  
	aSliderChange = #orientationChange
		ifTrue: [self updateSupportMorphs].
	
	(#(rangeChange valueChange) includes: aSliderChange)
		ifTrue: [self updateSliderButton].! !

!UiSlider methodsFor: 'accessing' stamp: 'mt 2/19/2011 13:00:27.935'!
sliderDown: aBoolean
	"Overridden to pass it to the slider button."
	
	super sliderDown: aBoolean.
	sliderButton down: aBoolean.  ! !

!UiSlider methodsFor: 'actions' stamp: 'mt 3/1/2011 14:11:52.387'!
startSliderMovement: aMouseEvent

	mouseDownEventPosition := aMouseEvent position.
	
	((self orientation = #horizontal and: [aMouseEvent position x between: sliderButton left and: sliderButton right])
		or: [self orientation = #vertical and: [aMouseEvent position y between: sliderButton top and: sliderButton bottom]])
		ifTrue: [
			"Start slider dragging."
			mouseDownSliderPosition := self sliderPosition.
			self sliderDown: true]
		ifFalse: [
			"Start slider stepping."
			self startStepping]

! !

!UiSlider methodsFor: 'stepping' stamp: 'mt 3/1/2011 14:17:50.008'!
step
	"Automatically move the slider page-step-wise."
	
	| value |
	mouseDownEventPosition ifNil: [^ self stopStepping].
	
	self orientation = #horizontal
		ifTrue: [value := self pixelToValue: mouseDownEventPosition x - self left - (sliderButton width / 2)]
		ifFalse: [value := self pixelToValue: self bottom - mouseDownEventPosition y - (sliderButton height / 2)].
		
	value < self sliderPosition
		ifTrue: [self sliderPosition: (self sliderPosition - self pageStep max: value)]
		ifFalse: [self sliderPosition: (self sliderPosition + self pageStep min: value)].! !

!UiSlider methodsFor: 'stepping' stamp: 'mt 2/25/2011 11:19:16.835'!
stepTime

	^ 100! !

!UiSlider methodsFor: 'updating' stamp: 'mt 2/19/2011 12:44:26.793'!
updatePagingMorph

	pagingMorph
		extent: 5@5.

	self orientation = #horizontal ifTrue: [
		pagingMorph
			hResizing: #spaceFill;
			vResizing: #rigid].
		
	self orientation = #vertical ifTrue: [
		pagingMorph
			hResizing: #rigid;
			vResizing: #spaceFill].! !

!UiSlider methodsFor: 'updating' stamp: 'mt 2/27/2011 15:42:53.077'!
updateSliderButton
	"Reposition the support morph for the slider to #sliderPosition."
	
	| pt |
	self sliderPosition ifNil: [^ self].
	
	sliderButton orientation: self orientation.
	
	self orientation = #horizontal	ifTrue: [
		sliderButton extent: 10@self height.

		pt := ((self valueToPixel: self sliderPosition)
			"Add the offset considering the current position."
			+ self left + (sliderButton width / 2))
			@ self center y.
			
		sliderButton center: pt].
		
	self orientation = #vertical ifTrue: [
		sliderButton extent: self width@10.
		
		pt := self center x @(
			self bottom
			- (self valueToPixel: self sliderPosition)
			- (sliderButton height / 2)).
			
		sliderButton center: pt].! !

!UiSlider methodsFor: 'updating' stamp: 'mt 2/26/2011 17:01:19.52'!
updateSupportMorphs

	self orientation = #horizontal ifTrue: [self listDirection: #topToBottom].  		
	self orientation = #vertical ifTrue: [self listDirection: #leftToRight].

	self
		updatePagingMorph;
		updateSliderButton.! !

!UiSlider methodsFor: 'converting' stamp: 'mt 2/27/2011 15:27:35.001'!
valueToPixel: aNumber
	"Converts a given number to a portion of the extent of this slider."

	| abs |
	self range extent = 0 ifTrue: [^ 0].

	abs := ((aNumber - self minimum) / self range extent).
	
	self orientation = #horizontal
		ifTrue: [^ abs * (pagingMorph width - sliderButton width)].
		
	self orientation = #vertical
		ifTrue: [^ abs * (pagingMorph height - sliderButton height)].
		
	^ 0! !

Morph subclass: #UiButton
	instanceVariableNames: 'checkable checked down icon text autoExclusive enabled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiButton commentStamp: 'mt 6/20/2010 20:42:54.272' prior: 0!
Abstract button.!

!UiButton class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:28:38.684'!
widgetCategory
	
	^ 'Buttons'! !

!UiButton class methodsFor: 'properties' stamp: 'mt 2/19/2011 12:56:08.001'!
widgetProperties

	^ super widgetProperties, {{'Button'. #yourself} -> {
		#enabled -> Boolean.
		#icon -> Form.
		#text -> String.
		#color -> Object.
		#checkable -> Boolean.
		#checked -> Boolean.
		#autoExclusive -> Boolean}}! !

!UiButton class methodsFor: 'properties' stamp: 'mt 11/14/2010 15:58:39.299'!
widgetPropertiesToIgnore

	^ #('TableLayout' 'ProportionalLayout' cellPositioning cellSpacing listSpacing minCellSize maxCellSize wrapCentering wrapDirection)! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/21/2010 10:28:44.532'!
autoExclusive

	^ autoExclusive ifNil: [autoExclusive := false]! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/21/2010 10:31:03.542'!
autoExclusive: aBoolean

	autoExclusive == aBoolean ifTrue: [^ self].
	
	autoExclusive := aBoolean.
	
	aBoolean ifTrue: [self checkable: true].! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 19:51:52.944'!
checkable

	^ checkable ifNil: [checkable := false]! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 19:48:57.047'!
checkable: aBoolean

	aBoolean == checkable ifTrue: [^ self].

	checkable := aBoolean.
	
	aBoolean ifFalse: [checked := false].! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 19:51:46.981'!
checked

	^ checked ifNil: [checked := false]! !

!UiButton methodsFor: 'accessing' stamp: 'mt 10/27/2010 10:10:10.273'!
checked: aBoolean

	(aBoolean == checked or: [self checkable not])
		ifTrue: [^ self].	
	
	checked := aBoolean.
	
	self toggled: aBoolean.
	
	self redrawLater.
	
	"Handle auto-exclusiveness."
	(self autoExclusive and: [self checked])
		ifTrue: [self owner ifNotNil: [:group |
			group submorphs
				select: [:m | (m isKindOf: UiButton) and: [m ~= self]]
				thenDo: [:m | m checked: false]]].! !

!UiButton methodsFor: 'signals' stamp: 'mt 6/20/2010 19:35:59.853'!
clicked: isChecked

	self emit.! !

!UiButton methodsFor: 'actions' stamp: 'mt 3/2/2011 11:27:17.367'!
doClick
	"#click is reserved for morphic event handling. Simulate a button click."
	
	self toggle.
	self clicked: self checked.! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 19:51:42.188'!
down

	^ down ifNil: [down := false]! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 21:21:04.758'!
down: aBoolean

	aBoolean == down ifTrue: [^ self].
	
	down := aBoolean.
	
	self redrawLater.
	
	aBoolean
		ifFalse: [self toggle].! !

!UiButton methodsFor: 'accessing' stamp: 'mt 11/22/2010 10:44:41.643'!
enabled

	^ enabled ifNil: [enabled := true]! !

!UiButton methodsFor: 'accessing' stamp: 'mt 11/22/2010 10:44:56.115'!
enabled: aBoolean

	enabled := aBoolean.! !

!UiButton methodsFor: 'drawing' stamp: 'mt 2/26/2011 16:07:11.041'!
gradient
	"Generate a gradient based on the current color and geometry."

	| inverted baseColor |
	inverted := self down or: [self checked].
	baseColor := 	self hasKeyboard
		ifTrue: [(Color r: 0.5 g: 0.5 b: 1.0) muchLighter]
		ifFalse: [self color].
	baseColor := inverted
		ifTrue: [baseColor darker]
		ifFalse: [baseColor].
	
	^ (GradientFillStyle ramp: {
			0.0 -> (baseColor adjustBrightness: 0.1).
			0.1-> (baseColor adjustBrightness: 0.05).
			0.6 -> (baseColor darker)})
		origin: (inverted
			ifTrue: [self innerBounds bottomLeft]
			ifFalse: [self innerBounds topLeft]);
		direction: (inverted
			ifTrue: [0 @ self innerBounds height negated]
			ifFalse: [0 @ self innerBounds height]).! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 19:34:04.67'!
icon

	^ icon! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 20:07:08.152'!
icon: aForm

	icon := aForm.! !

!UiButton methodsFor: 'initialization' stamp: 'mt 4/12/2011 13:53:59.122'!
initialize

	super initialize.
	
	self initializeEventHandler.
	
	self
		color: (Color gray: 0.85);
		borderColor: (Color gray: 0.3);
		borderWidth: 1;
		extent: 50@25.! !

!UiButton methodsFor: 'initialization' stamp: 'mt 4/14/2011 17:17:04.172'!
initializeEventHandler

	self eventHandler: UiEventHandler new.
	
	self eventHandler setProperty: #handlesMouseDown toValue: [self enabled].
	self eventHandler setProperty: #handlesMouseUp toValue: [self enabled].
	
	self connect: self eventHandler signal: #mousePressed: toSelector: #grabKeyboard.
	
	self connect: self eventHandler signal: #mousePressed: toSelector: #press:.
	self connect: self eventHandler signal: #mouseReleased: toSelector: #release:.
	
	self connect: self eventHandler signal: #returnPressed toSelector: #doClick.! !

!UiButton methodsFor: 'accessing' stamp: 'mt 8/1/2013 19:28:29.507'!
orientation

	^ #horizontal! !

!UiButton methodsFor: 'actions' stamp: 'mt 3/2/2011 11:04:58.085'!
press: aMouseEvent

	self
		down: true;
		pressed.! !

!UiButton methodsFor: 'signals' stamp: 'mt 6/20/2010 19:36:09.523'!
pressed

	self emit.! !

!UiButton methodsFor: 'actions' stamp: 'mt 3/29/2011 09:59:15.687'!
release: aMouseEvent

	(self bounds containsPoint: aMouseEvent position)
		ifTrue: [self down: false; released; clicked: self checked]
		ifFalse: [down := false. self redrawLater].! !

!UiButton methodsFor: 'signals' stamp: 'mt 6/20/2010 19:36:14.649'!
released

	self emit.! !

!UiButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 19:34:55.971'!
text

	^ text! !

!UiButton methodsFor: 'accessing' stamp: 'mt 10/27/2010 10:05:52.788'!
text: aStringOrText

	text := aStringOrText.! !

!UiButton methodsFor: 'actions' stamp: 'mt 6/21/2010 10:39:00.668'!
toggle

	(self autoExclusive and: [self checked])
		ifTrue: ["Must not toggle." ^ self].
	
	self checked: self checked not.! !

!UiButton methodsFor: 'signals' stamp: 'mt 6/20/2010 19:36:30.246'!
toggled: isChecked

	self emit.! !

UiButton subclass: #UiCompositeButton
	instanceVariableNames: 'box'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiCompositeButton commentStamp: 'mt 3/5/2011 11:53:19.193' prior: 0!
This is a button that has an additional button embedded.!

UiCompositeButton subclass: #UiCheckBox
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiCheckBox class methodsFor: 'testing' stamp: 'mt 6/29/2010 10:04:30.463'!
isWidgetClass

	^ true! !

!UiCheckBox class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:19.195'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiCheckBoxesIcon! !

!UiCheckBox class methodsFor: 'accessing' stamp: 'mt 6/30/2010 21:23:28.921'!
widgetClassName

	^ 'Check Box'! !

!UiCheckBox class methodsFor: 'properties' stamp: 'mt 10/28/2010 09:51:08.562'!
widgetPropertiesToIgnore

	^ super widgetPropertiesToIgnore, #('checkable')! !

!UiCheckBox methodsFor: 'initialization' stamp: 'mt 3/5/2011 11:55:37.001'!
boxClass

	^ UiCheckButtonBox! !

!UiCheckBox methodsFor: 'accessing' stamp: 'mt 3/22/2011 08:12:55.763'!
checked: aBoolean
	"Add explicit switching behavior with new signals."
	
	| old |
	old := self checked.
	
	super checked: aBoolean.
	
	old ~= aBoolean ifTrue: [
		aBoolean
			ifTrue: [self turnedOn]
			ifFalse: [self turnedOff]].
! !

!UiCheckBox methodsFor: 'drawing' stamp: 'mt 2/26/2011 15:33:20.135'!
fullDrawOn: aCanvas

	super fullDrawOn: aCanvas.
	
	"Indicate checking state with an additional tick."
	box notNil & self checked ifTrue: [
		aCanvas
			line: box topLeft + (0@(box height // 2)) + (2@0)
			to: box bottomLeft + ((box width // 2)@0) - (0@2)
			width: 2
			color: (Color gray: 0.3).
		aCanvas
			line: box bottomLeft + ((box width // 2)@0) - (0@2)
			to: box topRight - (2@ -2)
			width: 2
			color: (Color gray: 0.3)].! !

!UiCheckBox methodsFor: 'initialization' stamp: 'mt 3/5/2011 11:57:22.644'!
initialize

	super initialize.
		
	self text: 'CheckBox'.! !

!UiCheckBox methodsFor: 'initialization' stamp: 'mt 7/16/2011 17:06:23.754'!
initializeNoText
	"UiCheckBox basicNew initializeNoText. Performance reasons."
	
	super initialize.! !

!UiCheckBox methodsFor: 'signals' stamp: 'mt 3/22/2011 08:06:03.894'!
turnedOff
	"Emitted whenever the #checked: state changes. In contrast to #toggled:, explicitly maps the state to the signal name."
	
	self emit.! !

!UiCheckBox methodsFor: 'signals' stamp: 'mt 3/22/2011 08:05:56.578'!
turnedOn
	"Emitted whenever the #checked: state changes. In contrast to #toggled:, explicitly maps the state to the signal name."

	self emit.! !

!UiCheckBox methodsFor: 'updating' stamp: 'mt 2/16/2012 16:03:59.912'!
updateSubmorphs

	super updateSubmorphs.
	box autoRaise: false.! !

!UiCompositeButton methodsFor: 'initialization' stamp: 'mt 3/5/2011 11:53:57.984'!
boxClass

	^ self subclassResponsibility! !

!UiCompositeButton methodsFor: 'accessing' stamp: 'mt 3/5/2011 11:55:43.574'!
checked: aBoolean

	super checked: aBoolean.
	box ifNotNil: [:b | b down: aBoolean].! !

!UiCompositeButton methodsFor: 'accessing' stamp: 'mt 3/5/2011 11:55:47.699'!
down: aBoolean

	super down: aBoolean.
	box ifNotNil: [:b | b down: aBoolean | self checked].! !

!UiCompositeButton methodsFor: 'drawing' stamp: 'mt 3/5/2011 11:56:17.88'!
drawOn: aCanvas

	"Invisible."! !

!UiCompositeButton methodsFor: 'accessing' stamp: 'mt 2/7/2012 16:03:35.243'!
enabled: aBoolean

	enabled = aBoolean ifTrue: [^ self].
	super enabled: aBoolean.
	box ifNotNil: [:b | b enabled: aBoolean].
	self updateSubmorphs.! !

!UiCompositeButton methodsFor: 'accessing' stamp: 'mt 4/17/2011 15:33:29.467'!
icon: aForm

	aForm == icon ifTrue: [^ self].

	super icon: aForm.
	self updateSubmorphs.! !

!UiCompositeButton methodsFor: 'initialization' stamp: 'mt 3/5/2011 12:00:36.429'!
initialize

	super initialize.
	
	self
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		changeTableLayout;
		listDirection: #leftToRight;
		cellInset: 4;
		layoutInset: (2@0 corner: 2@0).
		
	self 
		checkable: true;
		updateSubmorphs.! !

!UiCompositeButton methodsFor: 'initialization' stamp: 'mt 3/5/2011 11:56:43.076'!
initializeEventHandler

	super initializeEventHandler.
	
	self eventHandler mouseDownPriority: 1.! !

!UiCompositeButton methodsFor: 'accessing' stamp: 'mt 3/5/2011 11:55:56.181'!
text: aText

	super text: aText.
	self updateSubmorphs.! !

!UiCompositeButton methodsFor: 'updating' stamp: 'mt 3/5/2011 11:57:45.947'!
updateSubmorphs

	self removeAllMorphs.
	
	box ifNil: [box := self boxClass new].
	
	self addMorphBack: box.
	self icon ifNotNil: [:i |
		self enabled
				ifTrue: [self addMorphBack: i asMorph]
				ifFalse: [self addMorphBack: i asGrayScale2 asMorph]].
	self text ifNotNil: [:t | 
		self enabled
				ifTrue: [self addMorphBack: t asHtmlText asMorph lock]
				ifFalse: [self addMorphBack: ('<font color="#444444>', t asString, '</font>') asHtmlText asMorph lock]].! !

UiCompositeButton subclass: #UiRadioButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiRadioButton class methodsFor: 'testing' stamp: 'mt 3/5/2011 12:00:58.832'!
isWidgetClass

	^ true! !

!UiRadioButton class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:35.606'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiRadioButtonsIcon! !

!UiRadioButton class methodsFor: 'accessing' stamp: 'mt 3/5/2011 12:01:47.229'!
widgetClassName

	^ 'Radio Button'! !

!UiRadioButton class methodsFor: 'properties' stamp: 'mt 3/5/2011 12:02:08.606'!
widgetPropertiesToIgnore

	^ super widgetPropertiesToIgnore, #(checkable autoExclusive)! !

!UiRadioButton methodsFor: 'initialization' stamp: 'mt 3/5/2011 11:58:48.539'!
boxClass

	^ UiRadioButtonBox! !

!UiRadioButton methodsFor: 'drawing' stamp: 'mt 3/5/2011 11:59:23.645'!
fullDrawOn: aCanvas

	super fullDrawOn: aCanvas.
	
	box notNil & self checked ifTrue: [
		aCanvas
			fillOval: (box bounds insetBy: 3)
			color: (Color gray: 0.3)].! !

!UiRadioButton methodsFor: 'initialization' stamp: 'mt 3/5/2011 12:00:06.306'!
initialize

	super initialize.
		
	self
		autoExclusive: true;
		text: 'RadioButton'.! !

UiButton subclass: #UiPushButton
	instanceVariableNames: 'outerBorderWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiPushButton commentStamp: 'mt 12/1/2011 11:24:09.953' prior: 0!
This button triggers regular user interactions. Usually, it is used for confirmation or cancellation in dialog boxes.!

!UiPushButton class methodsFor: 'testing' stamp: 'mt 6/28/2010 21:57:11.625'!
isWidgetClass

	^ true! !

!UiPushButton class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:14:23.409'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiButtonIcon! !

!UiPushButton class methodsFor: 'accessing' stamp: 'mt 6/30/2010 21:10:37.376'!
widgetClassName

	^ 'Push Button'! !

!UiPushButton methodsFor: 'drawing' stamp: 'mt 1/2/2012 16:59:04.168'!
drawOn: aCanvas

	UiPainter default new
		drawPushButton: self on: aCanvas.! !

!UiPushButton methodsFor: 'accessing' stamp: 'mt 2/7/2012 16:03:21.39'!
enabled: aBoolean

	enabled = aBoolean ifTrue: [^ self].
	super enabled: aBoolean.
	self updateSubmorphs.! !

!UiPushButton methodsFor: 'layout-properties' stamp: 'mt 3/5/2013 12:54:17.719'!
hResizing: aSymbol

	(aSymbol ~= #shrinkWrap and: [self clipSubmorphs not])
		ifTrue: [self clipSubmorphs: true].
	
	^ super hResizing: aSymbol! !

!UiPushButton methodsFor: 'accessing' stamp: 'mt 4/17/2011 15:32:44.867'!
icon: aForm

	aForm == icon ifTrue: [^ self].

	super icon: aForm.
	self updateSubmorphs.! !

!UiPushButton methodsFor: 'initialization' stamp: 'mt 5/18/2011 13:48:30.291'!
initialize

	super initialize.
	
	self
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		changeTableLayout;
		listDirection: #leftToRight;
		cellInset: 4;
		layoutInset: (2@0 corner: 2@0).
	
	self
		color: (Color gray: 0.8);
		text: 'PushButton'.! !

!UiPushButton methodsFor: 'accessing' stamp: 'mt 2/26/2011 20:07:00.698'!
outerBorderWidth

	^ outerBorderWidth ifNil: [outerBorderWidth := 1]! !

!UiPushButton methodsFor: 'accessing' stamp: 'mt 2/26/2011 20:07:27.461'!
outerBorderWidth: anInteger

	outerBorderWidth = anInteger ifTrue: [^ self].
	outerBorderWidth := anInteger.
	self redrawLater.! !

!UiPushButton methodsFor: 'geometry' stamp: 'mt 2/26/2011 20:07:46.451'!
outerBounds
	"Used for extra outline. See #draw."
	
	^ self bounds outsetBy: self outerBorderWidth! !

!UiPushButton methodsFor: 'accessing' stamp: 'mt 6/28/2010 22:03:18.543'!
text: aString

	super text: aString.
	self updateSubmorphs.! !

!UiPushButton methodsFor: 'updating' stamp: 'mt 5/18/2011 13:48:21.805'!
updateSubmorphs

	self submorphs ifNotEmpty: [self removeAllMorphs].
		
	self icon ifNotNil: [:i | 
		self enabled
				ifTrue: [self addMorphBack: i asMorph]
				ifFalse: [self addMorphBack: i asGrayScale2 asMorph]].
	self text ifNotNil: [:t | 
		self enabled
				ifTrue: [self addMorphBack: t asHtmlText asMorph lock]
				ifFalse: [self addMorphBack: ('<font color="#444444>', t asString, '</font>') asHtmlText asMorph lock]].! !

UiPushButton subclass: #UiScrollBarArrowButton
	instanceVariableNames: 'fadeAnimation direction'
	classVariableNames: 'Arrows'
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiScrollBarArrowButton class methodsFor: 'testing' stamp: 'mt 2/27/2011 16:16:58.913'!
isWidgetClass

	^ false! !

!UiScrollBarArrowButton methodsFor: 'initialization' stamp: 'mt 8/21/2012 14:53:29.889'!
arrow: aSymbol
	"#up, #down, #left, #right"
	
	Arrows ifNil: [ 
		Arrows := Dictionary new.
		Arrows at: #left put: UiButtonIcons bulletArrowLeftSunkenIcon.
		Arrows at: #up put: UiButtonIcons bulletArrowUpSunkenIcon.
		Arrows at: #right put: UiButtonIcons bulletArrowRightSunkenIcon.
		Arrows at: #down put: UiButtonIcons bulletArrowDownSunkenIcon].
	
	^ Arrows at: aSymbol! !

!UiScrollBarArrowButton methodsFor: 'accessing' stamp: 'mt 3/28/2011 13:37:22.086'!
direction

	^ direction! !

!UiScrollBarArrowButton methodsFor: 'accessing' stamp: 'mt 5/18/2011 14:09:07.559'!
direction: aSymbol

	aSymbol = direction ifTrue: [^ self].
	
	direction := aSymbol.
	
	self submorphs ifNotEmpty: [self removeAllMorphs].
	self addMorph: ((self arrow: aSymbol) asMorph
		frameFractions: (0@0 corner: 1@1);
		ignoresColorMappings: true).  
	
	(#(left right) includes: aSymbol)
		ifTrue: [
			self
				hResizing: #rigid;
				vResizing: #spaceFill;
				layoutInset: (2@0 corner: 2@0)]
		ifFalse: ["up down"
			self
				hResizing: #spaceFill;
				vResizing: #rigid;
				layoutInset: (0@2 corner: 0@2)].! !

!UiScrollBarArrowButton methodsFor: 'animations' stamp: 'mt 4/5/2011 16:00:16.928'!
fadeIn

	self visible ifFalse: [^ self].
	
	fadeAnimation
		ifNil: [^ self]
		ifNotNil: [:anim |
			anim unregister.
			fadeAnimation := nil].
	
	super fadeIn.! !

!UiScrollBarArrowButton methodsFor: 'animations' stamp: 'mt 4/5/2011 16:00:33.533'!
fadeOut

	self visible ifFalse: [^ self].
	
	fadeAnimation ifNil: [fadeAnimation := (AnimAlphaBlendAnimation
		fadeOutOn: self
		duration: 200)
		register;
		start: #keepWhenFinished].! !

!UiScrollBarArrowButton methodsFor: 'animations' stamp: 'mt 5/18/2011 13:54:08.466'!
fadeOutInstantly

	self fadeOut.
	
	fadeAnimation ifNotNil: [fadeAnimation
		stop;
		toEnd].! !

!UiScrollBarArrowButton methodsFor: 'initialization' stamp: 'mt 1/2/2012 17:30:20.525'!
initialize

	super initialize.
	
	self
		color: (Color gray: 0.9);
		changeProportionalLayout;
		listCentering: #center;
		fadeOutInstantly.
		
	self eventHandler setProperty: #handlesMouseStillDown toValue: [true].
	self eventHandler setProperty: #handlesKeyboard toValue: [false].
	
	self connect: self eventHandler signal: #focusReceived toSelector: #passFocusToOwner.! !

!UiScrollBarArrowButton methodsFor: 'accessing' stamp: 'mt 1/2/2012 17:26:40.231'!
orientation

	^ (self direction == #left or: [self direction == #right])
		ifTrue: [#horizontal]
		ifFalse: [#vertical]! !

!UiScrollBarArrowButton methodsFor: 'initialization' stamp: 'mt 2/26/2011 21:07:13.055'!
passFocusToOwner

	self owner grabKeyboard.! !

!UiScrollBarArrowButton methodsFor: 'updating' stamp: 'mt 5/18/2011 14:01:07.175'!
updateSubmorphs

	"Ignore."! !

UiPushButton subclass: #UiSliderButton
	instanceVariableNames: 'orientation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiSliderButton commentStamp: 'mt 2/27/2011 15:45:52.758' prior: 0!
Special push button to be used in slider.!

UiSliderButton subclass: #UiScrollBarSliderButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiScrollBarSliderButton commentStamp: 'mt 2/26/2011 17:45:21.609' prior: 0!
Special push button to be used in scrollbar.!

!UiScrollBarSliderButton methodsFor: 'drawing' stamp: 'mt 5/20/2011 08:52:38.69'!
drawGripOn: aCanvas at: aPoint
	"Draws a single grip. Considers orientation."
	
	| rect |
	rect := (aPoint extent: (
		self orientation == #horizontal
			ifTrue: [6 @ self height]
			ifFalse: [self width @ 6])) insetBy: 2.
	
	aCanvas
		frameRectangle: rect
		color: (Color gray: 0.3).
	
		aCanvas
			line: rect bottomLeft + (1@ -1)
			to: rect bottomRight - (1@1)
			color: (Color gray: 0.6).
		
		aCanvas
			line: rect bottomRight - (1@1)
			to: rect topRight + (-1@1)
			color: (Color gray: 0.6).! !

!UiScrollBarSliderButton methodsFor: 'drawing' stamp: 'mt 1/4/2012 14:16:21.46'!
drawGripsOn: aCanvas

	1 to: 3 do: [:index |
		self
			drawGripOn: aCanvas
			at: (self orientation == #horizontal
				ifTrue: [self center x - 6 +
					(index - 1 * 3) @ self top]
				ifFalse: [self left @ (self center y - 6 +
					(index - 1 * 3))])]! !

!UiScrollBarSliderButton methodsFor: 'drawing' stamp: 'mt 1/4/2012 14:15:35.75'!
drawOn: aCanvas

	super drawOn: aCanvas.
	
	"Draw grips."
	((self orientation == #horizontal
		and: [self width >= 16]) or: [
		(self orientation == #vertical
			and: [self height >= 16])])
				ifTrue: [self drawGripsOn: aCanvas].! !

!UiScrollBarSliderButton methodsFor: 'geometry' stamp: 'mt 5/18/2011 14:37:28.164'!
extent: aPoint
	
	self orientation = #horizontal ifTrue: [
		super extent: (aPoint max: 10@0)].
	
	self orientation = #vertical ifTrue: [
		super extent: (aPoint max: 0@10)].! !

!UiSliderButton class methodsFor: 'testing' stamp: 'mt 2/27/2011 16:17:13.969'!
isWidgetClass

	^ false! !

!UiSliderButton methodsFor: 'initialization' stamp: 'mt 1/2/2012 17:29:40.021'!
initialize

	super initialize.
	
	self
		color: (Color gray: 0.9);
		listCentering: #center;
		cellInset: 1;
		layoutInset: 2;  
		orientation: #horizontal;
		disableTableLayout: true.
		
	self eventHandler setProperty: #handlesMouseDown toValue: [false].! !

!UiSliderButton methodsFor: 'accessing' stamp: 'mt 2/27/2011 15:41:01.067'!
orientation

	^ orientation ifNil: [#horizontal]! !

!UiSliderButton methodsFor: 'accessing' stamp: 'mt 2/27/2011 15:41:02.552'!
orientation: aSymbol
	"#horizontal, #vertical"
	
	aSymbol = orientation ifTrue: [^ self].
	
	orientation := aSymbol.
	
	self update.! !

!UiSliderButton methodsFor: 'updating' stamp: 'mt 2/27/2011 15:41:34.974'!
update

	self extent: 14@14.
	
	self orientation = #horizontal
		ifTrue: [self
			listDirection: #leftToRight;
			hResizing: #rigid;
			vResizing: #spaceFill].
		
	self orientation = #vertical
		ifTrue: [self
			listDirection: #topToBottom;
			hResizing: #spaceFill;
			vResizing: #rigid].! !

!UiSliderButton methodsFor: 'updating' stamp: 'mt 5/18/2011 14:15:25.46'!
updateSubmorphs

	"Ignore."! !

UiButton subclass: #UiRadioButtonBox
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiRadioButtonBox methodsFor: 'drawing' stamp: 'mt 3/5/2011 11:44:52.361'!
drawBorderOn: aCanvas

	| color1 color2 |
	(self down or: [self checked])
		ifTrue: [
			color1 := self color twiceDarker.
			color2 := self color twiceLighter]
		ifFalse: [
			color1 := self color twiceLighter.
			color2 := self color twiceDarker].
		
	aCanvas
		frameOval: self bounds
		color: color1.
		
	aCanvas clipBy: (self bounds translateBy: 0 @ (self bounds height * 0.75)) during: [:c1 |
		c1
			frameOval: self bounds
			color: color2].
		
	aCanvas clipBy: (self bounds translateBy: (self bounds width * 0.75) @ 0) during: [:c1 |
		c1
			frameOval: self bounds
			color: color2].! !

!UiRadioButtonBox methodsFor: 'drawing' stamp: 'mt 3/5/2011 12:04:13.007'!
drawOn: aCanvas

	self setProperty: #fillStyle toValue: self gradient.
	
	aCanvas
		fillOval: self bounds
		fillStyle: self fillStyle.

	self drawBorderOn: aCanvas.! !

!UiRadioButtonBox methodsFor: 'initialization' stamp: 'mt 3/5/2011 12:03:48.215'!
initialize

	super initialize.
	self extent: 12@12.! !

UiButton subclass: #UiToolButton
	instanceVariableNames: 'raised autoRaise buttonStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiToolButton commentStamp: 'mt 3/5/2011 11:22:33.102' prior: 0!
Supports #autoRaise and aligns icon and text according to different #buttonStyle.!

UiToolButton subclass: #UiCheckButtonBox
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiCheckButtonBox class methodsFor: 'testing' stamp: 'mt 3/5/2011 12:15:45.982'!
isWidgetClass

	^ false! !

!UiCheckButtonBox methodsFor: 'initialization' stamp: 'mt 3/5/2011 11:55:14.331'!
initialize

	super initialize.
	
	self 
		checkable: false;
		hResizing: #rigid;
		vResizing: #rigid;
		icon: nil; text: nil;
		extent: 12@12.! !

!UiToolButton class methodsFor: 'testing' stamp: 'mt 6/28/2010 21:32:01.69'!
isWidgetClass

	^ true! !

!UiToolButton class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:11.933'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiButtonToggleIcon! !

!UiToolButton class methodsFor: 'accessing' stamp: 'mt 6/30/2010 21:11:24.018'!
widgetClassName

	^ 'Tool Button'! !

!UiToolButton class methodsFor: 'properties' stamp: 'mt 10/22/2010 11:40:45.982'!
widgetProperties

	^ super widgetProperties, {{'Tool Button'. #yourself} -> {
		#autoRaise -> Boolean.
		#buttonStyle -> #(iconOnly textOnly textBesideIcon textUnderIcon)}}! !

!UiToolButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 20:08:09.225'!
autoRaise

	^ autoRaise ifNil: [autoRaise := false]! !

!UiToolButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 20:36:20.019'!
autoRaise: aBoolean

	aBoolean == autoRaise ifTrue: [^ self].

	autoRaise := aBoolean.
	
	self raised: aBoolean not.! !

!UiToolButton methodsFor: 'accessing' stamp: 'mt 6/28/2010 22:36:21.335'!
buttonStyle

	^ buttonStyle ifNil: [buttonStyle := #textBesideIcon]! !

!UiToolButton methodsFor: 'accessing' stamp: 'mt 6/28/2010 22:37:57.741'!
buttonStyle: aStyle
	"#iconOnly #textOnly #textBesideIcon #textUnderIcon"

	aStyle = buttonStyle ifTrue: [^ self].
	
	buttonStyle := aStyle.
	
	self updateSubmorphs.! !

!UiToolButton methodsFor: 'drawing' stamp: 'mt 1/2/2012 16:59:24.803'!
drawOn: aCanvas

	UiPainter default new
		drawToolButton: self on: aCanvas.! !

!UiToolButton methodsFor: 'accessing' stamp: 'mt 2/7/2012 16:03:26.031'!
enabled: aBoolean

	enabled = aBoolean ifTrue: [^ self].
	super enabled: aBoolean.
	self updateSubmorphs.! !

!UiToolButton methodsFor: 'actions' stamp: 'mt 2/26/2011 15:16:50.685'!
flatten

	self autoRaise ifTrue: [self raised: false].! !

!UiToolButton methodsFor: 'accessing' stamp: 'mt 4/17/2011 15:33:06.665'!
icon: aForm

	aForm == icon ifTrue: [^ self].

	super icon: aForm.
	self updateSubmorphs.! !

!UiToolButton methodsFor: 'initialization' stamp: 'mt 12/16/2011 11:53:36.773'!
initialize

	super initialize.
	
	self
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		changeTableLayout;
		cellInset: 4;
		layoutInset: (2@0 corner: 2@0).
	
	self
		color: (Color r: 1.0 g: 222/255 b: 0);
		raised: true;
		autoRaise: true;
		text: 'ToolButton';
		updateSubmorphs.! !

!UiToolButton methodsFor: 'initialization' stamp: 'mt 1/6/2012 10:11:15.868'!
initializeEventHandler

	super initializeEventHandler.
	self eventHandler setProperty: #handlesMouseOver toValue: [self enabled].
	
	self connect: self eventHandler signal: #mouseEntered: toSelector: #raise.
	self connect: self eventHandler signal: #mouseLeft: toSelector: #flatten.
	
	"No keyboard focus for tool buttons."
	self eventHandler disconnectSignal: #mousePressed: from: self selector: #grabKeyboard.! !

!UiToolButton methodsFor: 'geometry' stamp: 'mt 12/16/2011 11:49:10.929'!
outerBounds

	^ self bounds outsetBy: 1! !

!UiToolButton methodsFor: 'actions' stamp: 'mt 2/26/2011 15:17:01.592'!
raise

	self autoRaise ifTrue: [self raised: true].! !

!UiToolButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 20:20:48.182'!
raised

	^ raised ifNil: [raised := true]! !

!UiToolButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 20:55:53.198'!
raised: aBoolean

	aBoolean == raised ifTrue: [^ self].

	raised := aBoolean.
	self redrawLater.! !

!UiToolButton methodsFor: 'actions' stamp: 'mt 2/26/2011 15:17:36.308'!
release: aMouseEvent

	(self bounds containsPoint: aMouseEvent position)
		ifFalse: [self flatten].
		
	super release: aMouseEvent.! !

!UiToolButton methodsFor: 'accessing' stamp: 'mt 6/20/2010 20:16:46.914'!
text: aString

	super text: aString.
	self updateSubmorphs.! !

!UiToolButton methodsFor: 'updating' stamp: 'mt 7/8/2013 16:34:49.769'!
updateSubmorphs

	self removeAllMorphs.
	
	self buttonStyle = #textUnderIcon
		ifTrue: [self listDirection: #topToBottom]
		ifFalse: [self listDirection: #leftToRight].
	
	self icon ifNotNil: [:i | (self buttonStyle ~= #textOnly 
		or: [self text isNil]) ifTrue: [
			self enabled
				ifTrue: [self addMorphBack: i asMorph]
				ifFalse: [self addMorphBack: i asGrayScale2 asMorph]]].
	self text ifNotNil: [:t | (self buttonStyle ~= #iconOnly
		or: [self icon isNil]) ifTrue: [
			| preparedText |
			preparedText := self enabled
				ifTrue: [t asHtmlText]
				ifFalse: [('<font color="#444444>', t asString, '</font>') asHtmlText].
			preparedText addAttribute: (TextFontReference toFont: UiPlatform current standardButtonFont).
			self addMorphBack: preparedText asMorph lock]].! !

Morph subclass: #UiComboBox
	instanceVariableNames: 'button readOnlyButton arrowButton lineEdit listWidget converter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiComboBox commentStamp: 'mt 1/6/2012 09:53:49.843' prior: 0!
A combo box is a mixture of a list and a line edit field. It can be #readOnly in which case the edit field becomes a clickable button.

Use combo boxes to present choices.!

!UiComboBox class methodsFor: 'testing' stamp: 'mt 6/29/2010 16:31:40.618'!
isWidgetClass

	^ true! !

!UiComboBox class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:29:17.438'!
widgetCategory
	
	^ 'Input Widgets'! !

!UiComboBox class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:13:05.962'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiComboBoxIcon! !

!UiComboBox class methodsFor: 'accessing' stamp: 'mt 6/30/2010 21:23:12.475'!
widgetClassName

	^ 'Combo Box'! !

!UiComboBox class methodsFor: 'properties' stamp: 'mt 10/28/2010 16:52:41.237'!
widgetProperties

	^ super widgetProperties, {{'Combo Box'. #yourself} -> {
		#readOnly -> Boolean}}! !

!UiComboBox class methodsFor: 'properties' stamp: 'mt 12/17/2011 14:37:40.505'!
widgetPropertiesToIgnore

	^ #('TableLayout' 'ProportionalLayout' vResizing)! !

!UiComboBox methodsFor: 'adding' stamp: 'mt 3/29/2011 18:21:08.755'!
addIcon: anIcon text: aLabel

	listWidget addIcon: anIcon text: aLabel.
		
	(listWidget items size = 1 and: [self readOnly])
		ifTrue: [self currentIndex: 1]! !

!UiComboBox methodsFor: 'adding' stamp: 'mt 1/5/2012 15:39:05.183'!
addItemListFrom: someObjects using: aBlock
	"myComboBox addItemListFrom: #(a b c) using: [:symbol :listItem | listItem text: symbol]."

	| wasEmpty |
	wasEmpty := listWidget items isEmpty.

	listWidget
		addItemListFrom: someObjects
		using: aBlock.
		
	(wasEmpty and: [self readOnly])
		ifTrue: [self currentIndex: 1].! !

!UiComboBox methodsFor: 'adding' stamp: 'mt 3/29/2011 18:21:03.793'!
addText: aLabel

	listWidget addText: aLabel.
	
	(listWidget items size = 1 and: [self readOnly])
		ifTrue: [self currentIndex: 1]! !

!UiComboBox methodsFor: 'adding' stamp: 'mt 3/30/2011 10:06:01.265'!
addTextList: someLabels

	| wasEmpty |
	wasEmpty := listWidget items isEmpty.
	
	listWidget addTextList: someLabels.
	
	(wasEmpty and: [self readOnly])
		ifTrue: [self currentIndex: 1]! !

!UiComboBox methodsFor: 'accessing - private' stamp: 'mt 1/5/2012 16:20:10.654'!
arrowButton

	^ arrowButton! !

!UiComboBox methodsFor: 'accessing - private' stamp: 'mt 12/17/2011 14:10:56.951'!
button

	^ button! !

!UiComboBox methodsFor: 'initialization' stamp: 'mt 2/14/2012 16:18:23.562'!
changeTableLayout

	"Ignore."
	self flag: #workaround. "Backward compatibility with old UIs."! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 9/12/2013 14:31:20.118'!
clear

	listWidget clear.
	self updateCurrentRow.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 1/6/2012 09:37:43.674'!
color
	"Return button color if in read-only mode."

	^ self readOnly
		ifTrue: [readOnlyButton color]
		ifFalse: [super color]! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 1/6/2012 09:38:08.774'!
color: aColor
	"Base color of this combo box cannot be overridden. It should always be white."

	readOnlyButton color: aColor.
	arrowButton color: aColor.
	button color: aColor.
	listWidget borderColor: aColor muchDarker.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 6/30/2010 09:57:53.22'!
converter

	^ converter ifNil: [converter := [:item | item]]! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 6/30/2010 09:57:11.326'!
converter: aBlock

	converter := aBlock.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 6/30/2010 08:19:30.874'!
currentIndex

	^ listWidget currentRow! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 6/30/2011 10:59:50.545'!
currentIndex: anIndex

	listWidget currentRow: anIndex.! !

!UiComboBox methodsFor: 'signals' stamp: 'mt 6/29/2010 17:11:28.048'!
currentIndexChanged: index text: text

	self emit.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 5/17/2011 13:28:57.881'!
currentItem

	^ listWidget currentItem! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 6/30/2011 10:55:23.408'!
currentItem: aListWidgetItem

	listWidget currentItem: aListWidgetItem.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 6/30/2010 09:58:02.404'!
currentText

	^ self converter value: lineEdit theText! !

!UiComboBox methodsFor: 'signals' stamp: 'mt 3/29/2011 11:36:54.865'!
currentTextChanged: text

	self emitAlways.! !

!UiComboBox methodsFor: 'drawing' stamp: 'mt 1/5/2012 16:24:08.142'!
drawOn: aCanvas

	UiPainter default new
		drawComboBox: self on: aCanvas.! !

!UiComboBox methodsFor: 'drawing' stamp: 'mt 1/5/2012 16:02:15.687'!
drawOverlayOn: aCanvas

	UiPainter default new
		drawComboBox: self overlayOn: aCanvas.! !

!UiComboBox methodsFor: 'events' stamp: 'mt 6/23/2011 10:38:46.144'!
hasKeyboard

	^ super hasKeyboard or: [listWidget hasKeyboard]! !

!UiComboBox methodsFor: 'testing' stamp: 'mt 1/6/2012 09:47:31.166'!
hasUnacceptedChanges

	^ self readOnly not and: [lineEdit hasUnacceptedChanges]! !

!UiComboBox methodsFor: 'private' stamp: 'mt 1/6/2012 10:00:02.37'!
hideList

	button checked: false.
	arrowButton checked: false.
	readOnlyButton checked: false.

	"listWidget fadeOutThen: [listWidget show; delete]."
	listWidget delete.! !

!UiComboBox methodsFor: 'initialization' stamp: 'mt 1/6/2012 10:27:57.962'!
initialize

	super initialize.
	
	self
		initializeSupportMorphs;
		initializeEventHandler.
	
	self
		width: 150;
		layoutPolicy: UiComboBoxLayout new.
	super color: Color white.
		
	"Initialize the submorphs the first time."
	self readOnly: true.
		
	self connect: listWidget signal: #currentRowChanged: toSelector: #updateCurrentRow:.
	self connect: listWidget signal: #currentRowChanged: toSelector: #hideList.
	
	"TODO: use other signals for changing text"
	self connect: lineEdit signal: #returnPressed: toSelector: #currentTextChanged:.! !

!UiComboBox methodsFor: 'initialization' stamp: 'mt 1/6/2012 10:24:12.895'!
initializeEventHandler

	self eventHandler: UiEventHandler new.
	
	self connect: self eventHandler signal: #mousePressed: to: button selector: #press:.
	self connect: self eventHandler signal: #mousePressed: to: arrowButton selector: #press:.
	self connect: self eventHandler signal: #mousePressed: to: readOnlyButton selector: #press:.

	self connect: self eventHandler signal: #mouseReleased: toSelector: #passMouseUp:.
	self connect: self eventHandler signal: #mouseReleased: toSelector: #toggleList.! !

!UiComboBox methodsFor: 'initialization' stamp: 'mt 8/21/2012 09:49:44.756'!
initializeSupportMorphs
	
	button := UiPushButton new
		text: nil;
		checkable: true;
		vResizing: #rigid;
		icon: UiButtonIcons bulletArrowDownSunkenIcon;
		layoutInset: (3@4 corner: 3@4).
		
	arrowButton := UiToolButton new "for #autoRaise in read-write-mode"
		buttonStyle: #iconOnly; "no text"
		checkable: true;
		vResizing: #rigid;
		icon: UiButtonIcons bulletArrowDownSunkenIcon;
		color: (Color gray: 0.85);
		layoutInset: (3@4 corner: 3@4).
	self connect: arrowButton signal: #toggled: toSelector: #toggleList.

	lineEdit := UiLineEdit new
		outerBorderWidth: 1.

	listWidget := UiListWidget new
		outerBorderWidth: 1.
	"The list widget behaves like a submorph for keyboard focus change."
	self connect: listWidget eventHandler signal: #focusReceivedInSubmorph: toSelector: #keyboardFocusChange:inSubmorph: pattern: #(=true 1).
	self connect: listWidget eventHandler signal: #focusLostInSubmorph: toSelector: #keyboardFocusChange:inSubmorph: pattern: #(=false 1).
	
	readOnlyButton := UiPushButton new
		text: ' ';
		icon: nil;
		checkable: true;
		hResizing: #spaceFill.! !

!UiComboBox methodsFor: 'accessing - private' stamp: 'mt 12/17/2011 14:11:17.62'!
lineEdit

	^ lineEdit! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 6/30/2011 10:53:41.053'!
list

	^ listWidget! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 8/21/2012 09:37:23.428'!
outerBorderWidth

	^ 1  ! !

!UiComboBox methodsFor: 'geometry' stamp: 'mt 1/5/2012 16:30:56.239'!
outerBounds

	^ self bounds outsetBy: 1! !

!UiComboBox methodsFor: 'event handling' stamp: 'mt 1/6/2012 09:57:03.293'!
passMouseUp: aMouseEvent
	"Modify the event position so that the buttons will not ignore the mouse up."
	
	| oldPos |
	(self bounds containsPoint: aMouseEvent position)
		ifFalse: [^ false].
	
	oldPos := aMouseEvent position.
	
	aMouseEvent setPosition: button center.
	button release: aMouseEvent.
	aMouseEvent setPosition: arrowButton center.
	arrowButton release: aMouseEvent.
	
	aMouseEvent setPosition: readOnlyButton center.    
	readOnlyButton release: aMouseEvent.
	
	aMouseEvent setPosition: oldPos.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 6/29/2010 16:38:37.021'!
readOnly

	^ lineEdit readOnly! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 1/6/2012 10:25:32.079'!
readOnly: aBoolean

	lineEdit readOnly: aBoolean.
	
	self eventHandler mouseDownPriority: (aBoolean ifTrue: [1] ifFalse: [0]).
	
	self updateSubmorphs.! !

!UiComboBox methodsFor: 'accessing - private' stamp: 'mt 12/17/2011 14:11:04.989'!
readOnlyButton

	^ readOnlyButton! !

!UiComboBox methodsFor: 'private' stamp: 'mt 3/29/2011 10:32:20.376'!
showList

	listWidget size = 0
		ifTrue: [^ self].
	
	listWidget
		useOptimalSize;
		width: (self width max: listWidget width);
		height: (listWidget height min: 100);
		topLeft: (self localPointToGlobal: self bottomLeft);
		openInWorld.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:11:32.602'!
size

	^ listWidget size! !

!UiComboBox methodsFor: 'private' stamp: 'mt 1/6/2012 09:59:13.53'!
toggleList

	(button checked or: [arrowButton checked])
		ifTrue: [self showList]
		ifFalse: [self hideList].! !

!UiComboBox methodsFor: 'updating' stamp: 'mt 5/20/2011 09:52:00.318'!
updateCurrentRow

	self updateCurrentRow: self currentIndex.! !

!UiComboBox methodsFor: 'updating' stamp: 'mt 5/17/2011 10:19:31.707'!
updateCurrentRow: anIndex

	lineEdit theText: listWidget currentText.
	
	readOnlyButton text: listWidget currentText.
	readOnlyButton icon: listWidget currentIcon.
	
	self currentIndexChanged: anIndex text: (self converter value: lineEdit theText).
	self currentTextChanged: (self converter value: lineEdit theText).! !

!UiComboBox methodsFor: 'updating' stamp: 'mt 1/6/2012 10:29:30.531'!
updateSubmorphs

	self removeAllMorphs.
	
	self
		addMorphBack: (self readOnly ifTrue: [button] ifFalse: [arrowButton]);
		addMorphBack: (self readOnly ifTrue: [readOnlyButton] ifFalse: [lineEdit]).! !

Morph subclass: #UiItemViewMorph
	instanceVariableNames: 'form formPosition label labelColor labelPosition font'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiItemViewMorph commentStamp: 'mt 6/29/2012 16:24:47.693' prior: 0!
This is a simple version of an item.!

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 2/14/2013 14:36:31.634'!
color

	^ color! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 2/14/2013 14:36:10.755'!
color: aColor

	color := aColor.! !

!UiItemViewMorph methodsFor: 'drawing' stamp: 'mt 7/8/2013 17:07:35.226'!
drawOn: aCanvas

	self color ifNotNil: [:c |
		UiPlatform current
			canvas: aCanvas
			fillRoundRect: (self bounds insetBy: 1)
			radius: 3
			fillStyle: (SolidFillStyle color: c)].

	self form ifNotNil: [:f |
		aCanvas
			translucentImage: f
			at: self formPosition + self topLeft].
	
	self label ifNotNil: [:l |
		aCanvas
			drawString: l
			at: self labelPosition + self topLeft
			font: self font
			color: self labelColor].! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:28:27.288'!
font
	^ font! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:28:27.304'!
font: anObject
	font := anObject! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:28:27.194'!
form
	^ form! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:28:27.238'!
form: anObject
	form := anObject! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:30:02.805'!
formPosition
	^ formPosition! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:30:02.852'!
formPosition: anObject
	formPosition := anObject! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:28:27.257'!
label
	^ label! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:28:27.273'!
label: anObject
	label := anObject! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:30:32.089'!
labelColor
	^ labelColor! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:30:32.138'!
labelColor: anObject
	labelColor := anObject! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:30:02.875'!
labelPosition
	^ labelPosition! !

!UiItemViewMorph methodsFor: 'accessing' stamp: 'mt 6/29/2012 16:30:02.89'!
labelPosition: anObject
	labelPosition := anObject! !

Morph subclass: #UiItemViewport
	instanceVariableNames: 'rows columns hoveredCell hoveredDropMode selectionColor selection selectionMode selectionBehavior verticalHeader horizontalHeader rowOrder columnOrder isLayoutBeingModified groupSpec cellToBounds drawCache drawCacheActive dragDropSpec editTriggers selectBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiItemViewport commentStamp: 'mt 11/2/2010 12:48:55.736' prior: 0!
An item viewport represents the whole area where a model's items are layouted. Usually, items (just morphs) are created by using an item delegate that reads a model index and therefore model data. A viewport handles item selection.

There is a difference between a model index and a cell index. Cell indices are numbered in visual order (e.g., according to the current sorting) and may change as the sorting order changes, whereas model indices always address some data in the model and are not changed by the viewport (or view) only by the model itself. Additionally, cell indices to not have a hierarchy, i.e. a parent cell index, and are always two-dimensional.

All item morphs get additional properties, e.g., #modelIndex, #hasChildren, #isExpanded, to track different state or information changed or needed by event handling or drawing routines in the viewport.!

!UiItemViewport methodsFor: 'drag/drop' stamp: 'mt 8/22/2012 11:52:55.307'!
acceptDroppingMorph: aTransformMorph event: event

	| cell |
	cell := self cellIndexFromPoint: event position.
	cell ifNil: [
		^ self
			objectDropped: aTransformMorph passenger
			type: aTransformMorph dragTransferType
			from: aTransformMorph source].
	
	self dragDropSpec dropMode == #over ifTrue: [
		self
			objectDropped: aTransformMorph passenger
			type: aTransformMorph dragTransferType
			from: aTransformMorph source
			on: (self modelIndexFor: cell)].
		
	self dragDropSpec dropMode == #between ifTrue: [
		| cellBounds |
		cellBounds := self cellBoundsFor: cell.
		
		(self dropsBefore: event bounds: cellBounds)
			ifTrue: [
				self
					objectDropped: aTransformMorph passenger
					type: aTransformMorph dragTransferType
					from: aTransformMorph source
					before: (self modelIndexFor: cell)	]
			ifFalse: [
				self
					objectDropped: aTransformMorph passenger
					type: aTransformMorph dragTransferType
					from: aTransformMorph source
					after: (self modelIndexFor: cell)]].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 7/10/2012 11:22:36.37'!
activateCurrentCell

	self selection currentIndex ifNil: [^ self].

	self selection currentCell ifNotNil: [
		self
			activated: self selection currentIndex;
			returnPressed: self selection currentIndex].! !

!UiItemViewport methodsFor: 'signals - model index' stamp: 'mt 10/19/2010 10:28:08.197'!
activated: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'add/remove - appending' stamp: 'mt 6/26/2012 09:43:54.699'!
appendGroupSeparator: aGroupSeparator

	| row |
	row := rows add: UiViewRowInfo new.
	self columns do: [:column |
		column items addLast: aGroupSeparator.
		row items add: aGroupSeparator.
		self addMorphBack: aGroupSeparator].! !

!UiItemViewport methodsFor: 'add/remove - appending' stamp: 'mt 7/1/2012 11:00:21.281'!
appendItemRows: data
	"group -> row -> column -> model index -> item/morph"
	
	data withIndexDo: [:group :groupIndex |
		self appendGroupSeparator: group key.
		
		group value do: [:row | row do: [:indexToItem |
			self
				appendMorph: indexToItem value
				inRow: (self appendRowFor: indexToItem key)
				forIndex: indexToItem key]]].! !

!UiItemViewport methodsFor: 'add/remove - appending' stamp: 'mt 7/1/2012 11:02:16.873'!
appendMorph: aMorph forIndex: aModelIndex
	
	self
		appendMorph: aMorph
		inRow: (self appendRowFor: aModelIndex)
		forIndex: aModelIndex.! !

!UiItemViewport methodsFor: 'add/remove - appending' stamp: 'mt 7/1/2012 11:00:03.761'!
appendMorph: aMorph inRow: aRowInfo forIndex: aModelIndex
	"Optimized insertion method that assumes that the view is filled row-by-row from left to right."
	
	| col row |
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: false.
	aMorph setProperty: #hasChildren toValue: aModelIndex hasChildren.
	
	row := aRowInfo.
	row items add: aMorph.
	
	col := self columnFor: aModelIndex.
	col items addLast: aMorph.
	
	self addMorphBack: aMorph.
	
	aMorph
		setProperty: #indent
		toValue: (self calculateIndentFor: aMorph atIndex: aModelIndex column: col).! !

!UiItemViewport methodsFor: 'add/remove - appending' stamp: 'mt 4/23/2012 11:51:44.338'!
appendRowFor: aModelIndex
	"Optimized method that only appends a new row at the end of all rows."
	
	aModelIndex parent ifNotNil: [self error: 'You cannot use this method for non-toplevel indices.'].
	
	^ (rows notEmpty and: [rows last isRowFor: aModelIndex])
		ifTrue: [rows last "used for column-wise appending. see #appendItemRows:."]
		ifFalse: [rows add: UiViewRowInfo new].! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 3/7/2011 14:42:08.321'!
beginChangeLayout

	isLayoutBeingModified := true.! !

!UiItemViewport methodsFor: 'events - in-place editing' stamp: 'mt 2/6/2013 13:56:32.098'!
beginEditCell: aCell

	aCell ifNil: [^ self].
	^ self beginEditItem: (self morphAt: aCell)! !

!UiItemViewport methodsFor: 'events - in-place editing' stamp: 'mt 7/8/2012 14:54:45.397'!
beginEditItem: anItem

	| index editor |
	index := anItem valueOfProperty: #modelIndex.
	
	(index editable not or: [index isInEditMode])
		ifTrue: [^ false "no further editing possible"].
	
	editor := index editor.
	
	self replaceMorphWith: editor atIndex: index.
	
	index spec editorAcceptSignal ifNotNil: [:s |
		self connect: editor signal: s toSelector: #endEditItem:acceptChanges: pattern: #(0 =true)].
	index spec editorRevertSignal ifNotNil: [:s |
		self connect: editor signal: s toSelector: #endEditItem:acceptChanges: pattern: #(0 =false)].
	
	index isInEditMode: true.
	
	self updateLayout.
	
	editor grabKeyboard.
	
	^ true! !

!UiItemViewport methodsFor: 'geometry - cached' stamp: 'mt 7/3/2012 11:41:26.842'!
boundsAt: aCell

	| boundsData |
	boundsData := (cellToBounds at: aCell y) at: aCell x.
	^ boundsData first extent: boundsData second! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 8/30/2012 10:14:53.963'!
boundsFor: aCellIndex
	"Returns the relevant bounds for a given cell index based on the current selection behavior."

	aCellIndex ifNil: [^ 0@0 extent: 0@0].

	^ self selectionBehavior == #selectCells
		ifTrue: [self cellBoundsFor: aCellIndex]
		ifFalse: [self selectionBehavior == #selectRows
			ifTrue: [self rowBoundsAt: aCellIndex y]
			ifFalse: [self selectionBehavior == #selectColumns
				ifTrue: [self columnBoundsAt: aCellIndex x]
				ifFalse: [0@0 extent: 0@0]]]! !

!UiItemViewport methodsFor: 'add/remove - private' stamp: 'mt 8/27/2012 15:34:45.736'!
calculateIndentFor: aMorph atIndex: aModelIndex column: col

	| indent itemBefore |
	aModelIndex column = 1 ifFalse: [^ 0].
	(aModelIndex parent isNil and: [self rootIsDecorated not])
		ifTrue: [^ 0].
	
	indent := 0.
	itemBefore := aMorph.

	[(itemBefore := col itemBefore: itemBefore)
		ifNil: [indent := self indent. false] "root"
		ifNotNil: [:item |
			(item hasProperty: #isGroupSeparator) "== special sibling"
			ifTrue: ["Just skip group separators." true]
			ifFalse: ["The item before is my parent. Add additional indentation."
				(item valueOfProperty: #modelIndex) = aModelIndex parent
					ifTrue: [indent := (item valueOfProperty: #indent) + self indent. false]
					ifFalse: ["The item before is my sibling. Use its indentation."
						(item valueOfProperty: #modelIndex) parent = aModelIndex parent
							ifTrue: [indent := item valueOfProperty: #indent. false]
							ifFalse: [true]]]].
	] whileTrue.
	
	^ indent! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 6/30/2012 16:31:16.4'!
cellBoundsFor: aCellIndex

	(cellToBounds at: aCellIndex y ifAbsent: [^ 0@0 extent: 0@0])
		ifNil: [^ 0@0 extent: 0@0]
		ifNotNil: [:rowBounds | (rowBounds at: aCellIndex x ifAbsent: [^ 0@0 extent: 0@0])
			ifNil: [^ 0@0 extent: 0@0]
			ifNotNil: [:data | ^ data first extent: data second]].! !

!UiItemViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 7/6/2012 10:40:23.802'!
cellIndexAt: colAndRow

	| row column |
	row := colAndRow y.
	column := colAndRow x.

	^ (self columnOrder mapIndexOf: column)
		@ (self rowOrder mapIndexOf: row)! !

!UiItemViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 6/30/2012 11:44:01.136'!
cellIndexFromColumn: aColumnInfo

	^ self cellIndexAt: (columns indexOf: aColumnInfo) @ 1! !

!UiItemViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 7/6/2012 10:39:59.967'!
cellIndexFromModelIndex: aModelIndex

	| row column |
	aModelIndex ifNil: [^ nil].

	row := rows indexOf: (self rowFor: aModelIndex).
	column := columns indexOf: (self columnFor: aModelIndex).

	^ (self columnOrder mapIndexOf: column)
		@ (self rowOrder mapIndexOf: row)! !

!UiItemViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 7/19/2012 10:32:37.914'!
cellIndexFromPoint: aPoint
	"Given a point in pixel coordinates."
	
	| origin cellIndex |
	origin := self topLeft.
	cellIndex := 0@0.
	self rowsDo: [:rowInfo |
		cellIndex := 0 @ (cellIndex y + 1).
		rowInfo visible ifTrue: [
			self columnsDo: [:colInfo |
				cellIndex := cellIndex + (1@0).
				colInfo visible ifTrue: [
					(origin <= aPoint and: [aPoint < (origin + (colInfo width @ rowInfo height))])
						ifTrue: [^ cellIndex].
					origin := origin translateBy: colInfo width @ 0]].
			origin := self left @ (origin y + rowInfo height)]].
	
	^ nil! !

!UiItemViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 6/30/2012 11:44:24.376'!
cellIndexFromRow: aRowInfo

	^ self cellIndexAt: 1 @ (rows indexOf: aRowInfo)! !

!UiItemViewport methodsFor: 'geometry - cached' stamp: 'mt 7/3/2012 11:40:24.421'!
cellToBounds

	^ cellToBounds! !

!UiItemViewport methodsFor: 'initialization' stamp: 'mt 3/23/2012 10:18:38.055'!
clear

	self selection clear.
	
	columns do: [:col | col clear]. "Keep column-specific settings." self flag: #inconsistency.
	rows removeAll.
	
	self
		removeAllMorphs;
		extent: 1@1.! !

!UiItemViewport methodsFor: 'signals - model index' stamp: 'mt 10/8/2010 16:05:54.303'!
clicked: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 6/30/2012 15:58:06.383'!
columnAt: index

	^ self columnOrder items: columns at: index! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 8/30/2012 09:58:51.41'!
columnBoundsAt: aColIndex

	aColIndex > columns size
		ifTrue: [^ 0@0 extent: 0@0].

	^ self columnBoundsFor: (self columnAt: aColIndex)! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 8/30/2012 09:59:52.885'!
columnBoundsFor: aColumnInfo

	aColumnInfo visible
		ifFalse: [^ 0@0 extent: 0@0].
	
	^ (self leftForColumn: aColumnInfo) @ self top
		extent: aColumnInfo width @ self height! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 4/21/2011 10:29:09.758'!
columnCount

	^ columns size! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 7/19/2012 11:12:55.845'!
columnCountVisible

	^ self columns count: [:columnInfo | columnInfo visible]! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/9/2010 11:26:53.355'!
columnFor: aModelIndex

	^ columns
		at: aModelIndex column
		ifAbsentPut: [UiViewColumnInfo new]! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/16/2010 15:00:57.992'!
columnOrder

	^ columnOrder! !

!UiItemViewport methodsFor: 'accessing - private' stamp: 'mt 1/4/2011 10:23:35.14'!
columns
	"Returns all columns IGNORING CURRENT SORT ORDER."
	
	^ columns! !

!UiItemViewport methodsFor: 'enumerating - ordered' stamp: 'mt 10/17/2010 08:20:05.896'!
columnsDo: aBlock

	self columnOrder items: columns do: aBlock.! !

!UiItemViewport methodsFor: 'enumerating - ordered' stamp: 'mt 10/17/2010 08:20:10.134'!
columnsWithIndexDo: aBlock

	self columnOrder items: columns withIndexDo: aBlock.! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 2/6/2013 14:03:33.712'!
copyCurrentTextToClipboard

	self selection currentCell ifNotNil: [:cell |
		Clipboard clipboardText: ((self morphAt: cell)
				valueOfProperty: #text ifAbsent: ['']) copy].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 2/15/2013 15:35:08.077'!
copySelectedTextToClipboard

	Clipboard clipboardText: 
		(self selection cells inject: ''
			into: [:text :cell | text, ((self morphAt: cell) valueOfProperty: #text ifAbsent: ['']) asString, ' ']).! !

!UiItemViewport methodsFor: 'signals - model index' stamp: 'mt 10/20/2010 13:54:10.953'!
currentIndexChanged: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 2/6/2013 11:45:06.307'!
deselectCell: aCell event: aUserInputEvent

	| morph |
	morph := self morphAt: aCell.

	(morph valueOfProperty: #selectable) ifTrue: [
		self selection remove: aCell event: aUserInputEvent].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 8/31/2012 13:11:33.878'!
doHoverCell: evt

	self hoveredDropMode: #none.
	
	(self cellIndexFromPoint: evt position) ifNotNil: [:cell |		
		((self morphAt: cell) valueOfProperty: #selectable)
			ifFalse: [self hoveredCell: nil]
			ifTrue: [
				(evt hand submorphs notEmpty and: [self wantsDroppedMorph: evt hand submorphs first event: evt]) ifTrue: [
				self
					doHoverDropTarget: evt
					forCell: cell].
			
			(evt hand submorphs isEmpty or: [self dropEnabled])
				ifTrue: [self hoveredCell: cell]]]
	
	"Allow dropping in an empty view."
	ifNil: [
		self hoveredCell: nil.
		(evt hand submorphs notEmpty and: [self wantsDroppedMorph: evt hand submorphs first event: evt])
			ifTrue: [self hoveredDropMode: #full]].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 8/22/2012 11:48:27.533'!
doHoverDropTarget: evt forCell: cell

	"visualize drop target"
	self dragDropSpec ifNotNil: [:spec |
		spec dropMode == #over ifTrue: [
			self hoveredDropMode: #over].
		spec dropMode == #between ifTrue: [
			| cellBounds |
			cellBounds := self cellBoundsFor: cell.
			evt position y < (cellBounds top + (cellBounds height // 2))
				ifTrue: [self hoveredDropMode: #inFrontOf]
				ifFalse: [self hoveredDropMode: #after]]].! !

!UiItemViewport methodsFor: 'signals - model index' stamp: 'mt 10/8/2010 16:05:00.799'!
doubleClicked: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:19:04.808'!
dragDropSpec
	^ dragDropSpec! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:19:04.854'!
dragDropSpec: anObject
	dragDropSpec := anObject! !

!UiItemViewport methodsFor: 'drag/drop' stamp: 'mt 7/18/2012 10:05:24.441'!
dragEnabled

	^ self dragDropSpec
		ifNil: [false]
		ifNotNil: [:spec | spec dragEnabled]! !

!UiItemViewport methodsFor: 'drawing - cached' stamp: 'mt 7/3/2012 11:04:19.284'!
drawCacheActive

	^ drawCacheActive ifNil: [false]! !

!UiItemViewport methodsFor: 'drawing - cached' stamp: 'mt 7/3/2012 11:06:24.82'!
drawCacheActive: aBoolean

	aBoolean = drawCacheActive ifTrue: [^ self].
	
	drawCacheActive := aBoolean.
	
	drawCacheActive
		ifTrue: [self updateDrawCache].! !

!UiItemViewport methodsFor: 'drawing - cached' stamp: 'mt 7/3/2012 11:50:57.267'!
drawCell: aCell on: aCanvas
	
	aCell ifNil: [^ self].
	
	self selectionBehavior == #selectCells ifTrue: [
		aCanvas
			clipBy: (self boundsAt: aCell)
			during: [:c | c fullDrawMorph: (self morphAt: aCell)]].
		
	self selectionBehavior == #selectRows ifTrue: [
		self columnOrder items: (self rowAt: aCell y) items withIndexDo: [:morph :colIndex |
			aCanvas
				clipBy: (self boundsAt: colIndex @ aCell y)
				during: [:c | c fullDrawMorph: morph]]].
		
	self selectionBehavior == #selectColumns ifTrue: [
		self rowOrder items: (self columnAt: aCell x) items withIndexDo: [:morph :rowIndex |
			aCanvas
				clipBy: (self boundsAt: aCell x @ rowIndex)
				during: [:c | c fullDrawMorph: morph]]].! !

!UiItemViewport methodsFor: 'drawing - cached' stamp: 'mt 7/3/2012 11:55:08.352'!
drawCell: aCell whiteOn: aCanvas
	
	aCell ifNil: [^ self].
	
	self selectionBehavior == #selectCells ifTrue: [
		aCanvas
			clipBy: (self boundsAt: aCell)
			during: [:c | c fillColor: Color white]].
		
	self selectionBehavior == #selectRows ifTrue: [
		1 to: self columnCount do: [:colIndex |
			aCanvas
				clipBy: (self boundsAt: colIndex @ aCell y)
				during: [:c | c fillColor: Color white]]].
		
	self selectionBehavior == #selectColumns ifTrue: [
		1 to: self rowCount do: [:rowIndex |
			aCanvas
				clipBy: (self boundsAt: aCell x @ rowIndex)
				during: [:c | c fillColor: Color white]]].! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 2/6/2013 14:49:51.629'!
drawCurrentCellOn: aCanvas

	self selection currentCell ifNil: [^ self].
	
	((self selectionBehavior == #selectRows and: [self selection currentCell x > 1]) or: [
		self selectionBehavior == #selectColumns and: [self selection currentCell y > 1]])
		ifTrue: [^ self
			drawHighlightOn: aCanvas
			bounds: (self cellBoundsFor: self selection currentCell)
			color: self selectionColorAdjusted darker].
	
	(self selection cells noneSatisfy: [:cell | cell y = self selection currentCell y])
		ifTrue: [^ self
			drawHighlightFrameOn: aCanvas
			bounds: (self boundsFor: self selection currentCell)
			color: self selectionColorAdjusted darker].
	
	self
		drawHighlightOn: aCanvas
		bounds: (self boundsFor: self selection currentCell)
		color: self selectionColorAdjusted darker.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 7/8/2013 17:05:42.059'!
drawDropTargetOn: aCanvas

	| cellBounds |
	self hoveredDropMode == #none ifTrue: [^ self].
	
	"Visualize drop target."
	self hoveredDropMode == #full ifTrue: [
		| box |
		self flag: #slow. "mt: use cellToBounds cache here?"
		box := self submorphBounds.
		^ aCanvas
			line: self left @ box bottom
			to: self right @ box bottom
			width: 3
			color: self selectionColorAdjusted muchDarker].
	
	cellBounds := self boundsFor: self hoveredCell.
	
	self hoveredDropMode == #over ifTrue: [
		^ UiPlatform current
			canvas: aCanvas
			frameRoundRect: cellBounds
			radius: 3
			width: 2
			color: self selectionColorAdjusted muchDarker].
		
	self hoveredDropMode == #inFrontOf ifTrue: [
		^ aCanvas
			line: cellBounds topLeft + (2@0)
			to: cellBounds topRight - (2@0)
			width: 3
			color: self selectionColorAdjusted muchDarker].
		
	self hoveredDropMode == #after ifTrue: [
		^ aCanvas
			line: cellBounds bottomLeft + (2@0)
			to: cellBounds bottomRight - (2@0)
			width: 3
			color: self selectionColorAdjusted muchDarker].! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 11/16/2012 15:45:31.384'!
drawGroupSeparator: aMorph on: aCanvas
	"Draw something that expands the group separator item to the whole width."
	
	aCanvas fullDrawMorph: aMorph.
	
	(aMorph valueOfProperty: #drawLine) == true ifTrue: [
		aCanvas
			line: aMorph right @ (aMorph center y + self groupSpec lineOffset)
			to: self right @ (aMorph center y + self groupSpec lineOffset)
			color: (self groupSpec lineColor alpha: 0.4)].! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 7/8/2013 16:39:47.395'!
drawHelpTextOn: aCanvas

	(self dragDropSpec isNil or: [self dragDropSpec helpText isNil]) ifTrue: [^ self].
	(self rows anySatisfy: [:rowInfo | rowInfo visible])
		ifTrue: [^ self].

	self dragDropSpec helpText isString ifTrue: [  
		| font |	
		font := UiPlatform current standardListFont.

		^ aCanvas
			drawString: self dragDropSpec helpText asString
			in: (self topLeft + (5@0) extent: 500 @ font height)
			font: font
			color: (Color gray: 0.7)].
		
	self dragDropSpec helpText isText ifTrue: [
		| m |
		m := self dragDropSpec helpText asMorph.
		m topLeft: self topLeft + (5@0).
		m fullDrawOn: aCanvas].! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 7/8/2013 17:05:48.472'!
drawHighlightFrameOn: aCanvas bounds: drawBounds color: aColor
	
	UiPlatform current
		canvas: aCanvas
		frameRoundRect: (drawBounds outsetBy: (0@0 corner: 0@1))
		radius: 3
		width: 1
		color: aColor muchDarker.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 7/8/2013 17:07:41.433'!
drawHighlightOn: aCanvas bounds: drawBounds color: aColor
	
	| gradient |

	gradient := (GradientFillStyle ramp: {
			0.0 -> Color white.
			0.1-> (aColor adjustBrightness: 0.05).
			0.6 -> (aColor darker)})
		origin: drawBounds topLeft;
		direction: 0 @ drawBounds height.
		
	UiPlatform current
		canvas: aCanvas
		fillRoundRect: (drawBounds insetBy: (1@1 corner: 1@1))
		radius: 3
		fillStyle: gradient.
		
	self drawHighlightFrameOn: aCanvas bounds: drawBounds color: aColor.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 2/6/2013 14:50:38.851'!
drawHoveredCellOn: aCanvas

	self hoveredCell ifNil: [^ self].

	self
		drawHighlightOn: aCanvas
		bounds: (self boundsFor: self hoveredCell)
		color: self selectionColorAdjusted muchLighter.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 8/25/2013 19:48:34.99'!
drawOn: aCanvas
	
	self shouldDraw ifFalse: [^ self].
	
	super drawOn: aCanvas.
		
	self drawCacheActive ifFalse: [
		self
			drawHoveredCellOn: aCanvas;
			drawSelectionOn: aCanvas;
			drawCurrentCellOn: aCanvas;
			drawHelpTextOn: aCanvas].! !

!UiItemViewport methodsFor: 'drawing - cached' stamp: 'mt 8/25/2013 19:47:32.24'!
drawOverlayOn: aCanvas
	
	self shouldDraw ifFalse: [^ self].
	
	self drawCacheActive ifTrue: [
		| importantCells |		
		importantCells := (self selection cells, {self selection currentCell}, {self hoveredCell}) asSet.
		
		"Restore background."
		importantCells do: [:cell | self drawCell: cell whiteOn: aCanvas].
			
		self
			drawHoveredCellOn: aCanvas;
			drawSelectionOn: aCanvas;
			drawCurrentCellOn: aCanvas.
		
		"Fill foreground."	
		importantCells do: [:cell | self drawCell: cell on: aCanvas]].
		
	super drawOverlayOn: aCanvas.
	
	self drawDropTargetOn: aCanvas.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 2/6/2013 14:50:45.868'!
drawSelectionOn: aCanvas

	self selection cells do: [:cell |
		self
			drawHighlightOn: aCanvas
			bounds: (self boundsFor: cell)
			color: self selectionColorAdjusted].! !

!UiItemViewport methodsFor: 'drawing - cached' stamp: 'mt 7/12/2012 15:52:27.609'!
drawSubmorphsCachedOn: aCanvas
	
	aCanvas
		image: drawCache
		at: 0@0
		sourceRect: drawCache boundingBox
		rule: Form blend.! !

!UiItemViewport methodsFor: 'drawing' stamp: 'mt 8/25/2013 19:48:09.821'!
drawSubmorphsOn: aCanvas
	
	| drawBlock |
	self shouldDraw ifFalse: [^ self].

	self drawCacheActive
		ifTrue: [^ self drawSubmorphsCachedOn: aCanvas].
		
	drawBlock := [:canvas :morphs |
		morphs withIndexDo: [:morph :rowIndex |
			| rowInfo |
			"Ignores sort order."
			rowInfo := (self rows at: rowIndex).
			rowInfo visible ifTrue: [
				rowInfo isSeparator
					ifTrue: [self drawGroupSeparator: morph on: canvas ]
					ifFalse: [canvas fullDrawMorph: morph]]]].
	
	"Column-wise drawing with clipping."
	self columnsDo: [:columnInfo | 
		columnInfo visible ifTrue: [
			aCanvas
				clipBy: (self columnBoundsFor: columnInfo)
				during: [:c | drawBlock value: c value: columnInfo items]]].! !

!UiItemViewport methodsFor: 'drag/drop' stamp: 'mt 7/18/2012 10:09:04.313'!
dropEnabled

	^ self dragDropSpec
		ifNil: [false]
		ifNotNil: [:spec | spec dropEnabled]! !

!UiItemViewport methodsFor: 'drag/drop' stamp: 'mt 8/22/2012 11:52:32.377'!
dropsBefore: evt bounds: cellBounds

	^ evt position y < (cellBounds top + (cellBounds height // 2))! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 2/6/2013 13:09:41.195'!
editTriggers

	^ editTriggers ifNil: [editTriggers := #(editKeyPressed doubleClicked)]! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 2/6/2013 14:15:44.071'!
editTriggers: someSymbols
	"#currentChanged, #doubleClicked, #selectedClicked, #editKeyPressed, #alphaNumericKeyPressed"

	editTriggers := someSymbols.! !

!UiItemViewport methodsFor: 'signals - model index' stamp: 'mt 7/8/2012 14:08:08.569'!
edited: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 8/25/2013 19:41:02.808'!
endChangeLayout

	self updateLayout.! !

!UiItemViewport methodsFor: 'events - in-place editing' stamp: 'mt 7/11/2012 14:48:50.078'!
endEditItem: anItem acceptChanges: aBoolean

	| index editedItem |
	index := anItem valueOfProperty: #modelIndex.
	
	index isInEditMode ifFalse: [^ false].

	aBoolean ifTrue: [	
		"Write changes to model."
		index spec editorFinisher
			value: anItem
			value: index node].

	aBoolean ifFalse: [
		"Abort editing."
		editedItem := index item.
		self replaceMorphWith: editedItem atIndex: index.
		self updateLayout].

	index isInEditMode: false.
	aBoolean ifTrue: [self edited: index].
	
	self grabKeyboard.
	
	^ true! !

!UiItemViewport methodsFor: 'initialization' stamp: 'mt 3/23/2012 10:21:39.279'!
ensureColumnCount: numberOfColumns
	"Create columns on-the-fly."
	
	numberOfColumns > columns size
		ifTrue: [numberOfColumns - columns size timesRepeat: [columns add: UiViewColumnInfo new]].! !

!UiItemViewport methodsFor: 'add/remove - private' stamp: 'mt 6/26/2012 11:56:25.365'!
ensureFirstRowFor: aParentModelIndex
	
	^ rows
		add: UiViewRowInfo new
		afterIndex: (rows indexOf: (rows detect: [:row | row firstModelIndex = aParentModelIndex]))! !

!UiItemViewport methodsFor: 'add/remove - private' stamp: 'mt 7/8/2013 14:02:07.412'!
ensureRowFor: aModelIndex
	"Adds a new row if needed. Returns the correct one if possible. Assumption: Parent node is exanded."
	
	| row rowIndex isCorrectRow isPreviousSibling isParentRow isFirstInGroup currentPath |
	isCorrectRow := [:r | ((r isSeparator not
		and: [r parentModelIndex = aModelIndex parent])
		and: [r groupIndex = aModelIndex groupIndex])
		and: [r firstModelIndex row = aModelIndex row]].
	isPreviousSibling := [:r | (r isSeparator not
		and: [r parentModelIndex = aModelIndex parent])
		and: [r groupIndex = aModelIndex groupIndex and: [r firstModelIndex row = (aModelIndex row - 1)]]].
	isParentRow := [:r | r isSeparator not and: [r firstModelIndex = aModelIndex parent]].
	isFirstInGroup := [:separator | ((separator parentModelIndex = aModelIndex parent
		and: [separator groupIndex = aModelIndex groupIndex])
		and: [aModelIndex row = 1])
		and: [aModelIndex column = 1]].
		
	rowIndex := 0.
	
	currentPath := Stack new.
	rows detect: [:r | rowIndex := rowIndex + 1.
		
		(isCorrectRow value: r)
			ifTrue: [row := r. true]
			ifFalse: [
				(r isSeparator and: [isFirstInGroup value: r])
					ifTrue: [row := rows add: UiViewRowInfo new afterIndex: rowIndex. true]
					ifFalse: [	(currentPath isEmpty and: [(isParentRow value: r) or: [isPreviousSibling value: r]])
						ifTrue: [currentPath push: r. false]
						ifFalse: [	currentPath isEmpty ifTrue: [false] ifFalse: [
							""
							[currentPath isEmpty or: [currentPath top firstModelIndex = r parentModelIndex]]
								whileFalse: [currentPath pop].
					
							currentPath isEmpty
								ifTrue: [
									row := rows add: UiViewRowInfo new beforeIndex: (
										(rows at: rowIndex - 1) isSeparator
											ifTrue: [rowIndex - 1]
											ifFalse: [rowIndex]).
									true "first child of parent"]
								ifFalse: [(isPreviousSibling value: r)
									ifTrue: [currentPath := Stack new. currentPath push: r. false]
									ifFalse: [currentPath push: r. false]]]]]]
		] ifNone: [row := rows add: UiViewRowInfo new].
	
	^ row! !

!UiItemViewport methodsFor: 'geometry' stamp: 'mt 7/13/2013 12:49:50.214'!
extent: aPoint

	(self shouldUpdateLayoutForNewExtent: aPoint) in: [:updateNeeded |
		super extent: aPoint.
		self resizedToWidth: aPoint x height: aPoint y.
		updateNeeded ifTrue: [self updateLayout]].! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 10/11/2010 17:18:00.533'!
focusChanged: aRectangle
	"Indicate that a scrollable container should ensure the given area as visible."
	
	self emit.! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 6/26/2012 10:56:44.548'!
groupSpec

	^ groupSpec! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 6/26/2012 10:56:53.843'!
groupSpec: aGroupSpec

	groupSpec := aGroupSpec.! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 3/18/2011 11:08:13.203'!
handleMouseMove: anEvent
	"Receive any mouse move event."
	
	super handleMouseMove: anEvent.
	
	anEvent wasHandled not
		ifTrue: [self mouseMove: anEvent. anEvent wasHandled: true].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 3/7/2013 08:09:09.891'!
hasSubmorphKeyboard
	"Improve lookup speed because there could be many submorphs to check."
	
	self flag: #optimize. "mt: Only if rowCount > 50?"
	^ false! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 8/22/2012 10:04:35.816'!
helpText

	^ self dragDropSpec
		ifNil: [nil]
		ifNotNil: [:spec | spec helpText]! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 9/7/2012 10:19:45.671'!
helpText: aString

	(self dragDropSpec isNil and: [aString isNil or: [aString isEmpty]])
		ifTrue: [^ self].

	^ (self dragDropSpec ifNil: [self dragDropSpec: UiItemViewDragDropSpec new])
		helpText: aString! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:00:05.566'!
horizontalHeader

	^ horizontalHeader! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:00:11.666'!
horizontalHeader: aMorph

	horizontalHeader := aMorph.! !

!UiItemViewport methodsFor: 'signals - model index' stamp: 'mt 8/16/2012 14:42:39.107'!
hovered: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'geometry' stamp: 'mt 8/16/2012 14:49:14.138'!
hoveredBounds

	| hb |
	hb := self boundsFor: self hoveredCell.
	^ (self localPointToGlobal: hb topLeft)
		extent: hb extent  ! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/11/2010 14:39:13.856'!
hoveredCell

	^ hoveredCell! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 8/16/2012 15:42:48.777'!
hoveredCell: newCellIndex
		
	newCellIndex = hoveredCell
		ifTrue: [^ self].
				
	self
		invalidRect: (self boundsFor: newCellIndex);
		invalidRect: (self boundsFor: hoveredCell).

	"Show/hide checkboxes."
	self updateVisibilityForCheckBoxesFor: newCellIndex old: hoveredCell.
	
	"Set new value."
	hoveredCell := newCellIndex.
	
	self hovered: (self modelIndexFor: hoveredCell).! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 7/18/2012 11:04:28.575'!
hoveredDropMode
	"#inFrontOf, #after, #over, #none"
	
	^ hoveredDropMode ifNil: [#none]! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 7/19/2012 09:13:45.09'!
hoveredDropMode: aSymbol

	hoveredDropMode == aSymbol ifTrue: [^ self].
	
	"Need some extra pixels in the vertical to visizualize #between."
	self invalidRect: ((self boundsFor: self hoveredCell) outsetBy: (0@2 corner: 0@2)).
	
	hoveredDropMode := aSymbol.! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/16/2010 08:01:57.895'!
indent
	"Defines how much items of the first column will be moved to the left for each level - including the first one."
	
	^ 0! !

!UiItemViewport methodsFor: 'initialization' stamp: 'mt 1/18/2012 10:37:56.468'!
initialize

	super initialize.
	
	self initializeEventHandler.
	
	self color: Color transparent.
		
	rows := OrderedCollection new.
	columns := OrderedCollection new.
	
	rowOrder := UiItemOrder new.
	columnOrder := UiItemOrder new.
	
	self selectionMode: UiViewSingleSelection.! !

!UiItemViewport methodsFor: 'initialization' stamp: 'mt 2/15/2013 15:09:10.987'!
initializeEventHandler

	self eventHandler: UiEventHandler new.

	self eventHandler
		mouseDownPriority: 25; "Catch all clicks for my cells to trigger context menu."
		setProperty: #handlesMouseWheel toValue: [true];
		grabKeyboardOnMouseEnter: true.
	
	self connect: self eventHandler signal: #mouseLeft: toSelector: #hoveredCell: pattern: #(=nil).
	self connect: self eventHandler signal: #mouseLeft: toSelector: #hoveredDropMode: pattern: #(=none).
	self connect: self eventHandler signal: #mouseLeftDragging: toSelector: #hoveredCell: pattern: #(=nil).
	self connect: self eventHandler signal: #mouseLeftDragging: toSelector: #hoveredDropMode: pattern: #(=none).

	self connect: self eventHandler signal: #mousePressed: toSelector: #mousePressCell:.
	self connect: self eventHandler signal: #mouseReleased: toSelector: #mouseReleaseCell:.
	self connect: self eventHandler signal: #mouseClicked: toSelector: #mouseClickCell:.
	self connect: self eventHandler signal: #mouseDoubleClicked: toSelector: #mouseDoubleClickCell:.
	self connect: self eventHandler signal: #mouseDragStarted: toSelector: #mouseStartDragCell:.
	self connect: self eventHandler signal: #mouseMoved: toSelector: #doHoverCell:.
	
	self connect: self eventHandler signal: #returnStroked toSelector: #activateCurrentCell.
	self connect: self eventHandler signal: #keyStroked: toSelector: #navigateByKey:.

	"Pass some events back to the view."
	self connect: self eventHandler signal: #mouseWheelUp: toSelector: #mouseWheelUp.
	self connect: self eventHandler signal: #mouseWheelDown: toSelector: #mouseWheelDown.

	self connect: self eventHandler signal: #keyStroked: toSelector: #onKeyStroked:.
	self connect: self eventHandler signal: #keyPressed: toSelector: #onKeyPressed:.	

	"Keyboard shortcuts"
	self flag: #hardcoded. "mt: Shortcuts should be configurable? Esp. the edit key? This is normally F2 under MS Windows..."
	self eventHandler on: #cmd -> $a send: #selectAll to: self.
	self eventHandler on: #cmd -> $c send: #copySelectedTextToClipboard to: self.
	self eventHandler on: Character space send: #onEditKeyPressed to: self.! !

!UiItemViewport methodsFor: 'add/remove - inserting' stamp: 'mt 6/26/2012 09:37:53.458'!
insertGroupSeparator: aGroupSeparator at: aRowIndex

	| row |
	row := rows at: aRowIndex.
	self columns do: [:column |
		"Insert separator in column."
		column items 
			add: aGroupSeparator
			beforeIndex: aRowIndex.
		
		"Row was empty so just append."
		row items add: aGroupSeparator.
		
		self addMorphBack: aGroupSeparator].! !

!UiItemViewport methodsFor: 'add/remove - inserting' stamp: 'mt 6/26/2012 12:02:04.455'!
insertItemRows: data
	"group -> row -> column -> (model index -> item/morph)"
	
	| firstRow firstRowIndex newRowsCount newItems tail rowOffset indent |
	
	"Step 1 - Add as much rows as needed."
	firstRow := data first value
		ifEmpty: [self ensureFirstRowFor: (data first key valueOfProperty: #parentModelIndex)]
		ifNotEmpty: [self ensureRowFor: data first value "group" first "row" first "column" key "model index"].

	firstRowIndex := rows indexOf: firstRow.
	newRowsCount := (data inject: 0 into: [:sum :group | sum + 1 "separator for each group" + group value size]).

	newRowsCount - 1 "first row already there"
		timesRepeat: [rows add: UiViewRowInfo new afterIndex: firstRowIndex].
	
	"Step 2 - Prepare items."
	newItems := Array new: (newRowsCount - data size "see #insertGroupSeparatorAt:") * self columnCount.
	tail := 1. "Next slot to fill. Just real items. No group separators."
	rowOffset := 0.
	data withIndexDo: [:group :groupIndex |
		self
			insertGroupSeparator: group key "separator item"
			at: firstRowIndex + rowOffset.
		rowOffset := rowOffset + 1 "separator row".
		
		group value do: [:itemRow | 
			itemRow do: [:indexToItem | | col |
				newItems
					at: tail
					put: indexToItem value.
				tail := tail + 1.
					
				(rows at: firstRowIndex + rowOffset)
					items add: indexToItem value. "column-wise filing left-to-right"
				
				col := self columnFor: indexToItem key.
				col items
					add: indexToItem value
					beforeIndex: firstRowIndex + rowOffset.
			
				"Same indent for all items."
				indent ifNil: [indent := self
					calculateIndentFor: indexToItem value
					atIndex: indexToItem key
					column: col].
			
				indexToItem value
					setProperty: #modelIndex toValue: indexToItem key;
					setProperty: #isExpanded toValue: false;
					setProperty: #hasChildren toValue: indexToItem key hasChildren;
					setProperty: #indent toValue: (indexToItem key column = 1 ifTrue: [indent] ifFalse: [0])].
			rowOffset := rowOffset + 1. "next row"]].
	
	"Step 3 - Add new submorphs."
	self addAllMorphs: newItems.! !

!UiItemViewport methodsFor: 'add/remove - inserting' stamp: 'mt 8/22/2012 14:53:31.587'!
insertMorph: aMorph forIndex: aModelIndex

	| col row |
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: false.
	aMorph setProperty: #hasChildren toValue: aModelIndex hasChildren.
	
	row := self ensureRowFor: aModelIndex.
	row items add: aMorph.
	
	col := self columnFor: aModelIndex.
	col items add: aMorph beforeIndex: (rows indexOf: row).
	
	self addMorphBack: aMorph.
	
	aMorph
		setProperty: #indent
		toValue: (self calculateIndentFor: aMorph atIndex: aModelIndex column: col).! !

!UiItemViewport methodsFor: 'testing' stamp: 'bs 8/3/2011 14:49:34.565'!
isExpanded: aModelIndex

	aModelIndex ifNil: [^ true] "the root is always expanded".
	^ (self isExpanded: aModelIndex parent) 
		and: [(self rowFor: aModelIndex) isExpanded]! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 1/18/2012 10:52:00.937'!
keyPressed: aKeyboardEvent on: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 7/22/2012 10:41:15.847'!
keyStroked: aKeyboardEvent on: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'geometry - cached' stamp: 'mt 7/3/2012 11:42:29.934'!
leftAt: aCell

	^ ((cellToBounds at: aCell y) at: aCell x) first x! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 6/30/2012 15:59:22.992'!
leftForColumn: aColumnInfo

	| result |
	aColumnInfo visible ifFalse: [^ 0].
	
	result := self left.
	self columnsDo: [:col |
		col visible ifTrue: [
			col = aColumnInfo ifTrue: [^ result].
			result := result + col width]].
	
	Error signal: 'Column info not found!!'.! !

!UiItemViewport methodsFor: 'initialization' stamp: 'mt 3/23/2012 10:25:04.47'!
limitColumnCount: numberOfColumns
	"Remove unnecessary columns."
	
	self ensureColumnCount: numberOfColumns.
	columns := columns first: numberOfColumns.! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 2/16/2012 10:32:55.566'!
menuRequestedAt: aPoint on: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'rows/columns - to model index' stamp: 'mt 9/15/2013 16:59:11.132'!
modelIndexFor: aCellIndex

	aCellIndex ifNil: [^ nil].
	aCellIndex y > self rowCount ifTrue: [^ nil].
	aCellIndex x > self columnCount ifTrue: [^ nil].
	
	^ (self rowOrder items: (self columnAt: aCellIndex x) items at: aCellIndex y)
		valueOfProperty: #modelIndex! !

!UiItemViewport methodsFor: 'rows/columns - to morph' stamp: 'mt 7/3/2012 11:36:59.782'!
morphAt: aCellIndex

	^ self columnOrder items: (self rowAt: aCellIndex y) items at: aCellIndex x! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 2/6/2013 14:11:59.867'!
mouseClickCell: evt

	| cell index |
	cell := self cellIndexFromPoint: evt position.
	index := self modelIndexFor: cell.
	index ifNil: [^ self].
	
	self
		clicked: index;
		activated: index.
	
	"Open context menu."
	evt yellowButtonPressed ifTrue: [self menuRequestedAt: evt position on: index].
	
	"Handle click on checkbox."
	self flag: #ugly.
	index checkable ifTrue: [((self morphAt: cell) valueOfProperty: #checkBox) ifNotNil: [:chk |
		(chk bounds containsPoint: evt position) ifTrue: [chk toggle]]].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 2/6/2013 13:32:47.547'!
mouseDoubleClickCell: evt

	| cell index |
	cell := self cellIndexFromPoint: evt position.
	index := self modelIndexFor: cell.
	index ifNil: [^ self].
	
	self
		doubleClicked: index;
		activated: index.
		
	(self editTriggers includes: #doubleClicked)
		ifTrue: [self beginEditItem: (self morphAt: cell)].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 2/6/2013 14:18:14.563'!
mousePressCell: evt

	| cell morph |
	cell := self cellIndexFromPoint: evt position.
	cell ifNil: [^ self].
	
	((self editTriggers includes: #selectedClicked)
		and: [self selection cells includes: cell])
		ifTrue: [self beginEditCell: cell].
	
	self selectCell: cell event: evt.
	
	"Pass event to item."
	morph := self morphAt: cell.
	morph mouseDown: evt.! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 2/6/2013 11:49:39.549'!
mouseReleaseCell: evt

	| cell morph |
	cell := self cellIndexFromPoint: evt position.
	cell ifNil: [^ self].

	self deselectCell: cell event: evt.

	"Pass event to item."
	morph := self morphAt: cell.
	morph mouseUp: evt.! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 7/24/2012 15:13:40.782'!
mouseStartDragCell: evt 
	
	evt hand hasSubmorphs ifTrue: [^ self].
	self dragEnabled ifFalse: [^ self].
	
	[ | nodes |

	"unboxing to not give model index to the outside"	
	(nodes := self selection selectedIndices collect: [:index | index node])
		ifEmpty: [^ self].

	evt hand grabMorph: (self dragDropSpec transferItemFor: nodes asOrderedCollection)]
		ensure: [Cursor normal show.
			evt hand releaseMouseFocus: self]! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 1/18/2012 10:43:34.324'!
mouseWheelDown

	self emitAlways.! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 1/18/2012 10:43:39.028'!
mouseWheelUp

	self emitAlways.! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 3/7/2013 09:58:32.696'!
navigateByKey: aKeyboardEvent

	| cc keyToActionMap |
	cc := self selection currentCell
		ifNil: [^ self selection add: 1@2 event: aKeyboardEvent].

	keyToActionMap := Dictionary newFrom: {
		Character arrowUp    -> #up.
		Character arrowDown  -> #down.
		Character arrowLeft  -> #left.
		Character arrowRight -> #right.
		Character pageUp     -> #pageUp.
		Character pageDown   -> #pageDown.
		Character home       -> #home.
		Character end        -> #end}.

	(keyToActionMap at: aKeyboardEvent keyCharacter ifAbsent: [])
		ifNotNil: [:direction |
			self selection add: (self nextCellFor: cc direction: direction) event: aKeyboardEvent].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 7/8/2013 14:12:01.002'!
nextCellFor: currentCell direction: aSymbol

	| deltaMap nextCellBlock skipBlock nextCell oldCell |	
	deltaMap := Dictionary newFrom: {
		#up       -> [:cell | cell - (0@1)].
		#down     -> [:cell | cell + (0@1)].
		#left     -> [:cell | ((self morphAt: cell) valueOfProperty: #hasChildren)
			ifTrue: [cell "collapse sub-tree"] ifFalse: [cell - (1@0)]].
		#right    -> [:cell | ((self morphAt: cell) valueOfProperty: #hasChildren)
			ifTrue: [cell "expand sub-tree"] ifFalse: [cell + (1@0)]].
		#pageUp   -> [:cell | cell - (0@3)].
		#pageDown -> [:cell | cell + (0@3)].
		#home     -> [:cell | cell x @ 2]. "first row; current column; skip separator"
		#end      -> [:cell | cell x @ rows size] "last row; current column"}.
	nextCellBlock := [:cell | ((deltaMap at: aSymbol) value: cell) min: columns size @ rows size max: 1@1].
	nextCell := nextCellBlock value: currentCell.
	
	"Skip separators but only if there are enough cells left."
	oldCell := nil.
	skipBlock := [(((self morphAt: nextCell) valueOfProperty: #selectable) not
		or: [(self rowAt: nextCell y) visible not])
		or: [(self columnAt: nextCell x) visible not]].
	[skipBlock value and: [nextCell ~= oldCell]] whileTrue: [
		oldCell := nextCell.
		nextCell := nextCellBlock value: nextCell].

	^ skipBlock value
		ifTrue: [currentCell]
		ifFalse: [nextCell]! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 8/8/2012 16:48:45.733'!
nextCellInDirection: aSymbol

	^ self selection currentCell
		ifNil: [self rowCount < 2
			ifTrue: []
			ifFalse: [1@2 "skip separator"]]
		ifNotNil: [:cc | self nextCellFor: cc direction: aSymbol]! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 2/8/2013 11:21:01.881'!
nodesVisible

	| result |
	result := OrderedCollection new.
	self rowsDo: [:row | row visible
		ifTrue: [row isSeparator ifFalse: [result add: row firstModelIndex node]]].
	^ result! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 8/22/2012 10:57:58.252'!
objectDropped: anObject type: aSymbol from: aSourceObject

	self emit.! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 7/18/2012 11:23:45.822'!
objectDropped: anObject type: aSymbol from: aSourceObject after: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 7/19/2012 10:17:12.593'!
objectDropped: anObject type: aSymbol from: aSourceObject before: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 7/18/2012 11:23:29.712'!
objectDropped: anObject type: aSymbol from: aSourceObject on: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 2/6/2013 13:44:35.516'!
onCurrentCellChangedFrom: oldCellIndex to: newCellIndex

	self
		invalidRect: (self boundsFor: oldCellIndex);
		invalidRect: (self boundsFor: newCellIndex).
		
	self updateVisibilityForCheckBoxesFor: newCellIndex old: oldCellIndex.

	self
		updateFocus;
		currentIndexChanged: self selection currentIndex.
		
	(newCellIndex notNil and: [self editTriggers includes: #currentChanged])
		ifTrue: [self beginEditItem: (self morphAt: newCellIndex)].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 2/6/2013 14:07:05.069'!
onEditKeyPressed

	| cell index |
	cell := self selection currentCell.
	index := self modelIndexFor: cell.
	index ifNil: [^ self].

	(self editTriggers includes: #editKeyPressed)
		ifTrue: [self beginEditCell: cell].
	
	"Toggle checkbox if any."
	self flag: #mixedFeatures. "mt: Really mix cell edit mode with #checked property?"
	index checkable ifTrue: [
		((self morphAt: cell) valueOfProperty: #checkBox) ifNotNil: [:chk | chk toggle]].! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 1/18/2012 10:50:40.505'!
onKeyPressed: aKeyboardEvent

	self keyPressed: aKeyboardEvent on: self selection currentIndex.! !

!UiItemViewport methodsFor: 'events' stamp: 'mt 2/6/2013 14:17:48.662'!
onKeyStroked: aKeyboardEvent

	self keyStroked: aKeyboardEvent on: self selection currentIndex.
	
	((self editTriggers includes: #alphaNumericKeyPressed)
		and: [aKeyboardEvent keyCharacter isAlphaNumeric])
		ifTrue: [self beginEditCell: self selection currentCell].! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 7/17/2012 16:16:36.254'!
onSelectionChangedFrom: oldCells to: newCells

	oldCells do: [:c | self invalidRect: (self boundsFor: c)].
	newCells do: [:c | self invalidRect: (self boundsFor: c)].

	self
		"updateFocus;" "never call here -> only on current cell change"
		selectionChanged: self selection selectedIndices.! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 6/26/2012 15:31:35.916'!
removeChildrenForRow: aRowInfo
	"Used for tree viewports."
	
	(self selectChildrenForRow: aRowInfo)
		do: [:row | self removeRow: row].! !

!UiItemViewport methodsFor: 'add/remove - private' stamp: 'mt 10/14/2010 11:37:04.05'!
removeRow: aRowInfo

	aRowInfo items withIndexDo: [:item :index |
		(columns at: index) items remove: item].
	
	self removeAllMorphsIn: aRowInfo items.
	rows remove: aRowInfo.! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 7/8/2013 14:08:29.23'!
removeRowFor: aParentModelIndex at: rowIndex inGroup: groupName

	self removeRow: (rows detect: [:row |
		((row isSeparator not
			and: [row parentModelIndex = aParentModelIndex])
			and: [row firstModelIndex group = groupName])
			and: [row firstModelIndex row = rowIndex]])! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 7/21/2012 14:33:31.641'!
replaceMorphWith: aMorph atIndex: aModelIndex

	| oldMorph col row |
	oldMorph := (rows detect: [:rowInfo | rowInfo includes: aModelIndex] ifNone: [^ self]) itemAt: aModelIndex.
		
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: (oldMorph valueOfProperty: #isExpanded).
	aMorph setProperty: #hasChildren toValue: aModelIndex hasChildren.
	
	row := self rowFor: aModelIndex.
	row items
		add: aMorph after: oldMorph;
		remove: oldMorph.
	
	col := self columnFor: aModelIndex.
	col items
		add: aMorph after: oldMorph;
		remove: oldMorph.
	
	self
		removeMorph: oldMorph;
		addMorphBack: aMorph.
	
	aMorph
		setProperty: #indent
		toValue: (self
			calculateIndentFor: aMorph
			atIndex: aModelIndex
			column: col).! !

!UiItemViewport methodsFor: 'signals - to view' stamp: 'mt 10/10/2010 13:02:41.787'!
resizedToWidth: newWidth height: newHeight

	self emit.! !

!UiItemViewport methodsFor: 'signals - model index' stamp: 'mt 2/28/2011 13:19:02.384'!
returnPressed: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'accessing - private' stamp: 'mt 8/27/2012 15:52:45.883'!
rootIsDecorated

	^ true! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/16/2010 19:56:47.691'!
rowAt: index

	^ self rowOrder items: rows at: index! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 8/30/2012 10:05:23.827'!
rowBoundsAt: aRowIndex

	| top |
	aRowIndex > rows size
		ifTrue: [^ 0@0 extent: 0@0].

	"use cache for better performance"
	top := ((cellToBounds at: aRowIndex)
		detect: [:cellBounds | cellBounds notNil] "skip hidden columns"
		ifNone: [^ 0@0 extent: 0@0]) first y.

	^ self rowBoundsFor: (self rowAt: aRowIndex) top: top! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 8/30/2012 10:06:18.693'!
rowBoundsFor: aRowInfo
		
	aRowInfo visible ifFalse: [^ 0@0 extent: 0@0].
	
	^ self rowBoundsFor: aRowInfo top: (self topForRow: aRowInfo)! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 8/30/2012 10:05:55.479'!
rowBoundsFor: aRowInfo top: top
		
	aRowInfo visible ifFalse: [^ 0@0 extent: 0@0].
	
	^ self left @ top
		extent: self width @ aRowInfo height! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 4/21/2011 10:28:59.349'!
rowCount

	^ rows size! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 7/19/2012 11:12:38.389'!
rowCountVisible

	^ self rows count: [:rowInfo | rowInfo visible]! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/9/2010 16:12:41.64'!
rowFor: aModelIndex

	^ rows detect: [:row | row includes: aModelIndex]! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/16/2010 15:00:50.256'!
rowOrder

	^ rowOrder! !

!UiItemViewport methodsFor: 'accessing - private' stamp: 'mt 1/4/2011 10:23:22.573'!
rows
	"Returning all rows IGNORING CURRENT SORT ORDER."
	
	^ rows! !

!UiItemViewport methodsFor: 'enumerating - ordered' stamp: 'mt 10/16/2010 19:06:41.893'!
rowsDo: aBlock

	self rowOrder items: rows do: aBlock.! !

!UiItemViewport methodsFor: 'enumerating - ordered' stamp: 'mt 10/16/2010 19:49:38.102'!
rowsWithIndexDo: aBlock

	self rowOrder items: rows withIndexDo: aBlock.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 7/8/2013 14:05:02.577'!
selectAll

	| cells |
	cells := OrderedCollection new.
	self rowsWithIndexDo: [:rowInfo :index |
		((rowInfo visible
			and: [rowInfo isSeparator not])
			and: [rowInfo items first valueOfProperty: #selectable])
				ifTrue: [cells add: 1@index]].
	self selection cells: cells.! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 2/8/2013 10:32:44.684'!
selectBlock
	"Filter nodes."
	
	^ selectBlock! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 2/11/2013 11:00:12.775'!
selectBlock: aBlock
	"[:node | node text size > 5]"
	
	selectBlock := aBlock.
	self
		selectNone; flag: #workaround; "mt: better update selection correctly than discarding it"
		updateLayout.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 2/6/2013 10:47:11.222'!
selectCell: aCell event: aUserInputEvent

	| morph |
	morph := self morphAt: aCell.
	
	"Update selection..."
	(morph valueOfProperty: #selectable) ifTrue: [
		"...for single cells."
		self selection add: aCell event: aUserInputEvent.
		(morph hasProperty: #isGroupSeparator) ifTrue: [
			"...for whole groups."
			self selection cells: (
				self selection cells, 
				((self selectChildrenForGroup: (self rowAt: aCell y))
					select: [:rowInfo | rowInfo visible]
					thenCollect: [:rowInfo | self cellIndexFromRow: rowInfo]))]].! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 7/8/2013 14:05:28.173'!
selectChildrenForGroup: aSeparatorRow
	
	| foundFirst predecessors rowsToSelect parentIndex groupIndex |
	aSeparatorRow isSeparator ifFalse: [^ self].
	
	parentIndex := aSeparatorRow parentModelIndex.
	groupIndex := aSeparatorRow items first valueOfProperty: #groupIndex.
	
	foundFirst := false.
	predecessors := {aSeparatorRow} asOrderedCollection.
	rowsToSelect := OrderedCollection new.

	rows anySatisfy: [:row |
		foundFirst
			ifTrue: [(((row isSeparator not and: [row parentModelIndex = parentIndex]) and: [row firstModelIndex groupIndex = groupIndex])
				or: [self flag: #slow. predecessors anySatisfy: [:pred | row parentModelIndex = pred firstModelIndex and: [row isSeparator or: [row firstModelIndex groupIndex = groupIndex]]]])
					ifTrue: [
						row isSeparator ifFalse: [predecessors add: row].
						rowsToSelect add: row.
						false]
					ifFalse: [true]]
			ifFalse: [row = aSeparatorRow ifTrue: [foundFirst := true]. false]].

	^ rowsToSelect! !

!UiItemViewport methodsFor: 'rows/columns - to info' stamp: 'mt 6/26/2012 16:23:52.89'!
selectChildrenForRow: aRowInfo
	
	| foundFirst predecessors rowsToSelect |
	foundFirst := false.
	predecessors := {aRowInfo} asOrderedCollection.
	rowsToSelect := OrderedCollection new.

	rows anySatisfy: [:row |
		self flag: #slow.
		(predecessors anySatisfy: [:pred | row parentModelIndex = pred firstModelIndex])
			ifTrue: [
				foundFirst := true.
				row isSeparator ifFalse: [predecessors add: row].
				rowsToSelect add: row.
				false]
			ifFalse: [foundFirst]].

	^ rowsToSelect! !

!UiItemViewport methodsFor: 'selection - convenience' stamp: 'mt 8/8/2012 15:56:18.84'!
selectNextColumn

	self selection add: (self nextCellInDirection: #right).! !

!UiItemViewport methodsFor: 'selection - convenience' stamp: 'mt 8/8/2012 15:55:54.636'!
selectNextRow

	self selection add: (self nextCellInDirection: #down).! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 8/29/2012 10:02:38.431'!
selectNone

	self selection clear.! !

!UiItemViewport methodsFor: 'selection - convenience' stamp: 'mt 8/8/2012 15:56:09.635'!
selectPreviousColumn

	self selection add: (self nextCellInDirection: #left).! !

!UiItemViewport methodsFor: 'selection - convenience' stamp: 'mt 8/8/2012 15:56:03.791'!
selectPreviousRow

	self selection add: (self nextCellInDirection: #up).! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/10/2010 20:03:34.835'!
selection

	^ selection! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 7/13/2011 14:34:02.764'!
selection: aSelection

	selection := aSelection.

	selection viewport: self.
	
	self connect: aSelection signal: #selectionChangedFrom:to: toSelector: #onSelectionChangedFrom:to:.
	self connect: aSelection signal: #currentCellChangedFrom:to: toSelector: #onCurrentCellChangedFrom:to:.
	
	self redrawLater.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/14/2010 13:50:42.369'!
selectionBehavior

	^ selectionBehavior ifNil: [selectionBehavior := #selectCells]! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:28:20.274'!
selectionBehavior: aSymbol
	"#selectCells, #selectRows, #selectColumns"

	selectionBehavior := aSymbol.
	self redrawLater.! !

!UiItemViewport methodsFor: 'signals - model index' stamp: 'mt 10/20/2010 13:45:49.758'!
selectionChanged: someModelIndices

	self emit.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 2/6/2013 14:49:21.688'!
selectionColor

	^ selectionColor ifNil: [selectionColor := Color r: 0.772 g: 0.858 b: 1.0]! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/8/2010 14:37:57.041'!
selectionColor: aColor

	selectionColor := aColor.! !

!UiItemViewport methodsFor: 'accessing - private' stamp: 'mt 2/6/2013 14:49:04.346'!
selectionColorAdjusted
	"Adjust selection color according to keyboard focus."
	
	^ self hasKeyboard
		ifFalse: [self selectionColor alpha: 0.3]
		ifTrue: [self selectionColor]! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:04:47.949'!
selectionMode

	^ selectionMode! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/11/2010 09:04:07.606'!
selectionMode: aSelectionClass

	selectionMode := aSelectionClass.
	self selection: aSelectionClass new.! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 6/30/2012 15:57:59.481'!
setHeaderItem: aMorph atColumn: anInteger

	(columns at: anInteger) headerItem: aMorph.! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 10/12/2010 11:50:38.889'!
setHeaderItem: aMorph atRow: anInteger

	(rows at: anInteger) headerItem: aMorph.! !

!UiItemViewport methodsFor: 'drawing - testing' stamp: 'mt 8/25/2013 19:54:21.178'!
shouldDraw
	"Drawing a viewport is can fail if involved caches are being updated."

	^ isLayoutBeingModified ~~ true! !

!UiItemViewport methodsFor: 'geometry - testing' stamp: 'mt 7/13/2013 12:57:21.735'!
shouldUpdateLayoutForNewExtent: aPoint

	^ ((self columns anySatisfy: [:col | col resizeMode == #spaceFill])
		and: [self rowCountVisible > 0])
		and: [self width ~= aPoint x]! !

!UiItemViewport methodsFor: 'geometry - cached' stamp: 'mt 7/3/2012 11:42:06.004'!
topAt: aCell

	^ ((cellToBounds at: aCell y) at: aCell x) first y! !

!UiItemViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 6/30/2012 15:59:33.029'!
topForRow: aRowInfo

	| result |
	aRowInfo visible ifFalse: [^ 0].
	
	result := self top.
	self rowsDo: [:row |
		row visible ifTrue: [
			row = aRowInfo ifTrue: [^ result].
			result := result + row height]].
	
	Error signal: 'Row info not found!!'.! !

!UiItemViewport methodsFor: 'layouting - deprecated' stamp: 'mt 6/26/2012 16:38:28.465'!
updateColumnsLayout

	| offset |
	self columnsDo: [:colInfo |
		colInfo visible ifTrue: [
			offset := 0.
			self rowOrder items: colInfo items withIndexDo: [:item :rowIndex |
				| height row |
				row := rows at: rowIndex.
				row visible ifTrue: [
					height := row height.
					colInfo vAlignment = #top
						ifTrue: [item top: offset].
					colInfo vAlignment = #center
						ifTrue: [item top: offset + (height - item height // 2)].
					colInfo vAlignment = #bottom
						ifTrue: [item bottom: offset + height].
							
					offset := offset + height]]]].! !

!UiItemViewport methodsFor: 'drawing - cached' stamp: 'mt 7/12/2012 15:46:00.292'!
updateDrawCache

	| canvas oldCacheFlag |
	oldCacheFlag := drawCacheActive.
	drawCacheActive := false.
	
	canvas := FormCanvas extent: self extent depth: 32 origin: 0@0 clipRect: (0@0 extent: self extent).
	canvas fillColor: Color white.
	self drawSubmorphsOn: canvas.
	drawCache := canvas form.

	drawCacheActive := oldCacheFlag.! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 10/16/2010 18:00:37.584'!
updateFirstColumnIndent

	columns first items do: [:item |
		item left: item left + (item valueOfProperty: #indent)].! !

!UiItemViewport methodsFor: 'updating' stamp: 'mt 10/16/2010 08:23:31.568'!
updateFocus
	"The important part is always the current cell."
	
	self selection currentCell ifNotNil: [:cc |
		self focusChanged: (self cellBoundsFor: cc)].! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 6/29/2012 10:28:47.865'!
updateHeaderLayout
	"Relayout changed header items."
	
	self horizontalHeader ifNotNil: [:hh |
		hh removeAllMorphs.
		self columnsDo: [:colInfo | colInfo headerItem ifNotNil: [:hi |
			hh addMorphBack: hi]]].
	self verticalHeader ifNotNil: [:vh |
		vh removeAllMorphs.
		self rowsDo: [:rowInfo | rowInfo headerItem ifNotNil: [:hi |
			vh addMorphBack: hi]]].! !

!UiItemViewport methodsFor: 'add/remove' stamp: 'mt 1/23/2012 10:56:06.884'!
updateIndex: newModelIndex oldIndex: formerModelIndex

	| morph |
	morph := (rows reverseDetect: [:rowInfo | rowInfo includes: formerModelIndex]) itemAt: formerModelIndex.
	morph setProperty: #modelIndex toValue: newModelIndex.! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 12/13/2012 14:42:38.365'!
updateItemLayout

	| vOffset hOffset cell |
	cellToBounds := Array new: self rowCount.
	cell := 0@0.
	
	vOffset := 0.
	hOffset := 0.
	self rowsDo: [:rowInfo |		
		"Prepare bounds cache for current row."
		cell := 0 @ (cell y + 1).
		cellToBounds at: cell y put: (Array new: self columnCount).
		
		rowInfo visible ifTrue: [
			| deltaHeight |
			deltaHeight := rowInfo height.
			hOffset := 0.

			self columnOrder items: rowInfo items withIndexDo: [:item :colIndex |
				| deltaWidth columnInfo |
				columnInfo := columns at: colIndex.
				deltaWidth := columnInfo width.
				
				"Advance cell."
				cell := cell + (1@0).
				
				columnInfo visible ifTrue: [
					columnInfo hAlignment == #left ifTrue: [item left: hOffset].
					columnInfo hAlignment == #center ifTrue: [item left: hOffset + (deltaWidth - item width // 2)].
					columnInfo hAlignment == #right ifTrue: [item right: hOffset + deltaWidth].
					columnInfo vAlignment == #top ifTrue: [item top: vOffset].
					columnInfo vAlignment == #center ifTrue: [item top: vOffset + (deltaHeight - item height // 2)].
					columnInfo vAlignment == #bottom ifTrue: [item bottom: vOffset + deltaHeight].
					
					"Fill bounds cache."
					(cellToBounds at: cell y) at: cell x put: {hOffset@vOffset.deltaWidth@deltaHeight}.

					hOffset := hOffset + deltaWidth]].
			rowInfo isSeparator ifTrue: [rowInfo items first left: 0].
			vOffset := vOffset + deltaHeight]].

	"Use offsets of last visible row."
	self extent: hOffset@(vOffset+2).! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 4/28/2012 18:33:39.85'!
updateItemOrder

	"Sort after first column for now."
	self rowOrder updateWith: columns first items.! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 8/25/2013 19:40:44.521'!
updateLayout

	[
	
	columns ifEmpty: [^ self].
	isLayoutBeingModified := true.
	
	self
		updateRowsVisibility;
		updateSizesOfRowsAndColumns;
		updateItemOrder;
		updateItemLayout;
		updateFirstColumnIndent;
		updateHeaderLayout.
	
	] ensure: [
		isLayoutBeingModified := false.
		self drawCacheActive
			ifTrue: [self updateDrawCache]].! !

!UiItemViewport methodsFor: 'layouting - deprecated' stamp: 'mt 6/26/2012 16:39:05.641'!
updateRowsLayout

	| offset |
	self rowsDo: [:rowInfo |
		rowInfo visible ifTrue: [
			rowInfo isSeparator
				ifTrue: [rowInfo items first left: 0]
				ifFalse: [
					offset := 0.
					self columnOrder items: rowInfo items withIndexDo: [:item :colIndex |
						| width hAlignment column |
						column := columns at: colIndex.
						column visible ifTrue: [
							width := column width.
							hAlignment := (columns at: colIndex) hAlignment.
							hAlignment = #left
								ifTrue: [item left: offset].
							hAlignment = #center
								ifTrue: [item left: offset + (width - item width // 2)].
							hAlignment = #right
								ifTrue: [item right: offset + width].
				
							offset := offset + width]]]]].! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 2/8/2013 10:56:49.6'!
updateRowsVisibility

	self rows ifEmpty: [^ self].
	
	self rows do: [:row | row visible: (
		self selectBlock ifNil: [true] ifNotNil: [:filter |
			row isSeparator ifTrue: [true] ifFalse: [
				filter value: row firstModelIndex node]])].
	
	self flag: #wrong. "mt: You need to check row visibility in order to determine empty groups."
	self rows overlappingPairsDo: [:rowInfo1 :rowInfo2 |
		rowInfo1 isSeparator ifTrue: [
			(rowInfo1 items first valueOfProperty: #hideAlways) == true
				ifTrue: [rowInfo1 visible: false]
				ifFalse: [((rowInfo2 isSeparator or: [rowInfo1 parentModelIndex ~= rowInfo2 parentModelIndex])
					and: [(rowInfo1 items first valueOfProperty: #hideIfEmpty) == true])
						ifTrue: [rowInfo1 visible: false]]].
					
		rowInfo2 isSeparator ifTrue: [
			(rowInfo1 parentModelIndex ~= rowInfo2 parentModelIndex
				and: [(rowInfo2 items first valueOfProperty: #hideIfFirst) == true])
					ifTrue: [rowInfo2 visible: false]]].

	self rows first isSeparator ifTrue: [
		((self rows first items first valueOfProperty: #hideAlways) == true
			or: [(self rows first items first valueOfProperty: #hideIfFirst) == true])
			ifTrue: [self rows first visible: false]].
	
	self rows last isSeparator ifTrue: [
		((self rows last items first valueOfProperty: #hideAlways) == true
			or: [(self rows last items first valueOfProperty: #hideIfEmpty) == true])
			ifTrue: [self rows last visible: false]].
! !

!UiItemViewport methodsFor: 'layouting' stamp: 'mt 9/13/2012 16:32:32.653'!
updateSizesOfRowsAndColumns

	| spaceFillColumns usedSpace |
	usedSpace := 0.
	spaceFillColumns := columns select: [:column | 
		self flag: #invisbleColumns. "mt: Consider this."
		column resizeMode == #spaceFill
			ifTrue: [true]
			ifFalse: [
				column updateSize.
				usedSpace := usedSpace + column width.
				false]].
	spaceFillColumns ifNotEmpty: [:cols |
		| spacePerCol |
		spacePerCol := self width - usedSpace // cols size.
		cols do: [:column | column resize: spacePerCol; updateItemSize]].

	rows do: [:row | row updateSize].! !

!UiItemViewport methodsFor: 'updating' stamp: 'mt 6/30/2012 11:55:40.942'!
updateVisibilityForCheckBoxesFor: newCell old: oldCell

	(oldCell notNil and: [oldCell y <= rows size]) ifTrue: [
		(self rowAt: oldCell y) items do: [:item | (item valueOfProperty: #checkBox)
			ifNotNil: [:chk | chk checked ifFalse: [chk hide]]]].

	(newCell notNil and: [newCell y <= rows size]) ifTrue: [
		(self rowAt: newCell y) items do: [:item | (item valueOfProperty: #checkBox)
			ifNotNil: [:chk | chk show]]].! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 13:59:56.367'!
verticalHeader

	^ verticalHeader! !

!UiItemViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 13:59:50.126'!
verticalHeader: aMorph

	verticalHeader := aMorph.! !

!UiItemViewport methodsFor: 'drag/drop' stamp: 'mt 7/8/2013 14:06:19.352'!
wantsDroppedMorph: aMorph event: event

	^ ((super wantsDroppedMorph: aMorph event: event)
		and: [aMorph isKindOf: TransferMorph])
		and: [self dragDropSpec acceptBlock value: aMorph]! !

UiItemViewport subclass: #UiListViewport
	instanceVariableNames: 'orientation maxItemExtent vRowCount vColumnCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiListViewport commentStamp: 'mt 11/2/2010 12:46:22.16' prior: 0!
A specialized item viewport that has different layout strategies for its items as they are only a list. See the #orientation: commment for more information about possible layouting.!

!UiListViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 8/31/2012 13:45:38.572'!
boundsFor: aCellIndex
	"Returns the relevant bounds for a given cell index based on the current selection behavior."

	self orientation == #default
		ifTrue: [^ super boundsFor: aCellIndex].
	
	aCellIndex ifNil: [^ 0@0 extent: 0@0].

	self assert: self selectionBehavior == #selectCells.

	^ self cellBoundsFor: aCellIndex! !

!UiListViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 8/31/2012 13:25:07.256'!
cellIndexAt: colAndRow

	Error signal: 'Undefined.'! !

!UiListViewport methodsFor: 'enumeration' stamp: 'mt 8/31/2012 13:51:44.947'!
cellIndexDetect: aBlock

	^ self
		cellIndexDetect: aBlock
		ifNone: []! !

!UiListViewport methodsFor: 'enumeration' stamp: 'mt 8/31/2012 13:51:32.082'!
cellIndexDetect: aBlock ifNone: alternative

	cellToBounds withIndexDo: [:row :rowIndex | row withIndexDo: [:cell :colIndex |
		(aBlock value: cell) ifTrue: [^ colIndex @ rowIndex]]].
	
	^ alternative value! !

!UiListViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 8/31/2012 13:24:41.006'!
cellIndexFromColumn: aColumnInfo

	self orientation == #default
		ifTrue: [^ super cellIndexFromColumn: aColumnInfo].
	
	^ 1@1! !

!UiListViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 8/31/2012 13:57:20.873'!
cellIndexFromModelIndex: aModelIndex

	self orientation == #default
		ifTrue: [^ super cellIndexFromModelIndex: aModelIndex].
		
	aModelIndex ifNil: [^ nil].

	^ self cellIndexDetect: [:cell | cell notNil and: [cell third "row info" firstModelIndex = aModelIndex]]! !

!UiListViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 7/8/2013 14:09:59.251'!
cellIndexFromPoint: aPoint
	"Given a point in pixel coordinates."
	
	| cell |
	self orientation = #default ifTrue: [
		| offset cellIndex |
		offset := self topLeft.
		cellIndex := 1@0.
		self rowsDo: [:rowInfo | 
			cellIndex := cellIndex + (0@1).
			rowInfo visible ifTrue: [
				(offset <= aPoint and: [aPoint < (offset + (self width @ rowInfo height))])
					ifTrue: [^ cellIndex].
				offset := offset translateBy: 0 @ rowInfo height]].
	
		^ nil].
	
	self orientation = #horizontalSingleRow ifTrue: [
		| localPt | 
		localPt := aPoint - self topLeft.
		cellToBounds first withIndexDo: [:cellSpec :colIndex |
			cellSpec ifNotNil: [cellSpec first x + cellSpec second x >= localPt x ifTrue: [^ colIndex @ 1]]].
		^ nil].
	
	cell := ((aPoint - self topLeft) // maxItemExtent) + (1@1).

	((cell x > vColumnCount or: [cell y > vRowCount])
		or: [((cellToBounds at: cell y) at: cell x) isNil])
			ifTrue: [^ nil].
	
	^ cell! !

!UiListViewport methodsFor: 'rows/columns - to cell' stamp: 'mt 8/31/2012 13:23:13.457'!
cellIndexFromRow: aRowInfo

	self orientation == #default
		ifTrue: [^ super cellIndexFromRow: aRowInfo].
		
	^ self cellIndexFromModelIndex: aRowInfo firstModelIndex  ! !

!UiListViewport methodsFor: 'enumeration' stamp: 'mt 8/31/2012 13:55:58.42'!
cellIndexSelect: aBlock

	| result |
	result := OrderedCollection new.
	cellToBounds withIndexDo: [:row :rowIndex | row withIndexDo: [:cell :colIndex |
		(aBlock value: cell) ifTrue: [result add: colIndex @ rowIndex]]].
	^ result! !

!UiListViewport methodsFor: 'rows/columns - to info' stamp: 'mt 8/31/2012 14:17:11.959'!
columnAtCell: aCellIndex
	"There is only one column."
	
	^ self listColumn! !

!UiListViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 8/30/2012 10:20:27.252'!
columnBoundsFor: aColumnInfo

	^ self bounds! !

!UiListViewport methodsFor: 'events' stamp: 'mt 8/23/2012 10:41:55.452'!
doHoverDropTarget: evt forCell: cell

	self orientation == #default
		ifTrue: [^ super doHoverDropTarget: evt forCell: cell].

	"visualize drop target"
	self dragDropSpec ifNotNil: [:spec |
		spec dropMode == #over ifTrue: [
			self hoveredDropMode: #over].
		spec dropMode == #between ifTrue: [
			| cellBounds |
			cellBounds := self cellBoundsFor: cell.
			
			self orientation == #vertical ifTrue: [
				evt position x < (cellBounds left + (cellBounds width // 2))
					ifTrue: [self hoveredDropMode: #inFrontOf]
					ifFalse: [self hoveredDropMode: #after]].
			self orientation == #horizontal ifTrue: [
				evt position y < (cellBounds top + (cellBounds height // 2))
					ifTrue: [self hoveredDropMode: #inFrontOf]
					ifFalse: [self hoveredDropMode: #after]]]].! !

!UiListViewport methodsFor: 'drawing' stamp: 'mt 2/6/2013 14:51:15.921'!
drawDropTargetOn: aCanvas

	| cellBounds |
	self hoveredDropMode == #none ifTrue: [^ self].
	self orientation == #default ifTrue: [^ super drawDropTargetOn: aCanvas].
	self hoveredDropMode == #over ifTrue: [^ super drawDropTargetOn: aCanvas].

	"Visualize drop target."
	cellBounds := self boundsFor: self hoveredCell.
	
	self orientation == #vertical ifTrue: [
		self hoveredDropMode == #inFrontOf ifTrue: [
			^ aCanvas
				line: cellBounds topLeft + (2@0)
				to: cellBounds bottomLeft + (2@0)
				width: 3
				color: self selectionColorAdjusted muchDarker].
		
		self hoveredDropMode == #after ifTrue: [
			^ aCanvas
				line: cellBounds topRight - (2@0)
				to: cellBounds bottomRight - (2@0)
				width: 3
				color: self selectionColorAdjusted muchDarker].
			
		self hoveredDropMode == #full ifTrue: [
			| box |
			box := cellToBounds
				ifEmpty: [self bounds]
				ifNotEmpty: [:ctb | (ctb last reverse detect: [:cell | cell notNil])
					ifNotNil: [:cell | cell first + (cell second x @ 0) extent: 2 @ cell second y]].
			^ aCanvas
				line: box topLeft
				to: box bottomLeft
				width: 3
				color: self selectionColorAdjusted muchDarker]].
		
	self flag: #unfinished:. "mt: handling for horizontal orientation is missing."! !

!UiListViewport methodsFor: 'drawing' stamp: 'mt 8/16/2013 20:45:49.871'!
drawGroupSeparator: aMorph on: aCanvas
	"Override to consider maxItemExtent for line."
	
	self orientation == #horizontalSingleRow
		ifTrue: [aCanvas fullDrawMorph: aMorph. ^ self].
	
	self orientation == #default
		ifTrue: [^ super drawGroupSeparator: aMorph on: aCanvas].
	
	self orientation == #vertical
		ifTrue: [^ aCanvas
			translateBy: (0@ (maxItemExtent y - aMorph height))
			during: [:c | super drawGroupSeparator: aMorph on: c]].
			
	"horizontal"
	aCanvas fullDrawMorph: aMorph.
	
	(aMorph valueOfProperty: #drawLine) == true ifTrue: [
		aCanvas
			line: aMorph right @ aMorph center y
			to: aMorph right + maxItemExtent x - aMorph width @ aMorph center y
			color: (self groupSpec lineColor alpha: 0.4)].! !

!UiListViewport methodsFor: 'drag/drop' stamp: 'mt 8/22/2012 11:55:53.133'!
dropsBefore: evt bounds: cellBounds

	self orientation == #vertical
		ifFalse: [^ super dropsBefore: evt bounds: cellBounds]
		ifTrue: [^ evt position x < (cellBounds left + (cellBounds width // 2))]! !

!UiListViewport methodsFor: 'add/remove - private' stamp: 'mt 7/8/2013 14:02:18.829'!
ensureRowFor: aModelIndex
	"Adds a new row if needed. Returns the correct one if possible."
	
	| row rowIndex isCorrectRow isPreviousSibling isFirstInGroup |
	isCorrectRow := [:r | r firstModelIndex group = aModelIndex group
		and: [r firstModelIndex row = aModelIndex row]].
	isPreviousSibling := [:r | r firstModelIndex group = aModelIndex group
		and: [r firstModelIndex row = (aModelIndex row - 1)]].
	isFirstInGroup := [:separator | (((separator items first valueOfProperty: #groupIndex) = aModelIndex groupIndex)
		and: [aModelIndex row = 1])
		and: [aModelIndex column = 1]].
	
	rowIndex := 0.
	
	rows detect: [:r | rowIndex := rowIndex + 1.
		r isSeparator ifTrue: [
			(isFirstInGroup value: r)
				ifTrue: [
					row := rows add: UiViewRowInfo new afterIndex: rowIndex.
					true]
				ifFalse: [false]] ifFalse: [
		(isCorrectRow value: r)
			ifTrue: [row := r. true]
			ifFalse: [(isPreviousSibling value: r)
				ifTrue: [row := rows add: UiViewRowInfo new after: r. true]
				ifFalse: [false]]
		]] ifNone: [row := rows add: UiViewRowInfo new].
	
	^ row! !

!UiListViewport methodsFor: 'initialization' stamp: 'mt 10/14/2010 20:57:45.903'!
initialize

	super initialize.
	maxItemExtent := 1@1.
	self orientation: #default.! !

!UiListViewport methodsFor: 'accessing - private' stamp: 'mt 8/30/2012 10:42:54.27'!
listColumn

	^ columns first! !

!UiListViewport methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:33:09.036'!
maxItemExtent

	^ maxItemExtent! !

!UiListViewport methodsFor: 'rows/columns - to model index' stamp: 'mt 8/31/2012 14:21:00.779'!
modelIndexFor: aCellIndex

	self orientation = #default
		ifTrue: [^ super modelIndexFor: aCellIndex].

	aCellIndex ifNil: [^ nil].
	
	^ (self rowAtCell: aCellIndex) ifNotNil: [:rowInfo | rowInfo firstModelIndex]! !

!UiListViewport methodsFor: 'rows/columns - to morph' stamp: 'mt 8/31/2012 14:20:56.563'!
morphAt: aCellIndex

	self orientation == #default
		ifTrue: [^ super morphAt: aCellIndex].

	aCellIndex ifNil: [^ nil].
	
	^ (self rowAtCell: aCellIndex) ifNotNil: [:rowInfo | rowInfo items first]! !

!UiListViewport methodsFor: 'events' stamp: 'mt 7/8/2013 14:12:19.832'!
nextCellFor: currentCell direction: aSymbol

	| deltaMap nextCellBlock skipBlock nextCell oldCell |	
	self orientation == #default ifTrue: [^ super nextCellFor: currentCell direction: aSymbol].
	
	deltaMap := Dictionary newFrom: {
		#up       -> [:cell | cell - (0@1)].
		#down     -> [:cell | cell + (0@1)].
		#left     -> [:cell | cell - (1@0)].
		#right    -> [:cell | cell + (1@0)].
		#pageUp   -> [:cell | cell - (0@3)].
		#pageDown -> [:cell | cell + (0@3)].
		#home     -> [:cell | self cellIndexDetect: [:c | c notNil and: [c third "row info" isSeparator not]]].
		#end      -> [:cell | vColumnCount @ vRowCount]}.
	nextCellBlock := [:cell | ((deltaMap at: aSymbol) value: cell) min: vColumnCount @ vRowCount max: 1@1].
	nextCell := nextCellBlock value: currentCell.
	
	"Skip separators but only if there are enough cells left."
	oldCell := nil.
	skipBlock := [
		| item |
		item := self morphAt: nextCell.
		(item isNil
			or: [(item valueOfProperty: #selectable) not])
			or: [(self rowAtCell: nextCell) visible not]].
	[skipBlock value and: [nextCell ~= oldCell]] whileTrue: [
		oldCell := nextCell.
		nextCell := nextCellBlock value: nextCell].

	^ skipBlock value
		ifTrue: [currentCell]
		ifFalse: [nextCell]! !

!UiListViewport methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:53:11.793'!
optimalSize

	columns ifEmpty: [^ 1@1].

	self orientation = #default ifTrue: [
		^ columns first optimalSize @ self height].
	
	self orientation = #vertical ifTrue: [
		^ maxItemExtent x @ self height].
	
	self orientation = #horizontal ifTrue: [
		^ self width @ maxItemExtent y].! !

!UiListViewport methodsFor: 'accessing' stamp: 'mt 10/14/2010 17:21:19.074'!
orientation

	^ orientation! !

!UiListViewport methodsFor: 'accessing' stamp: 'mt 5/22/2013 17:21:21.698'!
orientation: aSymbol
	"See UiListView>>#listOrientation:."

	aSymbol = orientation ifTrue: [^ self].
	orientation := aSymbol.
	
	orientation = #default
		ifTrue: [self selectionBehavior: #selectRows]
		ifFalse: [self selectionBehavior: #selectCells].! !

!UiListViewport methodsFor: 'rows/columns - to info' stamp: 'mt 8/31/2012 14:17:52.285'!
rowAtCell: aCellIndex

	^ ((cellToBounds at: aCellIndex y) at: aCellIndex x)
		ifNotNil: [:cell | cell third]! !

!UiListViewport methodsFor: 'selection' stamp: 'mt 7/8/2013 14:05:10.694'!
selectAll

	self orientation == #default
		ifTrue: [^ super selectAll].
		
	self selection cells: (self cellIndexSelect: [:cell | cell notNil and: [
		(cell third visible
			and: [cell third isSeparator not])
			and: [cell third items first valueOfProperty: #selectable]]]).! !

!UiListViewport methodsFor: 'selection' stamp: 'mt 8/31/2012 13:43:59.397'!
selectionBehavior

	^ self orientation == #default
		ifTrue: [super selectionBehavior]
		ifFalse: [#selectCells]! !

!UiListViewport methodsFor: 'selection' stamp: 'mt 8/31/2012 13:44:37.619'!
selectionBehavior: aSymbol

	self orientation == #default
		ifTrue: [^ super selectionBehavior: aSymbol].
		
	aSymbol ~~ self selectionBehavior
		ifTrue: [Error signal: 'Selection behavior is not configurable for lists.'].! !

!UiListViewport methodsFor: 'geometry - testing' stamp: 'mt 7/12/2013 11:48:45.419'!
shouldUpdateLayoutForNewExtent: aPoint

	^ (super shouldUpdateLayoutForNewExtent: aPoint)
		or: ((self orientation == #vertical and: [self width ~= aPoint x])
			or: [self orientation == #horizontal and: [self height ~= aPoint y]])! !

!UiListViewport methodsFor: 'layouting' stamp: 'mt 8/29/2013 16:26:59.573'!
updateLayout

	"There is no list wrapping in #default layout. Just use the super implementation."
	self orientation = #default
		ifTrue: [^ super updateLayout].	
	
	[
	
	self columnCountVisible == 0 ifTrue: [^ self].
	isLayoutBeingModified := true.
	
	self
		updateRowsVisibility;
		updateItemOrder;
		updateMaxItemExtent;
		updateSingleColumnLayout.
		
	] ensure: [
		isLayoutBeingModified := false.
		self drawCacheActive
			ifTrue: [self updateDrawCache]].! !

!UiListViewport methodsFor: 'layouting' stamp: 'mt 8/31/2012 11:29:52.277'!
updateMaxItemExtent

	maxItemExtent := 1@1.
	self rows do: [:rowInfo |
		rowInfo isSeparator ifFalse: [
			maxItemExtent := maxItemExtent max: rowInfo items first extent]].! !

!UiListViewport methodsFor: 'layouting' stamp: 'mt 8/31/2012 13:18:10.499'!
updateSingleColumnHorizontalLayout
	
	| cell hAlignment |
	
	self assert: self orientation == #horizontal.
	
	"Compute virtual row and column count."
	vRowCount := self height // maxItemExtent y.
	vRowCount == 0 ifTrue: [^ self "skip layouting"].
	vColumnCount := (self rowCountVisible / vRowCount) ceiling.
	
	"Prepare cache and running variables."
	cellToBounds := Array new: vRowCount.
	1 to: vRowCount do: [:row | cellToBounds at: row put: (Array new: vColumnCount)].
	cell := 0@0.
	hAlignment := self listColumn hAlignment.

	self rowsDo: [:rowInfo | rowInfo visible ifTrue: [
		| item vOffset hOffset |

		(cell y == vRowCount or: [cell y == 0]) "column break?"
			ifTrue: [cell := cell x + 1 @ 0].
		cell := cell + (0@1).
		
		item := rowInfo items first.
		hOffset := cell x-1 * maxItemExtent x.
		vOffset := cell y-1 * maxItemExtent y.
			
		hAlignment = #left
			ifTrue: [item topLeft: hOffset @ vOffset].
		hAlignment == #center
			ifTrue: [item topLeft: hOffset + (maxItemExtent x - item width // 2) @ vOffset].
		hAlignment == #right
			ifTrue: [item topRight: hOffset + maxItemExtent x @ vOffset].

		"Fill bounds cache."
		(cellToBounds at: cell y) at: cell x put: {hOffset@vOffset.maxItemExtent.rowInfo}]].
			
	"Update dynamically-sized dimension."
	self width: self submorphBounds width.! !

!UiListViewport methodsFor: 'layouting' stamp: 'mt 5/22/2013 17:17:20.849'!
updateSingleColumnHorizontalSingleRowLayout

	| cell hOffset previousItemWidth |
	
	self assert: self orientation == #horizontalSingleRow.
	
	"Compute virtual row and column count."
	vRowCount := 1.
	vColumnCount := self rowCountVisible.
	
	"Prepare cache and running variables."
	cellToBounds := {Array new: vColumnCount}. "one row only"
	cell := 0@1.
	hOffset := 0.
	previousItemWidth := nil.

	self rowsDo: [:rowInfo | rowInfo visible ifTrue: [
		| item |

		cell := cell + (1 @ 0).
		
		item := rowInfo items first.
		previousItemWidth ifNotNil: [:w | hOffset := hOffset + w].
		previousItemWidth := item width.

		"Special handling for hAlignment makes no sense here."		
		item topLeft: hOffset @ 0.

		"Fill bounds cache."
		cellToBounds first at: cell x put: {hOffset@0.item extent x @ maxItemExtent y.rowInfo}]].
			
	"Update dynamically-sized dimension."
	self width: self submorphBounds width.! !

!UiListViewport methodsFor: 'layouting' stamp: 'mt 5/22/2013 17:18:20.758'!
updateSingleColumnLayout
	
	self orientation == #vertical ifTrue: [
		^ self updateSingleColumnVerticalLayout].
	self orientation == #horizontal ifTrue: [
		^ self updateSingleColumnHorizontalLayout].
	self orientation == #horizontalSingleRow ifTrue: [
		^ self updateSingleColumnHorizontalSingleRowLayout].! !

!UiListViewport methodsFor: 'layouting' stamp: 'mt 8/19/2013 10:05:17.774'!
updateSingleColumnVerticalLayout
	
	| cell hAlignment vAlignment currentGroupItemCount |
	
	self assert: self orientation == #vertical.
	
	"Compute virtual row and column count."
	vRowCount := 0.
	vColumnCount := self width // maxItemExtent x.
	vColumnCount == 0 ifTrue: [^ self "skip layouting"].
	currentGroupItemCount := 0.

	self rows do: [:rowInfo | rowInfo visible ifTrue: [
		rowInfo isSeparator
			ifTrue: [
				vRowCount := vRowCount + 1 + (currentGroupItemCount / vColumnCount) ceiling.
				currentGroupItemCount := 0]
			ifFalse: [
				currentGroupItemCount := currentGroupItemCount + 1]]].
	vRowCount := vRowCount + (currentGroupItemCount / vColumnCount) ceiling.
	
	"Prepare cache and running variables."
	cellToBounds := Array new: vRowCount.
	cell := 0@0.
	hAlignment := self listColumn hAlignment.
	vAlignment := self listColumn vAlignment.

	self rowsDo: [:rowInfo | rowInfo visible ifTrue: [
		| item vOffset hOffset |

		((rowInfo isSeparator or: [cell x == vColumnCount]) or: [cell x == 0]) "line break?"
			ifTrue: [
				cell := 0 @ (cell y + 1).
				cellToBounds at: cell y put: (Array new: vColumnCount)].
		cell := cell + (1@0).
		
		item := rowInfo items first.
		hOffset := cell x-1 * maxItemExtent x.
		vOffset := cell y-1 * maxItemExtent y.
			
		(hAlignment == #left or: [rowInfo isSeparator])
			ifTrue: [item left: hOffset]
			ifFalse: [
				hAlignment == #center
					ifTrue: [item left: hOffset + (maxItemExtent x - item width // 2)].
				hAlignment == #right
					ifTrue: [item right: hOffset + maxItemExtent x]].

		(vAlignment == #top or: [rowInfo isSeparator])
			ifTrue: [item top: vOffset]
			ifFalse: [
				vAlignment == #bottom
					ifTrue: [item bottom: vOffset + maxItemExtent y].
				vAlignment == #center
					ifTrue: [item top: vOffset + (maxItemExtent y - item height // 2)]].

		"Fill bounds cache."
		(cellToBounds at: cell y) at: cell x put: {hOffset@vOffset.maxItemExtent.rowInfo}.

		rowInfo isSeparator "carriage return"
			ifTrue: [cell := 0 @ cell y]]].
			
	"Update dynamically-sized dimension."
	self height: self submorphBounds height.! !

UiItemViewport subclass: #UiTableViewport
	instanceVariableNames: 'showGrid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiTableViewport commentStamp: 'mt 11/2/2010 12:46:43.653' prior: 0!
A specialized viewport that can draw a grid.!

!UiTableViewport methodsFor: 'drawing' stamp: 'mt 10/14/2010 14:00:23.706'!
drawGridOn: aCanvas

	| offset |
	offset := self left.
	self columnsDo: [:colInfo | 
		offset := offset  + colInfo width.
		aCanvas
			line: offset @ self top
			to: offset @ self bottom
			color: (Color gray: 0.9)].
		
	offset := self top.
	self rowsDo: [:rowInfo | 
		offset := offset  + rowInfo height.
		aCanvas
			line: self left @ offset
			to: self right @ offset
			color: (Color gray: 0.9)].! !

!UiTableViewport methodsFor: 'drawing' stamp: 'mt 8/25/2013 19:48:56.921'!
drawOn: aCanvas

	self shouldDraw ifFalse: [^ self].
	self showGrid ifTrue: [self drawGridOn: aCanvas].
	super drawOn: aCanvas.! !

!UiTableViewport methodsFor: 'accessing' stamp: 'mt 8/7/2012 11:05:17.45'!
showGrid
	^ showGrid ifNil: [true]! !

!UiTableViewport methodsFor: 'accessing' stamp: 'mt 8/7/2012 11:05:11.599'!
showGrid: anObject
	showGrid := anObject! !

UiItemViewport subclass: #UiTreeViewport
	instanceVariableNames: 'indent rootIsDecorated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiTreeViewport commentStamp: 'mt 11/2/2010 12:47:08.195' prior: 0!
A specialized item viewport that handles hierarchical items in the first column.!

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 6/26/2012 16:46:47.224'!
bottomMostVisibleCellIndex: aCanvas
	| bottomLeftCell bottomLeft |
	bottomLeft := self bounds pointNearestTo: aCanvas clipRect bottomLeft.
	bottomLeftCell := self cellIndexFromPoint: bottomLeft.
	^ bottomLeftCell
		ifNil: [self rows size]
		ifNotNil: [:cell | cell y]! !

!UiTreeViewport methodsFor: 'signals' stamp: 'mt 10/14/2010 13:13:30.709'!
decorationClicked: aModelIndex
	"Whenever the user clicks the expand/collapse decoration in front of an item."
	
	self emit.! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 8/25/2013 19:48:49.916'!
drawOn: aCanvas
	
	self shouldDraw ifFalse: [^ self].
	
	super drawOn: aCanvas.
	
	self drawCacheActive ifFalse: [
		self drawTreeStructureOn: aCanvas].! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 8/25/2013 19:49:02.869'!
drawOverlayOn: aCanvas

	self shouldDraw ifFalse: [^ self].
	super drawOverlayOn: aCanvas.
	  
	self drawCacheActive ifTrue: [
		self drawTreeStructureOn: aCanvas].! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 8/30/2012 10:14:40.139'!
drawTreeStructureOn: aCanvas
	
	columns ifEmpty: [^ self].
	
	(self columnBoundsAt: 1) right < aCanvas clipRect left
		ifTrue: [^ self].
	
	((self visibleRowsIndices: aCanvas)
		collect: [:rowIndex | rowIndex -> (self rowAt: rowIndex)])
		select: [:rowAssoc | rowAssoc value visible
			and: [self rootIsDecorated or: [rowAssoc value parentModelIndex notNil]]]
		thenDo: [:rowAssoc |
		| colIndex item cellBounds form |
		colIndex := self columnOrder unmapIndex: 1.
		cellBounds := self cellBoundsFor: colIndex@rowAssoc key.
		item := self columnOrder items: rowAssoc value items at: colIndex.
		form := (item valueOfProperty: #hasChildren)
			ifFalse: [nil]
			ifTrue: [(item valueOfProperty: #isExpanded)
				ifFalse: [UiButtonIcons bulletArrowRightSunkenIcon]
				ifTrue: [UiButtonIcons bulletArrowDownSunkenIcon]].

		form ifNotNil: [
			aCanvas
				translucentImage: form
				at: (cellBounds left + 4 + ((item valueOfProperty: #indent) - 12))
					@ (cellBounds top + (cellBounds height - form height // 2))]].! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 8/27/2012 15:44:38.307'!
indent

	^ indent ifNil: [indent := 12]! !

!UiTreeViewport methodsFor: 'initialization' stamp: 'mt 10/18/2010 19:09:04.846'!
initialize

	super initialize.
	
	rowOrder := UiTreeItemOrder new.
	
	self selectionBehavior: #selectRows.! !

!UiTreeViewport methodsFor: 'initialization' stamp: 'mt 1/18/2012 11:30:35.395'!
initializeEventHandler

	super initializeEventHandler.
	
	self connect: self eventHandler signal: #mouseClicked: toSelector: #mouseClickDecoration:.! !

!UiTreeViewport methodsFor: 'event handling' stamp: 'mt 3/18/2011 11:25:54.178'!
mouseClickDecoration: evt

	| cell item |
	cell := self cellIndexFromPoint: evt position.
	cell ifNil: [^ self].
	
	item := (self rowAt: cell y) items at: 1.

	"Recognize a click on the decoration."
	(evt position x < item left
		and: [evt position x >= (item left - self indent)])
		ifTrue: [self decorationClicked: (item valueOfProperty: #modelIndex)].! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 6/26/2012 17:21:42.471'!
rootIsDecorated

	^ rootIsDecorated ifNil: [true]! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 7/3/2013 10:09:28'!
rootIsDecorated: aBoolean

	aBoolean = rootIsDecorated ifTrue: [^ self].
	
	rootIsDecorated := aBoolean.
	
	self columns ifEmpty: [
		self updateLayout.
		^ self].
	
	rootIsDecorated
		ifTrue: [
			"Add root indent to all items in first column."
			self columns first items do: [:morph |
				(morph hasProperty: #isGroupSeparator) ifFalse: [
					morph
						setProperty: #indent
						toValue: (morph valueOfProperty: #indent) + self indent]]]
		ifFalse: [
			"Remove root indent to all items in first column."
			self columns first items do: [:morph |
				(morph hasProperty: #isGroupSeparator) ifFalse: [
					morph
						setProperty: #indent
						toValue: (morph valueOfProperty: #indent) - self indent]]].

	self updateLayout.
! !

!UiTreeViewport methodsFor: 'rows/columns - to pixel' stamp: 'mt 8/30/2012 10:16:33.394'!
rowBoundsAt: aRowIndex

	| rowBounds indent row |
	rowBounds := super rowBoundsAt: aRowIndex.
	rowBounds width = 0 ifTrue: [^ rowBounds].
	
	row := self rowAt: aRowIndex.
	
	self flag: #magicNumber. "mt: 12 is the size of the tree structure decoration."
	indent := (self rootIsDecorated or: [row parentModelIndex notNil])
		ifTrue: [(row items first valueOfProperty: #indent) - 12]
		ifFalse: [row items first valueOfProperty: #indent].
	
	^ rowBounds insetBy: (indent @ 0 corner: 0 @ 0)! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 6/26/2012 16:47:03.261'!
topMostVisibleCellIndex: aCanvas
	| topLeft topLeftCell | 
	topLeft := self bounds pointNearestTo: aCanvas clipRect topLeft.
	topLeftCell := self cellIndexFromPoint: topLeft.
	^ topLeftCell
		ifNil: [1]
		ifNotNil: [:cell | cell y]! !

!UiTreeViewport methodsFor: 'drawing' stamp: 'mt 6/26/2012 16:49:43.663'!
visibleRowsIndices: aCanvas

	^ (self topMostVisibleCellIndex: aCanvas)
		to: (self bottomMostVisibleCellIndex: aCanvas)! !

Morph subclass: #UiLabel
	instanceVariableNames: 'form text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiLabel commentStamp: '<historical>' prior: 0!
I am a descriptive widget that explains some content of another widget near me.
I can show a form or a text. But not both at the same time.!

!UiLabel class methodsFor: 'testing' stamp: 'mt 6/29/2010 19:06:41.567'!
isWidgetClass

	^ true! !

!UiLabel class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:29:53.936'!
widgetCategory
	
	^ 'Display Widgets'! !

!UiLabel class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:13:53.245'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiLabelIcon! !

!UiLabel class methodsFor: 'accessing' stamp: 'mt 6/30/2010 21:22:24.431'!
widgetClassName

	^ 'Label'! !

!UiLabel class methodsFor: 'accessing' stamp: 'mt 10/28/2010 16:52:55.134'!
widgetProperties

	^ super widgetProperties, {{'Label'. #yourself} -> {
		#text -> String.
		#form -> Form}}! !

!UiLabel methodsFor: 'accessing' stamp: 'mt 5/1/2011 12:21:40.506'!
color: aColor

	self text addAttribute: (TextColor color: aColor).! !

!UiLabel methodsFor: 'accessing' stamp: 'mt 7/4/2010 08:35:24.982'!
form

	^ form! !

!UiLabel methodsFor: 'accessing' stamp: 'mt 7/4/2010 08:36:48.648'!
form: aForm

	form := aForm.
	self updateSubmorphs.! !

!UiLabel methodsFor: 'layout-properties' stamp: 'mt 7/22/2013 11:41:03.67'!
hResizing: aSymbol

	aSymbol = self hResizing ifTrue: [^ self].
	super hResizing: aSymbol.
	
	aSymbol = #shrinkWrap
		ifTrue: [
			text
				hResizing: #rigid;
				wrapFlag: false]
		ifFalse: [
			text
				hResizing: #spaceFill;
				wrapFlag: true].	! !

!UiLabel methodsFor: 'accessing - convenience' stamp: 'mt 7/22/2013 11:30:18.445'!
icon
	"Convenience."
	
	^ self form! !

!UiLabel methodsFor: 'accessing - convenience' stamp: 'mt 7/22/2013 11:30:17.081'!
icon: aForm
	"Convenience."
	
	^ self form: aForm! !

!UiLabel methodsFor: 'initialization' stamp: 'mt 5/1/2011 12:01:37.953'!
initialize

	super initialize.
	
	text := TextMorph new.
	text contents: 'Label'.
	text lock.
	
	super color: Color transparent.
	
	self
		changeTableLayout;
		listDirection: #leftToRight;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		updateSubmorphs.! !

!UiLabel methodsFor: 'accessing' stamp: 'mt 7/4/2010 08:33:10.538'!
text

	^ text contents! !

!UiLabel methodsFor: 'accessing' stamp: 'mt 7/22/2013 11:48:13.115'!
text: aText

	self hResizing = #shrinkWrap
		ifTrue: [text contents: aText asHtmlText]
		ifFalse: [text contentsWrapped: aText asHtmlText].
	self updateSubmorphs.! !

!UiLabel methodsFor: 'accessing - private' stamp: 'mt 7/4/2010 08:46:07.552'!
textMorph

	^ text! !

!UiLabel methodsFor: 'accessing - convenience' stamp: 'mt 7/22/2013 11:28:37.405'!
theText
	"Convenience. Consistent with UiTextEdit API."
	
	^ self text! !

!UiLabel methodsFor: 'accessing - convenience' stamp: 'mt 7/22/2013 11:28:23.407'!
theText: stringOrText
	"Convenience. Consistent with UiTextEdit API."
	
	^ self text: stringOrText! !

!UiLabel methodsFor: 'updating' stamp: 'mt 7/22/2013 11:43:18.683'!
updateSubmorphs

	self flag: #smell. "mt: Why not just integrate into accessors?"
	((self submorphs notEmpty and: [self form isNil]) and: [self submorphs first == text])
		ifTrue: [^ self].

	self removeAllMorphs.
	
	self form
		ifNil: [self addMorphBack: text]
		ifNotNil: [self addMorphBack: form asMorph].! !

Morph subclass: #UiProgressBar
	instanceVariableNames: 'text minimum maximum currentValue textColor backgroundColor backgroundTextColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiProgressBar commentStamp: '<historical>' prior: 0!
I am like a slide that is not interactive and has a label... and kind of a different appearance. :D

Actually, I am no slider. However, my API resembles that of one. I have a minimum (0 by default), a maximum (100 by default), and a currentValue.

My drawing code is optimized for speed. Thus, I do not support rich text but only simple messages.!

!UiProgressBar class methodsFor: 'testing' stamp: 'mt 8/1/2013 19:01:19.487'!
isWidgetClass

	^ true! !

!UiProgressBar class methodsFor: 'accessing' stamp: 'mt 8/1/2013 18:57:03.923'!
widgetCategory

	^ 'Display Widgets'! !

!UiProgressBar class methodsFor: 'accessing' stamp: 'mt 8/1/2013 18:57:40.386'!
widgetClassIcon

	^ UiFugueIcons uiProgressBarIcon! !

!UiProgressBar class methodsFor: 'accessing' stamp: 'mt 8/1/2013 18:57:55.65'!
widgetClassName

	^ 'Progress Bar'! !

!UiProgressBar class methodsFor: 'accessing' stamp: 'mt 8/2/2013 13:28:19.633'!
widgetProperties

	^ super widgetProperties, {{self widgetClassName. #yourself} -> {
		#text -> String.
		#minimum -> Integer.
		#maximum -> Integer.
		#currentValue -> Integer.
		}.
		{self widgetClassName, ' - Colors'. #yourself} -> {
		#color -> Object.
		#textColor -> Object.
		#backgroundColor -> Object.
		#backgroundTextColor -> Object}}! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/2/2013 13:22:42.213'!
backgroundColor

	^ backgroundColor! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/2/2013 13:24:11.644'!
backgroundColor: aColor

	backgroundColor = aColor
		ifTrue: [^ self].
	backgroundColor := aColor.
	self redrawLater.! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/2/2013 13:23:17.472'!
backgroundTextColor

	^ backgroundTextColor! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/2/2013 13:23:44.121'!
backgroundTextColor: aColor

	backgroundTextColor = aColor
		ifTrue: [^ self].
	backgroundTextColor := aColor.
	self redrawLater.! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/1/2013 18:55:33.05'!
currentValue

	^ currentValue! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/14/2013 13:06:19.415'!
currentValue: aNumber

	currentValue == aNumber ifTrue: [^ self].	
	currentValue := self isIndeterminate
		ifTrue: [aNumber]
		ifFalse: [aNumber min: maximum max: minimum].
	self redrawLater.! !

!UiProgressBar methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:36:48.131'!
drawOn: aCanvas

	UiPainter default new
		drawProgressBar: self on: aCanvas.! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/14/2013 12:53:06.541'!
fraction

	^ maximum
		ifNil: [1.0]
		ifNotNil: [(currentValue - minimum) / (maximum - minimum)]! !

!UiProgressBar methodsFor: 'initialization' stamp: 'mt 8/2/2013 13:22:34.712'!
initialize

	super initialize.
	
	text := 'Progress...'.
	minimum := 0.
	maximum := 100.
	currentValue := 50.
	
	backgroundColor := Color gray: 0.5.
	backgroundTextColor := Color gray: 0.9.
	textColor := Color gray: 0.1.
	
	self
		color: (Color r: 1.0 g: 222/255 b: 0);
		extent: 100@20.! !

!UiProgressBar methodsFor: 'testing' stamp: 'mt 8/14/2013 12:55:31.908'!
isIndeterminate

	^ maximum isNil! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/1/2013 18:53:27.793'!
maximum

	^ maximum! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/1/2013 18:53:20.388'!
maximum: aNumber

	maximum == aNumber ifTrue: [^ self].
	maximum := aNumber.
	self redrawLater.! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/1/2013 18:52:48.171'!
minimum

	^ minimum! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/1/2013 18:52:41.356'!
minimum: aNumber

	minimum == aNumber ifTrue: [^ self].
	minimum := aNumber.
	self redrawLater.! !

!UiProgressBar methodsFor: 'geometry' stamp: 'mt 8/1/2013 19:58:33.366'!
outerBounds

	^ self bounds outsetBy: 1! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/1/2013 18:51:55.071'!
text

	^ text! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/1/2013 19:47:01.32'!
text: stringOrText

	text == stringOrText ifTrue: [^ self].
	text := stringOrText.
	self redrawLater.! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/2/2013 13:23:09.391'!
textColor

	^ textColor! !

!UiProgressBar methodsFor: 'accessing' stamp: 'mt 8/2/2013 13:23:58.798'!
textColor: aColor

	textColor = aColor
		ifTrue: [^ self].
	textColor := aColor.
	self redrawLater.! !

Morph subclass: #UiScrollArea
	instanceVariableNames: 'horizontalScrollBar horizontalScrollBarPolicy verticalScrollBar verticalScrollBarPolicy cornerWidget transformMorph outerBorderWidth widgetResizable areaResizable scrollBarThickness'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiScrollArea commentStamp: 'mt 3/7/2011 14:29:02.055' prior: 0!
Just set a #widget: and use it. :)!

UiScrollArea subclass: #UiAbstractItemView
	instanceVariableNames: 'model columnSpecs verticalHeader horizontalHeader vHeaderScroller hHeaderScroller animated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

UiAbstractItemView class
	instanceVariableNames: 'signalConnections'!

!UiAbstractItemView commentStamp: 'mt 1/17/2012 13:40:22.701' prior: 0!
There is a difference between nodes and model indices. A node is an element in a tree-structured database. A model index corresponds to a node and one of multiple properties of a node. Model indices are tree-structured as well. (A cell in the viewport is a flattened model index for drawing and sorting reasons.)

The view should do node boxing and unboxing - not the viewport or the client.!

UiAbstractItemView class
	instanceVariableNames: 'signalConnections'!

!UiAbstractItemView class methodsFor: 'testing' stamp: 'mt 3/7/2011 14:34:32.325'!
isWidgetClass

	^ false! !

!UiAbstractItemView class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:33:18.144'!
widgetCategory
	
	^ 'Item Views (Model-Based)'! !

!UiAbstractItemView class methodsFor: 'properties' stamp: 'mt 12/17/2012 15:17:59.77'!
widgetProperties

	^ super widgetProperties, {{'Item View'. #yourself} -> {
		#modelClass -> Object.
		#horizontalHeaderVisible -> Boolean.
		#verticalHeaderVisible -> Boolean.
		#selectionBehavior -> #(selectCells selectRows selectColumns).
		"#selectionColor -> Object."
		#selectionMode -> Object}}! !

!UiAbstractItemView class methodsFor: 'properties' stamp: 'mt 4/6/2011 14:08:30.911'!
widgetPropertiesToIgnore

	^ #('TableLayout' 'ProportionalLayout' widgetResizable)! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 2/2/2012 10:37:43.479'!
activated: aNode
	"Single mouse click or pressing the return key."
	
	self emit.! !

!UiAbstractItemView methodsFor: 'rows/columns - specs' stamp: 'mt 1/23/2012 13:52:18.38'!
addColumn: aColumnSpec
	"Append a new column according to the given specification. Make sure that your model satisfies this specification."
	
	self addColumns: {aColumnSpec}.! !

!UiAbstractItemView methodsFor: 'rows/columns - specs - convenience' stamp: 'bs 1/26/2012 20:39:25.86'!
addColumn: headerText for: aColumnSpec
	
	self addColumn: (
		aColumnSpec 
			headerText: headerText;
			yourself).! !

!UiAbstractItemView methodsFor: 'rows/columns - specs - convenience' stamp: 'bs 1/26/2012 20:39:29.342'!
addColumn: headerText textSelector: aSymbol
	
	self addColumn: (UiItemViewColumnSpec new
		headerText: headerText;
		textSelector: aSymbol).! !

!UiAbstractItemView methodsFor: 'rows/columns - specs - convenience' stamp: 'bs 1/23/2012 16:36:24.376'!
addColumn: headerText textSelector: aSymbol and: selectorMap
	"This is a convenient method for adding a new column that just displays text."
	
	self addColumn: (
		(UiItemViewColumnSpec newFrom: selectorMap)
			headerText: headerText;
			textSelector: aSymbol).! !

!UiAbstractItemView methodsFor: 'rows/columns - specs - convenience' stamp: 'bs 1/26/2012 20:40:05.89'!
addColumnFor: aColumnSpec
	
	self addColumn: aColumnSpec! !

!UiAbstractItemView methodsFor: 'rows/columns - specs' stamp: 'mt 1/23/2012 13:47:53.468'!
addColumns: someColumnSpecs
	
	self columnSpecs addAll: someColumnSpecs.
	self reset.! !

!UiAbstractItemView methodsFor: 'headers - add/remove' stamp: 'mt 4/12/2011 10:58:36.739'!
addHorizontalHeader

	self addMorph: hHeaderScroller.
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - specs - convenience' stamp: 'bs 1/26/2012 20:40:00.992'!
addTextColumn: headerText selector: aSymbol
	
	self addColumn: (UiItemViewColumnSpec new
		headerText: headerText;
		textSelector: aSymbol).! !

!UiAbstractItemView methodsFor: 'headers - add/remove' stamp: 'mt 10/19/2010 14:11:36.704'!
addVerticalHeader

	self addMorph: vHeaderScroller.
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 1/18/2012 09:51:24.607'!
animated

	^ animated ifNil: [animated := false]! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 2/20/2011 18:49:46.161'!
animated: aBoolean

	animated := aBoolean.! !

!UiAbstractItemView methodsFor: 'rows/columns - sorting' stamp: 'mt 7/22/2012 10:02:40.445'!
autoSortBlock
	
	^ self viewport rowOrder sortBlock! !

!UiAbstractItemView methodsFor: 'rows/columns - sorting' stamp: 'mt 3/25/2011 09:35:55.905'!
autoSortBlock: aBlock
	"Specify the way rows will be sorted"
	
	self viewport rowOrder sortBlock: aBlock.
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 7/13/2013 17:26:35.702'!
changeAllItems
	"Update all visible items."

	| item useAnimation |
	useAnimation := self isAnimated.
		
	self columnSpecs withIndexDo: [:spec :id |
		(self viewport columns at: id) indices do: [:index |
			item := index item.
			self viewport replaceMorphWith: item atIndex: index.
			useAnimation ifTrue: [item fadeIn]]].
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 7/13/2013 17:26:55.507'!
changeAllItemsWithProperties: someProperties
	"Update all columns that contain at least one of the given properties."
	
	| item useAnimation |
	useAnimation := self isAnimated.
	
	self columnSpecs withIndexDo: [:spec :id |
		(spec includesAnyProperty: someProperties) ifTrue: [
			(self viewport columns at: id) indices do: [:index |
				item := index item.
				self viewport replaceMorphWith: item atIndex: index.
				useAnimation ifTrue: [item fadeIn]]]].
	
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 7/20/2012 16:23:27.529'!
changeItemFor: aModelIndex

	| item |
	"Check if node is visible."
	(self viewport isExpanded: aModelIndex parent)
		ifFalse: [^ self].
	
	item := aModelIndex item.
	self viewport replaceMorphWith: item atIndex: aModelIndex.
	self isAnimated ifTrue: [item fadeIn].
	
	self
		updateViewportLayout;
		changed: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 4/26/2012 13:27:06.863'!
changeRowFor: aModelIndex

	"Check if node is visible."
	(self viewport isExpanded: aModelIndex parent)
		ifFalse: [^ self].
	
	self columnSpecs withIndexDo: [:spec :id |
		| item index |
		index := UiModelIndex
			row: aModelIndex row
			column: id
			group: aModelIndex group
			groupIndex: aModelIndex groupIndex
			parent: aModelIndex parent
			node: aModelIndex node
			spec: spec.
		self connectIndex: index.
				
		item := index item.
		self viewport replaceMorphWith: item atIndex: index.
		self isAnimated ifTrue: [item fadeIn]].
	
	self
		updateViewportLayout;
		changed: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 7/19/2012 10:54:34.331'!
changeRowsFor: aParentModelIndex from: start to: end inGroup: groupName

	| index item node |
	(self viewport isExpanded: aParentModelIndex)
		ifFalse: [^ self].
		
	"node unboxing"
	node := aParentModelIndex
		ifNil: [self model]
		ifNotNil: [aParentModelIndex node].
	
	start to: end do: [:row |
		| nextChild |
		self flag: #ugly. "Reuse child nodes from present indices? How to access those nodes?"
		nextChild := node childAt: row inGroup: groupName.
		
		self columnSpecs withIndexDo: [:spec :id |
			index := UiModelIndex
				row: row
				column: id
				group: groupName
				groupIndex: (node groups indexOf: groupName)
				parent: aParentModelIndex
				node: nextChild
				spec: spec.
			self connectIndex: index.
				
			item := index item.
			self viewport replaceMorphWith: item atIndex: index.
			self isAnimated ifTrue: [item fadeIn]]].
	
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 2/16/2012 16:30:37.612'!
changed: aNode
	"Emitted whenever a node changes. These changes are manifested in model code."
		
	self emit.! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 2/2/2012 10:37:47.454'!
clicked: aNode

	self emit.! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 12/18/2012 09:56:25.598'!
color: aColor

	self transformMorph ifNotNil: [:tm |
		tm color: aColor].
	
	vHeaderScroller color: aColor.
	hHeaderScroller color: aColor.
	
	verticalScrollBar color: aColor.
	horizontalScrollBar color: aColor.! !

!UiAbstractItemView methodsFor: 'as yet unclassified' stamp: 'mt 8/25/2013 13:33:12.707'!
columnCount

	^ self columnSpecs size! !

!UiAbstractItemView methodsFor: 'rows/columns - specs' stamp: 'mt 1/23/2012 09:32:26.118'!
columnSpecs

	^ columnSpecs ifNil: [columnSpecs := OrderedCollection new]! !

!UiAbstractItemView methodsFor: 'model - initialization' stamp: 'mt 4/27/2012 11:04:53.805'!
connectIndex: index

	"For all columns resp. whole rows."
	index column = 1 ifTrue: [
		self connect: index signal: #modelReset toSelector: #resetChildrenFor: pattern: #(0).
		self connect: index signal: #rowsInsertedFrom:to:inGroup: toSelector: #insertRowsFor:from:to:inGroup: pattern: #(0 1 2 3).
		self connect: index signal: #rowsChangedFrom:to:inGroup: toSelector: #changeRowsFor:from:to:inGroup: pattern: #(0 1 2 3).
		self connect: index signal: #rowsRemovedFrom:to:inGroup: toSelector: #removeRowsFor:from:to:inGroup: pattern: #(0 1 2 3).
		self connect: index signal: #rowChanged toSelector: #changeRowFor: pattern: #(0)].
	
	"For a single column resp. a single cell."
	self connect: index signal: #itemChanged toSelector: #changeItemFor: pattern: #(0).
! !

!UiAbstractItemView methodsFor: 'model - initialization' stamp: 'mt 4/22/2012 16:11:59.008'!
connectRoot

	self model ifNotNil: [:m |
		"root signals"
		self connect: m signal: #modelReset toSelector: #reset.
		self connect: m signal: #childrenInsertedFrom:to:inGroup: toSelector: #insertRowsFor:from:to:inGroup: pattern: {#=. nil. 1. 2. 3}.
		self connect: m signal: #childrenChangedFrom:to:inGroup: toSelector: #changeRowsFor:from:to:inGroup: pattern: {#=. nil. 1. 2. 3}.
		self connect: m signal: #childrenRemovedFrom:to:inGroup: toSelector: #removeRowsFor:from:to:inGroup: pattern: {#=. nil. 1. 2. 3}.
		
		self connect: m signal: #propertiesChanged toSelector: #changeAllItems.
		self connect: m signal: #propertiesChanged: toSelector: #changeAllItemsWithProperties:].! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 2/2/2012 10:44:25.981'!
currentNode

	^ self selection currentIndex
		ifNotNil: [:index | index node]! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 2/6/2012 17:28:56.97'!
currentNode: aNode

	| indices |
	indices := self modelIndicesFor: aNode.
	self selection currentIndex: (indices
		ifEmpty: []
		ifNotEmpty: [indices first]).! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 2/7/2012 16:25:07.193'!
currentNodeChanged: aNode

	self emit.! !

!UiAbstractItemView methodsFor: 'rows/columns - sorting' stamp: 'mt 3/25/2011 09:36:39.029'!
disableAutoSort
	"No row sorting."
	
	self viewport rowOrder sortBlock: nil.
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'as yet unclassified' stamp: 'mt 8/16/2013 11:36:33.84'!
disconnectDefaultSignals
	"Nothing. No default signals connected."! !

!UiAbstractItemView methodsFor: 'as yet unclassified' stamp: 'mt 8/16/2013 20:03:42.062'!
disconnectRoot

	self model ifNotNil: [:m |
		"root signals"
		m disconnectSignal: #modelReset from: self selector: #reset.
		m disconnectSignal: #childrenInsertedFrom:to:inGroup: from: self selector: #insertRowsFor:from:to:inGroup:.
		m disconnectSignal: #childrenChangedFrom:to:inGroup: from: self selector: #changeRowsFor:from:to:inGroup:.
		m disconnectSignal: #childrenRemovedFrom:to:inGroup: from: self selector: #removeRowsFor:from:to:inGroup:.
		
		m disconnectSignal: #propertiesChanged from: self selector: #changeAllItems.
		m disconnectSignal: #propertiesChanged: from: self selector: #changeAllItemsWithProperties:].! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 2/2/2012 10:38:17.895'!
doubleClicked: aNode

	self emit.! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 7/18/2012 11:32:18.728'!
dragDropSpec

	^ self viewport dragDropSpec! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 7/18/2012 11:32:34.493'!
dragDropSpec: aDragDropSpec

	self viewport dragDropSpec: aDragDropSpec.! !

!UiAbstractItemView methodsFor: 'model - actions' stamp: 'mt 7/6/2012 11:35:31.221'!
edit: aNode property: aProperty
	"Start the edit mode for the property in the given node."

	| rowIndex columnIndex |
	rowIndex := self rowIndexFor: aNode.
	columnIndex := self viewport columnOrder mapIndexOf: (self columnSpecs indexOf: (self columnSpecs detect: [:spec | spec includesProperty: aProperty])).

	self viewport beginEditCell: columnIndex @ rowIndex.! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 2/6/2013 13:15:58.25'!
editTriggers

	^ self viewport editTriggers! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 2/6/2013 14:29:46.303'!
editTriggers: someSymbols
	"#currentChanged, #doubleClicked, #selectedClicked, #editKeyPressed, #alphaNumericKeyPressed"
	
	self viewport editTriggers: someSymbols.! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 7/8/2012 14:08:59.029'!
edited: aNode
	"Emitted whenever the user edits an item via in-place editors in the view."
	
	self emit.! !

!UiAbstractItemView methodsFor: 'rows/columns - sorting' stamp: 'mt 12/12/2012 11:06:43.689'!
enableAutoSort
	"Sort rows alphabetically in ascending order."
	
	self autoSortBlock: [:node1 :node2 | node1 asString <= node2 asString].
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'model - initialization' stamp: 'mt 7/15/2013 16:45:05.74'!
fillViewportFor: aModelIndex animated: aBoolean

	| index item groupItem newRows node |
	"Node unboxing"
	node := aModelIndex
		ifNil: [self model "root addressed"]
		ifNotNil: [aModelIndex node].
	
	"Create row-specific data as nested array:
	group -> row -> column -> model index -> item/morph."
	newRows := node groups withIndexCollect: [:group :groupIndex |
		groupItem := self groupSpec itemFor: node groupIndex: groupIndex isTopLevel: aModelIndex isNil.
		groupItem
			setProperty: #isGroupSeparator toValue: true;
			setProperty: #indent toValue: 0;
			setProperty: #hasChildren toValue: false;
			setProperty: #parentModelIndex toValue: aModelIndex;
			setProperty: #groupIndex toValue: groupIndex.
		
		groupItem -> (
		(1 to: (node childCountInGroup: group)) collect: [:row |
			| nextChild |
			"share instance of next child among all columns in one row"
			nextChild := node childAt: row inGroup: group.

			self columnSpecs withIndexCollect: [:spec :id |
				"Node boxing"
				index := UiModelIndex
					row: row
					column: id
					group: group
					groupIndex: groupIndex
					parent: aModelIndex
					node: nextChild
					spec: spec.
				self connectIndex: index.

				item := index item.
				aBoolean ifTrue: [item fadeIn].
				index -> item]])].
	
	newRows ifEmpty: [^ #()].
	
	aModelIndex
		ifNil: [self viewport appendItemRows: newRows] "top-level"
		ifNotNil: [self viewport insertItemRows: newRows].
		
	^ newRows! !

!UiAbstractItemView methodsFor: 'events' stamp: 'mt 5/24/2011 16:13:00.1'!
grabKeyboard

	self viewport grabKeyboard.! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 6/26/2012 11:28:53.334'!
groupSpec

	^ self viewport groupSpec! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 6/26/2012 11:29:50.737'!
groupSpec: aGroupSpec
	"Configure the way how group separator items are drawn or groups behave."
	
	self viewport groupSpec: aGroupSpec.
	self reset.! !

!UiAbstractItemView methodsFor: 'headers - testing' stamp: 'mt 10/14/2010 13:04:29.063'!
hasHorizontalHeader

	^ hHeaderScroller owner notNil! !

!UiAbstractItemView methodsFor: 'headers - testing' stamp: 'mt 10/14/2010 13:03:13.199'!
hasVerticalHeader

	^ vHeaderScroller owner notNil! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 8/22/2012 10:31:37.655'!
helpText
	"Shortcut for help text in the drag drop spec."
	
	^ self viewport helpText! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 9/7/2012 10:19:56.32'!
helpText: aString
	"Shortcut for help text in the drag drop spec."
	
	self viewport helpText: aString.
	self redrawLater.! !

!UiAbstractItemView methodsFor: 'rows/columns - alignment' stamp: 'mt 8/19/2013 14:46:33.077'!
horizontalAlignmentForColumn: anIndex

	self flag: #deprecated. "mt: Use column spec instead."
	^ (self viewport columnAt: anIndex)
		hAlignment! !

!UiAbstractItemView methodsFor: 'headers - add/remove' stamp: 'mt 10/28/2010 10:07:05.122'!
horizontalHeaderVisible
		
	^ self hasHorizontalHeader! !

!UiAbstractItemView methodsFor: 'headers - add/remove' stamp: 'mt 10/28/2010 10:06:33.964'!
horizontalHeaderVisible: aBoolean

	aBoolean
		ifTrue: [self addHorizontalHeader]
		ifFalse: [self removeHorizontalHeader].! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 8/16/2012 14:44:00.851'!
hovered: aNode

	self emit.! !

!UiAbstractItemView methodsFor: 'initialization' stamp: 'mt 8/22/2012 11:04:56.33'!
initialize

	"Initialize headers."
	verticalHeader := Morph new color: Color transparent; changeTableLayout; listDirection: #topToBottom; hResizing: #shrinkWrap; cellInset: -1 "hacky".
	horizontalHeader := Morph new color: Color transparent; changeTableLayout; listDirection: #leftToRight; vResizing: #shrinkWrap.
	vHeaderScroller := UiTransformMorph new.
	hHeaderScroller := UiTransformMorph new.

	super initialize.
	
	self
		borderWidth: 0;
		color: Color white.
	super color: Color transparent.
		
	self
		verticalScrollBarPolicy: #asNeeded;
		horizontalScrollBarPolicy: #asNeeded.
		
	"Initialize viewport."
	self widget: self viewportClass new.
	self widget groupSpec: UiItemViewGroupSpec new. "for creating group separator items"
	
	self connect: self viewport signal: #focusChanged: toSelector: #scrollToShow:.
	self connect: self viewport signal: #hovered: toSelector: #onHovered:.
	self connect: self viewport signal: #clicked: toSelector: #onClicked:.
	self connect: self viewport signal: #doubleClicked: toSelector: #onDoubleClicked:.
	self connect: self viewport signal: #selectionChanged: toSelector: #onSelectionChanged:.
	self connect: self viewport signal: #currentIndexChanged: toSelector: #onCurrentIndexChanged:.
	self connect: self viewport signal: #activated: toSelector: #onActivated:.
	self connect: self viewport signal: #returnPressed: toSelector: #onReturnPressed:.
	self connect: self viewport signal: #menuRequestedAt:on: toSelector: #showMenuAt:on:.
	self connect: self viewport signal: #edited: toSelector: #onEdited:.
	self connect: self viewport signal: #objectDropped:type:from:on: toSelector: #onObjectDropped:type:from:on:.
	self connect: self viewport signal: #objectDropped:type:from:before: toSelector: #onObjectDropped:type:from:before:.
	self connect: self viewport signal: #objectDropped:type:from:after: toSelector: #onObjectDropped:type:from:after:.
	self connect: self viewport signal: #objectDropped:type:from: toSelector: #objectDropped:type:from:.

	self connect: self viewport signal: #keyStroked:on: toSelector: #onKeyStroked:on:.

	"Needs to be redefined here beacuse the viewport catches keyboard and mouse events."
	self connect: self viewport signal: #mouseWheelUp to: self verticalScrollBar selector: #decreaseValueBySingleStep.
	self connect: self viewport signal: #mouseWheelDown to: self verticalScrollBar selector: #increaseValueBySingleStep.

	self viewport verticalHeader: verticalHeader.
	self viewport horizontalHeader: horizontalHeader.
	
	"Vertical scrolling is dependent from default item height."
	self verticalScrollBar singleStep: UiItemViewColumnSpec defaultItemExtent y.
	
	"Initialize headers - Finish."
	verticalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #height: pattern: #(2).
	vHeaderScroller connect: self transformMorph signal: #verticalOffsetChanged: toSelector: #verticalOffset:.
	self connect: vHeaderScroller signal: #areaChanged toSelector: #resizeVerticalHeaderScroller.
	vHeaderScroller addMorph: verticalHeader.

	horizontalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #width:.
	hHeaderScroller connect: self transformMorph signal: #horizontalOffsetChanged: toSelector: #horizontalOffset:.
	self connect: hHeaderScroller signal: #areaChanged toSelector: #resizeHorizontalHeaderScroller.
	hHeaderScroller addMorph: horizontalHeader.! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 7/8/2013 14:03:13.104'!
insertRowsFor: aParentModelIndex from: start to: end inGroup: groupName

	| index item isTopLevelAppending node |
	end < start ifTrue: [^ self].
	
	(self viewport isExpanded: aParentModelIndex)
		ifFalse: [^ self].
		
	"Node unboxing."
	node := aParentModelIndex
		ifNil: [self model]
		ifNotNil: [aParentModelIndex node].

	"Update trailing model indices."
	end + 1 to: (node childCountInGroup: groupName) do: [:row |
		| nextChild |
		self flag: #ugly. "Reuse child nodes from trailing indices? How to access those nodes?"
		nextChild := node childAt: row inGroup: groupName.
		
		self columnSpecs withIndexDo: [:spec :id | | oldIndex |
			index := UiModelIndex
				row: row column: id group: groupName
				groupIndex: (node groups indexOf: groupName)
				parent: aParentModelIndex
				node: nextChild
				spec: spec.
			oldIndex := UiModelIndex
				row: row - 1 - (end - start)
				column: id
				group: groupName
				groupIndex: (node groups indexOf: groupName)
				parent: aParentModelIndex
				node: nil
				spec: spec.
			
			self connectIndex: index.
			self viewport updateIndex: index oldIndex: oldIndex]].

	"Insert new rows."
	isTopLevelAppending := (groupName = node groups last
		and: [aParentModelIndex isNil])
		and: [(node childCountInGroup: groupName) == start].
	
	start to: end do: [:row |
		| nextChild |
		nextChild := node childAt: row inGroup: groupName.
		
		self columnSpecs withIndexDo: [:spec :id |
			index := UiModelIndex
				row: row column: id group: groupName
				groupIndex: (node groups indexOf: groupName)
				parent: aParentModelIndex
				node: nextChild
				spec: spec.
			self connectIndex: index.

			item := index item.
			isTopLevelAppending
				ifTrue: [self viewport appendMorph: item forIndex: index]
				ifFalse: [self viewport insertMorph: item forIndex: index].
			self isAnimated ifTrue: [item fadeIn]]].
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'testing' stamp: 'mt 7/13/2013 17:25:20.545'!
isAnimated
	"Show fading animations for item updates?"
	
	^ self animated and: [
		self viewport isNil or: [
			self viewport drawCacheActive not]]! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 7/20/2012 17:29:10.429'!
keyStroked: aKeyboardevent on: aNode

	self emit.! !

!UiAbstractItemView methodsFor: 'model - accessing' stamp: 'mt 3/7/2011 11:47:04.514'!
model

	^ model! !

!UiAbstractItemView methodsFor: 'model - accessing' stamp: 'mt 8/16/2013 20:04:30.611'!
model: aModel

	self disconnectRoot.
	
	model := aModel.

	self
		reset;
		connectRoot.! !

!UiAbstractItemView methodsFor: 'model - accessing' stamp: 'mt 10/28/2010 10:04:51.755'!
modelClass

	^ self model
		ifNil: [nil]
		ifNotNil: [:m | m class]! !

!UiAbstractItemView methodsFor: 'model - accessing' stamp: 'bs 1/20/2012 13:02:07.612'!
modelClass: aClass

	aClass
		ifNil: [self model: nil]
		ifNotNil: [:c | self model: c new].! !

!UiAbstractItemView methodsFor: 'model - to index' stamp: 'mt 5/2/2012 13:18:10.183'!
modelIndicesFor: aNode
	"Return a list of all model indices that are associated with the given node."
	
	self viewport rows ifEmpty: [^ #()].

	self viewport rows do: [:row |
		"Model index in a row can be nil for group separators."
		row firstModelIndex ifNotNil: [:index |
			index node = aNode ifTrue: [^ row indices]]].
	
	^ #()! !

!UiAbstractItemView methodsFor: 'rows/columns - sorting' stamp: 'mt 1/18/2012 11:43:31.377'!
moveColumn: sourceIndex to: targetIndex
	"Allows resorting of columns. Indices apply to ACTUALLY VISIBLE indices - not real model data indices."
	
	| map |
	map := self viewport columnOrder map
		ifNil: [(1 to: (self model columnCountFor: nil)) collect: [:i | i]].
	
	map
		at: targetIndex put: sourceIndex;
		at: sourceIndex put: targetIndex.
		
	self viewport columnOrder map: map.
	
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 8/22/2012 10:58:27.667'!
objectDropped: anObject type: aSymbol from: aSourceObject

	self emitAlways.! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 7/18/2012 11:46:41.132'!
objectDropped: anObject type: aSymbol from: aSourceObject afterNode: aNode

	self emit.! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 7/19/2012 10:18:19.233'!
objectDropped: anObject type: aSymbol from: aSourceObject beforeNode: aNode

	self emit.! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 7/18/2012 11:46:45.04'!
objectDropped: anObject type: aSymbol from: aSourceObject onNode: aNode

	self emit.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 2/2/2012 10:36:39.198'!
onActivated: aModelIndex

	self activated: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 2/2/2012 10:36:46.451'!
onClicked: aModelIndex

	self clicked: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 2/6/2012 17:33:20.361'!
onCurrentIndexChanged: aModelIndex

	self currentNodeChanged: (aModelIndex ifNotNil: [:index | index node]).! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 2/2/2012 10:37:05.079'!
onDoubleClicked: aModelIndex

	self doubleClicked: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 7/8/2012 14:08:36.707'!
onEdited: aModelIndex

	self edited: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 8/16/2012 15:43:27.032'!
onHovered: aModelIndex

	self hovered: (aModelIndex ifNotNil: [:mi | mi node]).! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 7/20/2012 17:36:12.359'!
onKeyStroked: aKeyboardEvent on: aModelIndex

	aModelIndex ifNil: [^ self].
	self keyStroked: aKeyboardEvent on: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 7/18/2012 11:28:31.733'!
onObjectDropped: anObject type: aSymbol from: aSourceObject after: aModelIndex

	self objectDropped: anObject type: aSymbol from: aSourceObject afterNode: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 7/19/2012 10:18:12.145'!
onObjectDropped: anObject type: aSymbol from: aSourceObject before: aModelIndex

	self objectDropped: anObject type: aSymbol from: aSourceObject beforeNode: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 7/18/2012 11:27:41.986'!
onObjectDropped: anObject type: aSymbol from: aSourceObject on: aModelIndex

	self objectDropped: anObject type: aSymbol from: aSourceObject onNode: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 2/2/2012 10:37:11.951'!
onReturnPressed: aModelIndex

	self returnPressed: aModelIndex node.! !

!UiAbstractItemView methodsFor: 'signals - unboxing' stamp: 'mt 2/2/2012 10:37:26.324'!
onSelectionChanged: someModelIndices

	self selectionChanged: (someModelIndices collect: [:index | index node]).! !

!UiAbstractItemView methodsFor: 'geometry' stamp: 'mt 9/13/2012 16:49:54.685'!
optimalWidth

	| w |
	w := 0.
	self viewport columns do: [:column |
		column resizeMode == #spaceFill
			ifTrue: [w := w + column optimalSize]
			ifFalse: [w := w + column width]].
	self flag: #magicNumber. "mt: Why is scrollbar thickness not sufficient here?"
	^ w + self scrollBarThickness + 5! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 2/16/2012 11:47:28.902'!
overrideMenuTarget: aMenuItem

	aMenuItem target: self model.
	aMenuItem subMenu ifNotNil: [:menu |
		menu items do: [:item | self overrideMenuTarget: item]].! !

!UiAbstractItemView methodsFor: 'rows/columns - specs' stamp: 'mt 8/16/2013 16:25:17.117'!
removeAllColumns

	columnSpecs := nil.
	self reset.! !

!UiAbstractItemView methodsFor: 'headers - add/remove' stamp: 'mt 10/19/2010 14:11:54.15'!
removeHorizontalHeader

	self removeMorph: hHeaderScroller.
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 4/27/2012 09:29:48.263'!
removeRowsFor: aParentModelIndex from: start to: end inGroup: groupName

	| index node |
	(self viewport isExpanded: aParentModelIndex)
		ifFalse: [^ self].
	
	start to: end do: [:row |
		self viewport removeRowFor: aParentModelIndex at: row inGroup: groupName].
	
	"node unboxing"
	node := aParentModelIndex
		ifNil: [self model]
		ifNotNil: [aParentModelIndex node].
	
	"Update trailing model indices."
	start to: (node childCountInGroup: groupName) do: [:row |
		| nextChild |
		self flag: #ugly. "Reuse child nodes from trailing indices? How to access those nodes?"
		nextChild := node childAt: row inGroup: groupName.
		
		self columnSpecs withIndexDo: [:spec :id | | oldIndex |
			index := UiModelIndex
				row: row column: id group: groupName
				groupIndex: (node groups indexOf: groupName)
				parent: aParentModelIndex
				node: nextChild
				spec: spec.
			oldIndex := UiModelIndex
				row: row + 1 + (end - start)
				column: id
				group: groupName
				groupIndex: (node groups indexOf: groupName)
				parent: aParentModelIndex
				node: nil
				spec: spec.
			
			self connectIndex: index.
			self viewport updateIndex: index oldIndex: oldIndex]].
	
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'headers - add/remove' stamp: 'mt 10/19/2010 14:11:43.32'!
removeVerticalHeader

	self removeMorph: vHeaderScroller.
	self updateLayout.! !

!UiAbstractItemView methodsFor: 'model - initialization' stamp: 'mt 8/30/2013 10:26:21.585'!
reset

	| selectedIndices currentIndex |
	selectedIndices := self selection selectedIndices.
	currentIndex := self selection currentIndex.

	self viewport clear.
	
	"Ensure the correct number of columns."
	self viewport limitColumnCount: self columnSpecs size.
	self columnSpecs with: self viewport columns do: [:spec :info |
		info hAlignment: spec hAlignment.
		info vAlignment: spec vAlignment].
	self resetHeaderItems.
	
	(self model notNil and: [self columnSpecs notEmpty]) ifTrue: [		
		self fillViewportFor: nil animated: false].

	"Restore selection if any."
	selectedIndices
		ifEmpty: [self updateViewportLayout]
		ifNotEmpty: [ | parentIndices |
			parentIndices := ((selectedIndices collect: [:i | i parent]) reject: [:i | i isNil]) asSet.

			self flag: #expensive. "mt: Improve performance here?"
			[
				parentIndices
					ifEmpty: [self updateViewportLayout]
					ifNotEmpty: [self expandIndices: parentIndices].
			
				self selection selectedIndices: selectedIndices.
				self selection currentIndex ifNil: [self selection currentIndex: currentIndex].
			] on: Error do: []].! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 4/27/2012 11:12:24.555'!
resetChildrenFor: aParentModelIndex

	| row |
	row := self viewport rowFor: aParentModelIndex.
	row isExpanded ifFalse: [^ self].
		
	self viewport beginChangeLayout.
	
	self viewport removeChildrenForRow: row.
	
	self
		fillViewportFor: aParentModelIndex
		animated: self isAnimated.
	
	self viewport endChangeLayout.! !

!UiAbstractItemView methodsFor: 'model - initialization' stamp: 'mt 8/21/2012 11:00:15.819'!
resetHeaderItems
	
	"Vertical Header - just a default implementation"
	"Layout/Height is wrong and increases height of top-level rows somehow... See UiItemRowInfo>>#optimalSize."
	"(1 to: self model childCount) do: [:row | | item |
		item := UiViewHeaderItem new orientation: #vertical.
		item addMorph: row asString asText asMorph lock.
		self viewport setHeaderItem: item atRow: row.
		
		self connect: item signal: #resized: toSelector: #setHeightForRow:to: pattern: {#=. row. 1}.
		self connect: item signal: #resizedToShrinkWrap toSelector: #setOptimalHeightForRow: pattern: {#=. row}.]."
	
	"Horizontal Header"
	self columnSpecs withIndexDo: [:spec :id | | item |
		item := UiViewHeaderItem new orientation: #horizontal.
		item addMorph: spec headerItem.
		self viewport setHeaderItem: item atColumn: id.
		
		self connect: item signal: #resized: toSelector: #setWidthForColumn:to: pattern: {#=. id. 1}.
		self connect: item signal: #resizedToShrinkWrap toSelector: #setOptimalWidthForColumn: pattern: {#=. id}].! !

!UiAbstractItemView methodsFor: 'headers - geometry' stamp: 'mt 3/25/2011 10:28:43.826'!
resizeHorizontalHeaderScroller

	hHeaderScroller
		height: horizontalHeader height.! !

!UiAbstractItemView methodsFor: 'headers - geometry' stamp: 'mt 3/25/2011 10:29:05.826'!
resizeVerticalHeaderScroller

	vHeaderScroller
		width: verticalHeader width.! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 2/2/2012 10:38:23.483'!
returnPressed: aNode

	self emit.! !

!UiAbstractItemView methodsFor: 'as yet unclassified' stamp: 'mt 8/16/2013 16:05:57.53'!
rootIsDecorated

	^ true! !

!UiAbstractItemView methodsFor: 'as yet unclassified' stamp: 'mt 8/16/2013 16:04:58.507'!
rootIsDecorated: aBoolean

	"Not functional."! !

!UiAbstractItemView methodsFor: 'model - to index' stamp: 'mt 7/6/2012 11:30:54.412'!
rowIndexFor: aNode
	
	self viewport rowsWithIndexDo: [:row :rowIndex |
		"Model index in a row can be nil for group separators."
		row firstModelIndex ifNotNil: [:index |
			index node = aNode ifTrue: [^ rowIndex]]].
	
	^ nil! !

!UiAbstractItemView methodsFor: 'scrolling' stamp: 'mt 12/18/2012 10:02:20.175'!
scrollToShowCurrentNode

	self scrollToShow: self selection currentBounds.! !

!UiAbstractItemView methodsFor: 'scrolling' stamp: 'mt 12/18/2012 10:06:43.641'!
scrollToShowSelection

	self scrollToShow: self selection selectionBounds.! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 8/29/2012 10:02:56.877'!
selectAll

	self viewport selectAll.! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 2/8/2013 10:57:27.078'!
selectBlock

	^ self viewport selectBlock! !

!UiAbstractItemView methodsFor: 'accessing' stamp: 'mt 2/8/2013 10:58:05.628'!
selectBlock: aBlock
	"Filter nodes resp. control node visibility, e.g., [:node | node text size > 5]."

	self viewport selectBlock: aBlock.! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 8/8/2012 15:58:34.853'!
selectNextColumn

	self viewport selectNextColumn.! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 8/8/2012 15:58:28.412'!
selectNextRow

	self viewport selectNextRow.! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 8/29/2012 10:02:51.887'!
selectNone

	self viewport selectNone.! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 8/8/2012 15:58:44.321'!
selectPreviousColumn

	self viewport selectPreviousColumn.! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 8/8/2012 15:58:50.532'!
selectPreviousRow

	self viewport selectPreviousRow.! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 6/26/2012 14:04:29.478'!
selectedNodes

	^ (self selection selectedIndices collect: [:index |
		index node]) asSet asOrderedCollection! !

!UiAbstractItemView methodsFor: 'selection - manual' stamp: 'mt 7/5/2013 11:15:35.409'!
selectedNodes: someNodes

	self selectedNodes = someNodes
		ifTrue: [^ self].

	^ self selection selectedIndices: (
		someNodes
			inject: OrderedCollection new
			into: [:indices :node | indices, (self modelIndicesFor: node)])! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 4/6/2011 10:20:24.432'!
selection
	
	^ self viewport selection! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 2/20/2011 19:20:28.508'!
selectionBehavior
	
	^ self viewport selectionBehavior! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 10/19/2010 10:21:08.2'!
selectionBehavior: aSymbol
	"#selectCells, #selectRows, #selectColumns"
	
	self viewport selectionBehavior: aSymbol.! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 2/2/2012 10:38:28.683'!
selectionChanged: someNodes

	self emit.! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 2/20/2011 19:21:34.624'!
selectionColor

	^ self viewport selectionColor! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 10/19/2010 10:24:52.272'!
selectionColor: aColor

	self viewport selectionColor: aColor.! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 2/20/2011 19:23:16.66'!
selectionMode

	^ self viewport selectionMode! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 10/19/2010 10:26:27.009'!
selectionMode: aSelectionClass
	"Use can use all subclasses of UiViewSelection here."

	self viewport selectionMode: aSelectionClass.! !

!UiAbstractItemView methodsFor: 'rows/columns - alignment' stamp: 'mt 8/19/2013 14:46:36.919'!
setHAlignmentForColumn: anIndex to: aSymbol
	"#left, #center, #right"
	
	self flag: #deprecated. "mt: Use column spec instead."
	(self viewport columnAt: anIndex)
		hAlignment: aSymbol.
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - alignment' stamp: 'mt 8/19/2013 14:46:40.336'!
setHAlignmentTo: aSymbol
	"#left, #center, #right"

	self flag: #deprecated. "mt: Use column spec instead."	
	self viewport columns do: [:columnInfo |
		columnInfo hAlignment: aSymbol].
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 3/25/2011 09:39:45.526'!
setHeightForRow: anIndex to: anInteger

	| row |
	row := self viewport rowAt: anIndex.
	row resizeMode ~= #rigid
		ifTrue: [^ self].
		
	row height: anInteger.
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 1/18/2012 09:40:43.874'!
setOptimalHeightForRow: anIndex
	"Resize a row to its contents."

	| row |
	row := self viewport rowAt: anIndex.
	row resizeMode ~= #rigid
		ifTrue: [^ self].
		
	row height: row optimalSize.
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 1/10/2012 17:24:11.118'!
setOptimalWidthForColumn: anIndex

	| col |
	col := self viewport columnAt: anIndex.
	col resizeMode ~= #rigid
		ifTrue: [^ self].
		
	col width: col optimalSize.
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 9/13/2012 16:34:59.894'!
setResizeModeForColumn: anIndex to: aSymbol

	(self viewport columnAt: anIndex)
		resizeMode: aSymbol.
		
	aSymbol == #spaceFill ifTrue: [
		self
			widgetResizable: true;
			horizontalScrollBarPolicy: #alwaysOff].
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 3/25/2011 09:39:56.474'!
setResizeModeForRow: anIndex to: aSymbol

	(self viewport rowAt: anIndex)
		resizeMode: aSymbol.
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - alignment' stamp: 'mt 8/19/2013 14:46:46.711'!
setVAlignmentForColumn: anIndex to: aSymbol
	"#top, #center, #bottom"

	self flag: #deprecated. "mt: Use column spec instead."	
	(self viewport columnAt: anIndex)
		vAlignment: aSymbol.
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - alignment' stamp: 'mt 8/19/2013 14:47:06.55'!
setVAlignmentTo: aSymbol
	"#top, #center, #bottom"

	self flag: #deprecated. "mt: Use column spec instead."		
	self viewport columns do: [:columnInfo |
		columnInfo vAlignment: aSymbol].
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'rows/columns - resizing' stamp: 'mt 3/25/2011 09:40:07.794'!
setWidthForColumn: anIndex to: anInteger

	| col |
	col := self viewport columnAt: anIndex.
	col resizeMode ~= #rigid
		ifTrue: [^ self].
		
	col width: anInteger.
		
	self updateViewportLayout.! !

!UiAbstractItemView methodsFor: 'model - callbacks' stamp: 'mt 2/16/2012 11:47:44.377'!
showMenuAt: aPoint on: aModelIndex

	aModelIndex menu ifNotNil: [:menu |
		"Overwrite target of menu items. Really good?"
		menu items do: [:item | self overrideMenuTarget: item].
		menu popUpNoKeyboard].! !

!UiAbstractItemView methodsFor: 'layouting' stamp: 'mt 9/5/2012 16:17:13.032'!
updateLayout
	
	super updateLayout.

	hHeaderScroller
		extent: self transformMorph width @ horizontalHeader height;
		topLeft: self transformMorph left @ self top.
		
	vHeaderScroller
		extent: verticalHeader width @ self transformMorph height;
		topLeft: self left @ self transformMorph top.

	self verticalScrollBar
		height: self transformMorph bottom - self top.
		
	self horizontalScrollBar
		width: self transformMorph right - self left.
		
	self updateViewportExtent.! !

!UiAbstractItemView methodsFor: 'layouting' stamp: 'mt 9/5/2012 16:42:09.742'!
updateViewportExtent
	"Make the viewport at least fill the transform morph. This is better for drag/drop operations."

	self widgetResizable ifTrue: [
		self horizontalScrollBarPolicy = #alwaysOff
			ifTrue: [self viewport height: (self transformMorph height / self transformMorph scale
				max: (self viewport rowCountVisible == 0
					ifTrue: [0]
					ifFalse: [self viewport submorphBounds height]))]
			ifFalse: [self verticalScrollBarPolicy = #alwaysOff
				ifTrue: [self viewport width: (self transformMorph width / self transformMorph scale
					max: (self viewport rowCountVisible == 0
						ifTrue: [0]
						ifFalse: [self viewport submorphBounds width]))]]].! !

!UiAbstractItemView methodsFor: 'layouting' stamp: 'mt 3/25/2011 09:34:25.29'!
updateViewportLayout

	self viewport ifNotNil: [:v | v updateLayout].! !

!UiAbstractItemView methodsFor: 'rows/columns - alignment' stamp: 'mt 8/19/2013 14:47:09.053'!
verticalAlignmentForColumn: anIndex

	self flag: #deprecated. "mt: Use column spec instead."		
	^ (self viewport columnAt: anIndex)
		vAlignment! !

!UiAbstractItemView methodsFor: 'headers - add/remove' stamp: 'mt 10/28/2010 10:06:51.888'!
verticalHeaderVisible

	^ self hasVerticalHeader! !

!UiAbstractItemView methodsFor: 'headers - add/remove' stamp: 'mt 10/28/2010 10:06:21.13'!
verticalHeaderVisible: aBoolean

	aBoolean
		ifTrue: [self addVerticalHeader]
		ifFalse: [self removeVerticalHeader].! !

!UiAbstractItemView methodsFor: 'accessing - private' stamp: 'mt 3/7/2011 11:50:41.566'!
viewport
	"Return the morph that is scrolled."
	
	^ self widget! !

!UiAbstractItemView methodsFor: 'geometry' stamp: 'mt 3/25/2011 10:17:15.12'!
viewportBounds

	| rect |
	rect := super viewportBounds.
	
	self hasVerticalHeader ifTrue: [
		rect := rect insetBy: (verticalHeader width @ 0 corner: 0@0)].

	self hasHorizontalHeader ifTrue: [
		rect := rect insetBy: (0 @ horizontalHeader height corner: 0@0)].

	^ rect! !

!UiAbstractItemView methodsFor: 'accessing - private' stamp: 'mt 10/14/2010 12:08:54.27'!
viewportClass

	self subclassResponsibility.! !

!UiAbstractItemView methodsFor: 'accessing - indices' stamp: 'mt 2/16/2012 17:00:17.665'!
visibleIndicesIn: aColumnNumber

	self flag: #badStyle.
	^ (self viewport columns at: aColumnNumber) indices! !

!UiAbstractItemView methodsFor: 'accessing - indices' stamp: 'mt 2/16/2012 17:00:24.076'!
visibleItemsIn: aColumnNumber

	self flag: #badStyle.
	^ (self viewport columns at: aColumnNumber) items! !

!UiAbstractItemView methodsFor: 'model - accessing' stamp: 'mt 2/8/2013 11:21:08.775'!
visibleNodes
	"Return a list of all nodes that are currently visible in the view. Current sort order retained."

	^ self viewport nodesVisible! !

UiAbstractItemView subclass: #UiListView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiListView commentStamp: 'mt 10/20/2010 14:02:59.798' prior: 0!
I am a list view that is able to break the line of items in one direction and shows as many items as possible.

I map some signals to ones that only indicate row indices, e.g., selected row indices instead of whole model indices.!

!UiListView class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:31:39.613'!
isWidgetClass

	^ true! !

!UiListView class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:14:02.112'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiListBoxIcon! !

!UiListView class methodsFor: 'accessing' stamp: 'mt 10/19/2010 14:06:46.933'!
widgetClassName

	^ 'List View'! !

!UiListView class methodsFor: 'accessing' stamp: 'mt 12/14/2012 11:19:34.097'!
widgetProperties

	^ super widgetProperties, {{'List View'. #yourself} -> {
		#listOrientation -> #(default horizontal vertical).
		#hAlignment -> #(left center right).
		#vAlignment -> #(top center bottom)}}! !

!UiListView methodsFor: 'as yet unclassified' stamp: 'mt 8/16/2013 16:24:10.894'!
addColumn: aColumnSpec
	
	self listSpec: aColumnSpec.! !

!UiListView methodsFor: 'as yet unclassified' stamp: 'mt 8/16/2013 16:23:17.53'!
addColumns: someColumnSpecs

	self listSpec: someColumnSpecs first.! !

!UiListView methodsFor: 'selection - manual' stamp: 'mt 8/7/2013 13:51:19.141'!
currentRow

	| numSeparators |
	self flag: #groupsMissing. "Needs groups support."
	numSeparators := 0.
	self viewport rows withIndexDo: [:rowInfo :rowIndex |
		rowInfo isSeparator ifTrue: [numSeparators := numSeparators  + 1].
		rowInfo firstModelIndex = self selection currentIndex ifTrue: [
			^ rowIndex - numSeparators]].! !

!UiListView methodsFor: 'selection - manual' stamp: 'mt 6/28/2012 17:11:16.56'!
currentRow: aRowIndex

	| numSeparators |
	self flag: #groupsMissing. "Needs groups support."
	numSeparators := 0.
	self viewport rows withIndexDo: [:rowInfo :rowIndex |
		rowInfo isSeparator ifTrue: [numSeparators := numSeparators  + 1].
		rowIndex - numSeparators = aRowIndex ifTrue: [
			self selection currentIndex: rowInfo firstModelIndex.
			^ self]].! !

!UiListView methodsFor: 'signals' stamp: 'mt 10/20/2010 13:53:08.525'!
currentRowChanged: newRowIndex

	self emit.! !

!UiListView methodsFor: 'model - initialization' stamp: 'mt 5/18/2012 11:32:04.866'!
fillViewportFor: aModelIndex animated: aBoolean
	"Optimized for performance."
	
	self model ifNil: [^ self].
	
	^ super fillViewportFor: aModelIndex animated: aBoolean! !

!UiListView methodsFor: 'accessing' stamp: 'mt 8/19/2013 14:48:11.796'!
hAlignment

	self flag: #deprecated. "mt: Use column spec instead."	
	self viewport ensureColumnCount: 1.
	^  self horizontalAlignmentForColumn: 1! !

!UiListView methodsFor: 'accessing' stamp: 'mt 8/19/2013 14:48:13.633'!
hAlignment: aSymbol

	self flag: #deprecated. "mt: Use column spec instead."	
	self viewport ensureColumnCount: 1.
	self setHAlignmentForColumn: 1 to: aSymbol.! !

!UiListView methodsFor: 'initialization' stamp: 'mt 7/21/2012 12:37:09.495'!
initialize

	super initialize.	
		
	self
		widgetResizable: true;
		listOrientation: #default;
		groupSpec: UiListViewGroupSpec new.! !

!UiListView methodsFor: 'accessing' stamp: 'mt 10/14/2010 15:27:12.646'!
listOrientation

	^ self viewport orientation! !

!UiListView methodsFor: 'accessing' stamp: 'mt 5/22/2013 17:19:47.891'!
listOrientation: aSymbol
	"#default ... no breaks, vertical scrolling if needed, full row selection
	#vertical ... items go from left to right and break at width -> vertical scrolling
	#horizontal ... items go from top to bottom and break at height -> horizontal scrolling
	#horizontalSingleRow ... like #horizontal but with only one row"

"	self listOrientation = aSymbol
		ifTrue: [^ self].
"	
	self viewport orientation: aSymbol.
	
	aSymbol = #default
		ifTrue: [self
			horizontalScrollBarPolicy: #alwaysOff;
			verticalScrollBarPolicy: #asNeeded].
		
	aSymbol = #vertical
		ifTrue: [self
			horizontalScrollBarPolicy: #alwaysOff;
			verticalScrollBarPolicy: #asNeeded].
		
	aSymbol = #horizontal
		ifTrue: [self
			horizontalScrollBarPolicy: #asNeeded;
			verticalScrollBarPolicy: #alwaysOff].
		
	self updateViewportLayout.! !

!UiListView methodsFor: 'rows/columns - specs' stamp: 'mt 1/23/2012 15:13:28.575'!
listSpec

	^ self columnSpecs
		ifEmpty: []
		ifNotEmpty: [self columnSpecs first]! !

!UiListView methodsFor: 'rows/columns - specs' stamp: 'mt 2/6/2013 17:00:58.158'!
listSpec: aColumnSpec

	columnSpecs := {aColumnSpec}.
	self reset.
	
	self flag: #tricky. "mt: Is this really the correct place to ensure the resize mode? Maybe we should preserve column-specific configurations AFTER a calling #reset... see UiColumnInfo class."
	self listOrientation == #default
		ifTrue: [self setResizeModeForColumn: 1 to: #spaceFill]  
		ifFalse: [self setResizeModeForColumn: 1 to: #shrinkWrap].! !

!UiListView methodsFor: 'signals -unboxing' stamp: 'mt 8/7/2013 13:52:10.756'!
onCurrentIndexChanged: aModelIndex
	"Map to the row index. Ignore the column in a list."
	
	super onCurrentIndexChanged: aModelIndex.
	self currentRowChanged: self currentRow.! !

!UiListView methodsFor: 'signals -unboxing' stamp: 'mt 1/17/2012 13:55:41.348'!
onSelectionChanged: someModelIndices

	super onSelectionChanged: someModelIndices.
	self selectedRowsChanged: (someModelIndices collect: [:i | i row]).! !

!UiListView methodsFor: 'geometry' stamp: 'mt 9/5/2012 13:42:59.347'!
optimalHeight

	self listOrientation == #default  ifTrue: [
		^ self viewport optimalSize y].
		
	self listOrientation == #vertical ifTrue: [
		^ self viewport optimalSize y].

	self listOrientation == #horizontal ifTrue: [
		^ self viewport optimalSize y + self scrollBarThickness +
			(self hasHorizontalHeader ifTrue: [horizontalHeader width] ifFalse: [0])].! !

!UiListView methodsFor: 'geometry' stamp: 'mt 9/5/2012 13:42:21.574'!
optimalWidth
	"Change the size of the view that only one kind/direction of scrolling is needed."

	self listOrientation == #default ifTrue: [
		^ self viewport optimalSize x + self scrollBarThickness + 5 + 
			(self hasVerticalHeader ifTrue: [verticalHeader width] ifFalse: [0])].
		
	self listOrientation == #vertical ifTrue: [
		^ self viewport optimalSize x + self scrollBarThickness + 5 +
			(self hasVerticalHeader ifTrue: [verticalHeader width] ifFalse: [0])].

	self listOrientation == #horizontal ifTrue: [
		^ self viewport optimalSize x].! !

!UiListView methodsFor: 'signals' stamp: 'mt 10/20/2010 14:00:30.024'!
selectedRowsChanged: someRowIndices

	self emit.! !

!UiListView methodsFor: 'geometry' stamp: 'mt 12/17/2012 15:59:04.571'!
useOptimalExtent
	"Change the size of the view that only one kind/direction of scrolling is needed."

	self
		useOptimalWidth;
		useOptimalHeight.! !

!UiListView methodsFor: 'geometry' stamp: 'mt 9/5/2012 13:43:08.932'!
useOptimalHeight
	"Change the size of the view that only one kind/direction of scrolling is needed."

	self height: self optimalHeight.! !

!UiListView methodsFor: 'geometry' stamp: 'mt 9/5/2012 10:12:28.946'!
useOptimalSize

	self flag: #deprecated.
	^ self useOptimalExtent.! !

!UiListView methodsFor: 'geometry' stamp: 'mt 9/5/2012 13:42:30.585'!
useOptimalWidth
	"Change the size of the view that only one kind/direction of scrolling is needed."

	self width: self optimalWidth.! !

!UiListView methodsFor: 'accessing' stamp: 'mt 8/19/2013 14:48:15.522'!
vAlignment

	self flag: #deprecated. "mt: Use column spec instead."	
	self viewport ensureColumnCount: 1.
	^ self verticalAlignmentForColumn: 1! !

!UiListView methodsFor: 'accessing' stamp: 'mt 8/19/2013 14:48:19.194'!
vAlignment: aSymbol

	self flag: #deprecated. "mt: Use column spec instead."	
	self viewport ensureColumnCount: 1.
	self setVAlignmentForColumn: 1 to: aSymbol.! !

!UiListView methodsFor: 'accessing - private' stamp: 'mt 11/2/2010 13:26:33.658'!
viewportClass

	^ UiListViewport! !

UiListView subclass: #UiListWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiListWidget commentStamp: 'mt 10/19/2010 19:32:52.216' prior: 0!
Simple list. Item based. No model required.!

!UiListWidget class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:33:00.509'!
widgetCategory
	
	^ 'Item Widgets (Item-Based)'! !

!UiListWidget class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:31.936'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiListBoxBlueIcon! !

!UiListWidget class methodsFor: 'accessing' stamp: 'mt 6/30/2010 21:21:46.085'!
widgetClassName

	^ 'List Widget'! !

!UiListWidget class methodsFor: 'properties' stamp: 'mt 4/6/2011 14:08:09.843'!
widgetPropertiesToIgnore

	^ super widgetPropertiesToIgnore, #(modelClass horizontalHeaderVisible verticalHeaderVisible selectionBehavior)! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:23:01.277'!
addIcon: anIcon text: aLabel

	self model
		addChild: (self createItem text: aLabel; icon: anIcon; yourself)
		inGroup: #items.
	
! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:23:06.158'!
addIcon: anIcon text: aLabel at: anIndex

	self model
		addChild: (self createItem new text: aLabel; icon: anIcon; yourself)
		beforeIndex: anIndex
		inGroup: #items.! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:11:43.264'!
addItem: aListWidgetItem

	self model addChild: aListWidgetItem inGroup: #items.! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:11:57.115'!
addItemList: someListWidgetItems

	self model addAllChildren: someListWidgetItems inGroup: #items.! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:25:49.376'!
addItemListFrom: someObjects using: aBlock
	"Constructs list widget items for each given objects and gives the chance to configure those items using the binary block. The string representation of an object will be used by default.
	
	self
		addItemListFrom: PackageOrganizer default packageNames
		using: [:pName :item | | mc |
			mc := MCPackage named: pName.
			mc hasWorkingCopy ifTrue: [item text: mc workingCopy description]]"
	
	self model addAllChildren: (someObjects collect: [:obj | | item |
		item := self createItem text: obj asString; yourself.
		aBlock value: obj value: item.
		item]) inGroup: #items.! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:13:07.114'!
addText: aLabel

	self addIcon: nil text: aLabel.! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:13:25.007'!
addText: aLabel at: anIndex

	self addIcon: nil text: aLabel at: anIndex.! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:14:37.47'!
addTextList: someTexts

	self
		addItemListFrom: someTexts
		using: [:text :item | item text: text].! !

!UiListWidget methodsFor: 'collection api' stamp: 'mt 6/30/2011 10:42:42.096'!
allSatisfy: aBlock

	self do: [:index | (aBlock value: index) ifFalse: [^ false]].
	^ true! !

!UiListWidget methodsFor: 'collection api' stamp: 'mt 6/30/2011 10:42:28.725'!
anySatisfy: aBlock

	self do: [:index | (aBlock value: index) ifTrue: [^ true]].
	^ false! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 1/27/2013 19:19:32.304'!
clear

	self model: UiPropertyItemNode new.
	self model addGroup: #items.
	self transformMorph offset: 0@0.! !

!UiListWidget methodsFor: 'collection api' stamp: 'mt 6/30/2011 10:42:12.144'!
count: aBlock

	| sum |
	sum := 0.
	self do: [:each | (aBlock value: each) ifTrue: [sum := sum + 1]].
	^ sum! !

!UiListWidget methodsFor: 'initialization' stamp: 'mt 1/27/2013 19:20:36.06'!
createItem

	| item |
	item := self itemClass new
		balloonText: nil;
		enabled: true;
		icon: nil;
		menu: nil;
		text: nil;
		yourself.
	self connect: item signal: #deleted to: self model selector: #removeChild:inGroup: pattern: #(0 =items).
	^ item! !

!UiListWidget methodsFor: 'selection - convenience' stamp: 'mt 10/20/2010 14:15:08.334'!
currentIcon

	^ self currentItem ifNotNil: [:i | i icon]! !

!UiListWidget methodsFor: 'selection' stamp: 'mt 10/20/2010 10:30:03.07'!
currentItem

	^ self currentRow ifNotNil: [:cc | self items at: cc]! !

!UiListWidget methodsFor: 'selection' stamp: 'mt 3/5/2013 10:50:48.136'!
currentItem: aListWidgetItem

	self currentRow: (self items indexOf: aListWidgetItem).! !

!UiListWidget methodsFor: 'selection - convenience' stamp: 'mt 10/20/2010 14:14:43.107'!
currentText

	^ self currentItem ifNotNil: [:i | i text]! !

!UiListWidget methodsFor: 'signals' stamp: 'mt 6/29/2010 09:53:38.414'!
currentTextChanged: aLabel

	self emit.! !

!UiListWidget methodsFor: 'collection api' stamp: 'mt 6/30/2011 10:23:27.492'!
detect: aBlock

	self do: [:index | (aBlock value: index) ifTrue: [^ index]].
	^ nil! !

!UiListWidget methodsFor: 'collection api' stamp: 'mt 1/27/2013 19:27:23.177'!
do: aBlock

	self items do: [:item | aBlock value: item].! !

!UiListWidget methodsFor: 'initialization' stamp: 'mt 2/2/2012 16:34:18.047'!
initialize
	
	super initialize.
	self
		listSpec: (UiItemViewColumnSpec newFrom: {
			#text -> #text.
			#icon -> #icon.
			#balloonText -> #balloonText.
			#menu -> #menu});
		clear.! !

!UiListWidget methodsFor: 'testing' stamp: 'mt 2/2/2012 16:30:33.535'!
isEmpty

	^ self items isEmpty! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 1/27/2013 19:00:45.93'!
itemClass

	^ UiPropertyItemNode! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 1/27/2013 19:02:02.506'!
items

	^ self model children at: #items! !

!UiListWidget methodsFor: 'model - accessing' stamp: 'mt 1/27/2013 19:53:48.368'!
modelClass: aClass

	"Ignore."! !

!UiListWidget methodsFor: 'signals - unboxing' stamp: 'mt 4/6/2011 13:28:49.453'!
onCurrentIndexChanged: aModelIndex

	super onCurrentIndexChanged: aModelIndex.
	self currentTextChanged: self currentText.! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:26:07.545'!
removeLastRow

	self model removeLastInGroup: #items.! !

!UiListWidget methodsFor: 'add/remove' stamp: 'mt 1/27/2013 19:26:33.256'!
removeRowAt: anIndex

	self model removeChildAt: anIndex inGroup: #items.! !

!UiListWidget methodsFor: 'collection api' stamp: 'mt 6/30/2011 10:23:03.193'!
select: aBlock

	| newCollection |
	newCollection := OrderedCollection new.
	self do: [:each | (aBlock value: each)
		ifTrue: [newCollection add: each]].
	^ newCollection! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:11:43.237'!
size

	^ self items size! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 4/6/2011 13:53:27.838'!
textList

	^ self items collect: [:item | item text]! !

UiAbstractItemView subclass: #UiTableView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTableView commentStamp: 'mt 10/19/2010 10:51:11.46' prior: 0!
I have a vertical header by default and are used for table-like models.!

!UiTableView class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:31:43.968'!
isWidgetClass

	^ true! !

!UiTableView class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:14:50.365'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiScrollPaneTableIcon! !

!UiTableView class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:32:09.963'!
widgetClassName

	^ 'Table View'! !

!UiTableView methodsFor: 'initialization' stamp: 'mt 6/28/2012 16:53:09.266'!
initialize

	super initialize.
		
	self groupSpec: UiTableViewGroupSpec new.
		
	self
		addMorph: vHeaderScroller;
		addMorph: hHeaderScroller;
		updateLayout.! !

!UiTableView methodsFor: 'accessing' stamp: 'mt 8/8/2012 10:36:20.461'!
showGrid

	^ self viewport showGrid! !

!UiTableView methodsFor: 'accessing' stamp: 'mt 8/8/2012 10:36:26.335'!
showGrid: aBoolean

	self viewport showGrid: aBoolean.! !

!UiTableView methodsFor: 'accessing - private' stamp: 'mt 10/14/2010 13:57:19.079'!
viewportClass

	^ UiTableViewport! !

UiAbstractItemView subclass: #UiTreeView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTreeView commentStamp: 'mt 10/19/2010 10:51:34.42' prior: 0!
I am a view that supports tree-like models and a tree in the first column.!

!UiTreeView class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:31:46.249'!
isWidgetClass

	^ true! !

!UiTreeView class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:15.068'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiScrollPaneTreeIcon! !

!UiTreeView class methodsFor: 'accessing' stamp: 'mt 10/19/2010 13:32:03.461'!
widgetClassName

	^ 'Tree View'! !

!UiTreeView methodsFor: 'expand/collapse - model index' stamp: 'mt 7/10/2012 12:12:33.224'!
collapseIndex: aModelIndex

	| row |
	(aModelIndex isNil or: [aModelIndex hasChildren not])
		ifTrue: [^ self].
	
	row := self viewport rowFor: aModelIndex.
	row isExpanded ifFalse: [^ self].
		
	row items first setProperty: #isExpanded toValue: false.
	
	self viewport beginChangeLayout.
	self viewport removeChildrenForRow: row.
	self viewport endChangeLayout.! !

!UiTreeView methodsFor: 'expand/collapse' stamp: 'mt 2/3/2012 14:54:20.427'!
collapseNode: aNode

	self collapseIndex: (self modelIndicesFor: aNode) first.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 8/16/2013 11:36:54.722'!
disconnectDefaultSignals

	self disconnectSignal: #doubleClicked: from: self selector: #toggleNode:.! !

!UiTreeView methodsFor: 'expand/collapse - model index' stamp: 'mt 7/16/2013 09:40:28.774'!
expandAllIndices: aModelIndex
	"Expand the given model index and all sub-indices."
	
	| nextIndices |
	self viewport beginChangeLayout.
	
	[
	self flag: #decide. "mt: Which approach to use? Allow for user interruption?"
	"breadth-first"
	self expandAllIndicesSilentlyBreadthFirst: {aModelIndex}.
	"depth-first"
	"self expandAllIndicesSilently: {aModelIndex}." 
	
	] ensure: [self viewport endChangeLayout].! !

!UiTreeView methodsFor: 'expand/collapse - private' stamp: 'mt 7/15/2013 17:46:53.119'!
expandAllIndicesSilently: someModelIndices

	someModelIndices
		select: [:index | index hasChildren]
		thenDo: [:index |
		(self expandIndexSilently: index) in: [:newRows |
			self expandAllIndicesSilently: (newRows gather: [:group |
				group value collect: [:row | row first "column" key "indexToItem"]])]].! !

!UiTreeView methodsFor: 'expand/collapse - private' stamp: 'mt 7/17/2013 17:19:14.144'!
expandAllIndicesSilentlyBreadthFirst: someModelIndices

	| nextIndices |
	nextIndices := someModelIndices asOrderedCollection.
	[nextIndices notEmpty] whileTrue: [
		| nextIndex |
		nextIndex := nextIndices removeFirst.
		nextIndex hasChildren ifTrue: [
			(self expandIndexSilently: nextIndex) in: [:newRows |
				nextIndices addAll: (newRows gather: [:group |
					group value collect: [:row | row first "column" key "indexToItem"]])].
			"nextIndices size > 1000 ifTrue: [
				nextIndices := #().
				Transcript cr; show: 'Too many nodes (> 1000)!! Operation cancelled.']"]].! !

!UiTreeView methodsFor: 'expand/collapse - convenience' stamp: 'mt 7/18/2013 15:43:04.912'!
expandAllNodes

	self viewport beginChangeLayout.
	
	[self expandAllIndicesSilentlyBreadthFirst: (self visibleIndicesIn: 1)]
		ensure: [self viewport endChangeLayout].! !

!UiTreeView methodsFor: 'expand/collapse - convenience' stamp: 'mt 8/2/2013 07:36:33.155'!
expandAllTopLevelNodes

	self viewport beginChangeLayout.

	self model groups do: [:group |
		(1 to: (self model childCountInGroup: group))
			collect: [:row | (self modelIndicesFor: (self model childAt: row inGroup: group)) first]
			thenDo: [:index | index hasChildren ifTrue: [self expandIndexSilently: index]]].
		
	self viewport endChangeLayout.! !

!UiTreeView methodsFor: 'expand/collapse - model index' stamp: 'mt 7/10/2012 13:26:42.002'!
expandIndex: aModelIndex
	"Any given index can be expanded. Collapsed parents will be expanded automatically."
	
	self flag: #asynch. "mt: Should use cached values from morph in item view?"
	(aModelIndex isNil or: [aModelIndex hasChildren not]) ifTrue: [^ self].
	
	self viewport beginChangeLayout.
	
	self
		expandIndexSilently: aModelIndex
		checkParents: true.
	
	self viewport endChangeLayout.! !

!UiTreeView methodsFor: 'expand/collapse - private' stamp: 'mt 7/10/2012 11:52:14.78'!
expandIndexSilently: aModelIndex

	^ self
		expandIndexSilently: aModelIndex
		checkParents: false! !

!UiTreeView methodsFor: 'expand/collapse - private' stamp: 'mt 7/15/2013 17:42:40.601'!
expandIndexSilently: aModelIndex checkParents: aBoolean

	| row |
	aModelIndex ifNil: [^ #()].
	
	aBoolean ifTrue: [
		"Ensure that all parent indices are expanded too."
		aModelIndex parent ifNotNil: [:parent |
			self expandIndexSilently: parent checkParents: true]].
	
	row := self viewport rowFor: aModelIndex.
	row isExpanded ifTrue: [ ^ #()].

	row items first setProperty: #isExpanded toValue: true.

	^ self
		fillViewportFor: row firstModelIndex
		animated: self isAnimated! !

!UiTreeView methodsFor: 'expand/collapse - model index' stamp: 'mt 8/3/2012 11:19:27.05'!
expandIndices: someModelIndices

	self flag: #asynch. "mt: Should use cached values from morph in item view?"
	(someModelIndices select: [:index | index hasChildren]) ifEmpty: [^ self].

	self viewport beginChangeLayout.

	someModelIndices do: [:index | self
			expandIndexSilently: index checkParents: true]. 

	self viewport endChangeLayout.! !

!UiTreeView methodsFor: 'expand/collapse' stamp: 'mt 2/3/2012 14:54:33.87'!
expandNode: aNode

	self expandIndex: (self modelIndicesFor: aNode) first.! !

!UiTreeView methodsFor: 'expand/collapse - convenience' stamp: 'mt 5/2/2012 13:22:02.858'!
expandTopLevelRows: someRows

	self viewport beginChangeLayout.

	someRows do: [:row |
		self expandIndexSilently: (self modelIndicesFor: (self model childAt: row inGroup: self model groups first)) first].

	self viewport endChangeLayout.! !

!UiTreeView methodsFor: 'initialization' stamp: 'mt 7/20/2012 17:29:35.511'!
initialize

	super initialize.
	
	self connect: self signal: #doubleClicked: toSelector: #toggleNode:.
	
	self connect: self viewport signal: #decorationClicked: toSelector: #toggleIndex:.
	
	self addHorizontalHeader.! !

!UiTreeView methodsFor: 'event handling' stamp: 'mt 7/20/2012 17:30:27.712'!
onKeyStroked: aKeyboardEvent on: aModelIndex
	"Default keyboard interaction."

	aKeyboardEvent keyCharacter = $*
		ifTrue: [self expandAllIndices: aModelIndex].
	aKeyboardEvent keyCharacter = Character arrowRight
		ifTrue: [self expandIndex: aModelIndex].
	aKeyboardEvent keyCharacter = Character arrowLeft
		ifTrue: [self collapseIndex: aModelIndex].
		
	super onKeyStroked: aKeyboardEvent on: aModelIndex.! !

!UiTreeView methodsFor: 'accessing' stamp: 'mt 7/5/2012 17:28:57.25'!
rootIsDecorated
	"Should toplevel-items visualize expand/collapse operations?"

	^ self viewport rootIsDecorated! !

!UiTreeView methodsFor: 'accessing' stamp: 'mt 7/5/2012 17:28:50.976'!
rootIsDecorated: aBoolean

	self viewport rootIsDecorated: aBoolean.! !

!UiTreeView methodsFor: 'expand/collapse - model index' stamp: 'mt 6/26/2012 14:08:51.055'!
toggleIndex: aModelIndex

	| row |
	aModelIndex ifNil: [^ self].
	aModelIndex hasChildren ifFalse: [^ self].
	
	row := self viewport rowFor: aModelIndex.
	
	self viewport beginChangeLayout.
	
	row isExpanded
		ifTrue: [
			row items first setProperty: #isExpanded toValue: false.
			self viewport removeChildrenForRow: row]
		ifFalse: [
			row items first setProperty: #isExpanded toValue: true.
			self
				fillViewportFor: row firstModelIndex
				animated: self isAnimated].
	
	self viewport endChangeLayout.! !

!UiTreeView methodsFor: 'expand/collapse' stamp: 'mt 2/3/2012 14:53:36.991'!
toggleNode: aNode
	"The given node has to be visible."
	
	self toggleIndex: (self modelIndicesFor: aNode) first.! !

!UiTreeView methodsFor: 'expand/collapse - convenience' stamp: 'mt 4/23/2012 10:25:43.369'!
toggleTopLevelRow: aRowIndex

	self flag: #broken.
	self toggleIndex: (self model indexRow: aRowIndex column: 1 parent: nil).! !

!UiTreeView methodsFor: 'accessing - private' stamp: 'mt 10/14/2010 13:21:16.162'!
viewportClass

	^ UiTreeViewport! !

!UiScrollArea class methodsFor: 'testing' stamp: 'mt 3/7/2011 14:11:32.14'!
isWidgetClass

	^ true! !

!UiScrollArea class methodsFor: 'accessing' stamp: 'mt 3/7/2011 14:26:02.526'!
widgetCategory

	^ 'Containers & Spacer'! !

!UiScrollArea class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:14:28.586'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiScrollPaneBothIcon! !

!UiScrollArea class methodsFor: 'accessing' stamp: 'mt 3/7/2011 14:12:38.791'!
widgetClassName

	^ 'Scroll Area'! !

!UiScrollArea class methodsFor: 'properties' stamp: 'mt 4/18/2011 17:38:27.532'!
widgetProperties

	^ super widgetProperties, {{'Scroll Area'. #yourself} -> {
		#widgetResizable -> Boolean.
		#horizontalScrollBarPolicy -> #(asNeeded alwaysOn alwaysOff).
		#verticalScrollBarPolicy -> #(asNeeded alwaysOn alwaysOff).
		#outerBorderWidth -> Integer.
		#scrollBarThickness -> Integer}}! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 4/4/2011 16:21:32.334'!
areaResizable
	"Check whether width or height of this area can be adapted to when scroll bars are #alwaysOff."
	
	^ areaResizable ifNil: [false]! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 4/4/2011 16:21:06.845'!
areaResizable: aBoolean

	aBoolean = areaResizable ifTrue: [^ false].
	
	areaResizable := aBoolean.
	
	self updateLayout.! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:42:13.512'!
cornerWidget

	^ cornerWidget! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/21/2011 09:51:58.442'!
cornerWidget: anObject

	anObject ifNil: [^ self cornerWidget: (Morph new color: Color transparent)].

	cornerWidget := anObject.

	self updateLayout.! !

!UiScrollArea methodsFor: 'drawing' stamp: 'mt 7/8/2013 17:06:14.445'!
drawBorderOn: aCanvas

	self outerBorderWidth = 0 ifTrue: [^ self].
		
	UiPlatform current
		canvas: aCanvas
		frameRoundRect: self outerBounds
		radius: 3
		width: self outerBorderWidth
		color: (Color gray: 0.3).! !

!UiScrollArea methodsFor: 'drawing' stamp: 'mt 7/1/2011 22:32:45.937'!
drawOn: aCanvas

	super drawOn: aCanvas.
	self drawBorderOn: aCanvas.! !

!UiScrollArea methodsFor: 'geometry' stamp: 'mt 4/5/2011 14:09:09.387'!
extent: aPoint

	| extentWasChanged |
	extentWasChanged := self extent ~= aPoint.
	super extent: aPoint.

	extentWasChanged ifTrue: [self updateLayout].! !

!UiScrollArea methodsFor: 'layout-properties' stamp: 'mt 3/29/2011 15:22:52.221'!
hResizing: aSymbol

	aSymbol = self hResizing
		ifTrue: [^ self].
		
	super hResizing: aSymbol.
	self updateLayout.  ! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:42:13.015'!
horizontalScrollBar

	^ horizontalScrollBar! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:42:13.1'!
horizontalScrollBar: anObject

	horizontalScrollBar := anObject.! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/7/2011 09:47:50.482'!
horizontalScrollBarPolicy

	^ horizontalScrollBarPolicy ifNil: [#alwaysOn]! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/7/2011 10:56:19.44'!
horizontalScrollBarPolicy: aSymbol
	"#asNeeded, #alwaysOn, #alwaysOff"
	
	aSymbol = self horizontalScrollBarPolicy ifTrue: [^ self].
	
	horizontalScrollBarPolicy := aSymbol.
	
	self
		updateLayout;
		updateScrollBars.! !

!UiScrollArea methodsFor: 'initialization' stamp: 'mt 12/22/2011 11:53:28.232'!
initialize

	super initialize.
	
	self
		horizontalScrollBar: (UiScrollBar new orientation: #horizontal; buttonMode: #none; color: Color white; outerBorderWidth: 0);
		verticalScrollBar: (UiScrollBar new orientation: #vertical; buttonMode: #none; color: Color white; outerBorderWidth: 0);
		transformMorph: self transformMorphClass new.
		
	cornerWidget := Morph new color: Color white.
	
	self initializeEventHandler.
	
	self connect: self transformMorph signal: #areaChanged toSelector: #updateLayout.
	self connect: self transformMorph signal: #areaChanged toSelector: #updateScrollBars.

	self connect: self transformMorph signal: #verticalOffsetChanged: to: self verticalScrollBar selector: #currentValue:.
	self connect: self transformMorph signal: #horizontalOffsetChanged: to: self horizontalScrollBar selector: #currentValue:.

	self connect: self horizontalScrollBar signal: #valueChanged: to: self transformMorph selector: #horizontalOffset:.
	self connect: self verticalScrollBar signal: #valueChanged: to: self transformMorph selector: #verticalOffset:.
	
	self
		color: Color transparent;
		addMorphBack: self transformMorph;
		extent: 160@100.! !

!UiScrollArea methodsFor: 'initialization' stamp: 'mt 4/13/2011 10:51:11.794'!
initializeEventHandler

	self eventHandler: UiEventHandler new.
	
	self eventHandler setProperty: #handlesMouseWheel toValue: [true].
	
	self connect: self eventHandler signal: #mouseWheelUp: to: self verticalScrollBar selector: #decreaseValueBySingleStep.
	self connect: self eventHandler signal: #mouseWheelDown: to: self verticalScrollBar selector: #increaseValueBySingleStep.! !

!UiScrollArea methodsFor: 'testing' stamp: 'mt 3/7/2011 14:06:34.696'!
isHorizontalScrollBarVisible

	^ self horizontalScrollBarPolicy = #alwaysOn
		or: [self horizontalScrollBarPolicy = #asNeeded
			and: [self widget isNil or: [self transformMorph width < self widget width]]]! !

!UiScrollArea methodsFor: 'testing' stamp: 'mt 3/7/2011 14:06:22.788'!
isVerticalScrollBarVisible

	^ self verticalScrollBarPolicy = #alwaysOn
		or: [self verticalScrollBarPolicy = #asNeeded
			and: [self widget isNil or: [self transformMorph height < self widget height]]]! !

!UiScrollArea methodsFor: 'scrolling - private' stamp: 'mt 7/17/2012 16:14:49.128'!
offsetToShow: aRectangle
	"Calculate the offset necessary to show the rectangle."
	
	| offset |
	offset := self transformMorph offset.

	"Vertical Scrolling"
	(aRectangle top - offset y) < 0
		ifTrue: [offset := offset x @ (
			(aRectangle top min: self widget height - self transformMorph height))].
		
	((aRectangle bottom - offset y) > self transformMorph height and: [aRectangle height <= self transformMorph height])
		ifTrue: [offset := offset x @ (
					(aRectangle top - self transformMorph height + aRectangle height min: self widget height - self transformMorph height))].
	
	"Horizontal Scrolling"
	(aRectangle left - offset x) < 0
		ifTrue: [offset := (
			(aRectangle left min: self widget width - self transformMorph width)) @ offset y].
		
	((aRectangle right - offset x) > self transformMorph width and: [aRectangle width <= self transformMorph width])
		ifTrue: [offset := (
			(aRectangle left - self transformMorph width + aRectangle width min: self widget width - self transformMorph width)) @ offset y].

	^ offset! !

!UiScrollArea methodsFor: 'geometry' stamp: 'mt 7/19/2013 10:33:44.097'!
optimalHeight

	self flag: #magicNumber. "mt: Why is scrollbar thickness not sufficient here?"
	^ self widget
		ifNil: [0]
		ifNotNil: [:w |
			w fullBounds. "update layout"
			w height + self scrollBarThickness + 5]! !

!UiScrollArea methodsFor: 'geometry' stamp: 'mt 9/5/2012 14:32:08.813'!
optimalWidth

	self flag: #magicNumber. "mt: Why is scrollbar thickness not sufficient here?"
	^ self widget
		ifNil: [0]
		ifNotNil: [:w | w width + self scrollBarThickness + 5]! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/29/2011 10:30:04.954'!
outerBorderWidth

	^ outerBorderWidth ifNil: [outerBorderWidth := 0]! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/29/2011 10:28:48.321'!
outerBorderWidth: aNumber

	outerBorderWidth := aNumber.
	self redrawLater.! !

!UiScrollArea methodsFor: 'geometry' stamp: 'mt 3/29/2011 10:29:04.634'!
outerBounds

	^ self bounds outsetBy: self outerBorderWidth! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 6/10/2011 11:38:32.374'!
scrollBarThickness

	^ scrollBarThickness ifNil: [scrollBarThickness := 9]! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 4/8/2011 10:11:25.649'!
scrollBarThickness: anInteger

	anInteger = scrollBarThickness ifTrue: [^ self].
	
	scrollBarThickness := anInteger.
	self updateLayout.! !

!UiScrollArea methodsFor: 'scrolling' stamp: 'mt 4/18/2011 17:45:18.292'!
scrollToBottom

	self widget ifNil: [^ self].
	
	self scrollToShow: (self widget bottomLeft extent: 1@1).! !

!UiScrollArea methodsFor: 'scrolling' stamp: 'mt 5/2/2011 20:54:03.921'!
scrollToShow: aRectangle
	"Scroll to include as much of aRectangle as possible, where aRectangle is in the scroller's local space."

	self transformMorph offset: (self offsetToShow: aRectangle).! !

!UiScrollArea methodsFor: 'scrolling' stamp: 'mt 9/19/2012 10:08:16.23'!
scrollToShow: aRectangle duration: milliseconds
	
	self
		scrollToShow: aRectangle
		duration: milliseconds
		thenDo: nil! !

!UiScrollArea methodsFor: 'scrolling' stamp: 'mt 9/19/2012 14:16:03.889'!
scrollToShow: aRectangle duration: milliseconds thenDo: aBlock
	"Animated scrolling."
	
	| anim newOffset |
	newOffset := self offsetToShow: aRectangle.
	
	newOffset = self transformMorph offset
		ifTrue: [^ self].
	
	"Consider caching before starting the animation."
	self flag: #refactor.
	self transformMorph drawCacheActive
		ifTrue: [self transformMorph updateDrawCache].
	
	(self valueOfProperty: #scrollAnimation ifAbsent: [])
		ifNotNil: [:an | an unregister].
	
	anim := (AnimPropertyAnimation
		on: self transformMorph
		property: #offset
		start: self transformMorph offset
		end: newOffset  
		duration: milliseconds).
	anim easingCurve conversionBlock: [:x | (-1 * (x-1 raisedTo: 6)) + 1].
	anim
		register;
		finishBlock: aBlock;
		start: #deleteWhenFinished.
		
	self setProperty: #scrollAnimation toValue: anim.! !

!UiScrollArea methodsFor: 'scrolling' stamp: 'mt 9/19/2012 09:59:14.609'!
scrollToShowGlobal: aGlobalRectangle

	self scrollToShow: 
		(self widget bounds: aGlobalRectangle from: self world).! !

!UiScrollArea methodsFor: 'scrolling' stamp: 'mt 9/19/2012 10:00:03.41'!
scrollToShowGlobal: aGlobalRectangle duration: milliseconds

	self
		scrollToShow: (self widget bounds: aGlobalRectangle from: self world)
		duration: milliseconds.! !

!UiScrollArea methodsFor: 'scrolling' stamp: 'mt 9/19/2012 10:09:00.915'!
scrollToShowGlobal: aGlobalRectangle duration: milliseconds thenDo: aBlock

	self
		scrollToShow: (self widget bounds: aGlobalRectangle from: self world)
		duration: milliseconds
		thenDo: aBlock.! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:42:13.643'!
transformMorph

	^ transformMorph! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:42:13.719'!
transformMorph: anObject

	transformMorph := anObject.! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 7/1/2011 16:39:06.282'!
transformMorphClass

	^ UiTransformMorph! !

!UiScrollArea methodsFor: 'layouting' stamp: 'mt 1/4/2012 14:03:26.089'!
updateLayout

	"viewport bounds can only be calculated right if transform morph has correct size..."
	self transformMorph disconnectSignal: #areaChanged from: self selector: #updateLayout.
	self transformMorph bounds: self viewportBounds.
	self connect: self transformMorph signal: #areaChanged toSelector: #updateLayout.

	self isVerticalScrollBarVisible
		ifFalse: [self removeMorph: self verticalScrollBar]
		ifTrue: [self addMorph: self verticalScrollBar].  
	
	self verticalScrollBar
		extent: self scrollBarThickness @ self transformMorph height;
		topLeft: self transformMorph right @ self top.

	self isHorizontalScrollBarVisible
		ifFalse: [self removeMorph: self horizontalScrollBar]
		ifTrue: [self addMorph: self horizontalScrollBar].  
		
	self horizontalScrollBar
		extent: self transformMorph width @ self scrollBarThickness;
		topLeft: self left @ self transformMorph bottom.

	self isHorizontalScrollBarVisible & self isVerticalScrollBarVisible
		ifFalse: [self removeMorph: self cornerWidget]
		ifTrue: [self addMorph: self cornerWidget].  
		
	self cornerWidget
		extent: self scrollBarThickness @ self scrollBarThickness;
		topLeft: self transformMorph bottomRight.
		
	self widget ifNil: [^ self].
	
	"Intelligent layout mechanisms."
	self widgetResizable ifTrue: [
		self horizontalScrollBarPolicy = #alwaysOff
			ifTrue: [self widget width: self transformMorph width / self transformMorph scale]
			ifFalse: [self verticalScrollBarPolicy = #alwaysOff
				ifTrue: [self widget height: self transformMorph height / self transformMorph scale]]].
	
	self areaResizable ifTrue: [
		self verticalScrollBarPolicy = #alwaysOff
		ifTrue: [
			self transformMorph verticalOffset: 0.
			self height: self widget height * self transformMorph scale + (self isHorizontalScrollBarVisible
				ifTrue: [self scrollBarThickness + 1] ifFalse: [0])]
		ifFalse: [self horizontalScrollBarPolicy = #alwaysOff
			ifTrue: [
				self transformMorph horizontalOffset: 0.
				self width: self widget width * self transformMorph scale + (self isVerticalScrollBarVisible
					ifTrue: [self scrollBarThickness + 1] ifFalse: [0])]]].! !

!UiScrollArea methodsFor: 'layouting' stamp: 'mt 4/20/2011 14:51:07.678'!
updateScrollBars

	self widget ifNil: [^ self].
	
	self horizontalScrollBar
		maximum: (self widget width * self transformMorph scale - self transformMorph width max: 0);
		pageStep: self transformMorph width;
		currentValue: self transformMorph offset x.
	
	self verticalScrollBar
		maximum: (self widget height * self transformMorph scale - self transformMorph height max: 0);
		pageStep: self transformMorph height;
		currentValue: self transformMorph offset y.! !

!UiScrollArea methodsFor: 'layout-properties' stamp: 'mt 3/29/2011 15:23:07.214'!
vResizing: aSymbol

	aSymbol = self vResizing
		ifTrue: [^ self].
		
	super vResizing: aSymbol.
	self updateLayout.  ! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:42:13.303'!
verticalScrollBar

	^ verticalScrollBar! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 2/27/2011 16:42:13.362'!
verticalScrollBar: anObject

	verticalScrollBar := anObject.! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/7/2011 09:47:58.761'!
verticalScrollBarPolicy

	^ verticalScrollBarPolicy ifNil: [#alwaysOn]! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/7/2011 10:48:43.579'!
verticalScrollBarPolicy: aSymbol
	"#asNeeded, #alwaysOn, #alwaysOff"
	
	aSymbol = self verticalScrollBarPolicy ifTrue: [^ self].
	
	verticalScrollBarPolicy := aSymbol.
	
	self
		updateLayout;
		updateScrollBars.! !

!UiScrollArea methodsFor: 'geometry' stamp: 'mt 4/5/2011 15:44:18.529'!
viewportBounds

	^ self bounds insetBy: (0@0 corner:
		(self isVerticalScrollBarVisible ifTrue: [self scrollBarThickness + 1] ifFalse: [0])
		@
		(self isHorizontalScrollBarVisible ifTrue: [self scrollBarThickness + 1] ifFalse: [0]))! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/7/2011 09:44:22.696'!
widget
	"The morph that is scrolled."
	
	^ self transformMorph submorphs
		ifEmpty: [nil]
		ifNotEmpty: [self transformMorph submorphs first]! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/24/2011 15:16:16.979'!
widget: aMorph

	self transformMorph removeAllMorphs.
	
	aMorph ifNotNil: [:m |
		self transformMorph addMorph: aMorph].! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/29/2011 11:57:37.956'!
widgetResizable
	"Check whether width or height can be adapted to when scroll bars are #alwaysOff."
	
	^ widgetResizable ifNil: [false]! !

!UiScrollArea methodsFor: 'accessing' stamp: 'mt 3/29/2011 11:48:29.577'!
widgetResizable: aBoolean

	aBoolean = widgetResizable ifTrue: [^ false].
	
	widgetResizable := aBoolean.
	
	self updateLayout.! !

UiScrollArea subclass: #UiTextEdit
	instanceVariableNames: 'model originalText scrollingMode scrollingLimit colors displayMode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiTextEdit commentStamp: 'mt 4/8/2011 10:27:09.13' prior: 0!
At the moment, you should never activate the horizontal scrollbar as the text can only be wrapped.

If you want to disable the vertical scrollbar, enable #areaResizable too.

Do not make the vertical scrollbar #asNeeded. Needs to be fixed. ;)!

UiTextEdit subclass: #UiLineEdit
	instanceVariableNames: 'helpText hideCursor deferEditSignalTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiLineEdit commentStamp: 'mt 1/10/2012 11:23:35.803' prior: 0!
This is an input field that has one line only.!

!UiLineEdit class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:29.676'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiTextFieldIcon! !

!UiLineEdit class methodsFor: 'accessing' stamp: 'mt 3/23/2011 10:49:25.347'!
widgetClassName

	^ 'Line Edit'! !

!UiLineEdit class methodsFor: 'accessing' stamp: 'mt 4/28/2011 16:28:48.843'!
widgetProperties

	^ super widgetProperties, {{'Line Edit'. #yourself} -> {
		#helpText -> String.
		#hideCursor -> Boolean}}! !

!UiLineEdit class methodsFor: 'accessing' stamp: 'mt 4/18/2011 17:38:52.221'!
widgetPropertiesToIgnore

	^ super widgetPropertiesToIgnore, #(vResizing scrollBarThickness)! !

!UiLineEdit methodsFor: 'accessing' stamp: 'mt 6/3/2011 22:50:25.43'!
deferEditSignalTime

	^ deferEditSignalTime ifNil: [deferEditSignalTime := 0]! !

!UiLineEdit methodsFor: 'accessing' stamp: 'mt 6/7/2011 14:09:01.822'!
deferEditSignalTime: milliseconds

	deferEditSignalTime := milliseconds.! !

!UiLineEdit methodsFor: 'drawing' stamp: 'mt 1/10/2012 11:29:16.346'!
drawOn: aCanvas

	UiPainter default new
		drawLineEdit: self on: aCanvas.! !

!UiLineEdit methodsFor: 'signals' stamp: 'mt 7/8/2012 14:27:39.733'!
escapePressed: discardedText
	
	self emitAlways.! !

!UiLineEdit methodsFor: 'events' stamp: 'mt 4/28/2011 16:34:31.335'!
handleMouseMove: evt

	self restoreCursor.
	^ super handleMouseMove: evt    ! !

!UiLineEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:30:16.854'!
helpText

	^ helpText ifNil: [helpText := '']! !

!UiLineEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:30:22.53'!
helpText: aString
	"Drawn whenver #theText is empty."
	
	helpText := aString.
	self redrawLater.! !

!UiLineEdit methodsFor: 'accessing' stamp: 'mt 4/28/2011 16:37:02.88'!
hideCursor
	"Hide the cursor while typing."
	
	^ hideCursor ifNil: [true]! !

!UiLineEdit methodsFor: 'accessing' stamp: 'mt 4/28/2011 15:35:29.459'!
hideCursor: aBoolean

	hideCursor := aBoolean.! !

!UiLineEdit methodsFor: 'initialization' stamp: 'mt 7/8/2012 14:27:25.622'!
initialize

	super initialize.
	
	self
		hResizing: #spaceFill;
		verticalScrollBarPolicy: #alwaysOff;
		widgetResizable: false;  
		height: 20;
		acceptOnFocusLost: true.
	
	self textMorph
		width: 999999;
		acceptOnCR: true;
		revertOnEscape: true.
		
	self connect: self textMorph signal: #returnPressed: toSelector: #returnPressed:.
	self connect: self textMorph signal: #escapePressed: toSelector: #escapePressed:.
	self connect: self signal: #textEdited: toSelector: #tryHideCursor.! !

!UiLineEdit methodsFor: 'initialization' stamp: 'mt 4/28/2011 16:36:37.338'!
initializeEventHandler

	super initializeEventHandler.
	
	self eventHandler grabKeyboardOnMouseEnter: true.
	
	self connect: self eventHandler signal: #focusLost toSelector: #restoreCursor.! !

!UiLineEdit methodsFor: 'cursor' stamp: 'mt 4/28/2011 16:32:44.772'!
restoreCursor

	ActiveHand showTemporaryCursor: nil.! !

!UiLineEdit methodsFor: 'signals' stamp: 'mt 7/8/2012 14:27:47.001'!
returnPressed: acceptedText
	
	self emitAlways.! !

!UiLineEdit methodsFor: 'signals' stamp: 'mt 6/7/2011 14:15:15.527'!
textEdited: aText

	self deferEditSignalTime > 0 ifTrue: [
		self
			removeAlarm: #triggerDeferredEditSignal;
			addAlarm: #triggerDeferredEditSignal
			after: self deferEditSignalTime.
		^ self].
	
	self emitAlways.! !

!UiLineEdit methodsFor: 'accessing - private' stamp: 'mt 7/18/2012 16:05'!
textMorphClass

	^ UiTextLineMorph! !

!UiLineEdit methodsFor: 'actions' stamp: 'mt 6/9/2011 11:31:03.779'!
triggerDeferredEditSignal

	super textEdited: self theText.! !

!UiLineEdit methodsFor: 'cursor' stamp: 'mt 7/8/2013 14:06:04.216'!
tryHideCursor

	((self hideCursor
		and: [self bounds containsPoint: (self globalPointToLocal: ActiveHand position)])
		and: [ActiveHand temporaryCursor isNil])
		ifTrue: [ActiveHand showTemporaryCursor: Cursor blank].! !

!UiTextEdit class methodsFor: 'testing' stamp: 'mt 3/23/2011 10:48:23.69'!
isWidgetClass

	^ true! !

!UiTextEdit class methodsFor: 'accessing' stamp: 'mt 3/23/2011 10:48:41.087'!
widgetCategory

	^ 'Input Widgets'! !

!UiTextEdit class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:04.146'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiTextAreaIcon! !

!UiTextEdit class methodsFor: 'accessing' stamp: 'mt 3/23/2011 10:48:53.067'!
widgetClassName

	^ 'Text Edit'! !

!UiTextEdit class methodsFor: 'accessing' stamp: 'mt 4/28/2011 15:31:23.781'!
widgetProperties

	^ super widgetProperties, {{'Text Edit'. #yourself} -> {
		#theText -> String.
		#readOnly -> Boolean.
		#autoConvert -> Boolean.
		#autoAccept -> Boolean.
		#acceptOnFocusLost -> Boolean.
		#scrollingMode -> #(none never limit always).
		#scrollingLimit -> Integer}}! !

!UiTextEdit class methodsFor: 'accessing' stamp: 'mt 3/29/2011 12:12:37.553'!
widgetPropertiesToIgnore

	^ #('TableLayout' 'ProportionalLayout' widgetResizable horizontalScrollBarPolicy verticalScrollBarPolicy)! !

!UiTextEdit methodsFor: 'actions' stamp: 'mt 9/8/2013 11:40:28.115'!
acceptChanges

	self originalText: self theText copy.
	self editingFinished: self theText copy.! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/23/2011 09:21:54.406'!
acceptOnFocusLost
	"Should the contents be accepted if the keyboard focus gets lost?"
	
	^ self textMorph acceptOnFocusLost! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/23/2011 09:21:43.524'!
acceptOnFocusLost: aBoolean

	self textMorph acceptOnFocusLost: aBoolean.! !

!UiTextEdit methodsFor: 'text changing' stamp: 'mt 6/27/2013 09:33:03.837'!
append: stringOrText

	self theText append: stringOrText.
	self textMorph recompose.
	
	self autoAccept
		ifTrue: [self textMorph editor accept].
		
	self textChanged: self theText.! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:11:29.281'!
autoAccept

	^ self textMorph autoAccept! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:11:21.722'!
autoAccept: aBoolean

	self textMorph autoAccept: aBoolean! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 17:46:43.014'!
autoConvert

	^ self textMorph autoConvert! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 17:46:34.323'!
autoConvert: aBoolean

	self textMorph autoConvert: aBoolean.  ! !

!UiTextEdit methodsFor: 'actions' stamp: 'mt 4/6/2011 15:42:04.336'!
clear

	self theText: ''.! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:01:16.688'!
colorBackground

	^ self colors
		at: #background
		ifAbsentPut: [Color white]! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:03:40.305'!
colorBackground: aColor

	self colors
		at: #background
		put: aColor! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:01:09.252'!
colorBorder

	^ self colors
		at: #border
		ifAbsentPut: [Color gray: 0.3]! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:03:56.543'!
colorBorder: aColor

	self colors
		at: #border
		put: aColor! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 5/10/2011 17:33:24.116'!
colorChangedBackground

	^ self colors
		at: #changedBackground
		ifAbsentPut: [Color yellow muchLighter paler]! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:04:09.806'!
colorChangedBackground: aColor

	self colors
		at: #changedBackground
		put: aColor! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 5/10/2011 17:33:27.92'!
colorChangedBorder

	^ self colors
		at: #changedBorder
		ifAbsentPut: [Color yellow muchDarker]! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:04:24.817'!
colorChangedBorder: aColor

	self colors
		at: #changedBorder
		put: aColor! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:00:07.404'!
colorFocusBackground

	^ self colors
		at: #focusBackground
		ifAbsentPut: [(Color r: 0.0 g: 0.0 b: 0.5) muchLighter paler]! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:04:36.158'!
colorFocusBackground: aColor

	self colors
		at: #focusBackground
		put: aColor! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:00:27.289'!
colorFocusBorder

	^ self colors
		at: #focusBorder
		ifAbsentPut: [(Color r: 0.5 g: 0.5 b: 1.0) muchDarker]! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 4/28/2011 17:04:51.483'!
colorFocusBorder: aColor

	self colors
		at: #focusBorder
		put: aColor! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 5/12/2011 15:32:33.026'!
colorHighlightedBackground

	^ self colors
		at: #colorHighlightedBackground
		ifAbsentPut: [Color orange muchLighter paler]! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 5/10/2011 16:46:24.392'!
colorHighlightedBackground: aColor

	^ self colors
		at: #colorHighlightedBackground
		put: aColor! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 5/12/2011 15:38:04.224'!
colorHighlightedBorder

	^ self colors
		at: #colorHighlightedBorder
		ifAbsentPut: [Color orange muchDarker]! !

!UiTextEdit methodsFor: 'accessing - colors' stamp: 'mt 5/10/2011 16:46:13.251'!
colorHighlightedBorder: aColor

	^ self colors
		at: #colorHighlightedBorder
		put: aColor! !

!UiTextEdit methodsFor: 'accessing - private' stamp: 'mt 4/28/2011 16:58:14.791'!
colors

	^ colors ifNil: [colors := Dictionary new]! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 18:29:16.694'!
converter
	
	^ self textMorph converter! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 18:29:11.591'!
converter: aBlock
	
	self textMorph converter: aBlock.  ! !

!UiTextEdit methodsFor: 'selection' stamp: 'mt 5/24/2011 15:47:53.806'!
cursorPosition

	^ self textMorph editor selectionInterval first! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 5/10/2011 17:13:05.772'!
displayMode
	"#normal ... normal mode with different colors for keyboard focus and changed state
	#highlight ... normal colors will be replaced with highlight ones."
	
	^ displayMode ifNil: [displayMode := #normal]! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 5/10/2011 17:13:38.21'!
displayMode: aSymbol

	displayMode = aSymbol ifTrue: [^ self].
	
	displayMode := aSymbol.! !

!UiTextEdit methodsFor: 'drawing' stamp: 'mt 1/10/2012 11:31:20.306'!
drawOn: aCanvas

	UiPainter default new
		drawTextEdit: self on: aCanvas.! !

!UiTextEdit methodsFor: 'signals' stamp: 'mt 3/23/2011 09:15:15.653'!
editingFinished: text
	"Emitted whenever the changed contents were accepted. This could happen with Cmd+S (Save operation), or focus lost."
	
	self emitAlways.! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 7/6/2012 13:05:43.63'!
font: aFont
	"Change the default font."
	
	self textMorph font: aFont.! !

!UiTextEdit methodsFor: 'actions' stamp: 'mt 4/4/2011 14:25:48.751'!
grabKeyboard
	"Pass-on keyboard focus to text morph."
	
	self textMorph grabKeyboard.! !

!UiTextEdit methodsFor: 'testing' stamp: 'mt 4/4/2011 14:26:57.511'!
hasKeyboard
	
	^ self textMorph hasKeyboard! !

!UiTextEdit methodsFor: 'testing' stamp: 'mt 4/18/2011 17:55:43.805'!
hasUnacceptedChanges

	^ self originalText asString ~= self theText asString! !

!UiTextEdit methodsFor: 'initialization' stamp: 'mt 6/27/2013 11:20:45.423'!
initialize

	horizontalScrollBarPolicy := #alwaysOff.
	verticalScrollBarPolicy := #asNeeded.
	widgetResizable := true.

	super initialize.
		
	self
		color: Color white;
		outerBorderWidth: 1.
		
	self transformMorph color: Color transparent.
	
	self verticalScrollBar
		singleStep: 20;
		outerBorderWidth: 0;
		color: Color transparent.  
	self verticalScrollBar eventHandler setProperty: #handlesKeyboard toValue: [false].
	
	self connect: self textMorph signal: #focusChanged: toSelector: #scrollToShow:.
	self connect: self textMorph signal: #textEdited: toSelector: #textEdited:.
	self connect: self textMorph signal: #textEdited: toSelector: #textChanged:.
	self connect: self textMorph signal: #accepted toSelector: #acceptChanges.
	self connect: self textMorph signal: #reverted toSelector: #revertChanges.
	self connect: self textMorph signal: #valueChanged: toSelector: #valueChanged:.! !

!UiTextEdit methodsFor: 'initialization' stamp: 'mt 6/27/2013 09:41:30.932'!
initializeEventHandler

	super initializeEventHandler.
	
	"Need the widget right before setting up the event handler connections."
	self
		model: self textModelClass new;
		widget: (self textMorphClass on: self).
	
	self connect: self eventHandler signal: #mouseClicked: toSelector: #grabKeyboard.
	self connect: self eventHandler signal: #mouseDoubleClicked: toSelector: #selectAll.

"	self connect: self eventHandler signal: #focusReceived toSelector: #updateFocusColor: pattern: #(=true).
	self connect: self eventHandler signal: #focusLost toSelector: #updateFocusColor: pattern: #(=false)."! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 7/5/2011 16:40:50.714'!
margins

	^ self textMorph margins! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 7/5/2011 16:40:59.522'!
margins: margins

	self textMorph margins: margins.! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 11/1/2012 16:42:47.563'!
model

	^ model! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 11/1/2012 16:42:55.201'!
model: aModel

	model := aModel.! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 5/11/2011 11:30:20.817'!
optimalWidth
	"Calculate the optimal with for the default font based of line breaks."
	
	| maxWidth |
	maxWidth := 0.
	(self theText asString copyReplaceAll: String tab with: '   ')
	   linesDo: [:line | 
			maxWidth := maxWidth max: (self textMorph textStyle defaultFont widthOfString: line)].
		
	^ maxWidth + self scrollBarThickness + 20 ! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/23/2011 09:26:42.103'!
originalText

	^ originalText ifNil: [originalText := '' asText]! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 18:11:05.141'!
originalText: stringOrText

	originalText := stringOrText.! !

!UiTextEdit methodsFor: 'accessing - private' stamp: 'mt 7/5/2011 16:41:44.253'!
preferredHeight

	^ (self scrollingLimit * self textMorph lineHeight)
		+ self margins top + self margins bottom! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:09:09.753'!
readOnly

	^ self textMorph readOnly! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:09:27.78'!
readOnly: aBoolean

	self textMorph readOnly: aBoolean! !

!UiTextEdit methodsFor: 'actions' stamp: 'mt 9/8/2013 11:40:36.16'!
revertChanges

	self theText: self originalText copy.
	self textReverted: self theText copy.! !

!UiTextEdit methodsFor: 'scrolling' stamp: 'mt 9/11/2012 13:24:27.15'!
scrollToShowSelection

	self scrollToShow: self textMorph editor selectionBlock.! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 4/28/2011 13:53:01.032'!
scrollingLimit
	"When #scrollingMode is not #never use this value to compute the height of this text edit field."
	
	^ scrollingLimit ifNil: [scrollingLimit := 10 "lines"]! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 4/28/2011 13:53:12.051'!
scrollingLimit: anInteger

	scrollingLimit = anInteger ifTrue: [^ self].

	scrollingLimit := anInteger.
	
	self updateLayout.! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 4/28/2011 13:53:15.977'!
scrollingMode
	"Decide when or if to show vertical scroll bar.
	#none ... deactivate any special handling
	#never ... expand vertically and never show scroll bar
	#limit ... use #scrollingLimit to decide when to add the scroll bar
	#always ... always show scroll bar and have the height specified by #scrollingLimit"
	
	^ scrollingMode ifNil: [scrollingMode := #none]! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 4/28/2011 13:53:21.905'!
scrollingMode: aSymbol

	scrollingMode = aSymbol ifTrue: [^ self].

	scrollingMode := aSymbol.
	
	self updateLayout.! !

!UiTextEdit methodsFor: 'selection' stamp: 'mt 1/18/2012 10:56:29.693'!
select: anInterval

	self textMorph editor selectInterval: anInterval. 
	self redrawLater.! !

!UiTextEdit methodsFor: 'selection' stamp: 'mt 1/18/2012 10:56:36.529'!
selectAll

	self textMorph editor selectAll.
	self redrawLater.! !

!UiTextEdit methodsFor: 'selection' stamp: 'mt 4/12/2013 21:50:26.339'!
selectLine

	self textMorph editor selectLine.
	self redrawLater.! !

!UiTextEdit methodsFor: 'selection' stamp: 'mt 4/6/2011 14:35:46.011'!
selectedText

	^ self theText copyFrom: self textMorph editor selectionInterval first to: self textMorph editor selectionInterval last! !

!UiTextEdit methodsFor: 'selection' stamp: 'mt 6/25/2012 17:05:58.527'!
selection

	^ self textMorph editor selectionInterval! !

!UiTextEdit methodsFor: 'signals' stamp: 'mt 3/23/2011 09:30:39.094'!
textChanged: text
	"Sent whenever the text was changed programmatically via #theText: or due to user input."
	
	self emitAlways.! !

!UiTextEdit methodsFor: 'signals' stamp: 'mt 3/23/2011 09:14:43.869'!
textEdited: text
	"Only emitted if text changed via user interaction and not programmatically."
	
	self emitAlways.! !

!UiTextEdit methodsFor: 'accessing - private' stamp: 'mt 11/1/2012 16:43:42.961'!
textModelClass

	^ UiTextModel! !

!UiTextEdit methodsFor: 'accessing - private' stamp: 'mt 3/21/2011 16:56:42.343'!
textMorph

	^ self widget! !

!UiTextEdit methodsFor: 'accessing - private' stamp: 'mt 7/18/2012 16:05'!
textMorphClass

	^ UiTextMorph! !

!UiTextEdit methodsFor: 'signals' stamp: 'mt 4/12/2011 17:47:19.641'!
textReverted: text

	self emit.! !

!UiTextEdit methodsFor: 'accessing' stamp: 'mt 3/22/2011 12:10:30.082'!
theText

	^ self textMorph text! !

!UiTextEdit methodsFor: 'accessing' stamp: 'bs 7/26/2011 14:15:02.945'!
theText: aValue

	aValue ifNil: [^ self theText: ''].

	aValue == self theText ifTrue: [^ self].
	
	self originalText: aValue.  
	
	"Scroll to the beginning."
	self transformMorph offset: 0@0.
	
	self textMorph newContents: aValue asStringOrText.

	self autoConvert
		ifTrue: [self textMorph convertText]
		ifFalse: [self updateScrollBars].
	
	self textChanged: self theText.
	
	self autoConvert
		ifFalse: [	self valueChanged: aValue].
			
	self textMorph style.
	
	self updateLayout.! !

!UiTextEdit methodsFor: 'actions' stamp: 'mt 5/10/2011 17:16:26.308'!
updateFocusColor: hasFocus

	hasFocus
		ifTrue: [self color: self colorFocusBackground]
		ifFalse: [
			self hasUnacceptedChanges
				ifTrue: [self color: self colorChangedBackground]
				ifFalse: [self displayMode = #highlight
					ifTrue: [self color: self colorHighlightedBackground]
					ifFalse: [self color: self colorBackground]]].! !

!UiTextEdit methodsFor: 'layouting' stamp: 'mt 4/28/2011 13:54:57.076'!
updateLayout

	self updateScrollingBehavior.
	super updateLayout.
! !

!UiTextEdit methodsFor: 'layouting' stamp: 'mt 7/22/2013 12:07:09.828'!
updateScrollingBehavior
	"Read #scrollingLimit and #scrollingMode and update height and scroll bar visibility."
	
	self scrollingMode ~= #none
		ifTrue: [
			areaResizable := true.
			verticalScrollBarPolicy := #asNeeded].
	
	self scrollingMode = #never
		ifTrue: [verticalScrollBarPolicy := #alwaysOff].
		
	self scrollingMode = #limit ifTrue: [
		self scrollingLimit < self textMorph lineCountComposed
			ifTrue: [
				verticalScrollBarPolicy := #alwaysOn.
				self height: self preferredHeight]
			ifFalse: [verticalScrollBarPolicy := #alwaysOff]].
		
	self scrollingMode = #always ifTrue: [
		verticalScrollBarPolicy := #alwaysOn.
		self height: self preferredHeight].! !

!UiTextEdit methodsFor: 'signals' stamp: 'mt 3/23/2011 09:38:35.655'!
valueChanged: convertedValue
	"Only emitted if set programmatically with #theText:."
	
	self emitAlways.! !

Morph subclass: #UiViewHeaderItem
	instanceVariableNames: 'orientation mouseDownPosition resizeable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiViewHeaderItem class methodsFor: 'testing' stamp: 'mt 1/10/2012 11:57:08.351'!
isWidgetClass

	^ false! !

!UiViewHeaderItem methodsFor: 'events-processing' stamp: 'mt 1/10/2012 13:47:46.124'!
handleMouseMove: anEvent
	"Receive any mouse move event."
	
	super handleMouseMove: anEvent.
	
	anEvent wasHandled not
		ifTrue: [self mouseMove: anEvent. anEvent wasHandled: true].! !

!UiViewHeaderItem methodsFor: 'initialization' stamp: 'mt 3/23/2012 10:08:19.817'!
initialize

	super initialize.
	self
		changeTableLayout.! !

!UiViewHeaderItem methodsFor: 'initialization' stamp: 'mt 3/23/2012 10:49:47.535'!
initializeEventHandler

	self eventHandler ifNotNil: [^ self].

	self eventHandler: UiEventHandler new.

	"Recognize drag start."
	self connect: self eventHandler signal: #mousePressed: toSelector: #updateMouseDownPosition:.

	"Update cursor."
	self connect: self eventHandler signal: #mouseMoved: toSelector: #updateMouseCursor:.
	self connect: self eventHandler signal: #mouseLeft: toSelector: #resetMouseCursor:.
	self connect: self eventHandler signal: #mouseReleased: toSelector: #resetMouseCursor:.

	"Trigger resize actions."
	self connect: self eventHandler signal: #mouseMoved: toSelector: #resizeByEvent:.
	self connect: self eventHandler signal: #mouseDoubleClicked: toSelector: #resizeToShrinkWrapByEvent:.! !

!UiViewHeaderItem methodsFor: 'testing' stamp: 'mt 1/10/2012 13:58:11.49'!
isMouseInResizeRect: aMouseEvent

	^ (self orientation == #horizontal and: [aMouseEvent position x + 5 > self right])
		or: [self orientation == #vertical and: [aMouseEvent position y + 5 > self bottom]]! !

!UiViewHeaderItem methodsFor: 'accessing' stamp: 'mt 1/10/2012 11:57:35.734'!
orientation

	^ orientation ifNil: [#horizontal]! !

!UiViewHeaderItem methodsFor: 'accessing' stamp: 'mt 1/10/2012 12:11:34.673'!
orientation: aSymbol
	"#horizontal, #vertical"

	orientation := aSymbol.
	self updateLayout.! !

!UiViewHeaderItem methodsFor: 'updating' stamp: 'mt 1/10/2012 14:10:06.792'!
resetMouseCursor: aMouseEvent

	mouseDownPosition := nil.
	aMouseEvent hand showTemporaryCursor: nil.! !

!UiViewHeaderItem methodsFor: 'actions' stamp: 'mt 3/23/2012 10:48:35.216'!
resize: widthOrHeight

	self orientation == #horizontal
		ifTrue: [self width: widthOrHeight]
		ifFalse: [self height: widthOrHeight].! !

!UiViewHeaderItem methodsFor: 'actions' stamp: 'mt 3/23/2012 10:46:39.905'!
resizeByEvent: aMouseEvent

	mouseDownPosition ifNil: [^ self].

	self resized: (self orientation == #horizontal
		ifTrue: [(aMouseEvent position - self left) x]
		ifFalse: [(aMouseEvent position - self top) y]).! !

!UiViewHeaderItem methodsFor: 'actions' stamp: 'mt 3/23/2012 10:46:47.668'!
resizeToShrinkWrapByEvent: aMouseEvent

	(self isMouseInResizeRect: aMouseEvent)
		ifTrue: [self resizedToShrinkWrap].! !

!UiViewHeaderItem methodsFor: 'accessing' stamp: 'mt 1/10/2012 17:13:51.191'!
resizeable
	"Anserws whether this item can be resized using a mouse or not."
	
	^ resizeable ifNil: [false]! !

!UiViewHeaderItem methodsFor: 'accessing' stamp: 'mt 1/10/2012 17:15:59.677'!
resizeable: aBoolean

	resizeable := aBoolean.
	aBoolean
		ifTrue: [self initializeEventHandler]
		ifFalse: [self eventHandler: nil].! !

!UiViewHeaderItem methodsFor: 'signals' stamp: 'mt 1/10/2012 14:20:55.469'!
resized: anInteger

	self emit.! !

!UiViewHeaderItem methodsFor: 'signals' stamp: 'mt 1/10/2012 17:17:36.411'!
resizedToShrinkWrap

	self emit.! !

!UiViewHeaderItem methodsFor: 'updating' stamp: 'mt 3/23/2012 10:07:20.019'!
updateLayout

	| c |
	c := Color gray: 0.85.
	
	self orientation == #horizontal ifTrue: [
		self
			listDirection: #leftToRight;
			hResizing: #rigid;
			vResizing: #shrinkWrap;
			fillStyle: ((GradientFillStyle ramp: {
				0.0 -> Color white.
				0.1-> (c adjustBrightness: 0.05).
				0.6 -> (c darker)})
				origin: self topLeft;
				direction: 0 @ self height);
			borderStyle: (BorderStyle raised width: 1).].
	
	self orientation == #vertical ifTrue: [
		self
			listDirection: #topToBottom;
			hResizing: #shrinkWrap;
			vResizing: #rigid;
			color: c;
			"fillStyle: ((GradientFillStyle ramp: {
				0.0 -> Color white.
				0.1-> (c adjustBrightness: 0.05).
				0.6 -> (c darker)})
				origin: self topLeft;
				direction: self width @ 0);"
			borderStyle: (BorderStyle raised width: 1)].! !

!UiViewHeaderItem methodsFor: 'updating' stamp: 'mt 1/10/2012 14:08:50.921'!
updateMouseCursor: aMouseEvent

	(aMouseEvent isMouseLeave or: [aMouseEvent isMouseUp]) ifTrue: [
		mouseDownPosition := nil.
		aMouseEvent hand showTemporaryCursor: nil.
		^ self].

	(mouseDownPosition notNil or: [self isMouseInResizeRect: aMouseEvent])
		ifTrue: [aMouseEvent hand showTemporaryCursor: (self orientation == #horizontal
			ifTrue: [Cursor resizeLeft]
			ifFalse: [Cursor resizeTop])]
		ifFalse: [aMouseEvent hand showTemporaryCursor: nil].! !

!UiViewHeaderItem methodsFor: 'updating' stamp: 'mt 1/10/2012 13:58:39.014'!
updateMouseDownPosition: aMouseEvent

	(self isMouseInResizeRect: aMouseEvent)
		ifTrue: [mouseDownPosition := aMouseEvent position]	.! !

Morph subclass: #UiWidgetPageButtonContainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiWidgetPageButtonContainer commentStamp: 'mt 7/14/2010 11:15:56.239' prior: 0!
I am a container for edit buttons that has a high mouse priority to accept mouse clicks.!

!UiWidgetPageButtonContainer methodsFor: 'events-processing' stamp: 'mt 7/16/2010 12:53:09.338'!
handlerForMouseDown: evt

	^ self submorphs
		detect: [:m | m bounds containsPoint: evt position] ifNone: [self].
		! !

!UiWidgetPageButtonContainer methodsFor: 'events-processing' stamp: 'mt 7/16/2010 12:53:21.162'!
handlerForYellowButtonDown: evt

	^ self submorphs
		detect: [:m | m bounds containsPoint: evt position] ifNone: [self].
		! !

!UiWidgetPageButtonContainer methodsFor: 'initialization' stamp: 'mt 7/16/2010 12:57:41.845'!
initialize

	super initialize.
	self
		color: (Color gray: 0.8);
		changeTableLayout;
		listDirection: #leftToRight;
		borderStyle: (BorderStyle raised width: 1);
		disableTableLayout: true;
		setToAdhereToEdge: #topRight;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap.! !

!UiWidgetPageButtonContainer methodsFor: 'events-processing' stamp: 'mt 7/14/2010 11:20:22.466'!
mouseDownPriority

	^ 150! !

Morph subclass: #UiWindow
	instanceVariableNames: 'ui'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiWindow commentStamp: 'mt 11/27/2010 14:01:27.968' prior: 0!
This is an abstract base class for a system window that uses a user interface generated by the Morphic Designer.

Implement #uiClass and override #title to configure the look of this window.!

!UiWindow class methodsFor: 'instance creation' stamp: 'mt 11/27/2010 13:55:25.929'!
open

	^ self new open! !

!UiWindow methodsFor: 'initialization' stamp: 'mt 11/27/2010 13:57:48.499'!
initialize

	super initialize.
	self ui setupUi: self.! !

!UiWindow methodsFor: 'open/close' stamp: 'mt 11/27/2010 13:55:39.471'!
open

	self openInWindowLabeled: self title.! !

!UiWindow methodsFor: 'accessing' stamp: 'mt 11/27/2010 17:32:06.393'!
title

	^ 'Unnamed Dialog'! !

!UiWindow methodsFor: 'accessing' stamp: 'mt 11/27/2010 13:56:20.01'!
ui

	^ ui ifNil: [ui := self uiClass new]! !

!UiWindow methodsFor: 'accessing' stamp: 'mt 11/27/2010 13:55:57.56'!
uiClass

	self subclassResponsibility.! !

!String methodsFor: '*widgets-text' stamp: 'mt 2/21/2013 14:35:29.699'!
asHtmlText

	^ UiHtmlTextFormatter toText: self! !

!String methodsFor: '*widgets-text' stamp: 'mt 5/2/2011 16:04:38.121'!
escapeHtmlText
	"Replaces all tokens that could be problematic when converting this string into a text using #asHtmlText."
	
	^ (self copyReplaceAll: '<' with: '\<')
		copyReplaceAll: '>' with: '\>'! !

!String methodsFor: '*widgets-support' stamp: 'mt 2/21/2013 11:50:16.724'!
withFirstCharacterUppercase

	| answer |
	self ifEmpty: [^ self copy].
	answer := self copy.
	answer at: 1 put: (answer at: 1) asUppercase.
	^ answer! !

!String methodsFor: '*widgets-support' stamp: 'mt 2/21/2013 11:00:09.828'!
withOnlyFirstCharacterUppercase

	| answer |
	self ifEmpty: [^ self copy].
	answer := self asLowercase.
	answer at: 1 put: (answer at: 1) asUppercase.
	^ answer! !

!String methodsFor: '*widgets-support' stamp: 'mt 2/21/2013 14:35:14.248'!
withoutLineEndings

	^ self withSqueakLineEndings
		copyReplaceAll: String cr
		with: ' '
		asTokens: false! !

!String methodsFor: '*widgets-support' stamp: 'mt 2/21/2013 14:35:05.164'!
withoutLineEndingsTruncatedTo: aNumber

	^ self withoutLineEndings truncateWithElipsisTo: aNumber! !

Point subclass: #UiModelIndex
	instanceVariableNames: 'group groupIndex parent node spec isInEditMode signalConnections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiModelIndex commentStamp: 'mt 1/23/2012 14:12:54.254' prior: 0!
A model index is a wrapping help data structure for the item viewport. It should not be used outside this viewport.!

!UiModelIndex class methodsFor: 'instance creation' stamp: 'mt 4/23/2012 11:14:53.525'!
row: aRow column: aColumn group: aGroup groupIndex: aGroupIndex parent: aParentModelIndex node: anObject spec: aColumnSpec

	^ (self x: aColumn y: aRow)
		group: aGroup;
		groupIndex: aGroupIndex;
		parent: aParentModelIndex;
		node: anObject;
		spec: aColumnSpec! !

!UiModelIndex methodsFor: 'comparing' stamp: 'mt 7/8/2013 13:59:58.106'!
= anotherModelIndex

	^ (super = anotherModelIndex
		and: [self group = anotherModelIndex group])
		and: [self parent = anotherModelIndex parent]! !

!UiModelIndex methodsFor: 'data' stamp: 'mt 1/23/2012 14:13:21.191'!
balloonText

	^ self spec balloonTextFor: self node! !

!UiModelIndex methodsFor: 'data' stamp: 'mt 7/5/2012 15:41:51.029'!
checkable

	^ self spec checkableFor: self node! !

!UiModelIndex methodsFor: 'data' stamp: 'mt 1/23/2012 14:13:36.771'!
checked

	^ self spec checkedFor: self node! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 10/7/2010 18:41:30.667'!
column

	^ self x! !

!UiModelIndex methodsFor: 'signals - setup' stamp: 'mt 4/27/2012 11:05:26.544'!
connectNode
	"See UiAbstractItemView>>connectIndex:."
	
	self column = 1 ifTrue: [
		self connect: self node signal: #modelReset toSelector: #modelReset.
		self connect: self node signal: #childrenChangedFrom:to:inGroup: toSelector: #rowsChangedFrom:to:inGroup:.
		self connect: self node signal: #childrenInsertedFrom:to:inGroup: toSelector: #rowsInsertedFrom:to:inGroup:.
		self connect: self node signal: #childrenRemovedFrom:to:inGroup: toSelector: #rowsRemovedFrom:to:inGroup:.
		self connect: self node signal: #propertiesChanged toSelector: #rowChanged].
	
	self connect: self node signal: #propertiesChanged: toSelector: #onPropertiesChanged:.
! !

!UiModelIndex methodsFor: 'signals - processing' stamp: 'mt 7/15/2013 15:19:06.635'!
disconnectAllSignals

	signalConnections := nil.! !

!UiModelIndex methodsFor: 'data' stamp: 'mt 7/6/2012 09:33:19.134'!
editable

	^ self spec editableFor: self node! !

!UiModelIndex methodsFor: 'building' stamp: 'mt 7/6/2012 10:22:34.571'!
editor

	^ self spec editorFor: self node! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 4/22/2012 11:20:59.187'!
group

	^ group! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 4/22/2012 11:21:06.402'!
group: anInteger

	group := anInteger.! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 4/23/2012 10:36:15.925'!
groupIndex

	^ groupIndex! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 4/23/2012 10:36:24.123'!
groupIndex: anInteger

	groupIndex := anInteger.! !

!UiModelIndex methodsFor: 'testing' stamp: 'mt 8/1/2013 10:52:12.217'!
hasChildren
	"Requires the node to have the model trait implemented."

	^ self node hasChildren! !

!UiModelIndex methodsFor: 'signals - processing' stamp: 'mt 7/15/2013 15:18:52.688'!
hasSignalConnections

	^ signalConnections notNil and: [signalConnections notEmpty]! !

!UiModelIndex methodsFor: 'comparing' stamp: 'mt 4/22/2012 16:27:26.813'!
hash

	^ (super hash
		bitXor: self group hash)
		bitXor: self parent hash! !

!UiModelIndex methodsFor: 'data' stamp: 'mt 6/26/2012 13:15:50.445'!
icon

	^ self spec iconFor: self node! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 7/6/2012 10:00:52.224'!
isInEditMode

	^ isInEditMode ifNil: [false]! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 7/6/2012 10:00:59.973'!
isInEditMode: aBoolean

	isInEditMode  := aBoolean.! !

!UiModelIndex methodsFor: 'building' stamp: 'mt 2/3/2012 11:24:35.558'!
item

	^ self spec itemFor: self node! !

!UiModelIndex methodsFor: 'signals' stamp: 'mt 2/3/2012 12:00:38.341'!
itemChanged

	self emitAlways.! !

!UiModelIndex methodsFor: 'data' stamp: 'mt 1/23/2012 14:12:15.013'!
menu

	^ self spec menuFor: self node! !

!UiModelIndex methodsFor: 'signals' stamp: 'mt 4/27/2012 11:04:04.275'!
modelReset

	self emitAlways.! !

!UiModelIndex methodsFor: 'as yet unclassified' stamp: 'mt 8/14/2013 11:01:25.447'!
morph

	^ self spec morphFor: self node! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 1/23/2012 14:09:05.195'!
node

	^ node! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 4/26/2012 13:58:24.84'!
node: aNode

	node := aNode.
	aNode ifNotNil: [self connectNode].! !

!UiModelIndex methodsFor: 'signals - setup' stamp: 'mt 7/9/2012 17:15:42.898'!
onPropertiesChanged: someProperties

	(spec includesAnyProperty: someProperties inNode: self node)
		ifTrue: [self itemChanged].! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 10/7/2010 17:24:15.322'!
parent
	"Return parent model index."
	
	^ parent! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 10/7/2010 17:24:27.971'!
parent: aModelIndex
	
	parent := aModelIndex.! !

!UiModelIndex methodsFor: 'printing' stamp: 'mt 4/26/2012 14:21:36.215'!
printString
	"Print the complete hierarchy of this model index."
	
	^ self parent
		ifNil: [super printString, ' (', self group, ')']
		ifNotNil: [super printString, ' (', self group, ')', ' -> ', self parent printString]! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 12/13/2012 15:10:25.707'!
rootGroup

	| c |
	c := self.
	[c parent notNil] whileTrue: [c := c parent].
	^ c group! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 12/13/2012 15:11:26.826'!
rootGroupIndex

	| c |
	c := self.
	[c parent notNil] whileTrue: [c := c parent].
	^ c groupIndex! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 10/7/2010 18:41:24.938'!
row

	^ self y! !

!UiModelIndex methodsFor: 'signals' stamp: 'mt 2/3/2012 12:02:07.679'!
rowChanged

	self emitAlways.! !

!UiModelIndex methodsFor: 'signals' stamp: 'mt 4/22/2012 16:13:03.665'!
rowsChangedFrom: startIndex to: endIndex inGroup: groupName

	self emitAlways.! !

!UiModelIndex methodsFor: 'signals' stamp: 'mt 4/22/2012 16:13:17.123'!
rowsInsertedFrom: startIndex to: endIndex inGroup: groupName

	self emitAlways.! !

!UiModelIndex methodsFor: 'signals' stamp: 'mt 4/22/2012 16:13:25.954'!
rowsRemovedFrom: startIndex to: endIndex inGroup: groupName

	self emitAlways.! !

!UiModelIndex methodsFor: 'data' stamp: 'mt 6/26/2012 13:46:17.036'!
selectable

	^ self spec selectableFor: self node! !

!UiModelIndex methodsFor: 'signals - processing' stamp: 'mt 7/15/2013 15:19:17.518'!
signalConnections

	^ signalConnections ifNil: [signalConnections := Dictionary new]! !

!UiModelIndex methodsFor: 'signals - processing' stamp: 'mt 7/15/2013 15:18:14.254'!
signalMutex

	^ UiNullMutex! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 1/23/2012 14:09:30.963'!
spec

	^ spec! !

!UiModelIndex methodsFor: 'accessing' stamp: 'mt 1/23/2012 14:09:40.224'!
spec: aColumnSpec

	spec := aColumnSpec.! !

!UiModelIndex methodsFor: 'data' stamp: 'mt 1/23/2012 14:13:13.196'!
text

	^ self spec textFor: self node! !

TransformMorph subclass: #UiTransformMorph
	instanceVariableNames: 'drawCache usesDrawCache drawCacheActivationFlag drawCacheFlag drawCacheUpdateFlag drawCacheActive moveFlag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiTransformMorph commentStamp: 'mt 4/25/2011 08:34:54.693' prior: 0!
A transform morph that provides signals.

A draw cache offers an experimental feature that speeds-up redraw for large transform morphs with complex contents. Activate it via #drawCacheActive:.

A hand that contains submorphs (i.e., is dragging something) will trigger an offset change in corner areas.  !

!UiTransformMorph methodsFor: 'accessing' stamp: 'mt 2/27/2011 17:02:46.165'!
angle: aNumber

	self angle = aNumber ifTrue: [^ self] .
	super angle: aNumber.
	self angleChanged: self angle.  ! !

!UiTransformMorph methodsFor: 'signals' stamp: 'mt 2/27/2011 17:00:55.169'!
angleChanged: aNumber

	self emit.! !

!UiTransformMorph methodsFor: 'signals' stamp: 'mt 3/24/2011 14:50:28.433'!
areaChanged
	"Sent whenever the feasible area, the offset can be used for, changed."

	self emit.! !

!UiTransformMorph methodsFor: 'caching' stamp: 'mt 4/20/2011 14:14:22.928'!
drawCacheActive
	"Indicate whether drawing should gain a speedup using a cache."	

	^ drawCacheActive ifNil: [false]! !

!UiTransformMorph methodsFor: 'caching' stamp: 'mt 4/20/2011 14:15:23.428'!
drawCacheActive: aBoolean

	aBoolean = drawCacheActive ifTrue: [^ self].

	drawCacheActive := aBoolean.
	
	drawCacheActive
		ifTrue: [self startStepping]
		ifFalse: [self
			stopStepping;
			redrawLater].! !

!UiTransformMorph methodsFor: 'drawing - cached' stamp: 'mt 4/20/2011 13:47:28.557'!
drawCacheOn: aCanvas
	
	aCanvas
		transformBy: transform
		clippingTo: self innerBounds
		during: [:myCanvas | myCanvas translucentImage: drawCache at: 0@0]
		smoothing: smoothing.! !

!UiTransformMorph methodsFor: 'drawing' stamp: 'mt 4/20/2011 13:47:36.915'!
drawSubmorphsOn: aCanvas

	(self usesDrawCache not or: [drawCache isNil])
		ifTrue: [^ super drawSubmorphsOn: aCanvas].

	self drawCacheOn: aCanvas.  ! !

!UiTransformMorph methodsFor: 'submorphs-accessing' stamp: 'mt 3/24/2011 15:06:05.221'!
firstSubmorph
	
	^ self submorphs
		ifEmpty: []
		ifNotEmpty: [self submorphs first]! !

!UiTransformMorph methodsFor: 'layout' stamp: 'mt 4/20/2011 14:16:25.082'!
fullBounds

	| result |
	fullBounds ifNotNil: [^ fullBounds].
	
	result := super fullBounds.

	"Force re-layouting of one an only submorph."	
	self firstSubmorph ifNotNil: [:m |
		m fullBounds.
		self areaChanged.
		
		"IS this the right place to invalidate the cache?"
		drawCacheUpdateFlag := #tryUpdateCache].

	^ result! !

!UiTransformMorph methodsFor: 'drawing - cached' stamp: 'mt 5/19/2011 15:13:27.463'!
fullDrawCacheOn: aCanvas
	"Update the cache if cache is nil. then use the cache to draw."
	
	self
		updateDrawCache;
		drawCacheOn: aCanvas.! !

!UiTransformMorph methodsFor: 'events-processing' stamp: 'mt 7/8/2013 14:02:58.381'!
handleMouseMove: anEvent
	"Receive any mouse move event."
	
	super handleMouseMove: anEvent.
	
	((anEvent hand hasSubmorphs and: [self firstSubmorph notNil]) and: [self firstSubmorph dropEnabled])
		ifTrue: [moveFlag := #doMoveScrolling.
			self isStepping ifFalse: [self startStepping]]
		ifFalse: [moveFlag := #idle.
			self drawCacheActive ifFalse: [self stopStepping]].! !

!UiTransformMorph methodsFor: 'accessing' stamp: 'mt 3/7/2011 09:33:49.654'!
horizontalOffset

	^ self offset x! !

!UiTransformMorph methodsFor: 'accessing' stamp: 'mt 3/7/2011 09:36:11.99'!
horizontalOffset: aNumber

	aNumber = self horizontalOffset ifTrue: [^ self].
	
	self offset: aNumber @ self offset y.

	self horizontalOffsetChanged: self horizontalOffset.  ! !

!UiTransformMorph methodsFor: 'signals - specific' stamp: 'mt 3/7/2011 09:35:09.391'!
horizontalOffsetChanged: aNumber

	self emit.! !

!UiTransformMorph methodsFor: 'initialization' stamp: 'mt 4/22/2011 08:43:39.362'!
initialize

	super initialize.
	
	self
		color: Color white;
		drawCacheActive: false.  ! !

!UiTransformMorph methodsFor: 'testing' stamp: 'mt 6/13/2012 17:15:26.501'!
isTransformMorph

	^ true! !

!UiTransformMorph methodsFor: 'geometry' stamp: 'mt 4/27/2011 17:14:43.927'!
layoutChanged

	localBounds := nil.
	fullBounds := nil.! !

!UiTransformMorph methodsFor: 'accessing' stamp: 'mt 6/14/2012 09:34:21.145'!
offset: aPoint

	| tPoint |
	tPoint := aPoint truncated max: 0@0.
	
	tPoint = self offset ifTrue: [^ self].

	self drawCacheActive ifTrue: [
		drawCacheFlag := #raised.
		drawCache
			ifNil: [self scheduleDrawCacheUpdate]
			ifNotNil: [self usesDrawCache: true]].
	
	"super offset: tPoint."
	transform := transform withOffset: tPoint - self innerBounds topLeft.
	self redraw.

	self
		offsetChanged: self offset;
		horizontalOffsetChanged: self offset x;
		verticalOffsetChanged: self offset y.! !

!UiTransformMorph methodsFor: 'signals' stamp: 'mt 2/27/2011 17:00:35.308'!
offsetChanged: aPoint

	self emit.! !

!UiTransformMorph methodsFor: 'caching' stamp: 'mt 5/16/2011 17:38:03.529'!
processCacheFlags

	self drawCacheActive
		ifFalse: [^ self].

	self usesDrawCache ifFalse: [
		"If cache update is already defered, schedule it now."
		drawCacheUpdateFlag = #deferUpdateCache
			ifTrue: ["self scheduleDrawCacheUpdate"
				self flag: #tofix.
				drawCache := nil].
		
		"If someone wants to update the cache, try to defer it."
		drawCacheUpdateFlag = #tryUpdateCache
			ifTrue: [drawCacheUpdateFlag := #deferUpdateCache].
		
		] ifTrue: [
		drawCacheFlag = #raised
			ifTrue: [drawCacheFlag := #cleared]
			ifFalse: [	self usesDrawCache: false]].! !

!UiTransformMorph methodsFor: 'drag scrolling' stamp: 'mt 4/25/2011 10:49:07.838'!
processMoveFlag

	self firstSubmorph ifNil: [^ self].

	(ActiveHand hasSubmorphs and: [moveFlag = #doMoveScrolling])
		ifTrue: [self updateOffsetByHand: ActiveHand]
		ifFalse: [moveFlag := #idle.
			self drawCacheActive ifFalse: [self stopStepping]].! !

!UiTransformMorph methodsFor: 'drawing' stamp: 'mt 3/22/2012 15:21:52.755'!
redraw
	"Override to use double buffering with BitBlt."
	
	(self usesDrawCache not or: [drawCache isNil])
		ifTrue: [^ self redrawLater].
	
	Display deferUpdates: true.	
	self fullDrawOn: Display getCanvas.
	Display forceToScreen: (self visibleBounds insetBy: (0@0 corner: 1@1)).! !

!UiTransformMorph methodsFor: 'accessing' stamp: 'mt 2/27/2011 17:03:04.995'!
scale: aNumber

	self scale = aNumber ifTrue: [^ self] .
	super scale: aNumber.
	self scaleChanged: self scale.! !

!UiTransformMorph methodsFor: 'signals' stamp: 'mt 2/27/2011 17:00:48.104'!
scaleChanged: aNumber

	self emit.! !

!UiTransformMorph methodsFor: 'caching' stamp: 'mt 5/19/2011 14:15:03.752'!
scheduleDrawCacheUpdate

	drawCache := nil.
	drawCacheUpdateFlag := #idle.

	WorldState addDeferredUIMessage: [
		self usesDrawCache ifFalse: [self updateDrawCache]].! !

!UiTransformMorph methodsFor: 'stepping' stamp: 'mt 4/25/2011 08:31:49.851'!
step

	self
		processCacheFlags;
		processMoveFlag.! !

!UiTransformMorph methodsFor: 'stepping' stamp: 'mt 4/20/2011 13:38:06.041'!
stepTime

	^ 100! !

!UiTransformMorph methodsFor: 'drawing - cached' stamp: 'mt 6/19/2012 18:07:05.607'!
updateDrawCache

	| cacheCanvas |
	drawCache ifNotNil: [^ self].
	
	drawCache := Form extent: self firstSubmorph extent depth: 32.
	cacheCanvas := drawCache getCanvas.
	cacheCanvas fillColor: (Color gray: 0.3).
	submorphs reverseDo: [:m | m fullDrawCacheOn: cacheCanvas].
	
	drawCacheUpdateFlag := #idle.! !

!UiTransformMorph methodsFor: 'drag scrolling' stamp: 'mt 5/2/2011 11:30:37.883'!
updateOffsetByHand: aHand

	| threshold pos dx dy maxDelta |
	threshold := 50. "in pixel"
	maxDelta := 30. "will be interpolated linear in threshold"
	pos := self globalPointToLocal: aHand position.
	dx := 0. "horizontal offset delta"
	dy := 0. "vertical offset delta"
	
	(self bounds containsPoint: pos) ifTrue: [
		dx := dx - (((1.0 - ((pos - self left) x / threshold)) max: 0) * maxDelta).
		dx := dx + (((1.0 - ((self right - pos) x / threshold)) max: 0) * maxDelta).
		dy := dy - (((1.0 - ((pos - self top) y / threshold)) max: 0) * maxDelta).
		dy := dy + (((1.0 - ((self bottom - pos) y / threshold)) max: 0) * maxDelta)].
	
	self offset: ((self offset + (dx @ dy))
		min: self firstSubmorph extent - self extent
		max: 0@0).! !

!UiTransformMorph methodsFor: 'caching' stamp: 'mt 4/20/2011 14:14:35.12'!
usesDrawCache
	"Indicate if a cache is used just now."
	
	^ usesDrawCache ifNil: [usesDrawCache := false]! !

!UiTransformMorph methodsFor: 'caching' stamp: 'mt 5/3/2011 13:53:18.125'!
usesDrawCache: aBoolean

	aBoolean = usesDrawCache ifTrue: [^ self].

	usesDrawCache := aBoolean.
	
	usesDrawCache ifFalse: [self redraw].! !

!UiTransformMorph methodsFor: 'accessing' stamp: 'mt 3/7/2011 09:33:57.533'!
verticalOffset

	^ self offset y! !

!UiTransformMorph methodsFor: 'accessing' stamp: 'mt 3/7/2011 09:36:39.622'!
verticalOffset: aNumber

	aNumber = self verticalOffset ifTrue: [^ self].
	
	self offset: self offset x @ aNumber.
	
	self verticalOffsetChanged: self verticalOffset.  ! !

!UiTransformMorph methodsFor: 'signals - specific' stamp: 'mt 3/7/2011 09:35:04.739'!
verticalOffsetChanged: aNumber

	self emit.! !

BorderedMorph subclass: #UiContainer
	instanceVariableNames: 'editMode focusRectangle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiContainer commentStamp: 'mt 7/13/2010 09:34:37.877' prior: 0!
I am a, normally invisible, container or owner morph for various UI elements. If I am in #editMode, I support drag&drop operations.!

UiContainer subclass: #UiButtonGroup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiButtonGroup commentStamp: 'mt 7/13/2010 09:32:47.81' prior: 0!
I am a container for buttons that make my submorphs/buttons auto-exclusive.!

!UiButtonGroup class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:12:57.61'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiGroupBoxIcon! !

!UiButtonGroup class methodsFor: 'accessing' stamp: 'mt 7/1/2010 08:07:04.687'!
widgetClassName

	^ 'Button Group'! !

!UiButtonGroup methodsFor: 'submorphs-add/remove' stamp: 'mt 9/13/2013 10:43:04.817'!
addMorphBack: aButton

	super addMorphBack: aButton.
	
	aButton autoExclusive: true.
	
	self
		connect: aButton signal: #clicked: toSelector: #clickButton: pattern: #(0);
		connect: aButton signal: #pressed toSelector: #pressButton: pattern: #(0);
		connect: aButton signal: #released toSelector: #releaseButton: pattern: #(0);
		connect: aButton signal: #toggled: toSelector: #toggleButton: pattern: #(0).! !

!UiButtonGroup methodsFor: 'accessing' stamp: 'mt 7/1/2010 08:16:04.263'!
button: anId

	 self submorphs withIndexDo: [:m :i | i = anId ifTrue: [^ m]].
	
	^ nil! !

!UiButtonGroup methodsFor: 'signals' stamp: 'mt 7/1/2010 08:11:14.563'!
buttonClicked: aButton id: anId

	self emit.! !

!UiButtonGroup methodsFor: 'signals' stamp: 'mt 7/1/2010 08:11:30.004'!
buttonPressed: aButton id: anId

	self emit.! !

!UiButtonGroup methodsFor: 'signals' stamp: 'mt 7/1/2010 08:11:48.494'!
buttonReleased: aButton id: anId

	self emit.! !

!UiButtonGroup methodsFor: 'signals' stamp: 'mt 9/13/2013 10:44:28.581'!
buttonToggled: aButton id: anId

	self emit.! !

!UiButtonGroup methodsFor: 'accessing' stamp: 'mt 7/1/2010 08:10:28.017'!
checkedButton

	^ self submorphs detect: [:m | m isChecked] ifNone: [nil]! !

!UiButtonGroup methodsFor: 'accessing' stamp: 'mt 7/1/2010 08:10:20.631'!
checkedId

	self submorphs withIndexDo: [:m :i | m isChecked ifTrue: [^ i]].
	^ 0! !

!UiButtonGroup methodsFor: 'private' stamp: 'mt 7/1/2010 08:16:32.662'!
clickButton: aButton

	self buttonClicked: aButton id: (self id: aButton)! !

!UiButtonGroup methodsFor: 'signals' stamp: 'mt 9/13/2013 13:16:23.893'!
currentIndexChanged: id

	self emit.! !

!UiButtonGroup methodsFor: 'accessing' stamp: 'mt 9/13/2013 13:07:11.298'!
id: aButton

	^ self submorphs indexOf: aButton! !

!UiButtonGroup methodsFor: 'private' stamp: 'mt 7/1/2010 08:16:44.088'!
pressButton: aButton

	self buttonPressed: aButton id: (self id: aButton)! !

!UiButtonGroup methodsFor: 'private' stamp: 'mt 7/1/2010 08:16:54.688'!
releaseButton: aButton

	self buttonReleased: aButton id: (self id: aButton).! !

!UiButtonGroup methodsFor: 'private' stamp: 'mt 9/13/2013 13:16:57.057'!
toggleButton: aButton

	self buttonToggled: aButton id: (self id: aButton).
	aButton checked
		ifTrue: [self currentIndexChanged: (self id: aButton)].! !

!UiButtonGroup methodsFor: 'dropping/grabbing' stamp: 'mt 7/2/2013 12:59:24.762'!
wantsDroppedMorph: aMorph event: evt

	self editMode ifFalse: [^ false].

	(((aMorph isKindOf: TransferMorph)
		and: [UiButton subclasses includes: (aMorph passenger first at: #class)])
		or: [aMorph isKindOf: UiButton])
		ifTrue: [^ super wantsDroppedMorph: aMorph event: evt].
	
	^ false! !

!UiContainer class methodsFor: 'testing' stamp: 'mt 6/28/2010 21:36:20.651'!
isWidgetClass

	^ true! !

!UiContainer class methodsFor: 'accessing' stamp: 'mt 10/26/2010 19:45:30.19'!
widgetCategory
	
	^ 'Containers & Spacer'! !

!UiContainer class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:23.353'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiPanelIcon! !

!UiContainer class methodsFor: 'accessing' stamp: 'mt 6/30/2010 21:22:49.177'!
widgetClassName

	^ 'Widget Container'! !

!UiContainer class methodsFor: 'properties' stamp: 'mt 10/28/2010 16:52:48.311'!
widgetProperties

	^ super widgetProperties, {{'Container'. #yourself} -> {
		#color -> Object.
		#borderWidth -> Integer.
		#borderStyle2 -> #(simple inset raised).
		#borderColor -> Object}}! !

!UiContainer methodsFor: 'dropping/grabbing' stamp: 'mt 7/22/2012 09:22:01.902'!
acceptDroppingMorph: aMorph event: evt

	| result |
	(aMorph isKindOf: TransferMorph)
		"Hard-wired with UiWidgetModel. Baaaaad."
		ifTrue: [ (aMorph passenger first at: #class ifAbsent: []) ifNotNil: [:widgetClass |
			| widget |
			widget := widgetClass new position: evt position.
			self addMorphBack: widget.
			(widget isKindOf: UiContainer)
				ifTrue: [widget editMode: self editMode].

			self
				changeFocusTo: widget bounds;
				changeSelectionTo: widget.
				
			^ self]].

	result := super acceptDroppingMorph: aMorph event: evt.
	
	self
		changeFocusTo: aMorph bounds;
		changeSelectionTo: aMorph.
	
	^ result! !

!UiContainer methodsFor: 'accessing' stamp: 'mt 11/6/2012 16:19:40.984'!
addChild: aMorph
	"A way to fill the hierarchy. Used by Morphic Designer."

	^ self addMorphBack: aMorph! !

!UiContainer methodsFor: 'accessing' stamp: 'mt 10/28/2010 15:29:55.046'!
borderStyle2

	^ self borderColor isSymbol
		ifTrue: [self borderColor]
		ifFalse: [#simple]! !

!UiContainer methodsFor: 'accessing' stamp: 'mt 10/28/2010 15:31:30.474'!
borderStyle2: aSymbol

	aSymbol ifNil: [^ self].
	
	aSymbol = #simple
		ifTrue: [^ self borderColor: Color transparent].
	
	^ self borderColor: aSymbol! !

!UiContainer methodsFor: 'focus' stamp: 'mt 10/23/2010 09:50:30.956'!
changeFocusTo: aRectangle

	(self owner isKindOf: UiContainer)
		ifTrue: [^ self owner changeFocusTo: aRectangle].
	
	aRectangle = self bounds
		ifTrue: [focusRectangle := nil].
	
	focusRectangle := aRectangle ifNotNil: [:r | r outsetBy: (2@2)].
	self redrawLater.! !

!UiContainer methodsFor: 'selection' stamp: 'mt 11/6/2012 13:19:37.27'!
changeSelectionTo: aMorph
	"Talk to root container."

	(self owner isKindOf: UiContainer)
		ifTrue: [self owner changeSelectionTo: aMorph]
		ifFalse: [	self selectionChanged: aMorph].
	
	aMorph world ifNotNil: [
		aMorph addHalo].! !

!UiContainer methodsFor: 'accessing' stamp: 'mt 11/6/2012 16:10:06.062'!
children
	"Hierarchy used by the Morphic Designer to generate code."

	^ self submorphs! !

!UiContainer methodsFor: 'event handling' stamp: 'mt 10/24/2010 11:07:59.575'!
click: evt

"	| clickedMorph |
	clickedMorph := self submorphs detect: [:m | m bounds containsPoint: evt position] ifNone: [self].
	
	evt yellowButtonPressed ifTrue: [
		self showContextMenuForMorph: clickedMorph atHand: evt hand]."
	
	| clickedMorph |
	clickedMorph := self submorphs
		detect: [:m | m bounds containsPoint: evt position] ifNone: [self].
		
	self
		changeFocusTo: clickedMorph bounds;
		changeSelectionTo: clickedMorph.! !

!UiContainer methodsFor: 'event handling' stamp: 'mt 6/27/2010 13:43:16.726'!
doubleClick: evt! !

!UiContainer methodsFor: 'drawing' stamp: 'mt 10/26/2010 20:20:20.839'!
drawFocusOn: aCanvas rectangle: aRect
	
"	aCanvas
		frameRectangle: (aRect insetBy: 1)
		color: (Color r: 0.5 g: 0.5 b: 1.0 alpha: 0.8) muchDarker.
		
	aCanvas
		frameRoundRect: aRect
		radius: 3
		width: 1
		color: (Color r: 0.5 g: 0.5 b: 1.0 alpha: 0.5) muchDarker."! !

!UiContainer methodsFor: 'drawing' stamp: 'mt 10/28/2010 11:08:59.269'!
drawOn: aCanvas

	self editMode
		ifFalse: [^ super drawOn: aCanvas].
	
	aCanvas
		fillRectangle: self bounds
		fillStyle: ((self color = Color transparent)
			ifTrue: [SolidFillStyle color: ((Color gray: 0.3) alpha: 0.1)]
			ifFalse: [self fillStyle])
		borderStyle: self borderStyle.

	focusRectangle ifNotNil: [self drawFocusOn: aCanvas rectangle: focusRectangle].! !

!UiContainer methodsFor: 'accessing' stamp: 'mt 6/30/2010 09:17:26.068'!
editMode

	^ editMode ifNil: [editMode := false]! !

!UiContainer methodsFor: 'accessing' stamp: 'mt 10/28/2010 11:07:21.218'!
editMode: aBoolean

	editMode := aBoolean.
	
	self dropEnabled: editMode.
	self dragEnabled: editMode.
	
	self submorphs
		select: [:m | m isKindOf: UiContainer]
		thenDo: [:m | m editMode: aBoolean].! !

!UiContainer methodsFor: 'signals' stamp: 'mt 6/28/2010 11:03:51.082'!
focusChanged: aBoolean

	self emit.! !

!UiContainer methodsFor: 'event handling' stamp: 'mt 6/30/2010 09:18:46.354'!
handlesMouseDown: evt

	^ self editMode! !

!UiContainer methodsFor: 'initialization' stamp: 'mt 10/26/2010 20:01:29.289'!
initialize

	super initialize.
	self
		extent: 100@100;
		color: Color transparent;
		borderWidth: 0;
		cellInset: 4;
		layoutInset: 4.! !

!UiContainer methodsFor: 'testing' stamp: 'mt 10/27/2010 20:39:00.491'!
isUiContainer

	^ true! !

!UiContainer methodsFor: 'event handling' stamp: 'mt 6/28/2010 11:03:39.618'!
keyboardFocusChange: aBoolean

	self focusChanged: aBoolean.! !

!UiContainer methodsFor: 'event handling' stamp: 'mt 10/23/2010 10:06:01.973'!
mouseDown: evt

	"| clickedMorph |
	clickedMorph := self submorphs
		detect: [:m | m bounds containsPoint: evt position] ifNone: [self].
	
	self submorphs do: [:m |
		m ~~ clickedMorph
			ifTrue: [m setProperty: #hasFocus toValue: false]].
	self setProperty: #hasFocus toValue: false.
	
	clickedMorph setProperty: #hasFocus toValue: true.
	self changeSelectionTo: clickedMorph."

	evt hand
		waitForClicksOrDrag: self
		event: evt
		selectors: {#click:. nil. nil. #startDrag:}
		threshold: 1. "pixel threshold"
		
	"self redrawLater.
	(self owner isKindOf: UiContainer) ifTrue: [self owner redrawLater]."! !

!UiContainer methodsFor: 'events-processing' stamp: 'mt 6/30/2010 09:18:26.354'!
mouseDownPriority
	"Give me ALL events."
	
	self editMode ifFalse: [^ super mouseDownPriority].
	^ 100! !

!UiContainer methodsFor: 'geometry' stamp: 'mt 12/9/2011 13:26:25.452'!
outerBounds

	^ super outerBounds
	"(self owner isKindOf: UiContainer)
		ifTrue: [^ self bounds outsetBy: (3@3 corner: 3@3)]
		ifFalse: [^ self bounds]."! !

!UiContainer methodsFor: 'signals' stamp: 'mt 6/30/2010 09:29:37.854'!
selectionChanged: aMorph

	self emit.! !

!UiContainer methodsFor: 'context menu' stamp: 'mt 6/28/2010 11:13:33.993'!
showContextMenuForMorph: aMorph atHand: aHand

	| menu |
	menu := SBMenu new.
	
	self
		connect: (menu addAction: (SBAction new text: 'vResizing'))
		signal: #triggered
		to: aMorph
		selector: #vResizing:
		pattern: #(=#spaceFill).
	
	"Clean up the menu."
	menu connect: self signal: #focusChanged: toSelector: #delete.
	
	"Get focus and show menu."
	aHand newKeyboardFocus: self.
	menu popUp..! !

!UiContainer methodsFor: 'event handling' stamp: 'mt 10/24/2010 11:08:08.144'!
startDrag: evt

	| morphToDrag |
	morphToDrag := self submorphs
		detect: [:m | m bounds containsPoint: evt position]
		ifNone: [self dragEnabled
			ifTrue: [self]
			ifFalse: [nil]].
	
	self changeFocusTo: nil.
			
	morphToDrag ifNil: [^ self].
	
	evt redButtonPressed
		ifTrue: [evt hand grabMorph: morphToDrag].! !

!UiContainer methodsFor: 'dropping/grabbing' stamp: 'mt 6/28/2010 16:46:21.331'!
wantsDroppedMorph: aMorph event: evt

	self redrawLater.
	^ super wantsDroppedMorph: aMorph event: evt! !

UiContainer subclass: #UiSpacer
	instanceVariableNames: 'orientation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiSpacer class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:15:52.306'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiAccordionIcon! !

!UiSpacer class methodsFor: 'accessing' stamp: 'mt 10/26/2010 19:43:44.923'!
widgetClassName

	^ 'Spacer'! !

!UiSpacer class methodsFor: 'properties' stamp: 'mt 10/26/2010 20:07:37.101'!
widgetProperties

	^ super widgetProperties, {{'Spacer'. #yourself} -> {
		#orientation -> #(horizontal vertical)}}! !

!UiSpacer class methodsFor: 'properties' stamp: 'mt 10/27/2010 09:42:04.578'!
widgetPropertiesToIgnore

	^ #(hResizing vResizing 'TableLayout' 'ProportionalLayout')! !

!UiSpacer methodsFor: 'drawing' stamp: 'mt 10/28/2010 10:58:19.197'!
drawOn: aCanvas

	self editMode ifFalse: [^ self].
	
	self orientation = #horizontal ifTrue: [
		aCanvas
			line: self topLeft to: self bottomLeft color: self color;
			line: self topRight to: self bottomRight color: self color;
			line: self topLeft + (0@(self height // 2)) to: self topRight + (0@(self height // 2)) color: self color].
		
	self orientation = #vertical ifTrue: [
		aCanvas
			line: self topLeft to: self topRight color: self color;
			line: self bottomLeft to: self bottomRight color: self color;
			line: self topLeft + ((self width // 2) @ 0) to: self bottomLeft + ((self width // 2) @ 0) color: self color].! !

!UiSpacer methodsFor: 'initialization' stamp: 'mt 4/13/2011 12:48:43.871'!
initialize

	super initialize.
	self
		color: Color blue;
		orientation: #horizontal;
		extent: 50@20;
		layoutInset: 0;
		cellInset: 0.! !

!UiSpacer methodsFor: 'accessing' stamp: 'mt 10/26/2010 19:57:59.594'!
orientation

	^ orientation! !

!UiSpacer methodsFor: 'accessing' stamp: 'mt 10/26/2010 19:57:51.389'!
orientation: aSymbol
	"#horizontal, #vertical"

	orientation == aSymbol
		ifTrue: [^ self].
		
	orientation := aSymbol.
	self updateLayout.! !

!UiSpacer methodsFor: 'geometry' stamp: 'mt 10/26/2010 20:04:04.927'!
outerBounds

	^ self bounds outsetBy: (0@0 corner: 1@1)! !

!UiSpacer methodsFor: 'updating' stamp: 'mt 4/13/2011 12:48:05.116'!
updateLayout

	self orientation = #horizontal
		ifTrue: [self
			extent: self height @ self width;
			hResizing: #spaceFill;
			vResizing: #rigid].
		
	self orientation = #vertical
		ifTrue: [self
			extent: self height @ self width;
			hResizing: #rigid;
			vResizing: #spaceFill].
		
	self redrawLater.! !

!UiSpacer methodsFor: 'dropping/grabbing' stamp: 'mt 10/26/2010 19:37:26.982'!
wantsDroppedMorph: aMorph event: evt

	^ false! !

UiContainer subclass: #UiStackedWidget
	instanceVariableNames: 'pages currentIndex buttonContainer animated animationDuration animationDirection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiStackedWidget class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:14:45.977'!
widgetClassIcon

	^ (Smalltalk at: #UiFugueIcons ifAbsent: [^ nil]) uiLayeredPaneIcon! !

!UiStackedWidget class methodsFor: 'accessing' stamp: 'mt 10/19/2010 14:05:23.132'!
widgetClassName

	^ 'Stacked Widget'! !

!UiStackedWidget methodsFor: 'edit mode - dropping/grabbing' stamp: 'mt 11/6/2012 13:21:27.938'!
acceptDroppingMorph: aMorph event: evt

	(aMorph isKindOf: TransferMorph)
		ifTrue: [(aMorph passenger first at: #class ifAbsent: []) ifNotNil: [:widgetClass |
			| widget |
			widget := widgetClass new position: evt position.
			self currentPage addMorphBack: widget.
			(widget isKindOf: UiContainer)
				ifTrue: [widget editMode: self editMode].
				
			self
				changeFocusTo: widget bounds;
				changeSelectionTo: widget]]
		ifFalse: [
			self currentPage addMorphBack: aMorph.
			
			self
				changeFocusTo: aMorph bounds;
				changeSelectionTo: aMorph].
		
	"self updateCurrentPage."! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 11/6/2012 16:20:15.724'!
addChild: aMorph

	^ self addPage: aMorph! !

!UiStackedWidget methodsFor: 'add/remove' stamp: 'mt 11/6/2012 16:41:58.652'!
addPage
	"Add an empty page."

	self addPage: (UiWidgetPage new
		editMode: self editMode).! !

!UiStackedWidget methodsFor: 'add/remove' stamp: 'mt 5/8/2013 12:05:23.465'!
addPage: aMorph

	self
		addPageSilently: aMorph;	
		currentIndex: self pages size.! !

!UiStackedWidget methodsFor: 'add/remove' stamp: 'mt 5/8/2013 12:05:07.125'!
addPageSilently: aMorph

	aMorph
		hResizing: #spaceFill;
		vResizing: #spaceFill.

	self pages add: aMorph.! !

!UiStackedWidget methodsFor: 'updating' stamp: 'mt 3/6/2013 11:21:19.268'!
animatePageTurnFrom: oldIndex to: newIndex
	"Turn the page and animate."
	
	| buffer canvas start end anim transition |
	buffer := Form
		extent: self width * ((newIndex - oldIndex) abs + 1) @ self height
		depth: 32.
	canvas := buffer getCanvas.
	start := {oldIndex.newIndex} min.
	end := {oldIndex.newIndex} max.
		
	start to: end do: [:i |
		canvas
			translateBy: self topLeft negated + ((self width * (i - start)) @ 0)
			during: [:c | (self pages at: i)
				bounds: self bounds;
				fullDrawOn: c]].
	
	transition := buffer asMorph.
	transition disableTableLayout: true.
	newIndex > oldIndex
		ifTrue: [transition topLeft: self topLeft]
		ifFalse: [transition topRight: self topRight].
	self addMorphFront: transition.
		
	anim := (AnimPropertyAnimation
		on: transition
		property: (newIndex > oldIndex ifTrue: [#right] ifFalse: [#left])
		start: (newIndex > oldIndex ifTrue: [transition right] ifFalse: [transition left])
		end: (newIndex > oldIndex ifTrue: [self right] ifFalse: [self left])
		duration: self animationDuration).
	anim easingCurve conversionBlock: [:x | (-1 * (x-1 raisedTo: 6)) + 1].
	
	anim
		register;
		finishBlock: [transition delete];
		start: #deleteWhenFinished.! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 11/7/2012 13:36:39.121'!
animated
	"Animate page turns."
	
	^ animated ifNil: [animated := false]! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 11/7/2012 13:36:29.526'!
animated: aBoolean

	animated := aBoolean.! !

!UiStackedWidget methodsFor: 'as yet unclassified' stamp: 'mt 6/28/2013 16:26:47.777'!
animationDirection

	^ animationDirection ifNil: [#auto]! !

!UiStackedWidget methodsFor: 'as yet unclassified' stamp: 'mt 6/28/2013 16:27:22.855'!
animationDirection: aSymbol
	"#auto, #leftToRight"

	animationDirection := aSymbol.! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 11/7/2012 16:30:06.518'!
animationDuration

	^ animationDuration ifNil: [animationDuration := 250]! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 11/7/2012 16:29:50.785'!
animationDuration: milliseconds

	animationDuration := milliseconds.! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 11/6/2012 16:10:37.003'!
children

	^ self pages! !

!UiStackedWidget methodsFor: 'add/remove' stamp: 'mt 11/7/2012 11:09:52.342'!
clear

	self pages removeAll.
	
	(self pages isEmpty and: [self editMode])
		ifTrue: [self addPage].
	
	self currentPage ifNil: [self currentIndex: self pages size].
	self updateCurrentPage.! !

!UiStackedWidget methodsFor: 'edit mode - event handling' stamp: 'mt 10/23/2010 10:03:25.639'!
click: evt

	| clickedMorph |
	clickedMorph := self currentPage submorphs
		detect: [:m | m bounds containsPoint: evt position] ifNone: [self].
	
	clickedMorph == self
		ifTrue: [self
			changeFocusTo: self currentPage bounds;
			changeSelectionTo: self currentPage]
		ifFalse: [self
			changeFocusTo: clickedMorph bounds;
			changeSelectionTo: clickedMorph].! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 11/6/2012 11:50:20.162'!
currentIndex

	^ currentIndex ifNil: [currentIndex := 0]! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 5/8/2013 13:37:54.273'!
currentIndex: anInteger
	"Change the current page."

	| old |
	(anInteger < 1
		or: [anInteger > self pages size]) ifTrue: [^ self].
	
	old := currentIndex.
	currentIndex := anInteger.
	self updateCurrentPage.
	
	(self animated and: [old notNil])
		ifTrue: [self animatePageTurnFrom: old to: anInteger].! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 11/6/2012 11:50:41.168'!
currentPage

	^ self pages at: self currentIndex ifAbsent: []! !

!UiStackedWidget methodsFor: 'signals' stamp: 'mt 6/27/2013 09:45:24.341'!
currentPageChanged
	"Emitted whenever the currently visible page changes."
	
	self emit.! !

!UiStackedWidget methodsFor: 'edit mode - accessing' stamp: 'mt 7/5/2013 08:36:10.031'!
editMode: aBoolean

	| result |
	result := super editMode: aBoolean.
	
	aBoolean ifTrue: [
		"Initialize control bar."
		buttonContainer ifNil: [self initializeEditButtons].
		"Ensure at least one page."
		self currentPage ifNil: [self addPage].
		"Set edit mode in all available pages."
		self pages do: [:pg | pg editMode: aBoolean].
		"Show edit bar if hidden."
		self updateEditButtons].
	
	aBoolean ifFalse: [
		buttonContainer delete].
	
	^ result! !

!UiStackedWidget methodsFor: 'initialization' stamp: 'mt 11/7/2012 15:10:28.942'!
initialize

	super initialize.
	
	self
		changeTableLayout;
		layoutInset: 0;  
		clipSubmorphs: true.! !

!UiStackedWidget methodsFor: 'initialization' stamp: 'mt 7/15/2013 08:31:06.19'!
initializeEditButtons

	| nextPageButton previousPageButton addPageButton removePageButton stackButton |
	nextPageButton := UiToolButton new
		autoRaise: true;
		icon: UiButtonIcons bulletArrowRightSunkenIcon;
		buttonStyle: #iconOnly;
		setBalloonText: 'Next Page'.

	self connect: nextPageButton signal: #clicked: toSelector: #nextPage.
		
	previousPageButton := UiToolButton new
		autoRaise: true;
		icon: UiButtonIcons bulletArrowLeftSunkenIcon;
		buttonStyle: #iconOnly;
		setBalloonText: 'Previous Page'.
	
	self connect: previousPageButton signal: #clicked: toSelector: #previousPage.

	addPageButton := UiToolButton new
		autoRaise: true;
		icon: UiButtonIcons plusSmallIcon;
		buttonStyle: #iconOnly;
		setBalloonText: 'Insert Page'.

	self connect: addPageButton signal: #clicked: toSelector: #insertPage.

	removePageButton := UiToolButton new
		autoRaise: true;
		icon: UiButtonIcons minusSmallIcon;
		buttonStyle: #iconOnly;
		setBalloonText: 'Remove Page'.

	self connect: removePageButton signal: #clicked: toSelector: #removeCurrentPage.

	stackButton := UiToolButton new
		autoRaise: true;
		icon: UiButtonIcons icon0418Small;
		buttonStyle: #iconOnly;
		setBalloonText: 'Select stack'.

	self connect: stackButton signal: #clicked: toSelector: #updateSelfSelection.
	
	buttonContainer := UiWidgetPageButtonContainer new.
	
	buttonContainer
		addMorphBack: stackButton;  
		addMorphBack: addPageButton;
		addMorphBack: removePageButton;
		addMorphBack: previousPageButton;
		addMorphBack: nextPageButton.! !

!UiStackedWidget methodsFor: 'initialization' stamp: 'mt 5/8/2013 12:06:05.716'!
initializePages: somePages startIndex: anInteger

	| oldAnimated |
	oldAnimated := self animated.
	
	self
		clear;
		animated: false.
		
	somePages do: [:p | self addPageSilently: p].
	
	self
		currentIndex: anInteger;
		animated: oldAnimated.  
		! !

!UiStackedWidget methodsFor: 'add/remove' stamp: 'mt 7/15/2013 08:32:27.017'!
insertPage
	"Morphic designer only."
	
	self
		insertPage: (UiWidgetPage new editMode: self editMode)
		behind: self currentPage.! !

!UiStackedWidget methodsFor: 'add/remove' stamp: 'mt 7/15/2013 08:30:26.969'!
insertPage: aMorph behind: anotherPage

	aMorph 
		hResizing: #spaceFill;
		vResizing: #spaceFill.

	self pages add: aMorph after: anotherPage.
	self currentIndex: (self pages indexOf: aMorph).! !

!UiStackedWidget methodsFor: 'add/remove' stamp: 'mt 7/15/2013 08:30:22.386'!
insertPage: aMorph inFrontOf: anotherPage

	aMorph 
		hResizing: #spaceFill;
		vResizing: #spaceFill.

	self pages add: aMorph before: anotherPage.
	self currentIndex: (self pages indexOf: aMorph).! !

!UiStackedWidget methodsFor: 'actions' stamp: 'mt 7/13/2010 10:15:59.084'!
nextPage

	self currentIndex: (self currentIndex + 1 min: self pages size).! !

!UiStackedWidget methodsFor: 'accessing' stamp: 'mt 7/13/2010 09:36:45.708'!
pages

	^ pages ifNil: [pages := OrderedCollection new]! !

!UiStackedWidget methodsFor: 'actions' stamp: 'mt 7/13/2010 10:16:13.279'!
previousPage

	self currentIndex: (self currentIndex - 1 max: 1).! !

!UiStackedWidget methodsFor: 'add/remove' stamp: 'mt 11/6/2012 13:27:38.867'!
removeCurrentPage

	self currentPage ifNil: [^ self].
	self pages remove: self currentPage.
	
	(self pages isEmpty and: [self editMode])
		ifTrue: [self addPage].
	
	self currentPage ifNil: [self currentIndex: self pages size].
	self updateCurrentPage.! !

!UiStackedWidget methodsFor: 'edit mode - event handling' stamp: 'mt 7/13/2010 10:09:25.305'!
startDrag: evt

	| morphToDrag |
	morphToDrag := self currentPage submorphs
		detect: [:m | m bounds containsPoint: evt position]
		ifNone: [self dragEnabled
			ifTrue: [self]
			ifFalse: [nil]].
			
	morphToDrag ifNil: [^ self].
	
	evt redButtonPressed
		ifTrue: [evt hand grabMorph: morphToDrag].! !

!UiStackedWidget methodsFor: 'updating' stamp: 'mt 6/27/2013 09:45:58.174'!
updateCurrentPage

	self removeAllMorphs.
	
	self currentPage ifNil: [^ self].
	self addMorphBack: self currentPage.
	
	self
		updateCurrentPageSelection;
		updateEditButtons.
		
	self currentPageChanged.! !

!UiStackedWidget methodsFor: 'edit mode - updating' stamp: 'mt 11/6/2012 13:11:08.697'!
updateCurrentPageSelection

	self editMode ifFalse: [^ self].

	self
		changeFocusTo: self currentPage bounds;
		changeSelectionTo: self currentPage.! !

!UiStackedWidget methodsFor: 'edit mode - updating' stamp: 'mt 11/6/2012 16:02:22.598'!
updateEditButtons
	"Ensure that the edit buttons are visible if in edit mode. Hide otherwise."

	self editMode
		ifTrue: [self addMorphFront: buttonContainer].! !

!UiStackedWidget methodsFor: 'edit mode - updating' stamp: 'mt 11/6/2012 15:59:11.993'!
updateSelfSelection

	self editMode ifFalse: [^ self].

	self
		changeFocusTo: self bounds;
		changeSelectionTo: self.! !

UiContainer subclass: #UiWidgetPage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiWidgetPage commentStamp: 'mt 7/14/2010 11:14:35.743' prior: 0!
I am used in a UiStackedWidget.!

!UiWidgetPage class methodsFor: 'testing' stamp: 'mt 3/5/2013 10:37:04.77'!
isWidgetClass

	^ false! !

!UiWidgetPage class methodsFor: 'accessing' stamp: 'mt 3/5/2013 10:36:52.144'!
widgetClassName

	^ 'Widget Page'! !

!UiWidgetPage class methodsFor: 'properties' stamp: 'mt 11/6/2012 16:47:05.622'!
widgetProperties

	^ super widgetProperties
	
	"{{'Stacked Widget - Layout'. #owner} -> {
		#topLeft -> Point.
		#width -> Integer.
		#height -> Integer.
		#hResizing -> #(rigid spaceFill shrinkWrap).
		#vResizing -> #(rigid spaceFill shrinkWrap)}.
		{'Stacked Widget - Pages'. #owner} -> {
			#currentIndex -> Integer}}"! !

!UiWidgetPage class methodsFor: 'properties' stamp: 'mt 7/16/2010 11:29:42.685'!
widgetPropertiesToIgnore

	^ #(hResizing vResizing width height topLeft)! !

!UiWidgetPage methodsFor: 'accessing' stamp: 'mt 11/6/2012 16:41:28.529'!
editMode: aBoolean

	super editMode: aBoolean.  
	
	self dragEnabled: false.! !

TextMorph subclass: #UiMessage
	instanceVariableNames: 'killTime textColor closeButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiMessage commentStamp: 'mt 3/28/2011 12:25:57.85' prior: 0!
This is a textmorph that shows a short message for a short time and then disappears.!

!UiMessage class methodsFor: 'instance creation' stamp: 'mt 6/18/2010 10:11:19.893'!
error: aText

	^ self new
		contents: aText;
		color: Color red darker;
		textColor: Color yellow;
		fadeIn! !

!UiMessage class methodsFor: 'examples' stamp: 'mt 3/28/2011 12:20:31.259'!
example01Warning

	(UiMessage warning: 'Your <b>chocolate level</b> is <b>low</b>.' asHtmlText)
		center: ActiveWorld center;
		openInWorld.! !

!UiMessage class methodsFor: 'examples' stamp: 'mt 3/28/2011 12:22:01.003'!
example02Error

	(UiMessage error: '<b>Seriously...</b>
	
Do you really think you are working hard enough? =D' asHtmlText)
		center: ActiveWorld center;
		openInWorld.! !

!UiMessage class methodsFor: 'examples' stamp: 'mt 3/28/2011 12:22:47.475'!
example03Information

	(UiMessage information: 'Try to click on me and see what happens.')
		center: ActiveWorld center;
		openInWorld.! !

!UiMessage class methodsFor: 'instance creation' stamp: 'mt 6/18/2010 09:57:21.514'!
information: aText

	^ self new
		contents: aText;
		color: Color gray;
		fadeIn! !

!UiMessage class methodsFor: 'instance creation' stamp: 'mt 6/18/2010 10:11:24.471'!
warning: aText

	^ self new
		contents: aText;
		color: Color yellow darker;
		textColor: Color black;
		fadeIn! !

!UiMessage class methodsFor: 'as yet unclassified' stamp: 'mt 9/15/2013 10:00:48.155'!
warning: text at: point

	^ (self warning: text)
		popUpAt: point! !

!UiMessage class methodsFor: 'as yet unclassified' stamp: 'mt 9/15/2013 10:01:35.137'!
warning: text for: morph

	^ self
		warning: text
		at: morph boundsInWorld bottomLeft! !

!UiMessage methodsFor: 'drawing' stamp: 'mt 7/8/2013 17:06:07.307'!
drawOn: aCanvas
	
	| gradient fauxBounds |	
	gradient := (GradientFillStyle ramp: {
			0.0 -> Color white.
			0.1-> (self color adjustBrightness: 0.05).
			0.6 -> (self color darker)})
		origin: self topLeft;
		direction: 0@ self height.
		
	UiPlatform current
		canvas: aCanvas
		fillRoundRect: (self bounds insetBy: (1@1 corner: 1@1))
		radius: 3
		fillStyle: gradient;
		canvas: aCanvas
		frameRoundRect: self bounds
		radius: 3
		width: 1
		color: self color muchDarker.
		
	self setDefaultContentsIfNil.
	(self startingIndex > text size)
		ifTrue: [self drawNullTextOn: aCanvas].
	"Hack here:  The canvas expects bounds to carry the location of the text, but we also need to communicate clipping."
	fauxBounds := self bounds topLeft corner: self innerBounds bottomRight.
	aCanvas paragraph: self paragraph bounds: fauxBounds color: textColor! !

!UiMessage methodsFor: 'actions' stamp: 'mt 6/19/2010 19:38:47.167'!
fadeOutAndDelete

	self fadeOutThen: [self delete].! !

!UiMessage methodsFor: 'initialization' stamp: 'mt 3/28/2011 12:13:29.182'!
initialize

	super initialize.
	killTime := Time now addSeconds: 5.
	
	self
		color: Color gray;
		textColor: Color white;
		margins: (3@1 corner: 3@0);
		startStepping.! !

!UiMessage methodsFor: 'event handling' stamp: 'mt 3/28/2011 12:13:17.647'!
mouseDown: aMouseEvent

	self stopFadeOut.
	super mouseDown: aMouseEvent.  ! !

!UiMessage methodsFor: 'open' stamp: 'mt 3/28/2011 12:24:49.34'!
popUp

	self
		topLeft: ActiveHand position + (ActiveHand width @ 0);
		openInWorld.! !

!UiMessage methodsFor: 'open' stamp: 'mt 3/28/2011 12:25:20.397'!
popUpAt: aPoint

	self
		topLeft: aPoint;
		openInWorld.! !

!UiMessage methodsFor: 'updating' stamp: 'mt 6/19/2010 19:38:56.537'!
step
	
	killTime <= Time now ifTrue: [self stopStepping; fadeOutAndDelete]! !

!UiMessage methodsFor: 'updating' stamp: 'mt 6/15/2010 20:08:09.113'!
stepTime

	^ 100! !

!UiMessage methodsFor: 'actions' stamp: 'mt 8/21/2012 10:20:12.051'!
stopFadeOut

	self stopStepping.

	closeButton ifNil: [closeButton :=
		UiToolButton new
			icon: (UiButtonIcons crossSmallSunkenIcon clippedTo: 10@16);
			buttonStyle: #iconOnly;
			color: (Color gray: 0.85);
			autoRaise: true.
		self connect: closeButton signal: #clicked: toSelector: #fadeOutAndDelete.
		self addMorph: closeButton.
		
		closeButton center: self center - (((self width / 2) + (closeButton width / 2)) @ 0).
		closeButton fadeIn].! !

!UiMessage methodsFor: 'accessing' stamp: 'mt 6/18/2010 08:48:07.306'!
textColor: aColor

	textColor := aColor.! !

TextMorph subclass: #UiTextMorph
	instanceVariableNames: 'autoAccept autoConvert readOnly editView acceptOnCR revertOnEscape acceptOnFocusLost converter styler selection cursor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

UiTextMorph subclass: #UiTextLineMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiTextLineMorph methodsFor: 'private' stamp: 'mt 7/18/2012 16:04'!
editorClass

	^ UiTextLineEditor! !

!UiTextMorph class methodsFor: 'instance creation' stamp: 'mt 3/22/2011 18:09:32.392'!
on: aScrollArea

	^ self new editView: aScrollArea! !

!UiTextMorph methodsFor: 'editing' stamp: 'mt 4/19/2011 10:47:04.648'!
acceptContents

	super acceptContents.

	self
		convertText;
		accepted.! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:35:26.776'!
acceptOnCR

	^ acceptOnCR ifNil: [false]! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:35:40.691'!
acceptOnCR: aBoolean

	acceptOnCR := aBoolean.! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/23/2011 09:21:22.49'!
acceptOnFocusLost
	"Should the contents be accepted if the keyboard focus gets lost?"
	
	^ acceptOnFocusLost ifNil: [acceptOnFocusLost := false]! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/23/2011 09:21:04.688'!
acceptOnFocusLost: aBoolean

	acceptOnFocusLost := aBoolean.! !

!UiTextMorph methodsFor: 'signals' stamp: 'mt 3/23/2011 09:07:04.136'!
accepted

	self emit.! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:18:27.012'!
autoAccept

	^ autoAccept ifNil: [autoAccept := false]! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:18:32.127'!
autoAccept: anObject

	autoAccept := anObject.! !

!UiTextMorph methodsFor: 'converting - accessing' stamp: 'mt 3/22/2011 17:43:15.147'!
autoConvert

	^ autoConvert ifNil: [autoConvert := false]! !

!UiTextMorph methodsFor: 'converting - accessing' stamp: 'mt 3/22/2011 17:43:31.18'!
autoConvert: aBoolean
	"If enabled, converts the text after each keystroke that changes the contents."
	
	autoConvert := aBoolean.! !

!UiTextMorph methodsFor: 'converting' stamp: 'mt 3/23/2011 09:35:49.492'!
convertText
	
	| v |
	v := self convertedValue.

	self newContents: self convertedText.
	self editView updateScrollBars.
	
	self valueChanged: v.  ! !

!UiTextMorph methodsFor: 'converting' stamp: 'mt 4/19/2011 10:46:52.184'!
convertedText

	| v |
	v := self convertedValue.
	(v isString or: [v isText])
		ifTrue: [^ v]
		ifFalse: [^ v asString].
	! !

!UiTextMorph methodsFor: 'converting' stamp: 'mt 4/19/2011 10:43:16.211'!
convertedValue
	"Return the converted value nevertheless if it is a string or not."
	
	^ self converter value: self text! !

!UiTextMorph methodsFor: 'converting - accessing' stamp: 'mt 3/22/2011 17:44:16.966'!
converter
	"Used for conversion purposes of input data. The default converter is the indentity function."

	^ converter ifNil: [converter := [:raw | raw]]! !

!UiTextMorph methodsFor: 'converting - accessing' stamp: 'mt 3/22/2011 17:44:59.158'!
converter: aUnaryBlock
	"Sets the converter to a block that accepts one argument and returns a string."
	
	converter := aUnaryBlock.! !

!UiTextMorph methodsFor: 'drawing' stamp: 'mt 3/21/2011 16:28:02.463'!
drawNullTextOn: aCanvas

	"Invisible."! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/22/2011 17:37:49.311'!
editView
	"The associated scroll area that shows part of this text morph."
	
	^ editView! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/22/2011 17:38:03.872'!
editView: aScrollArea

	editView := aScrollArea.! !

!UiTextMorph methodsFor: 'private' stamp: 'mt 4/6/2011 14:54:02.557'!
editorClass

	^ UiTextEditor! !

!UiTextMorph methodsFor: 'signals' stamp: 'mt 7/8/2012 14:26:54.298'!
escapePressed: discardedText

	self emit.! !

!UiTextMorph methodsFor: 'signals' stamp: 'mt 3/22/2011 10:44:35.41'!
focusChanged: aRectangle
	"Signal the current area of interest."
	
	self emit.! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/24/2011 10:47:16.57'!
font: aFont

	super font: aFont.
	
	"Store given font index as default font in text style."
	textStyle defaultFontIndex: (textStyle fontIndexOf: aFont).
	
	self styler ifNotNil: [self styler font: aFont].! !

!UiTextMorph methodsFor: 'events-processing' stamp: 'mt 3/22/2011 17:41:12.909'!
handleKeystroke: anEvent 
	"Overridden to avoid tab handling."
	
	anEvent wasHandled
		ifTrue: [^ self].
	(self handlesKeyboard: anEvent)
		ifFalse: [^ self].
	anEvent wasHandled: true.
	^ self keyStroke: anEvent! !

!UiTextMorph methodsFor: 'initialization' stamp: 'mt 3/21/2011 16:59:22.374'!
initialize

	super initialize.
	self
		contents: '';
		autoFit: true;
		wrapFlag: true;
		margins: (3@0 corner: 3@0);
		installEditorToReplace: nil.! !

!UiTextMorph methodsFor: 'initialization' stamp: 'mt 6/16/2013 18:29:02.131'!
initializeParagraph: aParagraph
	"Hook to set some properties in the paragraph after creation."! !

!UiTextMorph methodsFor: 'private' stamp: 'mt 11/1/2012 16:49:07.508'!
installEditorToReplace: priorEditor

	| newEditor |
	newEditor := super installEditorToReplace: priorEditor.
	selection ifNotNil: [newEditor selectInterval: selection].

	"Completion."
	self editView ifNotNil: [:view |
		newEditor model: view model].

	^ newEditor! !

!UiTextMorph methodsFor: 'event handling' stamp: 'mt 5/24/2011 10:04:34.358'!
keyDown: aKeyboardEvent

	self editView keyDown: aKeyboardEvent.
	^ super keyDown: aKeyboardEvent.! !

!UiTextMorph methodsFor: 'event handling' stamp: 'mt 8/29/2013 11:01:18.624'!
keyStroke: aKeyboardEvent

	| old textEdited |
	old := self text asString.
	
	"Mouse wheel handling in view only."
	((aKeyboardEvent isKeystroke and: [aKeyboardEvent controlKeyPressed]) and: [
		aKeyboardEvent keyCharacter = Character arrowUp or: [
		aKeyboardEvent keyCharacter = Character arrowDown]])
		ifTrue: [^ self editView keyStroke: aKeyboardEvent].

	"Must be here and not in #handlesKeyboard: to catch mouse wheel input first. See above."
	self readOnly ifTrue: [
		self flag: #incomplete. "mt: Support more interactions with read-only text."
		(aKeyboardEvent commandKeyPressed and: [aKeyboardEvent keyCharacter = $c])
			ifTrue: [super keyStroke: aKeyboardEvent].
		^ self].
		
	super keyStroke: aKeyboardEvent.
	textEdited := self text ~= old.
	
	self autoConvert
		ifTrue: [self convertText].
	
	"Signal the new text (even if converted) if changed."
	textEdited ifTrue: [self textEdited: self text].
	
	self autoAccept
		ifTrue: [self editor accept].

	(self acceptOnCR and: [{Character lf. Character cr. Character enter} includes: aKeyboardEvent keyCharacter])
		ifTrue: [self editor accept. self returnPressed: self text].

	(self revertOnEscape and: [aKeyboardEvent keyCharacter == Character escape])
		ifTrue: [self revertContents. self escapePressed: self text].

	"Trigger keyboard navigation in scrollable view."
	self focusChanged: (UiPlatform current textEditorPointBlock: self editor).
	
	textEdited ifTrue: [self style].
	
	"Allow view to handle shortcuts too."
	self editView keyStroke: aKeyboardEvent.! !

!UiTextMorph methodsFor: 'event handling' stamp: 'mt 5/24/2011 10:04:57.838'!
keyUp: aKeyboardEvent

	self editView keyUp: aKeyboardEvent.
	^ super keyUp: aKeyboardEvent.! !

!UiTextMorph methodsFor: 'event handling' stamp: 'mt 8/29/2013 10:45:04.768'!
keyboardFocusChange: aBoolean

	self editView
		keyboardFocusChange: aBoolean.

	paragraph ifNotNil: [:p | p focused: aBoolean].
	aBoolean
		ifTrue: [self editor; startBlinking]
		ifFalse: [self stopBlinking].
	
	(aBoolean not & self readOnly not and: [self acceptOnFocusLost])
		ifTrue: [self editor accept].! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 4/28/2011 13:02:00.15'!
lineCount

	^ self text asString lines size! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 4/28/2011 13:48:54.043'!
lineCountComposed
	"Answer the visual lines which could differ from #lineCount because the text composer did some linebreaks."

	^ self height // self lineHeight! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 4/28/2011 14:00:07.33'!
lineHeight
	"Return the height of one line in pixel."
	
	self flag: #tofix. "calculation is wrong"
	^ self textStyle leading + self textStyle lineGrid + 1.1! !

!UiTextMorph methodsFor: 'event handling' stamp: 'mt 9/4/2012 11:28:12.305'!
mouseDown: evt
	
	evt yellowButtonPressed ifTrue: [
		^ self yellowButtonActivity: evt shiftPressed].
	
	self grabKeyboard.
	
	editor ifNotNil: [:e |
		self
			handleInteraction: [e mouseDown: evt]
			fromEvent: evt].! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/24/2011 10:37:29.489'!
newContents: stringOrText
	"Replace all line ending characters if #acceptOnCR is true. This makes sense because the user cannot type these characters anyway."

	self acceptOnCR
		ifTrue: [super newContents: stringOrText asString withoutLineEndings]  
		ifFalse: [super newContents: stringOrText].! !

!UiTextMorph methodsFor: 'private' stamp: 'mt 6/16/2013 18:29:19.585'!
paragraph

	paragraph ifNil: [
		"Paragraph will be lazy-instatiated."
		self initializeParagraph: super paragraph].
	^ super paragraph! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:18:36.853'!
readOnly

	^ readOnly ifNil: [readOnly := false]! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 3/22/2011 11:18:41.109'!
readOnly: aBoolean

	readOnly := aBoolean.! !

!UiTextMorph methodsFor: 'private' stamp: 'mt 6/19/2013 11:55:55.868'!
recompose
	"Re-compose the current contents."
	
	self
		releaseParagraph;
		paragraph;
		updateFromParagraph;
		style.! !

!UiTextMorph methodsFor: 'private' stamp: 'mt 9/6/2012 10:48:44.75'!
releaseEditor

	editor ifNotNil: [
		selection := self editor selectionInterval].
	
	super releaseEditor.! !

!UiTextMorph methodsFor: 'signals' stamp: 'mt 7/8/2012 14:26:39.589'!
returnPressed: acceptedText

	self emit.! !

!UiTextMorph methodsFor: 'editing' stamp: 'mt 7/8/2012 14:16:53.773'!
revertContents

	self reverted.! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 7/8/2012 14:13:33.022'!
revertOnEscape

	^ revertOnEscape ifNil: [false]! !

!UiTextMorph methodsFor: 'accessing' stamp: 'mt 7/8/2012 14:13:48.719'!
revertOnEscape: aBoolean

	revertOnEscape := aBoolean.! !

!UiTextMorph methodsFor: 'signals' stamp: 'mt 7/8/2012 14:16:26.745'!
reverted

	self emit.! !

!UiTextMorph methodsFor: 'styling' stamp: 'mt 4/28/2011 10:49:59.607'!
style

	self styler ifNil: [^ self].
	
	selection := self editor selectionInterval.
	
	self newContents: (self styler format: self contents).
	
	self contents size < 4096
		ifTrue: [self styler style: self contents]
		ifFalse: [self styler styleInBackgroundProcess: self contents].! !

!UiTextMorph methodsFor: 'styling - accessing' stamp: 'mt 3/24/2011 10:07:52.802'!
styler

	^ styler! !

!UiTextMorph methodsFor: 'styling - accessing' stamp: 'mt 3/24/2011 10:21:28.406'!
styler: aTextStyler

	styler := aTextStyler.
	
	"Setup target for callback functinos."
	styler ifNotNil: [styler
			view: self;
			font: textStyle defaultFont].! !

!UiTextMorph methodsFor: 'styling' stamp: 'mt 5/20/2011 16:55:21.425'!
stylerStyled: styledText

	self contents runs: styledText runs.
	self updateFromParagraph.
	
	selection ifNotNil: [:s |
		self editor
			selectInvisiblyFrom: s first
			to: s last;
			storeSelectionInParagraph;
			setEmphasisHere].

	self editor blinkParen.

	self textStyled: self text.! !

!UiTextMorph methodsFor: 'styling' stamp: 'mt 3/24/2011 10:33:33.377'!
stylerStyledInBackground: styledText

	selection := nil.
	
	self contents string = styledText string
		ifTrue: [self stylerStyled: styledText].! !

!UiTextMorph methodsFor: 'signals' stamp: 'mt 3/22/2011 17:51:17.271'!
textEdited: text
	"Sent whenever text changes during user interaction."
	
	self emit.! !

!UiTextMorph methodsFor: 'signals' stamp: 'mt 5/20/2011 16:55:02.741'!
textStyled: text
	"Sent whenever text was styled."
	
	self emit.! !

!UiTextMorph methodsFor: 'private' stamp: 'mt 3/24/2011 10:22:26.21'!
updateFromParagraph

	| oldEditor |
	paragraph ifNil: [^self].
	wrapFlag ifNil: [wrapFlag := true].

	editor ifNotNil: 	[
		oldEditor := editor.
		editor storeSelectionInParagraph].

	text := paragraph text.

	paragraph textStyle ~= textStyle ifTrue: [
		self
			text: text textStyle: paragraph textStyle;
			releaseParagraph].

	self fit.

	"Reinstate selection after, eg, style change"
	editor ifNil: [self installEditorToReplace: oldEditor].! !

!UiTextMorph methodsFor: 'signals' stamp: 'mt 3/23/2011 09:34:32.707'!
valueChanged: convertedValue

	self emit.! !

!UiTextMorph methodsFor: 'event handling' stamp: 'mt 4/22/2011 16:01:19.394'!
yellowButtonActivity: shiftKeyState! !

!Text methodsFor: '*widgets-text' stamp: 'mt 6/22/2010 23:13:43.314'!
asHtmlText
	"No change here."
	
	^ self! !

SmalltalkEditor subclass: #UiTextEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiTextEditor class methodsFor: 'keyboard shortcut tables' stamp: 'mt 8/8/2012 17:54:29.7'!
initializeCmdKeyShortcuts
	"Make space for new shortcuts."
	super initializeCmdKeyShortcuts.

	#($u $t $f $o $e $y $g $w $h $k) do: [ :shortcut | cmdActions at: shortcut asciiValue + 1 put: #noop: ].
		
	cmdActions at: $t asciiValue + 1 put: #traceIt:! !

!UiTextEditor class methodsFor: 'keyboard shortcut tables' stamp: 'lt 8/2/2012 10:46:44.547'!
initializeShiftCmdKeyShortcuts

	super initializeShiftCmdKeyShortcuts.
	
	shiftCmdActions at: $f asciiValue + 1 put: #noop:! !

!UiTextEditor methodsFor: 'accessing' stamp: 'mt 9/11/2012 13:24:10.422'!
selectionBlock

	^ self startBlock quickMerge: self stopBlock! !

!UiTextEditor methodsFor: 'actions' stamp: 'mt 2/15/2013 14:31:47.264'!
traceIt

	| rcvr ctxt |
	self lineSelectAndEmptyCheck: [^ ''].

	(model respondsTo: #doItReceiver) 
		ifTrue: [ rcvr := model doItReceiver.
				ctxt := model doItContext]
		ifFalse: [rcvr := ctxt := nil].

	SystemChangeNotifier uniqueInstance
		traceSourceCode: self selection asString
		context: ctxt
		receiver: rcvr.! !

!UiTextEditor methodsFor: 'editing keys' stamp: 'mt 7/18/2012 16:57:13.703'!
traceIt: characterStream 
	"CMD+T"
	
	Sensor keyboard.	
	self traceIt.
	^ true! !

UiTextEditor subclass: #UiTextLineEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!Object methodsFor: '*widgets-events' stamp: 'mt 2/26/2011 13:45:37.745'!
isEventHandler

	^ false! !

Object subclass: #UiButtonIcons
	instanceVariableNames: ''
	classVariableNames: 'Icons'
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiButtonIcons commentStamp: 'mt 8/21/2012 09:43:59.59' prior: 0!
This support structure allows for being independent from Icons package.!

!UiButtonIcons class methodsFor: 'accessing' stamp: 'mt 8/21/2012 09:48:45.709'!
bulletArrowDownSunkenIcon

	^ Icons
		at: #bulletArrowDownSunkenIcon
		ifAbsentPut:[self bulletArrowDownSunkenIconContents].! !

!UiButtonIcons class methodsFor: 'contents' stamp: 'mt 8/21/2012 09:44:41.359'!
bulletArrowDownSunkenIconContents

	^ (Form
	extent: 8@8
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 2573624934 3429262950 3429262950 3429262950 3429262950 3429262950 2573624934 0 1550214758 3429262950 4284900966 4284900966 4284900966 3429262950 1550214758 0 1560281087 1550214758 3429262950 4284900966 3429262950 1550214758 1560281087 0 0 1560281087 1550214758 3429262950 1550214758 1560281087 0 0 0 0 1560281087 3439329279 1560281087 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)! !

!UiButtonIcons class methodsFor: 'accessing' stamp: 'mt 8/21/2012 09:49:02.52'!
bulletArrowLeftSunkenIcon

	^ Icons
		at: #bulletArrowLeftSunkenIcon
		ifAbsentPut:[self bulletArrowLeftSunkenIconContents].! !

!UiButtonIcons class methodsFor: 'contents' stamp: 'mt 8/21/2012 09:46:51.122'!
bulletArrowLeftSunkenIconContents

	^ (Form
	extent: 8@8
	depth: 32
	fromArray: #( 0 0 0 0 1550214758 2573624934 0 0 0 0 0 1550214758 3429262950 3429262950 0 0 0 0 1550214758 3429262950 4284900966 3429262950 0 0 0 0 3429262950 4284900966 4284900966 3429262950 0 0 0 0 1550214758 3429262950 4284900966 3429262950 0 0 0 0 1560281087 1550214758 3429262950 3429262950 0 0 0 0 0 1560281087 1550214758 2573624934 0 0 0 0 0 0 1560281087 2583691263 0 0)
	offset: 0@0)! !

!UiButtonIcons class methodsFor: 'accessing' stamp: 'mt 8/21/2012 09:49:10.076'!
bulletArrowRightSunkenIcon

	^ Icons
		at: #bulletArrowRightSunkenIcon
		ifAbsentPut:[self bulletArrowRightSunkenIconContents].! !

!UiButtonIcons class methodsFor: 'contents' stamp: 'mt 8/21/2012 09:46:34.98'!
bulletArrowRightSunkenIconContents

	^ (Form
	extent: 8@8
	depth: 32
	fromArray: #( 0 0 2573624934 1550214758 0 0 0 0 0 0 3429262950 3429262950 1550214758 0 0 0 0 0 3429262950 4284900966 3429262950 1550214758 0 0 0 0 3429262950 4284900966 4284900966 3429262950 0 0 0 0 3429262950 4284900966 3429262950 1550214758 0 0 0 0 3429262950 3429262950 1550214758 1560281087 0 0 0 0 2573624934 1550214758 1560281087 0 0 0 0 0 2583691263 1560281087 0 0 0 0)
	offset: 0@0)! !

!UiButtonIcons class methodsFor: 'accessing' stamp: 'mt 8/21/2012 09:48:55.159'!
bulletArrowUpSunkenIcon

	^ Icons
		at: #bulletArrowUpSunkenIcon
		ifAbsentPut:[self bulletArrowUpSunkenIconContents].! !

!UiButtonIcons class methodsFor: 'contents' stamp: 'mt 8/21/2012 09:45:56.471'!
bulletArrowUpSunkenIconContents

	^ (Form
	extent: 8@8
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1550214758 3429262950 1550214758 0 0 0 0 1550214758 3429262950 4284900966 3429262950 1550214758 0 0 1550214758 3429262950 4284900966 4284900966 4284900966 3429262950 1550214758 0 2573624934 3429262950 3429262950 3429262950 3429262950 3429262950 2573624934 0 2583691263 3439329279 3439329279 3439329279 3439329279 3439329279 2583691263 0 0 0 0 0 0 0 0 0)
	offset: 0@0)! !

!UiButtonIcons class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:21:39.94'!
crossSmallSunkenIcon

	^ Icons
		at: #crossSmallSunkenIcon
		ifAbsentPut:[self crossSmallSunkenIconContents].! !

!UiButtonIcons class methodsFor: 'contents' stamp: 'mt 8/21/2012 10:20:31.193'!
crossSmallSunkenIconContents

	^ (Form
	extent: 16@16
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1214670438 3429262950 1214670438 0 1214670438 3429262950 1214670438 0 0 0 0 0 0 0 0 0 3429262950 4284900966 3429262950 2070308454 3429262950 4284900966 3429262950 0 0 0 0 0 0 0 0 0 1214670438 3429262950 4284900966 3429262950 4284900966 3429262950 862348902 0 0 0 0 0 0 0 0 0 1224736767 2070308454 3429262950 4284900966 3429262950 1902536294 872415231 0 0 0 0 0 0 0 0 0 1214670438 3429262950 4284900966 3429262950 4284900966 3429262950 1214670438 0 0 0 0 0 0 0 0 0 3429262950 4284900966 3429262950 1902536294 3429262950 4284900966 3429262950 0 0 0 0 0 0 0 0 0 1214670438 3429262950 862348902 1912602623 1214670438 3429262950 1214670438 0 0 0 0 0 0 0 0 0 1224736767 3439329279 872415231 0 1224736767 3439329279 1224736767 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)! !

!UiButtonIcons class methodsFor: 'accessing' stamp: 'mt 11/6/2012 15:56:48.704'!
icon0418Small

	^ Icons
		at: #diagonaIcon0418Small
		ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self icon0418SmallContents readStream) ].! !

!UiButtonIcons class methodsFor: 'contents' stamp: 'mt 11/6/2012 15:56:25.462'!
icon0418SmallContents
	"Auto-generated.

	Diagona Icons 1.0
	 2007 Yusuke Kamiyamane
	Website: http://p.yusukekamiyamane.com
	License: http://creativecommons.org/licenses/by/3.0"
	
	^ 'iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAADiSURBVHjahJC9CsIwFIWP4mAV
hwiC0kEEfYKgg1MrOPoafR7n+iKCgwXnjN1cdBAHf6BisH/G3lSFgmDgktxzvpxcUlJKwRVY
Q7hWf+rgvhOgVe9ybFcuwB3P4bDLJCZJYgkhMOllze2gi86kkadvUuJ8I1WSqp9FHjEVgqMo
Qvw04O+vkFIiCAK9zyyuPcDIwTAMEaXAwGRZR2Xq10gjj1YB3B2LifaY/wY7rT+J+YzA6VxM
HA0/M77BOI6RZiBjTJdp5omkkfcFs7/yFkvfarMamo2qNi63B45XqT3qXwIMAO/0rXdCOjiV
AAAAAElFTkSuQmCC'! !

!UiButtonIcons class methodsFor: 'class initialization' stamp: 'mt 8/21/2012 09:55:39.318'!
initialize

	Icons := IdentityDictionary new.
	Smalltalk addToStartUpList: self.! !

!UiButtonIcons class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:30:10.046'!
minusSmallIcon

	^ Icons
		at: #minusSmallIcon
		ifAbsentPut:[self minusSmallIconContents].! !

!UiButtonIcons class methodsFor: 'contents' stamp: 'mt 8/21/2012 10:29:38.998'!
minusSmallIconContents

	^ (Form
	extent: 8@8
	depth: 32
	fromArray: #( 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 2577793024 3433496576 3433496576 3433496576 3433496576 3433496576 2577793024 16777215 3432513536 4294401630 4294401630 4294401630 4294401630 4294401630 3432513536 16777215 2575564800 3430940672 3430940672 3430940672 3430940672 3430940672 2575564800 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215)
	offset: 0@0)! !

!UiButtonIcons class methodsFor: 'accessing' stamp: 'mt 8/21/2012 10:30:06.309'!
plusSmallIcon

	^ Icons
		at: #plusSmallIcon
		ifAbsentPut:[self plusSmallIconContents].! !

!UiButtonIcons class methodsFor: 'contents' stamp: 'mt 8/21/2012 10:29:22.486'!
plusSmallIconContents

	^ (Form
	extent: 8@8
	depth: 32
	fromArray: #( 40210 43028 2566957333 3422595349 2566957333 43028 40210 16777215 40210 41235 3422594068 4284804725 3422594068 41235 40210 16777215 2566954002 3422592274 3422592274 4283820902 3422592274 3422592274 2566954002 16777215 3422590224 4282509138 4282509138 4282509138 4282509138 4282509138 3422590224 16777215 2566950158 3422587918 3422587918 4281131581 3422587918 3422587918 2566950158 16777215 35854 34829 3422585612 4279951147 3422585612 34829 35854 16777215 35854 28935 2566941958 3422578949 2566941958 28935 35854 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215)
	offset: 0@0)! !

Object subclass: #UiDocNode
	instanceVariableNames: 'nodes parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Documentation'!

UiDocNode subclass: #UiDocImageNode
	instanceVariableNames: 'form'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Documentation'!

!UiDocImageNode class methodsFor: 'instance creation' stamp: 'mt 11/29/2011 14:51:53.104'!
fromBase64: aString

	^ self fromForm: (Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: aString readStream))! !

!UiDocImageNode class methodsFor: 'instance creation' stamp: 'mt 11/29/2011 14:50:45.437'!
fromForm: aForm

	^ self new
		form: aForm! !

!UiDocImageNode class methodsFor: 'instance creation' stamp: 'mt 11/29/2011 14:51:59.864'!
fromMorph: aMorph

	^ self fromForm: aMorph imageForm! !

!UiDocImageNode class methodsFor: 'instance creation' stamp: 'mt 11/29/2011 14:52:10.709'!
fromMorphClass: aMorphClass

	^ self fromMorph: aMorphClass new! !

!UiDocImageNode methodsFor: 'visitng' stamp: 'mt 12/1/2011 09:21:51.189'!
accept: aDocVisitor

	aDocVisitor visitImageNode: self.! !

!UiDocImageNode methodsFor: 'actions' stamp: 'mt 11/29/2011 14:49:08.614'!
exportAsPngTo: aFileName

	PNGReadWriter
		putForm: self form
		onFileNamed: aFileName.! !

!UiDocImageNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:48:51.368'!
form

	^ form! !

!UiDocImageNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:48:57.477'!
form: aForm

	form := aForm.! !

!UiDocImageNode methodsFor: 'testing' stamp: 'mt 11/29/2011 15:02:07.292'!
isImageNode

	^ true! !

!UiDocNode methodsFor: 'visitng' stamp: 'mt 12/1/2011 09:21:41.068'!
accept: aDocVisitor

	self subclassResponsibility.! !

!UiDocNode methodsFor: 'accessing' stamp: 'mt 12/1/2011 10:25:23.856'!
add: aNode

	self nodes add: aNode.
	aNode parent: self.! !

!UiDocNode methodsFor: 'testing' stamp: 'mt 11/29/2011 15:01:32.153'!
isImageNode

	^ false! !

!UiDocNode methodsFor: 'testing' stamp: 'mt 11/29/2011 15:01:46.546'!
isPropertyNode

	^ false! !

!UiDocNode methodsFor: 'testing' stamp: 'mt 11/29/2011 15:01:52.652'!
isSignalNode

	^ false! !

!UiDocNode methodsFor: 'testing' stamp: 'mt 11/29/2011 15:01:58.645'!
isWidgetNode

	^ false! !

!UiDocNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:32:18.755'!
nodes

	^ nodes ifNil: [nodes := OrderedCollection new]! !

!UiDocNode methodsFor: 'accessing' stamp: 'mt 12/1/2011 10:24:59.476'!
parent

	^ parent! !

!UiDocNode methodsFor: 'accessing' stamp: 'mt 12/1/2011 10:25:04.312'!
parent: aNode

	parent := aNode.! !

UiDocNode subclass: #UiDocPropertyNode
	instanceVariableNames: 'symbol comment type defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Documentation'!

UiDocPropertyNode subclass: #UiDocInheritedPropertyNode
	instanceVariableNames: 'widgetName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Documentation'!

!UiDocInheritedPropertyNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 15:01:05.292'!
widgetName
	^ widgetName! !

!UiDocInheritedPropertyNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 15:01:05.391'!
widgetName: anObject
	widgetName := anObject! !

!UiDocPropertyNode methodsFor: 'visitng' stamp: 'mt 12/1/2011 09:21:55.896'!
accept: aDocVisitor

	aDocVisitor visitPropertyNode: self.! !

!UiDocPropertyNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:35:46.73'!
comment
	^ comment! !

!UiDocPropertyNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:35:46.776'!
comment: anObject
	comment := anObject! !

!UiDocPropertyNode methodsFor: 'accessing' stamp: 'mt 12/1/2011 10:16:49.625'!
defaultValue
	^ defaultValue! !

!UiDocPropertyNode methodsFor: 'accessing' stamp: 'mt 12/1/2011 10:16:49.688'!
defaultValue: anObject
	defaultValue := anObject! !

!UiDocPropertyNode methodsFor: 'convenience' stamp: 'mt 12/1/2011 11:00:22.788'!
defaultValueString

	^ (self defaultValue isString or: [self defaultValue isText])
		ifTrue: ['''{1}''' format: {self defaultValue asString}]
		ifFalse: [self defaultValue asString]! !

!UiDocPropertyNode methodsFor: 'testing' stamp: 'mt 11/29/2011 15:02:18.075'!
isPropertyNode

	^ true! !

!UiDocPropertyNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:53:58.534'!
symbol
	^ symbol! !

!UiDocPropertyNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:53:58.612'!
symbol: anObject
	symbol := anObject! !

!UiDocPropertyNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:35:46.814'!
type
	^ type! !

!UiDocPropertyNode methodsFor: 'accessing' stamp: 'mt 12/1/2011 10:09:57.665'!
type: anObject
	
	type := anObject.! !

!UiDocPropertyNode methodsFor: 'convenience' stamp: 'mt 12/1/2011 10:12:39.821'!
typeString

	^ self type isArray
		ifTrue: ['Symbol']
		ifFalse: [self type asString]! !

UiDocNode subclass: #UiDocSignalNode
	instanceVariableNames: 'symbol comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Documentation'!

!UiDocSignalNode methodsFor: 'visitng' stamp: 'mt 12/1/2011 09:22:02.058'!
accept: aDocVisitor

	aDocVisitor visitSignalNode: self.! !

!UiDocSignalNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:54:30.493'!
comment
	^ comment! !

!UiDocSignalNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:54:30.541'!
comment: anObject
	comment := anObject! !

!UiDocSignalNode methodsFor: 'testing' stamp: 'mt 11/29/2011 15:02:30.832'!
isSignalNode

	^ true! !

!UiDocSignalNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:54:30.352'!
symbol
	^ symbol! !

!UiDocSignalNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:54:30.424'!
symbol: anObject
	symbol := anObject! !

UiDocNode subclass: #UiDocWidgetNode
	instanceVariableNames: 'class'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Documentation'!

!UiDocWidgetNode methodsFor: 'visitng' stamp: 'mt 12/1/2011 09:22:10.917'!
accept: aDocVisitor

	aDocVisitor visitWidgetNode: self.! !

!UiDocWidgetNode methodsFor: 'convenience' stamp: 'mt 12/1/2011 09:40:20.56'!
briefComment

	^ (self comment first: (self comment findString: '.'))
		ifEmpty: [self comment]! !

!UiDocWidgetNode methodsFor: 'convenience' stamp: 'mt 12/1/2011 09:35:45.216'!
comment

	^ self theClass instanceSide organization classComment asString withBlanksTrimmed! !

!UiDocWidgetNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 15:03:09.212'!
imageNodes

	^ self nodes select: [:node | node isImageNode]! !

!UiDocWidgetNode methodsFor: 'testing' stamp: 'mt 11/29/2011 15:02:40.19'!
isWidgetNode

	^ true! !

!UiDocWidgetNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 15:03:16.8'!
propertyNodes

	^ self nodes select: [:node | node isPropertyNode]! !

!UiDocWidgetNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 15:03:24.199'!
signalNodes

	^ self nodes select: [:node | node isSignalNode]! !

!UiDocWidgetNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:48:03.303'!
theClass

	^ class! !

!UiDocWidgetNode methodsFor: 'accessing' stamp: 'mt 11/29/2011 14:48:10.888'!
theClass: aClass

	class := aClass.! !

Object subclass: #UiDocVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Documentation'!

UiDocVisitor subclass: #UiDocTracWikiExporter
	instanceVariableNames: 'out mode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Documentation'!

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 11:15:46.034'!
classLabel: aClass
	
	^ (self isWidgetClass: aClass)
			ifTrue: ['[../{1} {2}]' format: {aClass name asLowercase. aClass name}]
			ifFalse: ['!!', aClass name].! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 11/29/2011 15:41:34.89'!
contents

	^ out contents! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 10:49:17.613'!
initialize

	super initialize.
	out := WriteStream with: ''.! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 11:20:53.149'!
isWidgetClass: aClass

	^ (PackageOrganizer default packageOfClass: aClass ifNone: [ nil ])
		= (PackageInfo named: #Widgets) or: [aClass == Morph]! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 10:04:54.047'!
mode

	^ mode ifNil: [mode := #toc]! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 10:05:08.469'!
mode: aSymbol
	"#toc, #content"

	mode := aSymbol.! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 11:16:03.559'!
nextPutClassLabel: aClass
	"Make a link for widget classes and none for others."
	
	out nextPutAll: (self classLabel: aClass).! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 15:36:54.801'!
nextPutWidgetDetails: aWidgetNode

	self mode: #details.
	
	out
		cr;
		nextPutAll: '== Detailed Description == #details';
		cr;
		nextPutAll: aWidgetNode comment.
	
	out
		cr;
		nextPutAll: '== Property Documentation =='.
		
	aWidgetNode propertyNodes do: [:node | self visitNode: node].

	out
		cr;
		nextPutAll: '== Method Documentation =='.
	
	aWidgetNode signalNodes do: [:node | self visitNode: node].! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 15:16:42.936'!
nextPutWidgetSummary: aWidgetNode

	out
		cr;
		nextPutAll: '''''''System Category:'''''' '.
	(aWidgetNode theClass category findTokens: '-')
		do: [:token | out nextPutAll: '!!', token]
		separatedBy: [out nextPutAll: ' - '].

	out
		cr;
		nextPutAll: ('= {1} (!!{2}) =' format: {aWidgetNode theClass widgetClassName. aWidgetNode theClass});
		cr;
		nextPutAll: aWidgetNode briefComment.
		
	aWidgetNode briefComment size < aWidgetNode comment size
		ifTrue: [out nextPutAll: ' [#details More...]'].
		
	out
		cr; cr;
		nextPutAll: '''''''Inherits:'''''' '.

	aWidgetNode theClass allSuperclasses
		do: [:cls | self nextPutClassLabel: cls]
		separatedBy: [out nextPutAll: ' -> '].

	out
		cr; cr;
		nextPutAll: '''''''Inherited by:'''''' '.
		
	aWidgetNode theClass subclasses
		do: [:cls | self nextPutClassLabel: cls]
		separatedBy: [out nextPutAll: ', '].! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 11:33:37.666'!
nextPutWidgetTOC: aWidgetNode

	self mode: #toc.
	
	out
		cr;
		nextPutAll: '== Properties =='.
		
	aWidgetNode propertyNodes do: [:node | self visitNode: node].
	
	aWidgetNode theClass allSuperclasses do: [:cls |
		(cls class selectors includes: #widgetProperties) ifTrue: [ | p1 p2 |
			p1 := cls widgetProperties gather: [:category | category value].
			p2 := (cls superclass class canUnderstand: #widgetProperties)
				ifTrue: [cls superclass widgetProperties gather: [:category | category value]]
				ifFalse: [#()].
			out
				cr;
				nextPutAll: (' * {1} properties inherited from {2}' format: {
					p1 count: [:prop | (p2 includes: prop) not].
					self classLabel: cls})]].

	out
		cr;
		nextPutAll: '== Signals =='.
	
	aWidgetNode signalNodes do: [:node | self visitNode: node].
	
	aWidgetNode theClass allSuperclasses do: [:cls | | signals |
		signals := cls signals.
		signals ifNotEmpty: [
			out
				cr;
				nextPutAll: (' * {1} signals inherited from {2}' format: {
					signals size.
					self classLabel: cls})]].! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 11:00:27.873'!
visitPropertyNode: aPropertyNode

	self mode == #toc ifTrue: [
		out
			cr;
			nextPutAll: (' * [#property{1} {2}] : {3}' format: {
				aPropertyNode symbol asLowercase.
				aPropertyNode symbol.
				aPropertyNode typeString})].

	self mode == #details ifTrue: [
		out
			cr;
			nextPutAll: ('=== {1} : {2} === #property{3}' format: {
				aPropertyNode symbol.
				aPropertyNode typeString.
				aPropertyNode symbol asLowercase});
			cr;
			nextPutAll: aPropertyNode comment;
			cr; cr;
			nextPutAll: ('By default, this property is ''''''{1}''''''.' format: {
				aPropertyNode defaultValueString}).
			"cr; cr;
			nextPutAll: '''''''Accessors:''''''';
			cr"
			
	].! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 10:40:17.354'!
visitSignalNode: aSignalNode

	self mode == #toc ifTrue: [
		out
			cr;
			nextPutAll: (' * [#signal{1} {2}]' format: {
				aSignalNode symbol asLowercase.
				aSignalNode symbol})].

	self mode == #details ifTrue: [
		out
			cr;
			nextPutAll: ('=== {1} (Signal) === #signal{2}' format: {
				aSignalNode symbol.
				aSignalNode symbol asLowercase});
			cr;
			nextPutAll: aSignalNode comment].! !

!UiDocTracWikiExporter methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 10:39:31.244'!
visitWidgetNode: aWidgetNode

	out nextPutAll: '[[PageOutline(1-2)]]'.

	self
		nextPutWidgetSummary: aWidgetNode;
		nextPutWidgetTOC: aWidgetNode;
		nextPutWidgetDetails: aWidgetNode.
! !

!UiDocVisitor methodsFor: 'visiting' stamp: 'mt 11/29/2011 15:31:42.479'!
visitImageNode: anImageNode! !

!UiDocVisitor methodsFor: 'visiting' stamp: 'mt 12/1/2011 09:22:58.445'!
visitNode: aDocNode

	aDocNode accept: self.! !

!UiDocVisitor methodsFor: 'visiting' stamp: 'mt 11/29/2011 15:31:57.077'!
visitPropertyNode: aPropertyNode! !

!UiDocVisitor methodsFor: 'visiting' stamp: 'mt 11/29/2011 15:32:03.266'!
visitSignalNode: aSignalNode! !

!UiDocVisitor methodsFor: 'visiting' stamp: 'mt 11/29/2011 15:32:09.749'!
visitWidgetNode: aWidgetNode! !

Object subclass: #UiDocumentationGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Documentation'!

!UiDocumentationGenerator methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 10:53:59.557'!
commentForMethod: aCompiledMethod

	| parser |
	parser := RBParser parseMethod: aCompiledMethod getSource asString.
	
	^ parser comments
		ifEmpty: ['']
		ifNotEmpty: [parser source
			"Adjust index to skip comment delimiters."
			copyFrom: parser comments first first + 1
			to: parser comments first last - 1]! !

!UiDocumentationGenerator methodsFor: 'as yet unclassified' stamp: 'mt 12/1/2011 15:30:36.677'!
generate

	| root exporter |
	root := UiDocNode new.
	"(PackageInfo named: #Widgets) classes"
	{Morph} do: [:cls | "((cls inheritsFrom: Morph) and: [cls isWidgetClass] and: [cls isExperimentalWidgetClass not])" true ifTrue: [
		| classNode |
		classNode := UiDocWidgetNode new theClass: cls.
		root add: classNode.
		
		((cls superclass canUnderstand: #widgetProperties)
			ifTrue: [cls widgetProperties copyWithoutAll: cls superclass widgetProperties]
			ifFalse: [cls widgetProperties]) do: [:category |
			category value do: [:property | | method |
				method := cls lookupSelector: property key.
				classNode add: (UiDocPropertyNode new
					symbol: property key;
					type: property value;
					comment: (self commentForMethod: method);
					defaultValue: (cls new perform: property key))]].
			
		cls signals do: [:selector | | method |
			method := cls lookupSelector: selector.
			classNode add: (UiDocSignalNode new
				symbol: selector;
				comment: (self commentForMethod: method))]
		]].

	exporter := UiDocTracWikiExporter new.
	"exporter visitNode: root." "root accept: exporter ???"
	root nodes first accept: exporter.
	^ exporter contents! !

Object subclass: #UiEventHandler
	instanceVariableNames: 'properties signalConnections shortcuts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Events'!

!UiEventHandler commentStamp: 'mt 3/28/2011 10:20:21.148' prior: 0!
An event handler that maps morphic events to signals.

Adds support for #handlesMouseUp: which is not present in Morphic.

An event handler has its own signal connections stored locally to improve lookup performance. As signal processing happens only in the "UI Process" a null mutex is used to secure the connections. Do not try to connect or emit those signals from within another process.!

!UiEventHandler methodsFor: 'private' stamp: 'mt 3/19/2011 09:30:45.989'!
asMouseWheelEvent: aKeyboardEvent
	"Converts the given keyboard event into a mouse wheel event. CHECK WITH #isMouseWheelEvent: FIRST BEFORE CALLING THIS FUNCTION!!"
	
	^ UiMouseWheelEvent new
		setType: #mouseWheel
		position: aKeyboardEvent position
		direction: (aKeyboardEvent keyCharacter = Character arrowUp
			ifTrue: [#up]
			ifFalse: [#down])
		buttons: aKeyboardEvent buttons
		hand: aKeyboardEvent hand
		stamp: aKeyboardEvent timeStamp! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 2/26/2011 13:37:37.495'!
click: aMouseEvent fromMorph: aMorph

	self mouseClicked: aMouseEvent.  ! !

!UiEventHandler methodsFor: 'signals - processing' stamp: 'mt 3/28/2011 10:29:15.009'!
disconnectAllSignals

	signalConnections := nil.! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 2/26/2011 13:37:50.079'!
doubleClick: aMouseEvent fromMorph: aMorph

	self mouseDoubleClicked: aMouseEvent.  ! !

!UiEventHandler methodsFor: 'mouse - accessing' stamp: 'mt 3/22/2011 21:46:58.592'!
doubleClickTimeout: anEvent fromMorph: aMorph

	^ (self valueOfProperty: #doubleClickTimeout) value! !

!UiEventHandler methodsFor: 'mouse - accessing' stamp: 'mt 2/26/2011 13:36:13.637'!
dragThreshold

	^ (self valueOfProperty: #dragThreshold) value! !

!UiEventHandler methodsFor: 'mouse - accessing' stamp: 'mt 2/26/2011 14:35:29.928'!
dragThreshold: anInteger

	self setProperty: #dragThreshold toValue: [anInteger].! !

!UiEventHandler methodsFor: 'keyboard - signals' stamp: 'mt 2/26/2011 14:00:32.752'!
focusLost

	self emit.! !

!UiEventHandler methodsFor: 'keyboard - signals' stamp: 'mt 5/6/2011 08:59:32.254'!
focusLostInSubmorph: aMorph
	"Emitted whenever a submorph (any depth) received keyboard focus."
	
	self emit.! !

!UiEventHandler methodsFor: 'keyboard - signals' stamp: 'mt 5/6/2011 09:46:43.349'!
focusReceived

	self emit.! !

!UiEventHandler methodsFor: 'keyboard - signals' stamp: 'mt 5/6/2011 08:59:42.347'!
focusReceivedInSubmorph: aMorph
	"Emitted whenever a submorph (any depth) received keyboard focus."
	
	self emit.! !

!UiEventHandler methodsFor: 'accessing' stamp: 'mt 2/26/2011 14:46:03.624'!
grabKeyboardOnMouseEnter

	^ self valueOfProperty: #grabKeyboardOnMouseEnter! !

!UiEventHandler methodsFor: 'accessing' stamp: 'mt 2/26/2011 14:44:44.938'!
grabKeyboardOnMouseEnter: aBoolean

	self setProperty: #grabKeyboardOnMouseEnter toValue: aBoolean.! !

!UiEventHandler methodsFor: 'keyboard - shortcuts' stamp: 'mt 8/13/2012 10:31:45.871'!
handleShortcut: aKeyboardEvent

	| map pattern |
	self shortcuts ifEmpty: [^ self].
	
	map := {
		aKeyboardEvent commandKeyPressed -> #cmd.
		aKeyboardEvent controlKeyPressed -> #ctrl.
		aKeyboardEvent shiftPressed -> #shift.
	}.
	pattern := OrderedCollection new.
	
	map do: [:assoc | assoc key ifTrue: [pattern add: assoc value]].
	
	pattern := pattern sorted asArray -> aKeyboardEvent keyCharacter asLowercase.
	
	(self shortcuts at: pattern ifAbsent: []) ifNotNil: [:target |
		target first first
			ifNotNil: [:receiver | receiver perform: target second]
			ifNil: [self shortcuts removeKey: pattern] "kind of garbage collection here"].! !

!UiEventHandler methodsFor: 'keyboard - testing' stamp: 'mt 3/19/2011 09:32:21.583'!
handlesKeyboard: aKeyboardEvent

	(self isMouseWheelEvent: aKeyboardEvent)
		ifTrue: [^ self handlesMouseWheel: (self asMouseWheelEvent: aKeyboardEvent)].
		
	^ (self valueOfProperty: #handlesKeyboard) cull: aKeyboardEvent! !

!UiEventHandler methodsFor: 'mouse - testing' stamp: 'mt 3/2/2011 10:58:01.646'!
handlesMouseDown: event

	^ (self valueOfProperty: #handlesMouseDown) cull: event! !

!UiEventHandler methodsFor: 'mouse - testing' stamp: 'mt 2/26/2011 12:22:20.428'!
handlesMouseOver: event

	^ (self valueOfProperty: #handlesMouseOver) cull: event! !

!UiEventHandler methodsFor: 'mouse - testing' stamp: 'mt 2/26/2011 12:22:15.214'!
handlesMouseOverDragging: event

	^ (self valueOfProperty: #handlesMouseOverDragging) cull: event! !

!UiEventHandler methodsFor: 'mouse - testing' stamp: 'mt 2/26/2011 12:22:10.553'!
handlesMouseStillDown: event

	^ (self valueOfProperty: #handlesMouseStillDown) cull: event! !

!UiEventHandler methodsFor: 'mouse - testing' stamp: 'mt 3/2/2011 11:02:56.866'!
handlesMouseUp: event

	^ (self valueOfProperty: #handlesMouseUp) cull: event! !

!UiEventHandler methodsFor: 'mouse - testing' stamp: 'mt 3/11/2011 16:42:36.799'!
handlesMouseWheel: event

	^ (self valueOfProperty: #handlesMouseWheel) cull: event! !

!UiEventHandler methodsFor: 'signals - processing' stamp: 'mt 3/28/2011 10:17:26.776'!
hasSignalConnections

	^ true! !

!UiEventHandler methodsFor: 'initialization' stamp: 'mt 7/8/2013 16:25:01.93'!
initialize

	super initialize.
	
	self
		mouseDownPriority: 0;  
		setProperty: #handlesKeyboard toValue: [true];
		setProperty: #handlesMouseDown toValue: [true];
		setProperty: #handlesMouseUp toValue: [true];
		setProperty: #handlesMouseOver toValue: [true];
		setProperty: #handlesMouseOverDragging toValue: [true];
		setProperty: #handlesMouseStillDown toValue: [false];
		setProperty: #handlesMouseWheel toValue: [false];
		setProperty: #doubleClickTimeout toValue: [20 "ms"];
		dragThreshold: UiPlatform current dragThreshold "pixel";
		grabKeyboardOnMouseEnter: false;
		releaseKeyboardOnMouseLeave: false;
		tabAmongFields: false. "not implemented yet"! !

!UiEventHandler methodsFor: 'testing' stamp: 'mt 2/26/2011 13:45:11.154'!
isEventHandler

	^ true! !

!UiEventHandler methodsFor: 'private' stamp: 'mt 7/8/2013 14:04:11.291'!
isMouseWheelEvent: aKeyboardEvent
	"Check whether the given keyboard event encoded a mouse wheel event."
	
	^ (aKeyboardEvent isKeystroke
		and: [aKeyboardEvent controlKeyPressed])
		and: [aKeyboardEvent keyCharacter = Character arrowUp
			or: [aKeyboardEvent keyCharacter = Character arrowDown]]! !

!UiEventHandler methodsFor: 'keyboard - handling' stamp: 'mt 3/2/2011 11:34:46.902'!
keyDown: aKeyboardEvent fromMorph: aMorph

	self keyPressed: aKeyboardEvent.
	
	(#(3 13 10) includes: aKeyboardEvent keyValue)
		ifTrue: [self returnPressed].! !

!UiEventHandler methodsFor: 'keyboard - signals' stamp: 'mt 2/26/2011 12:28:56.969'!
keyPressed: aKeyboardEvent

	self emit.! !

!UiEventHandler methodsFor: 'keyboard - signals' stamp: 'mt 2/26/2011 12:29:04.453'!
keyReleased: aKeyboardEvent

	self emit.! !

!UiEventHandler methodsFor: 'keyboard - handling' stamp: 'mt 3/19/2011 09:33:33.637'!
keyStroke: aKeyboardEvent fromMorph: aMorph

	"Handle VM-specific mapping from mouse wheel interaction to key stroke event: [Ctrl]+Up/Down"
	(self isMouseWheelEvent: aKeyboardEvent)
		ifTrue: [^ self mouseWheel: (self asMouseWheelEvent: aKeyboardEvent) fromMorph: aMorph].

	(self tabAmongFields and: [aKeyboardEvent keyCharacter = Character tab])
		ifTrue: [self shouldBeImplemented].
		
	self
		handleShortcut: aKeyboardEvent;
		keyStroked: aKeyboardEvent.
		
	(#(3 13 10) includes: aKeyboardEvent keyValue)
		ifTrue: [self returnStroked].! !

!UiEventHandler methodsFor: 'keyboard - signals' stamp: 'mt 2/26/2011 12:44:29.785'!
keyStroked: aKeyboardEvent

	self emit.! !

!UiEventHandler methodsFor: 'keyboard - handling' stamp: 'mt 2/26/2011 13:49:48.08'!
keyUp: aKeyboardEvent fromMorph: aMorph

	self keyReleased: aKeyboardEvent.! !

!UiEventHandler methodsFor: 'keyboard - handling' stamp: 'mt 5/6/2011 09:09:36.587'!
keyboardFocusChange: aBoolean fromMorph: aMorph

	"Give the morph a chance to update its appearance."
	aMorph redrawLater.

	aBoolean
		ifTrue: [self focusReceived]
		ifFalse: [self focusLost].
		
	aMorph owner ifNotNil: [:owner |
		owner keyboardFocusChange: aBoolean inSubmorph: aMorph].! !

!UiEventHandler methodsFor: 'keyboard - handling' stamp: 'mt 5/6/2011 10:21:15.427'!
keyboardFocusChange: aBoolean inSubmorph: aSubmorph fromMorph: aMorph

	aBoolean
		ifTrue: [self focusReceivedInSubmorph: aSubmorph]
		ifFalse: [self focusLostInSubmorph: aSubmorph].
		
	aMorph owner ifNotNil: [:owner |
		owner keyboardFocusChange: aBoolean inSubmorph: aSubmorph].! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 2/26/2011 13:24:29.576'!
mouseClicked: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 2/26/2011 13:24:34.692'!
mouseDoubleClicked: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 2/26/2011 13:37:10.381'!
mouseDown: aMouseEvent fromMorph: aMorph

	aMouseEvent hand
		waitForClicksOrDrag: aMorph
		event: aMouseEvent
		selectors: #(click: doubleClick: doubleClickTimeout: startDrag:)
		threshold: self dragThreshold.
		
	self mousePressed: aMouseEvent.  ! !

!UiEventHandler methodsFor: 'mouse - accessing' stamp: 'mt 2/26/2011 14:28:19.68'!
mouseDownPriority

	^ (self valueOfProperty: #mouseDownPriority) value! !

!UiEventHandler methodsFor: 'mouse - accessing' stamp: 'mt 2/26/2011 14:47:19.021'!
mouseDownPriority: anInteger

	self setProperty: #mouseDownPriority toValue: anInteger.! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 2/26/2011 13:41:28.994'!
mouseDragStarted: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 2/26/2011 14:50:57.641'!
mouseEnter: aMouseEvent fromMorph: aMorph

	self mouseEntered: aMouseEvent.
	
	self grabKeyboardOnMouseEnter
		ifTrue: [aMorph grabKeyboard].! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 4/25/2011 09:56:31.185'!
mouseEnterDragging: aMouseEvent fromMorph: aMorph

	self mouseEnteredDragging: aMouseEvent.  ! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 2/26/2011 13:54:22.461'!
mouseEntered: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 4/25/2011 09:55:56.594'!
mouseEnteredDragging: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 4/7/2011 14:25:49.3'!
mouseLeave: aMouseEvent fromMorph: aMorph

	self mouseLeft: aMouseEvent.

	self releaseKeyboardOnMouseLeave
		ifTrue: [aMorph releaseKeyboard].! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 4/25/2011 09:56:41.098'!
mouseLeaveDragging: aMouseEvent fromMorph: aMorph

	self mouseLeftDragging: aMouseEvent.  ! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 4/7/2011 14:25:48.943'!
mouseLeft: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 4/25/2011 09:56:02.024'!
mouseLeftDragging: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 2/26/2011 13:56:17.454'!
mouseMove: aMouseEvent fromMorph: aMorph

	self mouseMoved: aMouseEvent.! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 2/26/2011 13:56:01.172'!
mouseMoved: aMouseMoveEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 2/26/2011 13:24:21.32'!
mousePressed: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 2/26/2011 13:24:26.458'!
mouseReleased: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 2/26/2011 13:53:35.477'!
mouseStillDown: aMouseEvent fromMorph: aMorph

	self mousePressed: aMouseEvent.  ! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 3/22/2011 22:34:35.025'!
mouseUp: aMouseEvent fromMorph: aMorph

	(self handlesMouseUp: aMouseEvent)
		ifTrue: [self mouseReleased: aMouseEvent].! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 3/11/2011 16:48:14.378'!
mouseWheel: aMouseEvent fromMorph: aMorph

	(self handlesMouseWheel: aMouseEvent)
		ifTrue: [
			aMouseEvent direction = #up
				ifTrue: [self mouseWheelUp: aMouseEvent].
			aMouseEvent direction = #down
				ifTrue: [self mouseWheelDown: aMouseEvent]].! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 3/11/2011 16:40:59.573'!
mouseWheelDown: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'mouse - signals' stamp: 'mt 3/11/2011 16:40:55.772'!
mouseWheelUp: aMouseEvent

	self emit.! !

!UiEventHandler methodsFor: 'keyboard - shortcuts' stamp: 'mt 8/13/2012 10:33:39.384'!
on: keyPattern send: aSelector to: aReceiver
	"Add a keyboard shortcut that is handled inside this event handler. The case (lower/upper) of the character does not matter. Example:
	self eventHandler on: #ctrl -> $c send: #copy to: self.
	self eventHandler on: #(ctrl shift) -> Character arrowUp send: #pageUp to: self.
	self eventHandler on: Character enter send: #save to: self.
	
	Modifiers: #cmd, #ctrl, #shift"

	| normalizedPattern |
	normalizedPattern := keyPattern isVariableBinding "= association"
		ifTrue: [keyPattern key isSymbol
			ifTrue: [{keyPattern key} -> keyPattern value asLowercase]
			ifFalse: [keyPattern key sorted -> keyPattern value asLowercase]]
		ifFalse: [#() -> keyPattern asLowercase "character given only"].

	self shortcuts at: normalizedPattern put: {WeakArray with: aReceiver. aSelector}! !

!UiEventHandler methodsFor: 'accessing' stamp: 'mt 2/26/2011 11:53:31.141'!
properties

	^ properties ifNil: [properties := Dictionary new]! !

!UiEventHandler methodsFor: 'accessing' stamp: 'mt 2/26/2011 14:46:17.804'!
releaseKeyboardOnMouseLeave

	^ self valueOfProperty: #releaseKeyboardOnMouseLeave! !

!UiEventHandler methodsFor: 'accessing' stamp: 'mt 2/26/2011 14:45:45.391'!
releaseKeyboardOnMouseLeave: aBoolean

	self setProperty: #releaseKeyboardOnMouseLeave toValue: aBoolean.! !

!UiEventHandler methodsFor: 'keyboard - shortcuts' stamp: 'mt 3/6/2013 20:18:21.162'!
removeAllShortcuts

	shortcuts := nil.! !

!UiEventHandler methodsFor: 'properties' stamp: 'mt 2/26/2011 11:55:36.248'!
removeProperty: aSymbol

	self properties removeKey: aSymbol ifAbsent: [].! !

!UiEventHandler methodsFor: 'keyboard - signals' stamp: 'mt 3/2/2011 11:33:52.002'!
returnPressed
	
	self emit.! !

!UiEventHandler methodsFor: 'keyboard - signals' stamp: 'mt 3/2/2011 11:32:38.703'!
returnStroked
	"Emitted whenefer Character enter, cr, or lf is stroked."
	
	self emit.! !

!UiEventHandler methodsFor: 'properties' stamp: 'mt 2/26/2011 11:54:47.669'!
setProperty: aSymbol toValue: anObject

	anObject ifNil: [^ self removeProperty: aSymbol].
	self properties at: aSymbol put: anObject.! !

!UiEventHandler methodsFor: 'keyboard - shortcuts' stamp: 'mt 3/6/2013 20:18:09.597'!
shortcuts

	^ shortcuts ifNil: [shortcuts := Dictionary new]! !

!UiEventHandler methodsFor: 'signals - processing' stamp: 'mt 3/28/2011 10:17:00.743'!
signalConnections

	^ signalConnections ifNil: [signalConnections := Dictionary new]! !

!UiEventHandler methodsFor: 'signals - processing' stamp: 'mt 3/28/2011 10:17:47.015'!
signalMutex

	^ UiNullMutex! !

!UiEventHandler methodsFor: 'mouse - handling' stamp: 'mt 2/26/2011 13:41:18.965'!
startDrag: aMouseEvent fromMorph: aMorph

	self mouseDragStarted: aMouseEvent.  ! !

!UiEventHandler methodsFor: 'accessing' stamp: 'mt 2/26/2011 16:30:19.736'!
tabAmongFields
	"Pass keyboard focus to next morph?"
	
	^ self valueOfProperty: #tabAmongFields! !

!UiEventHandler methodsFor: 'accessing' stamp: 'mt 2/26/2011 16:29:49.718'!
tabAmongFields: aBoolean

	self setProperty: #tabAmongFields toValue: aBoolean.! !

!UiEventHandler methodsFor: 'properties' stamp: 'mt 2/26/2011 13:18:22.966'!
valueOfProperty: aSymbol

	^ self valueOfProperty: aSymbol ifAbsent: []! !

!UiEventHandler methodsFor: 'properties' stamp: 'mt 2/26/2011 11:56:30.935'!
valueOfProperty: aSymbol ifAbsent: aBlock

	^ self properties at: aSymbol ifAbsent: aBlock! !

Object subclass: #UiHtmlTextFormatter
	instanceVariableNames: 'offset runStack runArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Text'!

!UiHtmlTextFormatter commentStamp: 'mt 7/18/2012 15:38' prior: 0!
I create a text instance from a given string that contains html tags for emphasis. Examples:

'Hello, <b>World!!</b>' asHtmlText
'<i>Hello</i>, World!!' asHtmlText
'<font color="#00cc00">Hello!!</font>, World!!' asHtmlText
'<font color="#FF00FF">Ahahaha!! <b>Hello</b>, <i>World</i></font>!!' asHtmlText asMorph openInHand
'<font color="#0000ff">Ahahaha!! <b><font size="7">Hello</font></b></font>, <i><b>World</b></i>!!' asHtmlText
'<font color="#444444"><b>Hello</b> <i>World!!</i></font>' asHtmlText asMorph openInHand

Supports line breaks: 'Hello<br>World!!' asHtmlText asMorph openInHand.

Escape control characters with $\. Double-escape them if you use #format:.
('<b>Hello \\>\\> {1}!!</b>' format: {'World'}) asHtmlText!

!UiHtmlTextFormatter class methodsFor: 'formatting' stamp: 'mt 6/22/2010 22:28:43.544'!
toText: aString

	^ self new toText: aString! !

!UiHtmlTextFormatter methodsFor: 'initialization' stamp: 'mt 3/23/2012 14:53:57.086'!
initialize

	super initialize.
	offset := 0. "To ignore characters in the input string that are used by tags."
	runStack := Stack new.
	runArray := RunArray new.
	
	"{text attributes. start index. end index. number of open tags}"
	runStack push: {OrderedCollection new. 1. nil. 0}.! !

!UiHtmlTextFormatter methodsFor: 'mapping' stamp: 'mt 7/8/2013 16:37:35.498'!
mapTagToAttribute: aTag

	| rx strm result fontFamily fontSize |
	aTag = '<b>' ifTrue: [^ {TextEmphasis bold}].
	aTag = '<i>' ifTrue: [^ {TextEmphasis italic}].
	aTag = '<u>' ifTrue: [^ {TextEmphasis underlined}].
	aTag = '<code>' ifTrue: [^ {TextFontReference toFont: UiPlatform current standardCodeFont}].
	
	result := OrderedCollection new.
	
	"<font color=""#00FFCC"">"
	rx := 'color\s*=\s*"(#\w\w\w\w\w\w)' asRegex.
	strm := aTag readStream.
	[rx searchStream: strm]
		whileTrue: [ result add: (TextColor color: (Color fromString: (rx subexpression: 2)))].
	
	"<font family=""Bitmap SegoeUI Light"">"
	rx := 'family\s*=\s*"([^"]+)"' asRegex.
	strm := aTag readStream.
	[rx searchStream: strm]
		whileTrue: [fontFamily := rx subexpression: 2].

	"<font size=""7"">"
	rx := 'size\s*=\s*"(\d+)' asRegex.
	strm := aTag readStream.
	[rx searchStream: strm]
		whileTrue: [fontSize := rx subexpression: 2].

	fontSize ifNil: [fontSize := UiPlatform current standardDefaultTextFont pointSize].
	result add: (TextFontReference toFont: (fontFamily
		ifNil: [TextStyle default fontOfPointSize: fontSize]
		ifNotNil: [:ff | StrikeFont familyName: ff pointSize: fontSize])).

	^ result! !

!UiHtmlTextFormatter methodsFor: 'parsing' stamp: 'mt 7/8/2013 16:10:56.867'!
parseString: anHtmlString

	| tag lookForNewTag result start end attrs escapeNextCharacter |
	lookForNewTag := true.
	tag := OrderedCollection new.
	result := OrderedCollection new.
	escapeNextCharacter := false.
	
	anHtmlString withIndexDo: [:character :index |
		escapeNextCharacter
			ifTrue: [result add: character. escapeNextCharacter := false]
			ifFalse: [
				character = $\
					ifTrue: [offset := offset + 1. escapeNextCharacter := true]
					ifFalse: [
						character = $< ifTrue: [lookForNewTag := false].
						character = $> ifTrue: [lookForNewTag := true].
		
						(lookForNewTag and: [character ~= $>])
							ifTrue: [result add: character]
							ifFalse: [tag add: character. offset := offset + 1]..
			
						(tag notEmpty and: [tag last = $>]) ifTrue: [
							"Full tag like <b> or </b> found."
							|tmp|
							tmp := String withAll: tag.
							tmp second ~= $/
								ifTrue: [self processStartTag: tmp pos: index]
								ifFalse: [self processEndTag: tmp pos: index].			
							tag := OrderedCollection new.
							lookForNewTag := true]]]].
	
	"Add last run."
	
	"Set end index."
	runStack top at: 3 put: anHtmlString size - offset.
	
	"Write to run array."
	start := runStack top second.
	end := runStack top third.
	attrs := runStack top first.
	UiPlatform current
		runArray: runArray
		add: attrs
		withOccurrences: end - start + 1.
	
	^ String withAll: result! !

!UiHtmlTextFormatter methodsFor: 'parsing' stamp: 'mt 7/8/2013 16:11:07.103'!
processEndTag: aTag pos: index

	|start end attrs|
	"De-Accumulate adjacent tags."
	runStack top at: 4 put: runStack top fourth - 1.
	runStack top fourth > 0
		ifTrue: [^ self "not yet"].
		
	"Set end index."
	runStack top at: 3 put: index - offset.
	
	"Write to run array."
	start := runStack top second.
	end := runStack top third.
	attrs := runStack top first.
	UiPlatform current
		runArray: runArray
		add: attrs
		withOccurrences: end - start + 1.

	runStack pop.
	runStack top at: 2 put: index - offset + 1.! !

!UiHtmlTextFormatter methodsFor: 'parsing' stamp: 'mt 7/8/2013 16:11:01.967'!
processStartTag: aTag pos: index

	|start end attrs|
	"Accumulate adjacent tags."
	(runStack size > 1 and: [runStack top second = (index - offset + 1) "= adjacent start tags"])
		ifTrue: [
			runStack top at: 1 put: (runStack top first copy addAll: (self mapTagToAttribute: aTag); yourself).
			runStack top at: 4 put: (runStack top fourth + 1). "increase number of open tags"
			^self].
			
	"Set end index."
	runStack top at: 3 put: index - offset.
	"Write to run array."
	start := runStack top second.
	end := runStack top third.
	attrs := runStack top first.
	UiPlatform current
		runArray: 	runArray
		add: attrs
		withOccurrences: end - start + 1.
	"Remove start/end info to reuse attributes later."
	runStack top at: 2 put: nil.
	runStack top at: 3 put: nil.
	"Copy attr list and add new attr."
	runStack push: ({runStack top first copy addAll: (self mapTagToAttribute: aTag); yourself. index - offset + 1. nil. 1}).! !

!UiHtmlTextFormatter methodsFor: 'formatting' stamp: 'mt 6/24/2012 13:07:29.022'!
toText: aString
	"Parse all known HTML attributes in the given string to text attributes."
	
	^ [Text string: (self parseString: (aString copyReplaceAll: '<br>' with: String cr)) runs: runArray] 
		on: Error
		do: [:e | aString asText]! !

Object subclass: #UiItemModelNullNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiItemModelNullNode commentStamp: 'mt 4/12/2011 11:21:15.846' prior: 0!
A model index that stores dummy data to, e.g., compute the default item extent.!

!UiItemModelNullNode methodsFor: 'accessing' stamp: 'mt 1/23/2012 09:45:51.656'!
icon

	^ Form extent: 16@16 depth: 32! !

!UiItemModelNullNode methodsFor: 'accessing' stamp: 'mt 1/23/2012 09:45:57.42'!
text

	^ 'This <b>is</b> a <i>dummy item</i>!!' asHtmlText! !

Object subclass: #UiItemOrder
	instanceVariableNames: 'map sortBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiItemOrder commentStamp: 'mt 10/16/2010 14:45:30.134' prior: 0!
A cached order of some items. Needs to be updated frequently.!

!UiItemOrder methodsFor: 'accessing' stamp: 'mt 10/16/2010 19:08:23.107'!
items: someItems at: anIndex
	
	self map ifNil: [^ someItems at: anIndex].
	^ someItems at: (self map at: anIndex)! !

!UiItemOrder methodsFor: 'enumerating' stamp: 'mt 10/16/2010 15:03:57.143'!
items: someItems collect: aBlock

	self map ifNil: [^ someItems collect: aBlock].
	^ map collect: [:index | aBlock value: (someItems at: index)]! !

!UiItemOrder methodsFor: 'enumerating' stamp: 'mt 10/16/2010 15:03:42.612'!
items: someItems do: aBlock

	self map ifNil: [^ someItems do: aBlock].
	map do: [:index | aBlock value: (someItems at: index)]! !

!UiItemOrder methodsFor: 'enumerating' stamp: 'mt 10/16/2010 19:12:41.82'!
items: someItems withIndexDo: aBlock

	self map ifNil: [^ someItems withIndexDo: aBlock].
	map do: [:index | aBlock value: (someItems at: index) value: index]! !

!UiItemOrder methodsFor: 'accessing' stamp: 'mt 10/16/2010 14:57:00.465'!
map

	^ map! !

!UiItemOrder methodsFor: 'accessing' stamp: 'mt 10/16/2010 14:57:44.975'!
map: anArray
	"Example: #(5 4 3 2 1) will revert a list with 5 items."

	map := anArray.! !

!UiItemOrder methodsFor: 'accessing' stamp: 'mt 7/6/2012 10:38:13.264'!
mapAt: anIndex
	
	^ self map
		ifNil: [anIndex]
		ifNotNil: [:m | m at: anIndex]! !

!UiItemOrder methodsFor: 'accessing' stamp: 'mt 7/6/2012 10:38:32.475'!
mapIndexOf: aMappedIndex
	
	^ self map
		ifNil: [aMappedIndex]
		ifNotNil: [:m | m indexOf: aMappedIndex]! !

!UiItemOrder methodsFor: 'accessing' stamp: 'mt 10/16/2010 15:04:46.949'!
sortBlock

	^ sortBlock! !

!UiItemOrder methodsFor: 'accessing' stamp: 'mt 10/16/2010 14:44:45.717'!
sortBlock: aBlock

	sortBlock := aBlock.! !

!UiItemOrder methodsFor: 'manual mapping' stamp: 'mt 10/16/2010 20:12:05.571'!
unmapIndex: anIndex

	self map ifNil: [^ anIndex].
	^ self map indexOf: anIndex! !

!UiItemOrder methodsFor: 'updating' stamp: 'mt 12/11/2012 14:06:12.913'!
updateSortedResult: ungroupedSortedItems
	"An optional post-processing step. Restore groups. Optimized for speed."

	| groupedSortedItems currentItems nextItems sortedSize currentSize nextSize groupIndex |
	groupedSortedItems := Array new: ungroupedSortedItems size.
	currentItems := ungroupedSortedItems asArray.
	nextItems := Array new: ungroupedSortedItems size.
	
	sortedSize := nextSize := 0.
	currentSize := currentItems size.
	
	groupIndex := 1.
	
	[currentSize == 0] whileFalse: [
		| temp |
		1 to: currentSize do: [:currentIndex | 
			((currentItems at: currentIndex) valueOfProperty: #modelIndex) groupIndex = groupIndex
				ifTrue: [sortedSize := sortedSize + 1.
					groupedSortedItems at: sortedSize put: (currentItems at: currentIndex)]
				ifFalse: [nextSize := nextSize + 1.
					nextItems at: nextSize put: (currentItems at: currentIndex)]].
	
		"Swap buffers."
		temp := currentItems.
		currentItems := nextItems.
		nextItems := temp.
		
		"Copy and reset buffer sizes."
		currentSize := nextSize.
		nextSize := 0.
		
		groupIndex := groupIndex + 1].
	
	^ groupedSortedItems! !

!UiItemOrder methodsFor: 'updating' stamp: 'mt 7/9/2013 11:27:41.944'!
updateWith: someItems

	| sortedCopy separators separatorPositions lastPos |
	self sortBlock ifNil: [^ self map: nil].	
	someItems ifEmpty: [^ self map: nil].
	
	"link separators with groups"
	separators := someItems select: [:item | item hasProperty: #isGroupSeparator].
	
	"sort without separators"
	sortedCopy := (someItems reject: [:item | item hasProperty: #isGroupSeparator])
		"unbox morphs into nodes"
		sorted: [:item1 :item2 | self sortBlock
			value: (item1 valueOfProperty: #modelIndex) node
			value: (item2 valueOfProperty: #modelIndex) node].
	sortedCopy := (self updateSortedResult: sortedCopy) asOrderedCollection.
	
	"compute separator positions"
	separatorPositions := Array new: separators size.
	separatorPositions at: 1 put: 1.
	separators allButFirst withIndexDo: [:separator :sepId |
		(sortedCopy detect: [:item |
			((separator valueOfProperty: #parentModelIndex) = (item valueOfProperty: #modelIndex) parent
				and: [(separator valueOfProperty: #groupIndex) = (item valueOfProperty: #modelIndex) groupIndex])]
			ifNone: []) ifNotNil: [:item | separatorPositions at: sepId + 1 put: (sortedCopy indexOf: item)]].
	"fill seperator positions for separators not found"
	lastPos := separatorPositions last ifNil: [sortedCopy size + 1 "empty groups past the last item"].
	(separatorPositions size to: 1 by: -1) do: [:index | | pos |
		pos := separatorPositions at: index.
		pos
			ifNil: [separatorPositions at: index put: lastPos]
			ifNotNil: [lastPos := pos]].

	"insert separators again"
	separators withIndexDo: [:separator :sepId |
		sortedCopy add: separator beforeIndex: (separatorPositions at: sepId) + sepId - 1].		
	
	"update index map"
	map := Array new: someItems size.
	sortedCopy withIndexDo: [:item :index |
		map at: index put: (someItems indexOf: item)].! !

UiItemOrder subclass: #UiTreeItemOrder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiTreeItemOrder commentStamp: 'mt 11/2/2010 13:09:33.58' prior: 0!
An item order that considers the model index hierarchy (i.e., parent indices) while sorting.!

!UiTreeItemOrder methodsFor: 'updating' stamp: 'mt 12/12/2012 11:00:09.619'!
parent: currentParent list: someItems result: result

	| groupIndex maxGroupIndex |
	someItems ifEmpty: [^ self].
	
	groupIndex := 1.
	maxGroupIndex := 1.
	
	[groupIndex <= maxGroupIndex] whileTrue: [
		someItems do: [:each |
			| index |
			index := each valueOfProperty: #modelIndex.
			index parent = currentParent ifTrue: [
				index groupIndex = groupIndex ifTrue: [
					result add: each.
					(each valueOfProperty: #hasChildren)
						ifTrue: [self parent: index list: someItems result: result]].
				"During first iteration, this will climax."
				maxGroupIndex := maxGroupIndex max: index groupIndex]].
		groupIndex := groupIndex + 1].! !

!UiTreeItemOrder methodsFor: 'updating' stamp: 'mt 12/13/2012 14:57:15.228'!
updateSortedResult: sortedItems
	"rebuild tree-structure"

	| result |
	result := OrderedCollection new: sortedItems size.
	self parent: nil list: sortedItems result: result.
	
	self assert: sortedItems size == result size.
	^ result! !

Object subclass: #UiItemViewColumnSpec
	instanceVariableNames: 'textSelector iconSelector colorSelector balloonTextSelector checkedSelector menuSelector headerText headerIcon headerBalloonText itemOrientation balloonTextConverter textConverter iconConverter colorConverter overrideStandardListFont selectableSelector checkableSelector editableSelector editorCreator editorFinisher editorAcceptSignal editorRevertSignal morphSelector morphConverter vAlignment hAlignment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiItemViewColumnSpec commentStamp: 'mt 7/6/2012 10:30:16.426' prior: 0!
This specification is used by the view to access model data and create (morphic) items for the viewport PER COLUMN.

UiItemViewColumnSpec newFrom: {
	#text -> #authorName.
	#icon -> #statusForm "selector in model or key in UiPropertyItemNode".
	#balloonText -> 3. "3rd property in #properties of model"}.!

!UiItemViewColumnSpec class methodsFor: 'defaults' stamp: 'mt 1/23/2012 09:45:24.433'!
defaultItemExtent

	^ ((self newFrom: {#text -> #text. #icon -> #icon})
		itemFor: UiItemModelNullNode new) extent! !

!UiItemViewColumnSpec class methodsFor: 'instance creation' stamp: 'mt 8/14/2013 11:02:56.305'!
newFrom: selectorMap
	"Targets can be symbols OR integers. The latter one needs #properties to be implemented in all nodes. See TUiItemModel trait."

	selectorMap isArray
		"{#text -> #name. #icon -> #stateForm}"
		ifTrue: [^ self newFrom: (Dictionary newFrom: selectorMap)].
		
	^ self new
		"item data"
		textSelector: (selectorMap at: #text ifAbsent: []);
		iconSelector: (selectorMap at: #icon ifAbsent: []);
		colorSelector: (selectorMap at: #color ifAbsent: []);
		balloonTextSelector: (selectorMap at: #balloonText ifAbsent: []);
		menuSelector: (selectorMap at: #menu ifAbsent: []);
		checkedSelector: (selectorMap at: #checked ifAbsent: []);
		morphSelector: (selectorMap at: #morph ifAbsent: []);
		"header data"
		headerText: (selectorMap at: #headerText ifAbsent: []);
		headerIcon: (selectorMap at: #headerIcon ifAbsent: []);
		"flags"
		selectableSelector: (selectorMap at: #selectable ifAbsent: []);
		checkableSelector: (selectorMap at: #checkable ifAbsent: []);
		editableSelector: (selectorMap at: #editable ifAbsent: [])
		
! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 1/22/2012 17:08:34.301'!
balloonTextConverter

	^ balloonTextConverter! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 1/22/2012 17:07:22.391'!
balloonTextConverter: aBlock

	balloonTextConverter := aBlock.! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 2/14/2013 14:27:36.034'!
balloonTextFor: aNode

	^ self data: balloonTextSelector for: aNode! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.343'!
balloonTextSelector
	^ balloonTextSelector! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.373'!
balloonTextSelector: anObject
	balloonTextSelector := anObject! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 7/5/2012 15:49:06.61'!
checkableFor: aNode

	^ ((self data: checkableSelector for: aNode) ifNil: [false])
		and: [self checkedSelector notNil]
	! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 7/5/2012 15:39:11.629'!
checkableSelector
	^ checkableSelector! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 7/5/2012 15:39:19.169'!
checkableSelector: anObject
	checkableSelector := anObject! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 2/12/2012 15:58:00.989'!
checkedFor: aNode

	^ (self data: checkedSelector for: aNode) ifNil: [false]
	! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.403'!
checkedSelector
	^ checkedSelector! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.433'!
checkedSelector: anObject
	checkedSelector := anObject! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 2/14/2013 14:27:20.855'!
colorConverter

	^ colorConverter! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 2/14/2013 14:27:31.012'!
colorConverter: aBlock

	colorConverter := aBlock.! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 2/14/2013 14:27:51.664'!
colorFor: aNode

	^ self data: colorSelector for: aNode! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 2/14/2013 14:28:04.604'!
colorSelector
	^ colorSelector! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 2/14/2013 14:28:16.541'!
colorSelector: anObject
	colorSelector := anObject! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 7/6/2012 10:33:41.754'!
data: aSymbolOrIntegerOrBoolean for: aNode
	"Private. Access property directly or use a indirection via #properties in node. This makes tree nodes having different kinds of properties more flexible."
	
	| property |
	aSymbolOrIntegerOrBoolean ifNil: [^ nil].
	
	"Support hardcoded boolean defaults."
	aSymbolOrIntegerOrBoolean == true ifTrue: [^ true].
	aSymbolOrIntegerOrBoolean == false ifTrue: [^ false].
	
	"Support ordered properties referenced by an integer. Consider #properties in node."
	property := aSymbolOrIntegerOrBoolean isInteger
		ifTrue: [aNode properties at: aSymbolOrIntegerOrBoolean ifAbsent: []]
		ifFalse: [aSymbolOrIntegerOrBoolean].
	
	"Lookup property as accessor or key in dictionary."
	^ property ifNotNil: [:p | (p ~~ #yourself and: [aNode isDictionary])
		ifTrue: [aNode at: p ifAbsent: []]
		ifFalse: [aNode perform: p]]! !

!UiItemViewColumnSpec methodsFor: 'building' stamp: 'mt 1/22/2012 16:58:20.965'!
defaultItemExtent

	^ (self itemFor: UiItemModelNullNode new) extent! !

!UiItemViewColumnSpec methodsFor: 'building' stamp: 'mt 3/4/2013 17:14:56.449'!
doLayoutFor: aMorph

	| offset |
	self itemOrientation = #horizontal ifTrue: [
		"All submorphs vertically centered. Text morph dictates height."
		
		aMorph submorphs last isTextMorph
			ifTrue: [ | fontHeight |
				fontHeight := aMorph submorphs last textStyle defaultFont height.
				(fontHeight * 2) < aMorph submorphs last height
					"More than one line?"
					ifTrue: [aMorph height: aMorph submorphs last height]
					ifFalse: [aMorph height: fontHeight]]
			ifFalse: [
				aMorph height: aMorph submorphs last height].
				
		"Manual layouting."
		offset := (aMorph left + 3) @ 1.
		aMorph submorphs do: [:m |
			m left: offset x.
			offset := offset + ((m width + 2) @ 0).
			m top: aMorph top - (m height - aMorph height // 2) + offset y].
	
		"Expand item horizontally to its contents."
		aMorph
			width: aMorph submorphBounds width + 5; "font rendering bugfix"
			height: aMorph height + 1. "font rendering bugfix"
		
		^ self].

	self itemOrientation = #horizontalMaxHeight ifTrue: [
		"All submorphs vertically centered."
		
		aMorph height: (aMorph submorphs detectMax: [:m | m height]) height.
				
		"Manual layouting."
		offset := (aMorph left + 3) @ 1.
		aMorph submorphs do: [:m |
			m left: offset x.
			offset := offset + ((m width + 2) @ 0).
			m top: aMorph top - (m height - aMorph height // 2) + offset y].
	
		"Expand item horizontally to its contents."
		aMorph
			width: aMorph submorphBounds width + 5; "font rendering bugfix"
			height: aMorph height + 1. "font rendering bugfix"
		
		^ self].
	
	self itemOrientation = #vertical ifTrue: [
		aMorph width: aMorph submorphs last width + 10.
			
		"Manual layouting."
		offset := aMorph top + 3.
		aMorph submorphs do: [:m |
			m top: offset.
			offset := offset + m height + 2.
			m left: (aMorph left - (m width - aMorph width // 2))].
	
		aMorph height: aMorph submorphBounds height + 2].! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 7/6/2012 09:41:58.34'!
editableFor: aNode

	^ (self data: editableSelector for: aNode) ifNil: [false]
	! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 7/6/2012 09:36:44.919'!
editableSelector
	^ editableSelector! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 7/6/2012 09:36:38.296'!
editableSelector: anObject
	editableSelector := anObject! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 7/8/2012 14:48:27.207'!
editorAcceptSignal
	"Signal that should be listened for to finish the editing."
	
	^ editorAcceptSignal ifNil: [#returnPressed:]! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 7/8/2012 14:49:09.212'!
editorAcceptSignal: aSymbol
	editorAcceptSignal := aSymbol.! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 7/8/2013 16:39:04.26'!
editorCreator
	"A block that creates an editor given the node that needs to be edited."
	
	^ editorCreator ifNil: [editorCreator := [:spec :node |
			UiLineEdit new
				hideCursor: false;
				outerBorderWidth: 0;
				font: UiPlatform current standardListFont;
				theText: (spec textFor: node); selectAll]].! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 7/6/2012 11:42:07.863'!
editorCreator: aBlock	
	editorCreator := aBlock.! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 7/12/2012 10:29:57.951'!
editorFinisher
	"A block that should take editor data and write it back to the model."
	
	^ editorFinisher ifNil: [editorFinisher := [:editor :node |
			node perform: (self textSelector, #:) asSymbol withArguments: {editor theText}]].! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 7/6/2012 11:46:04.146'!
editorFinisher: aBlock
	editorFinisher := aBlock.! !

!UiItemViewColumnSpec methodsFor: 'building' stamp: 'mt 7/6/2012 11:43:21.232'!
editorFor: aNode

	| editor |
	editor := (self editorCreator value: self value: aNode)
		hResizing: #spaceFill;
		setProperty: #selectable toValue: true;
		yourself.
	^ editor! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 7/8/2012 14:49:32.343'!
editorRevertSignal
	
	^ editorRevertSignal ifNil: [#escapePressed:]! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 7/8/2012 14:49:43.667'!
editorRevertSignal: aSymbol
	
	editorRevertSignal := aSymbol.! !

!UiItemViewColumnSpec methodsFor: 'as yet unclassified' stamp: 'mt 8/19/2013 14:29:46.718'!
hAlignment

	^ hAlignment ifNil: [hAlignment := #left]! !

!UiItemViewColumnSpec methodsFor: 'as yet unclassified' stamp: 'mt 8/19/2013 14:30:44.51'!
hAlignment: symbol
	"#left, #center, #right"
	
	hAlignment := symbol.! !

!UiItemViewColumnSpec methodsFor: 'header' stamp: 'mt 1/22/2012 16:39:08.048'!
headerBalloonText

	^ headerBalloonText! !

!UiItemViewColumnSpec methodsFor: 'header' stamp: 'mt 1/22/2012 16:39:23.239'!
headerBalloonText: stringOrText

	headerBalloonText := stringOrText.! !

!UiItemViewColumnSpec methodsFor: 'header' stamp: 'mt 1/22/2012 16:38:42.192'!
headerIcon

	^ headerIcon! !

!UiItemViewColumnSpec methodsFor: 'header' stamp: 'mt 1/22/2012 16:38:31.156'!
headerIcon: aForm

	headerIcon := aForm.! !

!UiItemViewColumnSpec methodsFor: 'building' stamp: 'mt 7/8/2013 16:39:15.496'!
headerItem

	| icon text balloonText item |	
	icon := self headerIcon.
	text := self headerText.
	balloonText := self headerBalloonText.
	
	(text isNil and: [icon isNil]) ifTrue: [text := ' ' asText].
	
	item := Morph new color: Color transparent.
		
	icon ifNotNil: [item addMorphBack: icon asMorph].
	text ifNotNil: [
		text isString ifTrue: [text := text asText].
		text isString | text isText ifFalse: [text := text asString asText].
		self overrideStandardListFont ifFalse: [text addAttribute: (TextFontReference toFont: UiPlatform current standardListFont)].
		item addMorphBack: text asMorph lock].
	balloonText ifNotNil: [
		item balloonText: balloonText].
	
	self doLayoutFor: item.	

	^ item! !

!UiItemViewColumnSpec methodsFor: 'header' stamp: 'mt 1/22/2012 16:37:57.892'!
headerText

	^ headerText! !

!UiItemViewColumnSpec methodsFor: 'header' stamp: 'mt 1/22/2012 16:38:13.581'!
headerText: stringOrText

	headerText := stringOrText.! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 1/22/2012 17:08:20.423'!
iconConverter

	^ iconConverter! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 1/22/2012 17:07:45.402'!
iconConverter: aBlock

	iconConverter := aBlock.! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 2/12/2012 15:58:15.074'!
iconFor: aNode

	^ self data: iconSelector for: aNode

	! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.281'!
iconSelector
	^ iconSelector! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.312'!
iconSelector: anObject
	iconSelector := anObject! !

!UiItemViewColumnSpec methodsFor: 'testing' stamp: 'mt 1/23/2012 11:01:23.807'!
includesAnyProperty: someProperties
	
	^ someProperties anySatisfy: [:property | self includesProperty: property]! !

!UiItemViewColumnSpec methodsFor: 'testing' stamp: 'mt 2/12/2012 16:25:45.346'!
includesAnyProperty: someProperties inNode: aNode
	
	^ someProperties anySatisfy: [:property | self includesProperty: property inNode: aNode]! !

!UiItemViewColumnSpec methodsFor: 'testing' stamp: 'mt 7/6/2012 09:37:56.118'!
includesProperty: aProperty
	
	^ {textSelector. iconSelector. balloonTextSelector. checkedSelector. selectableSelector. checkableSelector. editableSelector}
		anySatisfy: [:selector | selector == aProperty]! !

!UiItemViewColumnSpec methodsFor: 'testing' stamp: 'mt 7/9/2012 17:15:29.478'!
includesProperty: aProperty inNode: aNode

	| checkBlock |
	checkBlock := [:symbolOrInteger | (symbolOrInteger isInteger
		ifTrue: [aNode properties at: symbolOrInteger ifAbsent: []]
		ifFalse: [symbolOrInteger]) = aProperty].
	
	^ {textSelector. iconSelector. balloonTextSelector. checkedSelector. selectableSelector. checkableSelector. editableSelector}
		anySatisfy: [:selector | checkBlock value: selector]! !

!UiItemViewColumnSpec methodsFor: 'building' stamp: 'mt 8/14/2013 11:05:11.469'!
itemFor: aNode

	| icon text balloonText item checkbox itemColor morph |	
	icon := self iconFor: aNode "ifNil: [Morph new extent: 16@16; color: Color transparent]".
	morph := self morphFor: aNode.
	text := self textFor: aNode.
	balloonText := self balloonTextFor: aNode.
	itemColor := self colorFor: aNode.
	
	"Configurable data conversion step."
	self iconConverter ifNotNil: [:c | icon := c value: icon].
	self textConverter ifNotNil: [:c | text := c value: text].
	self balloonTextConverter ifNotNil: [:c | balloonText := c value: balloonText].
	self colorConverter ifNotNil: [:c | itemColor := c value: itemColor].
	
	"Workaround for item layouting if item is empty."
	(text isNil and: [icon isNil]) ifTrue: [text := ' ' asText].
	
	item := Morph new color: (itemColor ifNil: [Color transparent]).
	
	"A checkbox is created if this column was configured to have checkable nodes."
	(self checkableFor: aNode) ifTrue: [
		checkbox := UiCheckBox basicNew initializeNoText.
		checkbox checked: (self checkedFor: aNode).
		self connect: checkbox signal: #toggled: to: aNode selector: (self checkedSelector, #:) asSymbol.
		self connect: checkbox signal: #toggled: to: checkbox selector: #visible:.
		item addMorphBack: checkbox].
	
	icon ifNotNil: [item addMorphBack: icon asMorph].
	morph ifNotNil: [item addMorphBack: morph].
	text ifNotNil: [
		text isString ifTrue: [text := text asText].
		text isString | text isText ifFalse: [text := text asString asText].
		self overrideStandardListFont ifFalse: [text addAttribute: (TextFontReference toFont: UiPlatform current standardListFont)].
		item addMorphBack: text asMorph lock].
	balloonText ifNotNil: [
		item balloonText: balloonText].
	
	"Store morphs in properties for later access."
	item
		setProperty: #text toValue: text;
		setProperty: #icon toValue: icon;
		setProperty: #balloonText toValue: balloonText;
		setProperty: #checkBox toValue: checkbox;
		setProperty: #selectable toValue: (self selectableFor: aNode).

	self doLayoutFor: item.	
	checkbox ifNotNil: [checkbox hide].
	
	^ item! !

!UiItemViewColumnSpec methodsFor: 'building' stamp: 'mt 1/22/2012 16:55:48.639'!
itemOrientation

	^ itemOrientation ifNil: [itemOrientation := #horizontal]! !

!UiItemViewColumnSpec methodsFor: 'building' stamp: 'mt 1/22/2012 16:56:11.312'!
itemOrientation: aSymbol
	"#vertical, #horizontal"

	 itemOrientation := aSymbol.! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 2/14/2012 17:17:42.851'!
menuFor: aNode

	^ [self data: menuSelector for: aNode]
		on: MessageNotUnderstood
		do: ["Allow only some nodes to implement a menuSelector. This DNU is not expensive."]

	! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.465'!
menuSelector
	^ menuSelector! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.495'!
menuSelector: anObject
	menuSelector := anObject! !

!UiItemViewColumnSpec methodsFor: 'as yet unclassified' stamp: 'mt 8/14/2013 11:00:22.499'!
morphConverter

	^ morphConverter! !

!UiItemViewColumnSpec methodsFor: 'as yet unclassified' stamp: 'mt 8/14/2013 11:00:32.686'!
morphConverter: aBlock

	morphConverter := aBlock.! !

!UiItemViewColumnSpec methodsFor: 'as yet unclassified' stamp: 'mt 8/14/2013 10:59:47.071'!
morphFor: aNode

	^ self data: morphSelector for: aNode! !

!UiItemViewColumnSpec methodsFor: 'as yet unclassified' stamp: 'mt 8/14/2013 10:59:58.536'!
morphSelector
	^ morphSelector! !

!UiItemViewColumnSpec methodsFor: 'as yet unclassified' stamp: 'mt 8/14/2013 11:00:11.339'!
morphSelector: anObject
	morphSelector := anObject.! !

!UiItemViewColumnSpec methodsFor: 'accessing' stamp: 'mt 6/26/2012 08:56:51.738'!
overrideStandardListFont
	"Check whether Preferences>>#standardListFont should NOT be used for all items, but custom text fonts are supported."
	
	^ overrideStandardListFont ifNil: [false]! !

!UiItemViewColumnSpec methodsFor: 'accessing' stamp: 'mt 6/26/2012 08:57:10.244'!
overrideStandardListFont: aBoolean

	overrideStandardListFont := aBoolean.! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 6/26/2012 13:11:09.727'!
selectableFor: aNode

	^ (self data: selectableSelector for: aNode) ifNil: [true]
	! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 6/26/2012 13:09:29.552'!
selectableSelector
	^ selectableSelector! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 6/26/2012 13:09:29.597'!
selectableSelector: anObject
	selectableSelector := anObject! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 1/22/2012 17:08:42.26'!
textConverter

	^ textConverter! !

!UiItemViewColumnSpec methodsFor: 'accessing - converting' stamp: 'mt 1/22/2012 17:07:33.389'!
textConverter: aBlock

	textConverter := aBlock.! !

!UiItemViewColumnSpec methodsFor: 'accessing - nodes' stamp: 'mt 8/14/2013 10:59:33.371'!
textFor: aNode

	^ self data: textSelector for: aNode! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.219'!
textSelector
	^ textSelector! !

!UiItemViewColumnSpec methodsFor: 'accessing - selectors' stamp: 'mt 1/22/2012 16:35:00.25'!
textSelector: anObject
	textSelector := anObject! !

!UiItemViewColumnSpec methodsFor: 'as yet unclassified' stamp: 'mt 8/19/2013 14:29:34.696'!
vAlignment

	^ vAlignment ifNil: [vAlignment := #top]! !

!UiItemViewColumnSpec methodsFor: 'as yet unclassified' stamp: 'mt 8/19/2013 14:30:48.916'!
vAlignment: symbol
	"#top, #center, #bottom"
	
	vAlignment := symbol.! !

UiItemViewColumnSpec subclass: #UiItemViewSimpleColumnSpec
	instanceVariableNames: 'labelColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiItemViewSimpleColumnSpec commentStamp: 'mt 8/16/2012 09:26:04.678' prior: 0!
Performance-optimized version of the default column specification.!

!UiItemViewSimpleColumnSpec methodsFor: 'building' stamp: 'mt 8/19/2013 15:13:19.008'!
doLayoutFor: aMorph

	| offset |

	(self itemOrientation = #horizontal or: [self itemOrientation = #horizontalMaxHeight]) ifTrue: [
		| formHeight |
		formHeight := aMorph form ifNil: [0] ifNotNil: [:f | f height].
		self itemOrientation = #horizontalMaxHeight
			ifTrue: [aMorph height: (aMorph font height max: formHeight)]
			ifFalse: [aMorph height: aMorph font height].
				
		"Manual layouting."
		offset := 3 @ 1.
		aMorph form ifNotNil: [
			aMorph formPosition: (offset + (0@(aMorph height - aMorph form height // 2))).
			offset := offset + ((aMorph form width + 2) @ 0)].
		
		aMorph label ifNotNil: [
			aMorph labelPosition: (offset + (0@(aMorph height - aMorph font height // 2))).
			offset := offset + (((aMorph font widthOfString: aMorph label) + 2) @ 0)].
		
		"Expand item horizontally to its contents."
		aMorph
			width: offset x; "font rendering bugfix"
			height: aMorph height + 1. "font rendering bugfix"
		
		^ self].
		
	self itemOrientation = #vertical ifTrue: [
		| formWidth labelWidth |
		formWidth := aMorph form ifNil: [0] ifNotNil: [:f | f width].
		labelWidth := aMorph label ifNil: [0] ifNotNil: [:l | aMorph font widthOfString: l].
		
		aMorph width: (formWidth max: labelWidth) + 10.
			
		"Manual layouting."
		offset := 3.
		aMorph form ifNotNil: [
			aMorph formPosition: (aMorph width - formWidth // 2)@offset.
			offset := offset + aMorph form height + 2].
		
		aMorph label ifNotNil: [
			aMorph labelPosition: (aMorph width - labelWidth // 2)@offset.
			offset := offset + aMorph font height + 2].		
		
		aMorph height: offset].! !

!UiItemViewSimpleColumnSpec methodsFor: 'building' stamp: 'mt 9/17/2013 10:37:30.053'!
headerItem

	| item |	
	item := UiItemViewMorph new
		form: self headerIcon;
		label: (self headerText ifNil: [''] ifNotNil: [:t | t asString]);
		font: UiPlatform current standardListFont;
		color: nil;
		labelColor: Color black;
		balloonText: self headerBalloonText;	
		yourself.

	self doLayoutFor: item.	
	
	^ item! !

!UiItemViewSimpleColumnSpec methodsFor: 'building' stamp: 'mt 8/19/2013 15:29:01.986'!
itemFor: aNode

	| item emph text |	
	text := self textFor: aNode.
	emph := text isText
		ifFalse: [0]
		ifTrue: [((text attributesAt: 1) detect: [:att | att emphasisCode ~= 0] ifNone: [])
			ifNil: [0]
			ifNotNil: [:att | att emphasisCode]].
		
	item := UiItemViewMorph new
		form: ((self morphFor: aNode)
			ifNil: [self iconFor: aNode]
			ifNotNil: [:morph | morph imageForm]);
		label: (text ifNotNil: [:t | t asString]);
		font: (UiPlatform current standardListFont emphasized: emph);
		color: (self colorFor: aNode);
		labelColor: self labelColor;
		balloonText: (self balloonTextFor: aNode);	
		setProperty: #selectable toValue: (self selectableFor: aNode);
		setProperty: #text toValue: text;
		yourself.

	self doLayoutFor: item.	
	
	^ item! !

!UiItemViewSimpleColumnSpec methodsFor: 'accessing' stamp: 'mt 8/16/2012 09:12:06.096'!
labelColor

	^ labelColor ifNil: [labelColor := Color black]! !

!UiItemViewSimpleColumnSpec methodsFor: 'accessing' stamp: 'mt 9/3/2013 14:00:56.861'!
labelColor: aColor

	aColor ifNil: [^ self].
	
	labelColor := aColor isString
		ifTrue: [Color fromString: aColor]
		ifFalse: [aColor].! !

Object subclass: #UiItemViewDragDropSpec
	instanceVariableNames: 'dragEnabled dropEnabled source dragTransferType textSelector iconSelector dropMode acceptBlock helpText'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/19/2012 11:07:48.681'!
acceptBlock
	"See TransferMorph for accessible fields. This block is called to check whether the object should be dropped in the view that uses this spec."

	^ acceptBlock ifNil: [acceptBlock := [:transferMorph | true]]! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/19/2012 09:23:40.744'!
acceptBlock: aBlock

	acceptBlock := aBlock.! !

!UiItemViewDragDropSpec methodsFor: 'accessing - nodes' stamp: 'mt 7/18/2012 10:28:38.24'!
data: aSymbolOrIntegerOrBoolean for: aNode
	"Private. Access property directly or use a indirection via #properties in node. This makes tree nodes having different kinds of properties more flexible."
	
	| property |
	aSymbolOrIntegerOrBoolean ifNil: [^ nil].
	
	"Support hardcoded boolean defaults."
	aSymbolOrIntegerOrBoolean == true ifTrue: [^ true].
	aSymbolOrIntegerOrBoolean == false ifTrue: [^ false].
	
	"Support ordered properties referenced by an integer. Consider #properties in node."
	property := aSymbolOrIntegerOrBoolean isInteger
		ifTrue: [aNode properties at: aSymbolOrIntegerOrBoolean ifAbsent: []]
		ifFalse: [aSymbolOrIntegerOrBoolean].
	
	"Lookup property as accessor or key in dictionary."
	^ property ifNotNil: [:p | (p ~~ #yourself and: [aNode isDictionary])
		ifTrue: [aNode at: p ifAbsent: []]
		ifFalse: [aNode perform: p]]! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:04:23.878'!
dragEnabled
	^ dragEnabled ifNil: [true]! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:04:08.007'!
dragEnabled: anObject
	dragEnabled := anObject! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/19/2012 11:06:30.291'!
dragTransferType
	"Can be used to lookup the kind of data that was picked up in the view that uses this spec."

	^ dragTransferType! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:21:32.514'!
dragTransferType: anObject
	dragTransferType := anObject! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:04:35.402'!
dropEnabled
	^ dropEnabled ifNil: [true]! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:04:08.045'!
dropEnabled: anObject
	dropEnabled := anObject! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:57:29.844'!
dropMode
	"#over, #between"
	
	^ dropMode ifNil: [dropMode := #between]! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:57:04.5'!
dropMode: anObject
	dropMode := anObject! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 8/22/2012 09:58:18.3'!
helpText

	^ helpText! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 8/22/2012 09:58:25.871'!
helpText: aString

	helpText := aString.! !

!UiItemViewDragDropSpec methodsFor: 'accessing - nodes' stamp: 'mt 7/18/2012 10:28:59.403'!
iconFor: aNode

	^ self data: iconSelector for: aNode! !

!UiItemViewDragDropSpec methodsFor: 'accessing - selectors' stamp: 'mt 7/18/2012 10:27:22.919'!
iconSelector
	^ iconSelector! !

!UiItemViewDragDropSpec methodsFor: 'accessing - selectors' stamp: 'mt 7/18/2012 10:27:22.966'!
iconSelector: anObject
	iconSelector := anObject! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:17:00.455'!
source
	^ source! !

!UiItemViewDragDropSpec methodsFor: 'accessing' stamp: 'mt 7/18/2012 10:17:00.503'!
source: anObject
	source := anObject! !

!UiItemViewDragDropSpec methodsFor: 'accessing - nodes' stamp: 'mt 7/25/2012 14:59:53.568'!
textFor: aNode

	^ (self data: textSelector for: aNode) ifNil: [aNode asString]! !

!UiItemViewDragDropSpec methodsFor: 'accessing - selectors' stamp: 'mt 7/18/2012 10:27:10.578'!
textSelector
	^ textSelector! !

!UiItemViewDragDropSpec methodsFor: 'accessing - selectors' stamp: 'mt 7/18/2012 10:27:10.622'!
textSelector: anObject
	textSelector := anObject! !

!UiItemViewDragDropSpec methodsFor: 'building' stamp: 'mt 8/21/2012 10:26:32.298'!
transferItemFor: someNodes

	| item draggedMorph |
	item := TransferMorph withPassenger: someNodes from: self source.
	item dragTransferType: self dragTransferType.
	
	draggedMorph := Morph new
		color: Color transparent;
		changeTableLayout;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap.
		
	someNodes size > 3
		ifTrue: [
			draggedMorph listDirection: #leftToRight.	
			(Smalltalk includesKey: #UiFugueIcons)
				ifTrue: [draggedMorph addMorphBack: ((Smalltalk at: #UiFugueIcons) documentsStackIcon asMorph)].
				
			draggedMorph
				addMorphBack: ((self textFor: someNodes first) asString, '(', (someNodes size - 1) asString, ' more)') asMorph]
		ifFalse: [
			draggedMorph listDirection: #topToBottom.
			someNodes do: [:node |
				| row |
				row := Morph new
					color: Color transparent;
					changeTableLayout;
					listDirection: #leftToRight;
					hResizing: #shrinkWrap;
					vResizing: #shrinkWrap.

				(self iconFor: node) ifNotNil: [:form |
					row addMorphBack: form asMorph].
				row addMorphBack: (self textFor: node) asMorph.
				draggedMorph addMorphBack: row]].
	
	item draggedMorph: draggedMorph.
	item addMorphBack: draggedMorph.
	item fullBounds.

	^ item! !

Object subclass: #UiItemViewGroupSpec
	instanceVariableNames: 'hideIfFirst hideIfEmpty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiItemViewGroupSpec methodsFor: 'building' stamp: 'mt 11/16/2012 15:47:59.671'!
doLayoutFor: aMorph

	| offset |
	aMorph submorphs last isTextMorph
		ifTrue: [ | fontHeight |
			fontHeight := aMorph submorphs last textStyle defaultFont height + 4.
			(fontHeight * 2) < aMorph submorphs last height
				"More than one line?"
				ifTrue: [aMorph height: aMorph submorphs last height]
				ifFalse: [aMorph height: fontHeight]]
		ifFalse: [
			aMorph height: aMorph submorphs last height].
	
	"Manual layouting."
	offset := (aMorph left + 3) @ 1.
	aMorph submorphs do: [:m |
		m left: offset x.
		offset := offset + ((m width + 2) @ 0).
		m top: aMorph top - (m height - aMorph height // 2) + offset y + 2].

	"Expand item horizontally to its contents."
	aMorph
		width: aMorph submorphBounds width + 5; "font rendering bugfix"
		height: aMorph height + 1. "font rendering bugfix"! !

!UiItemViewGroupSpec methodsFor: 'accessing' stamp: 'mt 12/12/2012 11:39:49.235'!
hideIfEmpty

	^ hideIfEmpty ifNil: [false]! !

!UiItemViewGroupSpec methodsFor: 'accessing' stamp: 'mt 12/12/2012 11:40:05.424'!
hideIfEmpty: aBoolean

	hideIfEmpty := aBoolean.! !

!UiItemViewGroupSpec methodsFor: 'accessing' stamp: 'mt 6/28/2012 17:16:58.572'!
hideIfFirst
	"Configure visiblity of first group separator for top-level items."

	^ hideIfFirst ifNil: [true]! !

!UiItemViewGroupSpec methodsFor: 'accessing' stamp: 'mt 6/28/2012 17:16:23.937'!
hideIfFirst: aBoolean

	hideIfFirst := aBoolean.! !

!UiItemViewGroupSpec methodsFor: 'building' stamp: 'mt 3/7/2013 10:00:41.634'!
itemFor: aNode groupIndex: aGroupIndex isTopLevel: aBoolean
	"Create a morph that separates groups in item views."
	
	| icon text balloonText item |	
	aBoolean ifFalse: [
		item := Morph new
			color: Color transparent;
			extent: 6@6;
			yourself.
		item
			setProperty: #selectable toValue: false;
			setProperty: #drawLine toValue: false;
			setProperty: #hideIfFirst toValue: true;
			setProperty: #hideIfEmpty toValue: true.
		^ item].
	
	icon := nil.
	text := ('<font family="Bitmap Segoe UI" size="10" color="#AAAAAA">', (aNode groups at: aGroupIndex), '</font>') asHtmlText.
	balloonText := nil.
		
	item := Morph new color: Color transparent.
		
	icon ifNotNil: [item addMorphBack: icon asMorph].
	text ifNotNil: [item addMorphBack: text asMorph lock].
	balloonText ifNotNil: [item balloonText: balloonText].
	
	self doLayoutFor: item.	
	
	item
		setProperty: #selectable toValue: false;
		setProperty: #drawLine toValue: true;
		setProperty: #hideIfFirst toValue: self hideIfFirst;
		setProperty: #hideIfEmpty toValue: self hideIfEmpty.
	
	^ item! !

!UiItemViewGroupSpec methodsFor: 'accessing' stamp: 'mt 11/16/2012 15:48:36.123'!
lineColor

	"Color r: 11/255 g: 97/255 b: 164/255"
	^ Color fromString: #AAAAAA! !

!UiItemViewGroupSpec methodsFor: 'accessing' stamp: 'mt 11/16/2012 15:46:40.281'!
lineOffset

	^ 2! !

UiItemViewGroupSpec subclass: #UiListViewGroupSpec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiListViewGroupSpec methodsFor: 'building' stamp: 'mt 8/31/2012 13:05:59.977'!
itemFor: aNode groupIndex: aGroupIndex isTopLevel: aBoolean
	"Create a morph that separates groups in item views."
	
	^ (super
		itemFor: aNode
		groupIndex: aGroupIndex
		isTopLevel: aBoolean)
			setProperty: #selectable toValue: false;
			yourself! !

UiItemViewGroupSpec subclass: #UiTableViewGroupSpec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTableViewGroupSpec methodsFor: 'building' stamp: 'mt 8/8/2012 13:06:25.896'!
itemFor: aNode groupIndex: aGroupIndex isTopLevel: aBoolean
	"Create a morph that separates groups in item views."
	
	| item |	
	item := Morph new
		color: Color transparent;
		extent: 6@1;
		yourself.
	item
		setProperty: #hideAlways toValue: true;
		setProperty: #selectable toValue: false.

	^ item! !

Object subclass: #UiIterator
	instanceVariableNames: 'root iteratorState iteratorPosition iteratorShadow process sortBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiIterator commentStamp: 'mt 8/8/2012 11:34:05.156' prior: 0!
An iterator for collections. Optionally concurrent.

f := UiIterator new.
f root: Smalltalk classNames.
self connect: f signal: #found: to: Transcript selector: #show:.
self connect: f signal: #found: to: Transcript selector: #show: pattern: {#=. String cr}.
self connect: f signal: #finished to: Transcript selector: #show: pattern: {#=. 'Done.'}.
f nextSatisfying: [:name | name includesSubstring: 'tool' caseSensitive: false] forkAt: 35.!

UiIterator subclass: #UiItemModelFinder
	instanceVariableNames: 'traversalMode rootIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiItemModelFinder commentStamp: 'mt 6/23/2011 17:18:46.568' prior: 0!
A helper class that allows to find model indices in an item model.

A column filter can be used to reduce the search operation to specific columns in a specific order. The amount of columns being searched has an impact on the performance.

The item model can be traversed #breadthFirst or #depthFirst (default).

A subset of the collection API is present (e.g., #do:, #detect:).

====
Example
====

| view model finder |
view := UiTreeView new.
model := UiSmalltalkModel new.
finder := UiItemModelFinder on: model.

finder
   columnFilter: #(1); "nur Spalte 1"
   traversalMode: #depthFirst. "standard; #breadthFirst mglich"

"Knoten finden und selektieren/ausklappen."
view currentIndex: (finder detect: [:modelIndex |
   modelIndex text asString startsWith: "Morph"]).

"Iterative Suche. Fr weiteren Index Code wiederholt ausfhren:"
view currentIndex: (finder nextSatisfying: [:modelIndex |
   modelIndex text asString startsWith: "Morph"]).
!

!UiItemModelFinder class methodsFor: 'instance creation' stamp: 'mt 2/2/2012 15:21:05.931'!
on: aNode

	^ self new
		root: aNode! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:31.949'!
allSatisfy: aBlock

	^ self allSatisfy: aBlock startingAt: self root! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.065'!
allSatisfy: aBlock startingAt: aNode

	self
		do: [:index | (aBlock value: index) ifFalse: [^ false]]
		startingAt: aNode.
		
	^ true! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.15'!
anySatisfy: aBlock

	^ self anySatisfy: aBlock startingAt: self root! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.237'!
anySatisfy: aBlock startingAt: aNode

	self
		do: [:index | (aBlock value: index) ifTrue: [^ true]]
		startingAt: aNode.
		
	^ false! !

!UiItemModelFinder methodsFor: 'collection api - private' stamp: 'mt 8/8/2012 11:02:35.254'!
childNodesFor: aParentNode

	^ aParentNode groups gather: [:group |
		(1 to: (aParentNode childCountInGroup: group)) collect: [:i | aParentNode childAt: i inGroup: group]]! !

!UiItemModelFinder methodsFor: 'iterator api - private' stamp: 'mt 7/9/2013 11:27:16.435'!
childrenFor: aParentModelIndex

	| node |
	node := aParentModelIndex
		ifNil: [self root]
		ifNotNil: [:index | index node].
	
	self flag: #performance. "mt: signal connections not necessary. see UiModelIndex>>connectIndex."
	^ (node groups gather: [:group | 
		| indices |
		indices := (1 to: (node childCountInGroup: group)) collect: [:i | 
			UiModelIndex
				row: i column: 1 group: group
				groupIndex: nil "not used"
				parent: aParentModelIndex
				node: (node childAt: i inGroup: group)
				spec: nil "not used"].
		self sortBlock ifNotNil: [:sb |
			"node unboxing"
			indices := indices sorted: [:i1 :i2 | sb value: i1 node value: i2 node]].
		indices]) asOrderedCollection! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.323'!
count: aBlock

	^ self count: aBlock startingAt: self root! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.408'!
count: aBlock startingAt: aNode

	| sum |
	sum := 0.
	self
		do: [:each | (aBlock value: each) ifTrue: [sum := sum + 1]]
		startingAt: aNode.
	^ sum! !

!UiItemModelFinder methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:06:35.642'!
current

	(iteratorPosition isNil or: [iteratorPosition = 0])
		ifTrue: [^ self root].

	^ self currentIndex ifNotNil: [:index | index node]! !

!UiItemModelFinder methodsFor: 'iterator api - private' stamp: 'mt 8/8/2012 11:08:21.665'!
currentIndex

	(iteratorPosition isNil or: [iteratorPosition = 0])
		ifTrue: [^ self rootIndex].
		
	iteratorPosition > iteratorState size
		ifTrue: [^ nil "at end"].
	
	^ iteratorState at: iteratorPosition! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.494'!
detect: aBlock

	^ self
		detect: aBlock
		ifNone: [Error signal: 'Object is not in the collection.']! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.579'!
detect: aBlock ifNone: exceptionBlock

	^ self
		detect: aBlock
		ifNone: exceptionBlock
		startingAt: self root
		! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.665'!
detect: aBlock ifNone: exceptionBlock startingAt: aNode

	self
		do: [:index | (aBlock value: index) ifTrue: [^ index]]
		startingAt: aNode.
		
	^ exceptionBlock value! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.75'!
detect: aBlock startingAt: aNode

	^ self
		detect: aBlock
		ifNone: [Error signal: 'Object is not in the collection.']
		startingAt: aNode! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.837'!
do: aBlock
	
	self do: aBlock startingAt: self root.! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:32.923'!
do: aBlock startingAt: aNode
	"Startng index WILL NOT be evaluated."
	
	self traversalMode == #depthFirst
		ifTrue: [self doDepthFirst: aBlock startingAt: aNode].
	self traversalMode == #breadthFirst
		ifTrue: [self doBreadthFirst: aBlock startingAt: aNode].! !

!UiItemModelFinder methodsFor: 'collection api - private' stamp: 'mt 8/8/2012 10:58:06.684'!
doBreadthFirst: aBlock startingAt: aNode
	"Breadth-first search."
	
	| nextChildren |
	nextChildren := (self childNodesFor: aNode) asOrderedCollection.
		
	[nextChildren notEmpty] whileTrue: [
		| nextChild |
		nextChild := nextChildren removeFirst.
		nextChildren addAll: (self childNodesFor: nextChild).
		aBlock value: nextChild].! !

!UiItemModelFinder methodsFor: 'collection api - private' stamp: 'mt 8/8/2012 10:58:11.368'!
doDepthFirst: aBlock startingAt: aNode
	"Depth-search."

	| nextChildren |
	nextChildren := (self childNodesFor: aNode) asOrderedCollection.
		
	[nextChildren notEmpty] whileTrue: [
		| nextChild |
		nextChild := nextChildren removeFirst.
		nextChildren addAllFirst: (self childNodesFor: nextChild).
		aBlock value: nextChild].! !

!UiItemModelFinder methodsFor: 'iterator api - signals' stamp: 'mt 8/8/2012 11:05:29.649'!
foundIndex: aModelIndex

	self emit.! !

!UiItemModelFinder methodsFor: 'iterator api - testing' stamp: 'mt 8/8/2012 11:04:45.763'!
hasPrevious

	^ self currentIndex ~= self rootIndex or: [self hasFinished]! !

!UiItemModelFinder methodsFor: 'iterator api - private' stamp: 'mt 8/8/2012 11:14:18.123'!
insertNextRows

	| newRows |
	newRows := self childrenFor: self currentIndex.

	self traversalMode = #depthFirst ifTrue: [
		newRows reverseDo: [:row |
			iteratorState add: row afterIndex: iteratorPosition]].
	self traversalMode = #breadthFirst ifTrue: [
		iteratorState addAll: newRows].! !

!UiItemModelFinder methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:09:40.15'!
next

	^ self nextIndex ifNotNil: [:index | index node]! !

!UiItemModelFinder methodsFor: 'iterator api - private' stamp: 'mt 8/8/2012 11:09:22.364'!
nextIndex

	"Already at end?"
	(iteratorPosition notNil and: [self currentIndex isNil])
		ifTrue: [^ self currentIndex "nil"].

	"Initialize after #reset."
	iteratorState ifNil: [iteratorState := OrderedCollection new].
	iteratorPosition ifNil: [iteratorPosition := 0].
	iteratorShadow ifNil: [iteratorShadow := 0].

	"Append new rows if necessary."
	iteratorPosition = iteratorShadow
		ifTrue: [self insertNextRows].
	
	self incrementIteratorPosition.
	
	^ self currentIndex! !

!UiItemModelFinder methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:12:37.654'!
nextSatisfying: aBlock forkAt: aPriority
	"Asynchronous version of #nextSatisfying:. The signal #indexFound: will be emitted when operation has finished."

	self fork: [
		| node |
		[(node := self next) notNil] whileTrue: [
			(aBlock value: node) ifTrue: [self found: node; foundIndex: self currentIndex]].
		self finished] at: aPriority.! !

!UiItemModelFinder methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:10:19.443'!
previous

	^ self previousIndex
		ifNil: [self root "the beginning"]
		ifNotNil: [:index | index node]! !

!UiItemModelFinder methodsFor: 'iterator api - private' stamp: 'mt 8/8/2012 11:10:27.701'!
previousIndex

	"Was resetted but not used yet?"
	iteratorPosition ifNil: [^ self rootIndex].
		
	self decrementIteratorPosition.
	
	^ self currentIndex! !

!UiItemModelFinder methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:13:15.792'!
previousSatisfying: aBlock forkAt: aPriority
	"Asynchronous version of #previousSatisfying:. The signal #indexFound: will be emitted when operation has finished."

	self fork: [
		| node |
		[(node := self previous) ~= self root] whileTrue: [
			(aBlock value: node) ifTrue: [self found: node; foundIndex: self currentIndex]].
		self finished] at: aPriority.! !

!UiItemModelFinder methodsFor: 'iterator api' stamp: 'mt 8/8/2012 13:00:04.099'!
reset

	iteratorState := nil.
	iteratorPosition := nil.
	iteratorShadow := nil.
	process ifNotNil: [
		process terminate.
		process := nil].! !

!UiItemModelFinder methodsFor: 'accessing' stamp: 'mt 2/7/2012 16:29:20.307'!
rootIndex

	^ rootIndex! !

!UiItemModelFinder methodsFor: 'accessing' stamp: 'mt 2/7/2012 16:34:29.816'!
rootIndex: aModelIndex

	rootIndex := aModelIndex.
	self root: aModelIndex node.! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:33.009'!
select: aBlock

	^ self select: aBlock startingAt: self root! !

!UiItemModelFinder methodsFor: 'collection api' stamp: 'mt 8/8/2012 11:02:33.097'!
select: aBlock startingAt: aNode

	| newCollection |
	newCollection := OrderedCollection new.
	self
		do: [:each | (aBlock value: each) ifTrue: [newCollection add: each]]
		startingAt: aNode.
		
	^ newCollection! !

!UiItemModelFinder methodsFor: 'accessing' stamp: 'mt 6/23/2011 14:27:35.279'!
traversalMode

	^ traversalMode ifNil: [traversalMode := #depthFirst]! !

!UiItemModelFinder methodsFor: 'accessing' stamp: 'mt 2/7/2012 15:48:34.252'!
traversalMode: aSymbol
	"#depthFirst, #breadthFirst"

	traversalMode = aSymbol ifTrue: [^ self].
	traversalMode := aSymbol.
	self reset.! !

!UiIterator methodsFor: 'iterator api - signals' stamp: 'mt 8/8/2012 10:56:53.059'!
cancelled

	self emit.! !

!UiIterator methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:08:52.98'!
current

	(iteratorPosition isNil or: [iteratorPosition = 0])
		ifTrue: [^ self root].
		
	iteratorPosition > iteratorState size
		ifTrue: [^ nil "at end"].
	
	^ iteratorState at: iteratorPosition! !

!UiIterator methodsFor: 'iterator api - private' stamp: 'mt 8/8/2012 10:56:50.666'!
decrementIteratorPosition

	iteratorPosition := iteratorPosition - 1 max: 0.! !

!UiIterator methodsFor: 'iterator api - signals' stamp: 'mt 8/8/2012 10:56:53.178'!
finished

	self emit.! !

!UiIterator methodsFor: 'iterator api - private' stamp: 'mt 8/8/2012 10:56:50.751'!
fork: aBlock at: aPriority

	process ifNotNil: [
		process terminate.
		self cancelled].

	process := aBlock forkAt: aPriority.! !

!UiIterator methodsFor: 'iterator api - signals' stamp: 'mt 8/8/2012 10:56:53.263'!
found: aNode

	self emit.! !

!UiIterator methodsFor: 'iterator api - testing' stamp: 'mt 8/8/2012 10:56:55.198'!
hasFinished

	^ iteratorPosition notNil and: [iteratorPosition > iteratorState size]! !

!UiIterator methodsFor: 'iterator api - testing' stamp: 'mt 8/8/2012 10:56:55.329'!
hasNext

	^ self hasFinished not! !

!UiIterator methodsFor: 'iterator api - testing' stamp: 'mt 8/8/2012 11:05:15.654'!
hasPrevious

	^ self current ~= self root first or: [self hasFinished]! !

!UiIterator methodsFor: 'iterator api - private' stamp: 'mt 8/8/2012 10:56:50.836'!
incrementIteratorPosition

	iteratorPosition := iteratorPosition + 1 min: iteratorState size + 1.
	
	iteratorShadow < iteratorPosition
		ifTrue: [iteratorShadow := iteratorPosition].! !

!UiIterator methodsFor: 'iterator api - private' stamp: 'mt 7/9/2013 11:27:57.313'!
insertNextRows

	iteratorState addAll: (self sortBlock
		ifNil: [self root asOrderedCollection]
		ifNotNil: [:sb | self root sorted: sb])
	! !

!UiIterator methodsFor: 'iterator api - testing' stamp: 'mt 8/8/2012 10:56:55.501'!
isRunning

	^ process notNil! !

!UiIterator methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:41:13.566'!
next

	"Already at end?"
	(iteratorPosition notNil and: [self current isNil])
		ifTrue: [^ self current "nil"].

	"Initialize after #reset."
	iteratorPosition ifNil: [iteratorPosition := 0].
	iteratorShadow ifNil: [iteratorShadow := 0].
	
	self incrementIteratorPosition.
	
	^ self current! !

!UiIterator methodsFor: 'iterator api' stamp: 'mt 8/8/2012 10:56:47.985'!
nextSatisfying: aBlock

	| node |
	[(node := self next) notNil] whileTrue: [
		(aBlock value: node) ifTrue: [^ node]].
	^ nil! !

!UiIterator methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:12:55.301'!
nextSatisfying: aBlock forkAt: aPriority
	"Asynchronous version of #nextSatisfying:. The signal #indexFound: will be emitted when operation has finished."

	self fork: [
		| node |
		[(node := self next) notNil] whileTrue: [
			(aBlock value: node) ifTrue: [self found: node]].
		self finished] at: aPriority.! !

!UiIterator methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:10:52.077'!
previous

	"Was resetted but not used yet?"
	iteratorPosition ifNil: [^ self root].
		
	self decrementIteratorPosition.
	
	^ self current! !

!UiIterator methodsFor: 'iterator api' stamp: 'mt 8/8/2012 10:56:48.284'!
previousSatisfying: aBlock

	| node |
	[(node := self previous) ~= self root] whileTrue: [
		(aBlock value: node) ifTrue: [^ node]].
	^ nil! !

!UiIterator methodsFor: 'iterator api' stamp: 'mt 8/8/2012 11:13:20.11'!
previousSatisfying: aBlock forkAt: aPriority
	"Asynchronous version of #previousSatisfying:. The signal #indexFound: will be emitted when operation has finished."

	self fork: [
		| node |
		[(node := self previous) ~= self root] whileTrue: [
			(aBlock value: node) ifTrue: [self found: node]].
		self finished] at: aPriority.! !

!UiIterator methodsFor: 'iterator api' stamp: 'mt 7/9/2013 11:28:13.158'!
reset

	process ifNotNil: [
		process terminate.
		self hasFinished ifFalse: [self cancelled].
		process := nil].


	iteratorState := (self sortBlock
		ifNil: [self root asOrderedCollection]
		ifNotNil: [:sb | self root sorted: sb]).
	
	iteratorPosition := nil.
	iteratorShadow := nil.
! !

!UiIterator methodsFor: 'accessing' stamp: 'mt 8/8/2012 10:56:08.628'!
root

	^ root! !

!UiIterator methodsFor: 'accessing' stamp: 'mt 8/8/2012 10:56:10.689'!
root: aNode

	root = aNode ifTrue: [^ self].
	root := aNode.
	self reset.! !

!UiIterator methodsFor: 'accessing' stamp: 'mt 8/8/2012 10:56:13.772'!
sortBlock

	^ sortBlock ifNil: [sortBlock := [:node1 :node2 | true]]! !

!UiIterator methodsFor: 'accessing' stamp: 'mt 8/8/2012 10:56:15.841'!
sortBlock: aBlock

	sortBlock := aBlock.
	self reset.! !

Object subclass: #UiNullMutex
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Events'!

!UiNullMutex commentStamp: 'mt 3/28/2011 10:14:51.28' prior: 0!
This is a null mutex that understands #critical: but does not secure anything.!

!UiNullMutex class methodsFor: 'critical' stamp: 'mt 3/28/2011 10:14:29.075'!
critical: aBlock

	^ aBlock value! !

Object subclass: #UiPainter
	instanceVariableNames: 'morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Painter'!

UiPainter subclass: #UiGlossyPainter
	instanceVariableNames: 'orientation fraction highlight bounds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Painter'!

!UiGlossyPainter methodsFor: 'geometry' stamp: 'mt 8/1/2013 19:31:47.831'!
bounds

	^ bounds ifNil: [super bounds]! !

!UiGlossyPainter methodsFor: 'colors' stamp: 'mt 1/6/2012 09:39:08.895'!
color

	self morph hasKeyboard
		ifTrue: [^ (Color r: 0.0 g: 0.0 b: 0.5) muchLighter paler].
		
	^ super color! !

!UiGlossyPainter methodsFor: 'colors' stamp: 'mt 1/2/2012 16:35:41.31'!
colorBorder
	"By default, the border color is computed from the base color."
	
	^ self color mixed: 0.65 with: Color black! !

!UiGlossyPainter methodsFor: 'colors' stamp: 'mt 1/6/2012 09:51:30.651'!
colorChanged

	^ Color yellow muchLighter paler! !

!UiGlossyPainter methodsFor: 'colors' stamp: 'mt 1/2/2012 16:40:59.129'!
colorHighlight
	"The highlight can be used to intensify the bevel effect."

	^ self morph
		valueOfProperty: #colorHighlight
		ifAbsentPut: [Color white]! !

!UiGlossyPainter methodsFor: 'highlight' stamp: 'mt 1/2/2012 16:46:41.465'!
disableHighlight

	fraction := 0.5.
	highlight := false.! !

!UiGlossyPainter methodsFor: 'drawing - specific' stamp: 'mt 1/5/2012 16:27:49.592'!
drawComboBoxEnabledOn: aCanvas

	self morph readOnly ifTrue: [^ self].
	
	self drawTextEdit: self morph on: aCanvas.! !

!UiGlossyPainter methodsFor: 'drawing - specific' stamp: 'mt 7/8/2013 14:01:09.711'!
drawComboBoxOverlayEnabledOn: aCanvas

	| b |
	self morph readOnly ifFalse: [
		((self morph arrowButton raised not
			and: [self morph arrowButton down not])
			and: [self morph arrowButton checked not]) ifTrue: [
			aCanvas
				fillRectangle: (self morph arrowButton topLeft - (2@0) corner: self morph arrowButton bottomLeft + (2@0))
				color: self color].
		
		^ self].
	
	
	b := self morph readOnly
		ifTrue: [self morph readOnlyButton outerBounds]
		ifFalse: [self morph lineEdit outerBounds].
	
	aCanvas
		line: b topCenter
		to: self morph button outerBounds topCenter
		width: 1
		color: (self color mixed: 0.65 with: Color black).
		
	aCanvas
		line: b bottomCenter - (0@1)
		to: self morph button outerBounds bottomCenter - (0@1)
		width: 1
		color: (self color mixed: 0.65 with: Color black).! !

!UiGlossyPainter methodsFor: 'drawing - specific' stamp: 'mt 1/10/2012 11:29:38.667'!
drawLineEditEnabledOn: aCanvas

	| font |
	self drawTextEditEnabledOn: aCanvas.
	
	"Draw help text."
	(self morph helpText isEmpty or: [self morph textMorph contents notEmpty]) ifTrue: [^ self].

	font := self morph textMorph textStyle defaultFont.

	aCanvas
		drawString: self morph helpText 
		in: (self bounds translateBy: 5 @ (self bounds height - font height // 2))
		font: font
		color: (Color gray: 0.7).! !

!UiGlossyPainter methodsFor: 'drawing - specific' stamp: 'mt 8/14/2013 13:09:46.27'!
drawProgressBarEnabledOn: aCanvas

	| upperFillStyleInnerBorder lowerFillStyleInnerBorder upperFillStyleBackground lowerFillStyleBackground colorBorder textPosition font |

	font := UiPlatform current standardButtonFont.
	self morph text ifNotNil: [:label |
		textPosition := self bounds topLeft + (((self bounds width // 2) - ((font widthOfString: label asString) // 2))
			@ ((self bounds height // 2) - (font height // 2)))].
	
	self morph fraction < 1.0 ifTrue: [
		"Draw background."
		aCanvas fillRectangle: self bounds color: self morph backgroundColor lighter.
		aCanvas fillRectangle: self innerBounds color: self morph backgroundColor darker.

		UiPlatform current
			canvas: aCanvas
			frameRoundRect: self outerBounds radius: 3 width: 1
			color: (self morph backgroundColor muchDarker alpha: 0.5).
		
		UiPlatform current
			canvas: aCanvas
			frameRoundRect: self outerBounds radius: 4 width: 1
			color: self morph backgroundColor muchDarker.

		"Draw background text."
		self morph text ifNotNil: [:label |
			aCanvas
				drawString: label asString
				at: textPosition
				font: font
				color: self morph backgroundTextColor].
	]. "morph fraction"

	"Draw bar."
	self morph fraction = 0 ifTrue: [^ self].
	
	bounds := self morph topLeft extent: (self morph extent * (self morph fraction @ 1)) rounded.
	self enableHighlight. "like a tool button"	
	upperFillStyleInnerBorder := self fillStyleUpper mixed: 0.3 with: self colorLighten.
	lowerFillStyleInnerBorder := self fillStyleLower mixed: 0.65 with: self colorLighten.
	upperFillStyleBackground := self fillStyleUpper.
	lowerFillStyleBackground := self fillStyleLower.
	colorBorder := self colorBorder.	
	
	"Bevel --- Needs to be overdrawn by background due to limitations in Balloon engine."
	aCanvas fillRectangle: self upperBounds fillStyle: upperFillStyleInnerBorder.
	aCanvas fillRectangle: self lowerBounds fillStyle: lowerFillStyleInnerBorder.

	"Background"
	aCanvas fillRectangle: self innerUpperBounds fillStyle: upperFillStyleBackground.
	aCanvas fillRectangle: self innerLowerBounds fillStyle: lowerFillStyleBackground.

	"Border"
	UiPlatform current
		canvas: aCanvas
		frameRoundRect: self outerBounds radius: 3 width: 1
		color: (colorBorder alpha: 0.5).
		
	UiPlatform current
		canvas: aCanvas
		frameRoundRect: self outerBounds radius: 4 width: 1
		color: colorBorder.
		
	"Draw bar text."
	self morph text ifNotNil: [:label |
		aCanvas clipBy: self innerBounds during: [:c | c
			drawString: label asString
			at: textPosition
			font: font
			color: self morph textColor]].! !

!UiGlossyPainter methodsFor: 'drawing - specific' stamp: 'mt 7/8/2013 17:04:48.091'!
drawPushButtonDisabledOn: aCanvas

	"Border outer"
	UiPlatform current
		canvas: aCanvas
		frameRoundRect: self outerBounds radius: 3 width: 1
		color: ((self colorBorder mixed: 0.8 with: self colorLighten) alpha: 0.5).
	
	UiPlatform current
		canvas: aCanvas
		frameRoundRect: self outerBounds radius: 4 width: 1
		color: (self colorBorder mixed: 0.8 with: self colorLighten).! !

!UiGlossyPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:29:10.311'!
drawPushButtonEnabledOn: aCanvas

	| upperFillStyleInnerBorder lowerFillStyleInnerBorder upperFillStyleBackground lowerFillStyleBackground colorBorder |
	orientation := self morph orientation.
	
	upperFillStyleInnerBorder :=
		self morph down ifTrue: [
			self fillStyleUpper mixed: 0.8 with: self colorDarken] ifFalse: [
		self morph checked ifTrue: [
			self fillStyleUpper mixed: 0.85 with: self colorDarken] ifFalse: [
		"normal"
			self fillStyleUpper mixed: 0.3 with: self colorLighten]].
	lowerFillStyleInnerBorder :=
		self morph down ifTrue: [
			self fillStyleLower mixed: 0.85 with: self colorDarken] ifFalse: [
		self morph checked ifTrue: [
			self fillStyleLower mixed: 0.9 with: self colorDarken] ifFalse: [
		"normal"
			self fillStyleLower mixed: 0.65 with: self colorLighten]].
	upperFillStyleBackground :=
		self morph down ifTrue: [
			self fillStyleUpper mixed: 0.9 with: self colorDarken] ifFalse: [
		self morph checked ifTrue: [
			self fillStyleUpper mixed: 0.95 with: self colorDarken] ifFalse: [
		"normal"
			self fillStyleUpper]].
	lowerFillStyleBackground :=
		self morph down ifTrue: [
			self fillStyleLower mixed: 0.9 with: self colorDarken] ifFalse: [
		self morph checked ifTrue: [
			self fillStyleLower mixed: 0.95 with: self colorDarken] ifFalse: [
		"normal"
			self fillStyleLower]].
	colorBorder :=
		self morph down ifTrue: [
			self colorBorder mixed: 0.9 with: self colorDarken] ifFalse: [
		self morph checked ifTrue: [
			self colorBorder mixed: 0.95 with: self colorDarken] ifFalse: [
		"normal"
			self colorBorder]].	
	
	"Bevel --- Needs to be overdrawn by background due to limitations in Balloon engine."
	aCanvas fillRectangle: self upperBounds fillStyle: upperFillStyleInnerBorder.
	aCanvas fillRectangle: self lowerBounds fillStyle: lowerFillStyleInnerBorder.

	"Background"
	aCanvas fillRectangle: self innerUpperBounds fillStyle: upperFillStyleBackground.
	aCanvas fillRectangle: self innerLowerBounds fillStyle: lowerFillStyleBackground.

	"Border"
	UiPlatform current
		canvas: aCanvas
		frameRoundRect: self outerBounds radius: 3 width: 1
		color: (colorBorder alpha: 0.5).
		
	UiPlatform current
		canvas: aCanvas
		frameRoundRect: self outerBounds radius: 4 width: 1
		color: colorBorder.
! !

!UiGlossyPainter methodsFor: 'drawing - specific' stamp: 'mt 7/8/2013 17:07:26.969'!
drawTextEditEnabledOn: aCanvas

	| fillStyleLower fillStyleUpper c |.
	c := (self morph hasKeyboard not and: [self morph hasUnacceptedChanges])
		ifTrue: [self colorChanged]
		ifFalse: [self color].
	
	self morph outerBorderWidth > 0 ifTrue: [
		fillStyleLower := self fillStyleLower colorRamp:  {
			0.0 -> (c mixed: 0.81 with: self colorDarken).
			1.0 -> (c mixed: 0.85 with: self colorDarken)}.
		fillStyleUpper := self fillStyleUpper colorRamp: {
			0.0 -> (c mixed: 0.72 with: self colorDarken).
			1.0 -> (c mixed: 0.76 with: self colorDarken)}.

		aCanvas
			fillRectangle: (self upperBounds outsetBy: (1@0 corner: 1@0))
			fillStyle: fillStyleUpper.
		
		aCanvas
			fillRectangle: (self lowerBounds outsetBy: (1@0 corner: 1@0))
			fillStyle: fillStyleLower.
		
		aCanvas
			line: self outerBounds topLeft + (1@0)
			to: self outerBounds topRight - (2@0)
			width: 1
			color: fillStyleUpper colorRamp first value.

		aCanvas
			line: self outerBounds bottomLeft + (1@ -1)
			to: self outerBounds bottomRight - (2@ 1)
			width: 1
			color: fillStyleLower colorRamp last value.
	].

	"Fill background."
	UiPlatform current
		canvas: aCanvas
		fillRoundRect: self bounds
		radius: 3
		fillStyle: (SolidFillStyle color: c).
! !

!UiGlossyPainter methodsFor: 'drawing - specific' stamp: 'mt 7/8/2013 14:01:18.336'!
drawToolButtonEnabledOn: aCanvas

	"Consider #autoRaise flag."
	((self morph checked not
		and: [self morph autoRaise])
		and: [self morph raised not])
			ifTrue: [^ self].

	"Reuse drawing code from push buttons."
	self enableHighlight.
	self drawPushButtonEnabledOn: aCanvas.! !

!UiGlossyPainter methodsFor: 'highlight' stamp: 'mt 1/2/2012 16:46:51.295'!
enableHighlight

	fraction := 0.4.
	highlight := true.! !

!UiGlossyPainter methodsFor: 'colors - fillstyles' stamp: 'mt 1/2/2012 16:57:27.324'!
fillStyleHighlight

	^ (GradientFillStyle ramp: {
		0.0 -> (self color mixed: 0.6 with: self colorLighten).
		0.8 -> (self color mixed: 0.8 with: self colorDarken)})
	origin: (self orientation == #horizontal
		ifTrue: [self innerLowerBounds bottomCenter]
		ifFalse: [self innerLowerBounds rightCenter]);
	radial: true;
	direction: (self orientation == #horizontal
		ifTrue: [0 @ self innerLowerBounds height negated]
		ifFalse: [self innerLowerBounds width negated @ 0]);
	normal: (self orientation == #horizontal
		ifTrue: [(self innerLowerBounds width * 2) @ 0]
		ifFalse: [0 @ (self innerLowerBounds height * 2)])! !

!UiGlossyPainter methodsFor: 'colors - fillstyles' stamp: 'mt 1/2/2012 17:02:19.14'!
fillStyleLower

	highlight == true ifTrue: [^ self fillStyleHighlight].

	^ (GradientFillStyle ramp: {
		0.0 -> (self color mixed: 0.85 with: self colorDarken).
		1.0 -> (self color mixed: 0.8 with: self colorDarken)})
	origin: self lowerBounds topLeft;
	direction: (self orientation == #horizontal
		ifTrue: [0 @ self lowerBounds height]
		ifFalse: [self lowerBounds width @ 0])! !

!UiGlossyPainter methodsFor: 'colors - fillstyles' stamp: 'mt 1/2/2012 16:48:31.811'!
fillStyleUpper

	^ (GradientFillStyle ramp: {
		0.0 -> (self color mixed: 0.6 with: self colorLighten).
		1.0 -> (self color mixed: 0.9 with: self colorDarken)})
	origin: self upperBounds topLeft;
	direction: (self orientation == #horizontal
		ifTrue: [0 @ self upperBounds height]
		ifFalse: [self upperBounds width @ 0])! !

!UiGlossyPainter methodsFor: 'accessing' stamp: 'mt 12/15/2011 15:16:42.956'!
fraction

	^ fraction ifNil: [fraction := 0.5]! !

!UiGlossyPainter methodsFor: 'geometry' stamp: 'mt 1/2/2012 16:09:18.494'!
innerLowerBounds

	^ self lowerBounds insetBy: (
		(self orientation == #horizontal ifTrue: [1@0] ifFalse: [0@1]) corner: 1@1)! !

!UiGlossyPainter methodsFor: 'geometry' stamp: 'mt 1/2/2012 16:09:25.369'!
innerUpperBounds

	^ self upperBounds insetBy: (1@1 corner: 
		(self orientation == #horizontal ifTrue: [1@0] ifFalse: [0@1]))! !

!UiGlossyPainter methodsFor: 'geometry' stamp: 'mt 1/2/2012 16:08:34.379'!
lowerBounds

	| b |
	b := self bounds.
	^ self orientation == #horizontal
		ifTrue: [b insetBy: (0@(b height * self fraction) corner: 0@0)]
		ifFalse: [b insetBy: ((b width * self fraction)@0 corner: 0@0)]! !

!UiGlossyPainter methodsFor: 'accessing' stamp: 'mt 8/1/2013 19:29:44.396'!
orientation

	^ orientation ifNil: [#horizontal]! !

!UiGlossyPainter methodsFor: 'geometry' stamp: 'mt 8/1/2013 19:56:14.46'!
outerBounds

	^ bounds
		ifNil: [super outerBounds]
		ifNotNil: [:b | b outsetBy: 1 "artificial"]! !

!UiGlossyPainter methodsFor: 'geometry' stamp: 'mt 1/2/2012 16:08:02.77'!
upperBounds

	| b |
	b := self bounds.
	^ self orientation == #horizontal
		ifTrue: [b insetBy: (0@0 corner: 0@(b height * (1 - self fraction)))]
		ifFalse: [b insetBy: (0@0 corner: (b width * (1 - self fraction))@ 0)]! !

!UiPainter class methodsFor: 'default painter' stamp: 'mt 1/2/2012 16:58:44.393'!
default

	^ UiGlossyPainter! !

!UiPainter class methodsFor: 'code generation' stamp: 'mt 8/1/2013 19:02:07.572'!
generateDrawMethods
	"self generateDrawMethods."

	(PackageInfo named: #Widgets) classes 
		select: [:cls | (cls inheritsFrom: Morph) and: [cls isWidgetClass]]
		thenDo: [:cls | | qname src msg |
			qname := cls name allButFirst: 2. "Remove 'Ui' prefix."
			msg := '"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."'.
			src := 'draw{1}: a{1} {2}: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: a{1}.
	a{1} enabled
		ifTrue: [self draw{1}{3}EnabledOn: aCanvas]
		ifFalse: [self draw{1}{3}DisabledOn: aCanvas].'.
	
			UiPainter
				compile: (src format: {qname. 'on'. ''})
				classified: #drawing;
				compile: (src format: {qname. 'overlayOn'. 'Overlay'})
				classified: #drawing;
				compile: ('draw{1}EnabledOn: aCanvas
	{2}' format: {qname. msg})
				classified: #'drawing - specific';
				compile: ('draw{1}DisabledOn: aCanvas
	{2}' format: {qname. msg})
				classified: #'drawing - specific';
				compile: ('draw{1}OverlayEnabledOn: aCanvas
	{2}' format: {qname. msg})
				classified: #'drawing - specific';
				compile: ('draw{1}OverlayDisabledOn: aCanvas
	{2}' format: {qname. msg})
				classified: #'drawing - specific'].! !

!UiPainter methodsFor: 'geometry' stamp: 'mt 1/2/2012 14:30:37.672'!
bounds

	^ self morph bounds! !

!UiPainter methodsFor: 'colors' stamp: 'mt 1/2/2012 16:30:53.124'!
color

	^ self morph color! !

!UiPainter methodsFor: 'colors' stamp: 'mt 1/2/2012 16:33:07.253'!
colorBorder

	^ self morph borderColor! !

!UiPainter methodsFor: 'colors' stamp: 'mt 1/6/2012 09:51:10.404'!
colorChanged
	"Indicate pending changes that have not been saved."

	^ self color! !

!UiPainter methodsFor: 'colors' stamp: 'mt 1/2/2012 16:35:02.762'!
colorDarken

	^ self morph
		valueOfProperty: #colorDarken
		ifAbsentPut: [Color black]! !

!UiPainter methodsFor: 'colors' stamp: 'mt 1/2/2012 16:35:13.358'!
colorLighten

	^ self morph
		valueOfProperty: #colorLighten
		ifAbsentPut: [Color white]! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.817'!
drawButtonGroup: aButtonGroup on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aButtonGroup.
	aButtonGroup enabled
		ifTrue: [self drawButtonGroupEnabledOn: aCanvas]
		ifFalse: [self drawButtonGroupDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.848'!
drawButtonGroup: aButtonGroup overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aButtonGroup.
	aButtonGroup enabled
		ifTrue: [self drawButtonGroupOverlayEnabledOn: aCanvas]
		ifFalse: [self drawButtonGroupOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.863'!
drawButtonGroupDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.856'!
drawButtonGroupEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.876'!
drawButtonGroupOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.87'!
drawButtonGroupOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.881'!
drawCheckBox: aCheckBox on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aCheckBox.
	aCheckBox enabled
		ifTrue: [self drawCheckBoxEnabledOn: aCanvas]
		ifFalse: [self drawCheckBoxDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.886'!
drawCheckBox: aCheckBox overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aCheckBox.
	aCheckBox enabled
		ifTrue: [self drawCheckBoxOverlayEnabledOn: aCanvas]
		ifFalse: [self drawCheckBoxOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.895'!
drawCheckBoxDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.891'!
drawCheckBoxEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.905'!
drawCheckBoxOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.9'!
drawCheckBoxOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.91'!
drawComboBox: aComboBox on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aComboBox.
	aComboBox enabled
		ifTrue: [self drawComboBoxEnabledOn: aCanvas]
		ifFalse: [self drawComboBoxDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.915'!
drawComboBox: aComboBox overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aComboBox.
	aComboBox enabled
		ifTrue: [self drawComboBoxOverlayEnabledOn: aCanvas]
		ifFalse: [self drawComboBoxOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.926'!
drawComboBoxDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.92'!
drawComboBoxEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.936'!
drawComboBoxOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.931'!
drawComboBoxOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.941'!
drawContainer: aContainer on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aContainer.
	aContainer enabled
		ifTrue: [self drawContainerEnabledOn: aCanvas]
		ifFalse: [self drawContainerDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.946'!
drawContainer: aContainer overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aContainer.
	aContainer enabled
		ifTrue: [self drawContainerOverlayEnabledOn: aCanvas]
		ifFalse: [self drawContainerOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.958'!
drawContainerDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.952'!
drawContainerEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.969'!
drawContainerOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.963'!
drawContainerOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.974'!
drawLabel: aLabel on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aLabel.
	aLabel enabled
		ifTrue: [self drawLabelEnabledOn: aCanvas]
		ifFalse: [self drawLabelDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:23.981'!
drawLabel: aLabel overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aLabel.
	aLabel enabled
		ifTrue: [self drawLabelOverlayEnabledOn: aCanvas]
		ifFalse: [self drawLabelOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.993'!
drawLabelDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.987'!
drawLabelEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.005'!
drawLabelOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:23.999'!
drawLabelOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.011'!
drawLineEdit: aLineEdit on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aLineEdit.
	aLineEdit enabled
		ifTrue: [self drawLineEditEnabledOn: aCanvas]
		ifFalse: [self drawLineEditDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.018'!
drawLineEdit: aLineEdit overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aLineEdit.
	aLineEdit enabled
		ifTrue: [self drawLineEditOverlayEnabledOn: aCanvas]
		ifFalse: [self drawLineEditOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.031'!
drawLineEditDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.025'!
drawLineEditEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.044'!
drawLineEditOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.038'!
drawLineEditOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.051'!
drawListView: aListView on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aListView.
	aListView enabled
		ifTrue: [self drawListViewEnabledOn: aCanvas]
		ifFalse: [self drawListViewDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.058'!
drawListView: aListView overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aListView.
	aListView enabled
		ifTrue: [self drawListViewOverlayEnabledOn: aCanvas]
		ifFalse: [self drawListViewOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.071'!
drawListViewDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.065'!
drawListViewEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.085'!
drawListViewOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.078'!
drawListViewOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.092'!
drawListWidget: aListWidget on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aListWidget.
	aListWidget enabled
		ifTrue: [self drawListWidgetEnabledOn: aCanvas]
		ifFalse: [self drawListWidgetDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.098'!
drawListWidget: aListWidget overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aListWidget.
	aListWidget enabled
		ifTrue: [self drawListWidgetOverlayEnabledOn: aCanvas]
		ifFalse: [self drawListWidgetOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.114'!
drawListWidgetDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.107'!
drawListWidgetEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.129'!
drawListWidgetOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.121'!
drawListWidgetOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.136'!
drawProgressBar: aProgressBar on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aProgressBar.
	aProgressBar enabled
		ifTrue: [self drawProgressBarEnabledOn: aCanvas]
		ifFalse: [self drawProgressBarDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.242'!
drawProgressBar: aProgressBar overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aProgressBar.
	aProgressBar enabled
		ifTrue: [self drawProgressBarOverlayEnabledOn: aCanvas]
		ifFalse: [self drawProgressBarOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.438'!
drawProgressBarDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.34'!
drawProgressBarEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.635'!
drawProgressBarOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.536'!
drawProgressBarOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.734'!
drawPushButton: aPushButton on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aPushButton.
	aPushButton enabled
		ifTrue: [self drawPushButtonEnabledOn: aCanvas]
		ifFalse: [self drawPushButtonDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.748'!
drawPushButton: aPushButton overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aPushButton.
	aPushButton enabled
		ifTrue: [self drawPushButtonOverlayEnabledOn: aCanvas]
		ifFalse: [self drawPushButtonOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.775'!
drawPushButtonDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.762'!
drawPushButtonEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.804'!
drawPushButtonOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.789'!
drawPushButtonOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.818'!
drawRadioButton: aRadioButton on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aRadioButton.
	aRadioButton enabled
		ifTrue: [self drawRadioButtonEnabledOn: aCanvas]
		ifFalse: [self drawRadioButtonDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.832'!
drawRadioButton: aRadioButton overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aRadioButton.
	aRadioButton enabled
		ifTrue: [self drawRadioButtonOverlayEnabledOn: aCanvas]
		ifFalse: [self drawRadioButtonOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.86'!
drawRadioButtonDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.846'!
drawRadioButtonEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.888'!
drawRadioButtonOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.874'!
drawRadioButtonOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.902'!
drawScrollArea: aScrollArea on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aScrollArea.
	aScrollArea enabled
		ifTrue: [self drawScrollAreaEnabledOn: aCanvas]
		ifFalse: [self drawScrollAreaDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.916'!
drawScrollArea: aScrollArea overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aScrollArea.
	aScrollArea enabled
		ifTrue: [self drawScrollAreaOverlayEnabledOn: aCanvas]
		ifFalse: [self drawScrollAreaOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.945'!
drawScrollAreaDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.931'!
drawScrollAreaEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.974'!
drawScrollAreaOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:24.959'!
drawScrollAreaOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:24.989'!
drawScrollBar: aScrollBar on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aScrollBar.
	aScrollBar enabled
		ifTrue: [self drawScrollBarEnabledOn: aCanvas]
		ifFalse: [self drawScrollBarDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.004'!
drawScrollBar: aScrollBar overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aScrollBar.
	aScrollBar enabled
		ifTrue: [self drawScrollBarOverlayEnabledOn: aCanvas]
		ifFalse: [self drawScrollBarOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.033'!
drawScrollBarDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.019'!
drawScrollBarEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.063'!
drawScrollBarOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.048'!
drawScrollBarOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.078'!
drawSlider: aSlider on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aSlider.
	aSlider enabled
		ifTrue: [self drawSliderEnabledOn: aCanvas]
		ifFalse: [self drawSliderDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.094'!
drawSlider: aSlider overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aSlider.
	aSlider enabled
		ifTrue: [self drawSliderOverlayEnabledOn: aCanvas]
		ifFalse: [self drawSliderOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.124'!
drawSliderDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.109'!
drawSliderEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.155'!
drawSliderOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.14'!
drawSliderOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.171'!
drawSpacer: aSpacer on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aSpacer.
	aSpacer enabled
		ifTrue: [self drawSpacerEnabledOn: aCanvas]
		ifFalse: [self drawSpacerDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.186'!
drawSpacer: aSpacer overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aSpacer.
	aSpacer enabled
		ifTrue: [self drawSpacerOverlayEnabledOn: aCanvas]
		ifFalse: [self drawSpacerOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.218'!
drawSpacerDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.202'!
drawSpacerEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.249'!
drawSpacerOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.234'!
drawSpacerOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.265'!
drawStackedWidget: aStackedWidget on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aStackedWidget.
	aStackedWidget enabled
		ifTrue: [self drawStackedWidgetEnabledOn: aCanvas]
		ifFalse: [self drawStackedWidgetDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.281'!
drawStackedWidget: aStackedWidget overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aStackedWidget.
	aStackedWidget enabled
		ifTrue: [self drawStackedWidgetOverlayEnabledOn: aCanvas]
		ifFalse: [self drawStackedWidgetOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.313'!
drawStackedWidgetDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.297'!
drawStackedWidgetEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.346'!
drawStackedWidgetOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.329'!
drawStackedWidgetOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.362'!
drawTableView: aTableView on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aTableView.
	aTableView enabled
		ifTrue: [self drawTableViewEnabledOn: aCanvas]
		ifFalse: [self drawTableViewDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.378'!
drawTableView: aTableView overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aTableView.
	aTableView enabled
		ifTrue: [self drawTableViewOverlayEnabledOn: aCanvas]
		ifFalse: [self drawTableViewOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.411'!
drawTableViewDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.395'!
drawTableViewEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.445'!
drawTableViewOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.428'!
drawTableViewOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 1/10/2012 10:28:19.671'!
drawTableWidget: aTableWidget on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aTableWidget.
	aTableWidget enabled
		ifTrue: [self drawTableWidgetEnabledOn: aCanvas]
		ifFalse: [self drawTableWidgetDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 1/10/2012 10:28:19.679'!
drawTableWidget: aTableWidget overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aTableWidget.
	aTableWidget enabled
		ifTrue: [self drawTableWidgetOverlayEnabledOn: aCanvas]
		ifFalse: [self drawTableWidgetOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 1/10/2012 10:28:19.695'!
drawTableWidgetDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 1/10/2012 10:28:19.687'!
drawTableWidgetEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 1/10/2012 10:28:19.71'!
drawTableWidgetOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 1/10/2012 10:28:19.703'!
drawTableWidgetOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.461'!
drawTextEdit: aTextEdit on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aTextEdit.
	aTextEdit enabled
		ifTrue: [self drawTextEditEnabledOn: aCanvas]
		ifFalse: [self drawTextEditDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.478'!
drawTextEdit: aTextEdit overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aTextEdit.
	aTextEdit enabled
		ifTrue: [self drawTextEditOverlayEnabledOn: aCanvas]
		ifFalse: [self drawTextEditOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.513'!
drawTextEditDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.495'!
drawTextEditEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.547'!
drawTextEditOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.53'!
drawTextEditOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.564'!
drawToolButton: aToolButton on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aToolButton.
	aToolButton enabled
		ifTrue: [self drawToolButtonEnabledOn: aCanvas]
		ifFalse: [self drawToolButtonDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.671'!
drawToolButton: aToolButton overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aToolButton.
	aToolButton enabled
		ifTrue: [self drawToolButtonOverlayEnabledOn: aCanvas]
		ifFalse: [self drawToolButtonOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.708'!
drawToolButtonDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.69'!
drawToolButtonEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.744'!
drawToolButtonOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.726'!
drawToolButtonOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.762'!
drawTreeView: aTreeView on: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aTreeView.
	aTreeView enabled
		ifTrue: [self drawTreeViewEnabledOn: aCanvas]
		ifFalse: [self drawTreeViewDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing' stamp: 'mt 8/1/2013 19:01:25.78'!
drawTreeView: aTreeView overlayOn: aCanvas
	"Auto-generated. Do not edit. See UiPainter>>generateDrawMethods."
	
	self morph: aTreeView.
	aTreeView enabled
		ifTrue: [self drawTreeViewOverlayEnabledOn: aCanvas]
		ifFalse: [self drawTreeViewOverlayDisabledOn: aCanvas].! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.817'!
drawTreeViewDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.799'!
drawTreeViewEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.854'!
drawTreeViewOverlayDisabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'drawing - specific' stamp: 'mt 8/1/2013 19:01:25.836'!
drawTreeViewOverlayEnabledOn: aCanvas
	"Auto-generated. Do not edit but override. See UiPainter>>generateDrawMethods."! !

!UiPainter methodsFor: 'geometry' stamp: 'mt 8/1/2013 19:54:59.616'!
innerBounds
	"Artificial. Could be used for bevel effect."
	
	^ self bounds insetBy: 1! !

!UiPainter methodsFor: 'accessing' stamp: 'mt 1/2/2012 16:13:03.943'!
morph

	^ morph! !

!UiPainter methodsFor: 'accessing' stamp: 'mt 1/2/2012 16:13:12.414'!
morph: aMorph

	morph := aMorph.! !

!UiPainter methodsFor: 'geometry' stamp: 'mt 1/2/2012 14:30:46.725'!
outerBounds

	^ self morph outerBounds! !

UiPainter subclass: #UiSqueakPainter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Painter'!

!UiSqueakPainter commentStamp: 'mt 8/20/2012 18:16:33.659' prior: 0!
Not functional.!

!UiSqueakPainter methodsFor: 'drawing' stamp: 'mt 7/8/2013 17:06:51.98'!
drawPushButton: aPushButton on: aCanvas
	"Adopted from PluggableButtonMorph>>draw:."
	
	| cc gradient borderColor |
	cc := aPushButton color.
	cc isTransparent ifTrue:[cc := Color gray: 0.9].
	aPushButton enabled ifFalse:[cc := Color lightGray].
	cc brightness > 0.9 ifTrue:[cc := cc adjustBrightness: 0.9 - cc brightness].
	aPushButton down ifTrue:[
		borderColor := cc muchDarker.
		gradient := GradientFillStyle ramp: {
			0.0 -> cc muchDarker.
			0.1-> (cc adjustBrightness: -0.2).
			0.5 -> cc.
			0.9-> (cc adjustBrightness: -0.1).
			1 -> cc muchDarker.
		}.
	] ifFalse:[
		borderColor := Color lightGray.
		gradient := GradientFillStyle ramp: {
			0.0 -> Color white.
			0.1-> (cc adjustBrightness: 0.05).
			0.6 -> (cc darker).
		}
	].
	gradient origin: aPushButton bounds topLeft.
	gradient direction: 0@self height.
	UiPlatform current
		canvas: aCanvas 
		frameAndFillRoundRect: aPushButton bounds 
		radius: 8 
		fillStyle: gradient 
		borderWidth: 1 
		borderColor: borderColor.! !

Object subclass: #UiPlatform
	instanceVariableNames: ''
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'Widgets-Platforms'!

!UiPlatform class methodsFor: 'accessing' stamp: 'mt 7/8/2013 15:50:58.712'!
current
	"UiPlatform current."
	^ Current! !

!UiPlatform class methodsFor: 'class initialization' stamp: 'mt 7/30/2013 10:42:32.807'!
initialize
	"self current."
	
	self == UiPlatform ifTrue: [^ self].
	Current ifNil: [Current := self new].! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2013 16:56:54.474'!
canvas: aCanvas
fillRoundRect: aRect
radius: aRadius
fillStyle: aFillStyle

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2013 16:56:52.254'!
canvas: aCanvas
frameAndFillRoundRect: aRect
radius: aRadius
fillStyle: aFillStyle
borderWidth: aWidth
borderColor: aColor

	self subclassResponsibility.  ! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2013 16:54:27.809'!
canvas: aCanvas
frameRoundRect: aRect
radius: aRadius
width: aWidth
color: aColor

	self subclassResponsibility.  ! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2013 16:24:10.105'!
dragThreshold

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/9/2013 10:26:36.314'!
keyboardEvent: anEvent setCharCodeFor: aCharacter

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2013 16:10:34.91'!
runArray: aRunArray add: value withOccurrences: anInteger

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2013 16:31:39.582'!
standardButtonFont

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2013 16:33:02.588'!
standardCodeFont

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2013 16:32:53.672'!
standardDefaultTextFont

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/8/2013 16:33:22.549'!
standardListFont

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/9/2013 09:49:58.308'!
string: aString includesSubstring: anotherString

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/9/2013 10:35:33.722'!
textEditorPointBlock: aTextEditor 

	self subclassResponsibility.! !

!UiPlatform methodsFor: 'as yet unclassified' stamp: 'mt 7/9/2013 10:52:20.725'!
textPrintHtmlString: aText

	self subclassResponsibility.! !

Object subclass: #UiTextModel
	instanceVariableNames: 'doItReceiver doItContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiTextModel methodsFor: 'accessing' stamp: 'mt 8/12/2013 11:28:38.692'!
bindings

	self flag: #orderBroken. "mt: order of temps gets lost"
	^ self doItContext
		ifNil: [Dictionary new]
		ifNotNil: [:ctxt |
			self flag: #shadowing. "mt: duplicate keys possible due to shadowin."
			Dictionary newFrom: (
			ctxt tempNames asSet asOrderedCollection withIndexCollect: [:nm :idx |
				nm -> (ctxt namedTempAt: idx)])]! !

!UiTextModel methodsFor: 'signals' stamp: 'mt 1/31/2013 14:31:29.128'!
contextChanged

	self emit.! !

!UiTextModel methodsFor: 'do-its' stamp: 'mt 11/1/2012 17:01:26.82'!
doItContext

	^ doItContext! !

!UiTextModel methodsFor: 'do-its' stamp: 'mt 1/31/2013 14:51:06.858'!
doItContext: aContext

	aContext == doItContext ifTrue: [^ self].
	doItContext := aContext.
	self contextChanged.! !

!UiTextModel methodsFor: 'do-its' stamp: 'mt 11/1/2012 17:01:35.033'!
doItReceiver

	^ doItReceiver! !

!UiTextModel methodsFor: 'do-its' stamp: 'mt 1/31/2013 14:51:05.337'!
doItReceiver: anObject

	anObject == doItReceiver ifTrue: [^ self].
	doItReceiver := anObject.
	self receiverChanged.! !

!UiTextModel methodsFor: 'signals' stamp: 'mt 1/31/2013 14:31:37.958'!
receiverChanged

	self emit.! !

!UiTextModel methodsFor: 'do-its' stamp: 'mt 11/1/2012 17:06:08.181'!
selectedClass

	^ self doItReceiver class! !

Object subclass: #UiViewInfo
	instanceVariableNames: 'size items resizeMode headerItem visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiViewInfo commentStamp: 'mt 11/2/2010 12:54:36.518' prior: 0!
A information holder for columns and rows in a viewport. Stores extent and resizing behavior.

Represents an index of a viewports items.!

UiViewInfo subclass: #UiViewColumnInfo
	instanceVariableNames: 'vAlignment hAlignment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiViewColumnInfo commentStamp: 'mt 10/9/2010 11:38:45.048' prior: 0!
Useful index element for tree viewport.!

!UiViewColumnInfo methodsFor: 'accessing' stamp: 'mt 10/14/2010 19:08:25.869'!
hAlignment

	^ hAlignment! !

!UiViewColumnInfo methodsFor: 'accessing' stamp: 'mt 10/14/2010 19:08:34.152'!
hAlignment: aSymbol

	hAlignment := aSymbol.! !

!UiViewColumnInfo methodsFor: 'initialization' stamp: 'mt 10/14/2010 19:08:54.097'!
initialize

	super initialize.
	self
		resize: 150;
		vAlignment: #top;
		hAlignment: #left.! !

!UiViewColumnInfo methodsFor: 'geometry' stamp: 'mt 6/29/2012 10:28:13.586'!
optimalSize
	"Return the width of the largest item."
	
	| max |
	max := 0.
	items do: [:item | 
		((item hasProperty: #isGroupSeparator) or: [item hResizing == #spaceFill])
			ifFalse: [max := max max: (item width + (item valueOfProperty: #indent))]].
	self headerItem ifNotNil: [:hi | hi submorphs ifNotEmpty: [
		max := max max: self headerItem submorphBounds width + 5. "font rendering bugfix" ]].
	^ max! !

!UiViewColumnInfo methodsFor: 'printing' stamp: 'mt 4/26/2012 14:25:30.738'!
printString

	^ 'COL:', super printString! !

!UiViewColumnInfo methodsFor: 'geometry' stamp: 'mt 7/6/2012 11:07:04.802'!
updateItemSize

	self items do: [:item | 
		item hResizing = #spaceFill
			ifTrue: [item width: self width]].! !

!UiViewColumnInfo methodsFor: 'accessing' stamp: 'mt 10/14/2010 19:08:11.098'!
vAlignment

	^ vAlignment! !

!UiViewColumnInfo methodsFor: 'accessing' stamp: 'mt 10/14/2010 19:08:17.674'!
vAlignment: aSymbol

	vAlignment := aSymbol.! !

!UiViewColumnInfo methodsFor: 'accessing' stamp: 'mt 10/10/2010 12:52:12.794'!
width

	^ size! !

!UiViewColumnInfo methodsFor: 'accessing' stamp: 'mt 10/9/2010 18:15:41.705'!
width: anInteger

	self resize: anInteger.! !

!UiViewInfo methodsFor: 'initialization' stamp: 'mt 10/11/2010 17:51:31.693'!
clear

	items removeAll.! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/10/2010 12:46:28.423'!
headerItem

	^ headerItem! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 3/23/2012 10:48:52.86'!
headerItem: aHeaderItem
	
	headerItem := aHeaderItem.
	aHeaderItem
		resizeable: self resizeMode == #rigid;
		resize: self pixelSize.! !

!UiViewInfo methodsFor: 'testing' stamp: 'mt 10/9/2010 19:57:43.188'!
includes: aModelIndex

	^ items anySatisfy: [:item | (item valueOfProperty: #modelIndex) = aModelIndex]! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 4/26/2012 13:26:19.981'!
indices

	^ (items collect: [:item | item valueOfProperty: #modelIndex])
		reject: [:index | index isNil]! !

!UiViewInfo methodsFor: 'initialization' stamp: 'mt 6/29/2012 10:30:05'!
initialize

	super initialize.
	
	items := OrderedCollection new.
	resizeMode := #shrinkWrap.! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/13/2010 15:42:39.671'!
itemAt: aModelIndex

	^ items detect: [:item | (item valueOfProperty: #modelIndex) = aModelIndex]! !

!UiViewInfo methodsFor: 'items query' stamp: 'mt 10/9/2010 18:19:16.151'!
itemBefore: anItem

	^ items before: anItem ifAbsent: []! !

!UiViewInfo methodsFor: 'items query' stamp: 'mt 10/9/2010 19:58:04.261'!
itemIndexFor: aModelIndex

	items withIndexDo: [:item :index |
		(item valueOfProperty: #modelIndex) = aModelIndex
			ifTrue: [^ index]].
	
	Error signal: 'Not found.'.! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/9/2010 18:13:57.129'!
items

	^ items! !

!UiViewInfo methodsFor: 'geometry' stamp: 'mt 10/9/2010 18:16:09.336'!
optimalSize

	self subclassResponsibility.	! !

!UiViewInfo methodsFor: 'geometry' stamp: 'mt 3/23/2012 10:49:22.44'!
pixelSize

	^ size ifNil: [0]! !

!UiViewInfo methodsFor: 'printing' stamp: 'mt 4/26/2012 14:25:08.637'!
printString

	^ self firstModelIndex printString! !

!UiViewInfo methodsFor: 'geometry' stamp: 'mt 7/6/2012 11:19:35.224'!
resize: aNumber

	size = aNumber ifTrue: [^ self].
	
	size := aNumber.
	self headerItem ifNotNil: [:hi |
		hi resize: aNumber].
	
	self updateItemSize.! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/9/2010 18:09:54.303'!
resizeMode

	^ resizeMode! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 6/29/2012 10:25:13.588'!
resizeMode: aSymbol
	"#rigid, #shrinkWrap, #spaceFill"

	resizeMode := aSymbol.
	self headerItem ifNotNil: [:hi |
		hi resizeable: resizeMode == #rigid].! !

!UiViewInfo methodsFor: 'geometry' stamp: 'mt 7/6/2012 11:14:43.445'!
updateItemSize
! !

!UiViewInfo methodsFor: 'geometry' stamp: 'mt 7/6/2012 11:19:52.032'!
updateSize

	self resizeMode == #shrinkWrap
		ifTrue: [self resize: self optimalSize]
		ifFalse: [self updateItemSize].! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 6/26/2012 16:26:55.266'!
visible

	^ visible ifNil: [true]! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 8/31/2012 14:37:35.851'!
visible: aBoolean
 
	self visible == aBoolean ifTrue: [^ self].
	
	visible := aBoolean.
	
	"Set flag in morphs to better support morphic event processing."
	self items do: [:morph | morph visible: visible].! !

UiViewInfo subclass: #UiViewRowInfo
	instanceVariableNames: 'height'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiViewRowInfo commentStamp: 'mt 10/9/2010 11:38:35.657' prior: 0!
Useful index element for tree viewport.!

!UiViewRowInfo methodsFor: 'accessing' stamp: 'mt 10/9/2010 18:21:06.63'!
firstModelIndex
	
	^ items
		ifEmpty: [nil]
		ifNotEmpty: [(items first valueOfProperty: #modelIndex)]! !

!UiViewRowInfo methodsFor: 'accessing' stamp: 'mt 12/13/2012 11:57:52.399'!
groupIndex
	
	self flag: #inconsistent. "mt: Why is the group index not cached in the item of a non-separator?"
	^ self isSeparator
		ifTrue: [items first valueOfProperty: #groupIndex]
		ifFalse: [self firstModelIndex groupIndex]! !

!UiViewRowInfo methodsFor: 'accessing' stamp: 'mt 10/10/2010 12:51:44.846'!
height

	^ size! !

!UiViewRowInfo methodsFor: 'accessing' stamp: 'mt 10/9/2010 18:20:41.803'!
height: anInteger

	self resize: anInteger.! !

!UiViewRowInfo methodsFor: 'accessing' stamp: 'mt 4/27/2012 11:36:31.027'!
indent
	
	^ items
		ifEmpty: [0]
		ifNotEmpty: [(items first valueOfProperty: #indent ifAbsent: [0])]! !

!UiViewRowInfo methodsFor: 'initialization' stamp: 'mt 7/8/2013 16:39:50.066'!
initialize

	super initialize.
	self resize: UiPlatform current standardListFont height.! !

!UiViewRowInfo methodsFor: 'testing' stamp: 'mt 10/10/2010 10:27:44.554'!
isCollapsed

	^ self isExpanded not! !

!UiViewRowInfo methodsFor: 'testing' stamp: 'mt 10/10/2010 10:27:36.67'!
isExpanded

	^ items notEmpty and: [items first valueOfProperty: #isExpanded]! !

!UiViewRowInfo methodsFor: 'testing' stamp: 'mt 4/23/2012 11:51:34.437'!
isRowFor: aModelIndex

	^ self firstModelIndex
		ifNil: [false "separator"]
		ifNotNil: [:index | index row = aModelIndex row]! !

!UiViewRowInfo methodsFor: 'testing' stamp: 'mt 4/26/2012 13:18:16.203'!
isSeparator

	^ self items first hasProperty: #isGroupSeparator! !

!UiViewRowInfo methodsFor: 'geometry' stamp: 'mt 7/6/2012 11:02:05.012'!
optimalSize
	"Return the height of the largest item."
	
	| max |
	max := 0.
	items do: [:item |
		item vResizing ~= #spaceFill
			ifTrue: [max := max max: item height]].
	self headerItem ifNotNil: [:hi | hi submorphs ifNotEmpty: [
		max := max max: self headerItem submorphBounds height]].
	^ max! !

!UiViewRowInfo methodsFor: 'accessing' stamp: 'mt 6/26/2012 10:37:29.918'!
parentModelIndex
	"All items in a row should have the same parent model index. So take the first item to access."
	
	^ items
		ifEmpty: [nil]
		ifNotEmpty: [self isSeparator
			ifTrue: [items first valueOfProperty: #parentModelIndex]
			ifFalse: [(items first valueOfProperty: #modelIndex) ifNotNil: [:index | index parent]]]! !

!UiViewRowInfo methodsFor: 'printing' stamp: 'mt 4/26/2012 14:25:46.805'!
printString

	^ 'ROW:', super printString! !

!UiViewRowInfo methodsFor: 'geometry' stamp: 'mt 7/6/2012 11:14:59.892'!
updateItemSize

	self items do: [:item | 
		item vResizing = #spaceFill
			ifTrue: [item height: self height]].! !

Object subclass: #UiViewSelection
	instanceVariableNames: 'viewport'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiViewSelection commentStamp: 'mt 4/6/2011 10:14:52.597' prior: 0!
A view selection keeps track of all selected cells in a viewport as well as the current cell which is normally only the last selected cell.

A reference to the viewport is needed to map cells to model indices.!

UiViewSelection subclass: #UiViewMultiSelection
	instanceVariableNames: 'cells currentCell cellToUnselect'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

UiViewMultiSelection subclass: #UiViewExtendedSelection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiViewExtendedSelection methodsFor: 'add/remove' stamp: 'mt 8/9/2012 10:10:55.898'!
add: aCellIndex

	(self cells includes: aCellIndex)
		ifTrue: [
			"Prepare to unselect the cell."
			cellToUnselect := aCellIndex]
		ifFalse: [
			"Select cell. Discard old selection."
			self cells: {aCellIndex}].
	
	self currentCell: aCellIndex.! !

!UiViewExtendedSelection methodsFor: 'add/remove' stamp: 'mt 8/9/2012 10:09:15.682'!
add: aCellIndex event: aUserInputEvent

	aUserInputEvent shiftPressed
		ifTrue: [^ super add: aCellIndex].

	self add: aCellIndex.! !

!UiViewExtendedSelection methodsFor: 'add/remove' stamp: 'mt 8/9/2012 10:25:56.274'!
remove: aCellIndex

	cellToUnselect = aCellIndex
		ifTrue: [
			cellToUnselect := nil.
			self cells: {aCellIndex}].! !

!UiViewExtendedSelection methodsFor: 'add/remove' stamp: 'mt 8/9/2012 10:13:20.263'!
remove: aCellIndex event: aUserInputEvent

	aUserInputEvent shiftPressed
		ifTrue: [^ super remove: aCellIndex].
		
	self remove: aCellIndex.  ! !

!UiViewMultiSelection methodsFor: 'add/remove' stamp: 'mt 8/9/2012 10:07:16.998'!
add: aCellIndex

	| oldCells |
	
	(self cells includes: aCellIndex)
		ifTrue: [
			"Prepare to unselect the cell."
			cellToUnselect := aCellIndex]
		ifFalse: [
			"Select cell."
			oldCells := self cells copy.
			self cells add: aCellIndex.
			self selectionChangedFrom: oldCells to: self cells copy].
	
	self currentCell: aCellIndex.! !

!UiViewMultiSelection methodsFor: 'accessing' stamp: 'mt 6/26/2012 13:54:26.263'!
cells

	^ cells ifNil: [cells := Set new]! !

!UiViewMultiSelection methodsFor: 'accessing' stamp: 'mt 7/12/2012 15:23:08.885'!
cells: somePoints

	| oldCells |
	oldCells := self cells.
	
	cells := somePoints
		ifEmpty: [nil]
		ifNotEmpty: [somePoints asSet].
		
	self selectionChangedFrom: oldCells to: self cells.! !

!UiViewMultiSelection methodsFor: 'accessing' stamp: 'mt 6/26/2012 13:51:14.871'!
currentCell

	^ currentCell! !

!UiViewMultiSelection methodsFor: 'accessing' stamp: 'mt 6/26/2012 13:52:34.805'!
currentCell: aPoint

	| oldCell |
	aPoint = currentCell ifTrue: [^ self].

	oldCell := currentCell.
	currentCell := aPoint.

	self currentCellChangedFrom: oldCell to: currentCell.! !

!UiViewMultiSelection methodsFor: 'add/remove' stamp: 'mt 8/9/2012 10:25:35.307'!
remove: aCellIndex

	| oldCells |
	cellToUnselect = aCellIndex
		"Unselect the cell."
		ifTrue: [
			cellToUnselect := nil.
			oldCells := self cells copy.
			self cells remove: aCellIndex.
			self selectionChangedFrom: oldCells to: self cells copy].! !

!UiViewMultiSelection methodsFor: 'accessing - model index' stamp: 'mt 7/9/2013 11:28:26.637'!
selectedIndices

	^ ((self cells asOrderedCollection sorted: [:cell1 :cell2 | cell1 y <= cell2 y or: [cell1 y == cell2 y and: [cell1 x <= cell2 x]]])
		collect: [:cellIndex | self viewport modelIndexFor: cellIndex])
		reject: [:modelindex | modelindex isNil]
! !

UiViewSelection subclass: #UiViewNullSelection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiViewNullSelection commentStamp: 'mt 10/11/2010 14:09:45.699' prior: 0!
No selection of cells possible.!

!UiViewNullSelection methodsFor: 'add/remove' stamp: 'mt 8/8/2012 15:53:23.817'!
add: aCellIndex

	"Ignore."! !

!UiViewNullSelection methodsFor: 'accessing' stamp: 'mt 10/11/2010 14:08:50.077'!
cells

	^ #()! !

!UiViewNullSelection methodsFor: 'accessing' stamp: 'mt 2/2/2012 10:35:30.303'!
cells: somePoints

	! !

!UiViewNullSelection methodsFor: 'accessing' stamp: 'mt 10/23/2010 10:00:00.921'!
currentCell

	^ nil! !

!UiViewNullSelection methodsFor: 'accessing' stamp: 'mt 10/23/2010 10:00:08.721'!
currentCell: anIndex

! !

!UiViewSelection methodsFor: 'add/remove' stamp: 'mt 8/8/2012 15:51:25.672'!
add: aCellIndex! !

!UiViewSelection methodsFor: 'add/remove' stamp: 'mt 8/8/2012 15:51:34.375'!
add: aCellIndex event: aUserInputEvent

	self add: aCellIndex.! !

!UiViewSelection methodsFor: 'accessing' stamp: 'mt 10/11/2010 09:13:15.101'!
cells
	"Return an array with all selected cells."
	
	self subclassResponsibility.! !

!UiViewSelection methodsFor: 'accessing' stamp: 'mt 2/2/2012 10:30:49.742'!
cells: somePoints
	
	self subclassResponsibility.! !

!UiViewSelection methodsFor: 'add/remove' stamp: 'mt 6/26/2012 14:27:00.825'!
clear

	self
		cells: #();
		currentCell: nil.! !

!UiViewSelection methodsFor: 'accessing - pixel' stamp: 'mt 8/13/2012 10:12:58.473'!
currentBounds
	"Returns the global bounds of the current cell bounds according to #selectionBehavior."
	
	| cb |
	cb := self viewport boundsFor: self currentCell.
	^ (self viewport localPointToGlobal: cb topLeft)
		extent: cb extent  ! !

!UiViewSelection methodsFor: 'accessing' stamp: 'mt 10/16/2010 08:15:39.961'!
currentCell
	"Return the current cell. Normally the last added one."

	self subclassResponsibility.! !

!UiViewSelection methodsFor: 'accessing' stamp: 'mt 10/16/2010 14:23:46.96'!
currentCell: aPoint

	self subclassResponsibility.! !

!UiViewSelection methodsFor: 'accessing - pixel' stamp: 'mt 8/13/2012 10:12:37.826'!
currentCellBounds
	"Returns the global bounds of the current cell bounds."
	
	| cb |
	cb := self viewport cellBoundsFor: self currentCell.
	^ (self viewport localPointToGlobal: cb topLeft)
		extent: cb extent  ! !

!UiViewSelection methodsFor: 'signals' stamp: 'mt 7/13/2011 14:25:23.278'!
currentCellChangedFrom: aPoint to: anotherPoint

	self emit.! !

!UiViewSelection methodsFor: 'accessing - model index' stamp: 'mt 4/6/2011 10:17:57.587'!
currentIndex

	^ self viewport modelIndexFor: self currentCell  ! !

!UiViewSelection methodsFor: 'accessing - model index' stamp: 'mt 6/23/2011 13:27:41.965'!
currentIndex: aModelIndex

	self currentCell:
		(self viewport cellIndexFromModelIndex: aModelIndex).
! !

!UiViewSelection methodsFor: 'testing' stamp: 'mt 3/7/2013 09:55:19.565'!
isEmpty

	^ self cells isEmpty! !

!UiViewSelection methodsFor: 'add/remove' stamp: 'mt 8/9/2012 10:03:45.564'!
remove: aCellIndex! !

!UiViewSelection methodsFor: 'add/remove' stamp: 'mt 8/9/2012 10:03:36.028'!
remove: aCellIndex event: aUserInputEvent

	self remove: aCellIndex.! !

!UiViewSelection methodsFor: 'accessing - model index' stamp: 'mt 8/2/2012 14:32:37.786'!
selectedIndices

	^ (self cells collect: [:cellIndex | self viewport modelIndexFor: cellIndex])
		reject: [:modelindex | modelindex isNil]! !

!UiViewSelection methodsFor: 'accessing - model index' stamp: 'mt 2/2/2012 10:33:15.876'!
selectedIndices: someModelIndices

	self cells: (someModelIndices collect: [:index |
		self viewport cellIndexFromModelIndex: index]).
! !

!UiViewSelection methodsFor: 'accessing - pixel' stamp: 'mt 7/5/2013 11:16:42.306'!
selectionBounds
	"Returns the global bounds of the current selection according to #selectionBehavior."
	
	| b |
	self cells ifEmpty: [^ 0@0 corner: 1@1].
	
	b := (self cells collect: [:cell | self viewport boundsFor: cell])
		fold: [:rect1 :rect2 | rect1 quickMerge: rect2].
		
	^ (self viewport localPointToGlobal: b topLeft)
		extent: b extent  ! !

!UiViewSelection methodsFor: 'signals' stamp: 'mt 7/13/2011 14:25:51.282'!
selectionChangedFrom: someOldPoints to: someNewPoints

	self emit.! !

!UiViewSelection methodsFor: 'accessing' stamp: 'mt 4/6/2011 10:14:27.303'!
viewport

	^ viewport! !

!UiViewSelection methodsFor: 'accessing' stamp: 'mt 4/6/2011 10:14:38.203'!
viewport: anItemViewport

	viewport := anItemViewport.! !

UiViewSelection subclass: #UiViewSingleSelection
	instanceVariableNames: 'selectedCell'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views-Support'!

!UiViewSingleSelection methodsFor: 'add/remove' stamp: 'mt 8/8/2012 15:53:32.938'!
add: aCellIndex

	| oldCell |
	aCellIndex ifNil: [^ self].
	aCellIndex = selectedCell ifTrue: [^ self].
	
	oldCell := selectedCell.
	
	self currentCell: aCellIndex.
	
	self selectionChangedFrom: {oldCell} to: {selectedCell}.! !

!UiViewSingleSelection methodsFor: 'accessing' stamp: 'mt 10/11/2010 14:35:26.499'!
cells

	^ selectedCell
		ifNil: [#()]
		ifNotNil: [{selectedCell}]! !

!UiViewSingleSelection methodsFor: 'accessing' stamp: 'mt 6/26/2012 13:56:35.69'!
cells: somePoints

	| oldCell |
	oldCell := selectedCell.
	
	selectedCell := somePoints
		ifEmpty: [nil]
		ifNotEmpty: [somePoints first].
		
	self selectionChangedFrom: {oldCell} to: self cells.! !

!UiViewSingleSelection methodsFor: 'accessing' stamp: 'mt 10/16/2010 08:16:06.717'!
currentCell

	^ selectedCell! !

!UiViewSingleSelection methodsFor: 'accessing' stamp: 'mt 7/13/2011 14:27:18.775'!
currentCell: aPoint

	| oldCell |
	aPoint = selectedCell ifTrue: [^ self].
	oldCell := selectedCell.
	
	selectedCell := aPoint.

	self currentCellChangedFrom: oldCell to: selectedCell.! !

Object subclass: #UserInterface
	instanceVariableNames: 'panel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UserInterface commentStamp: 'mt 6/27/2010 09:43:15.463' prior: 0!
I am the abstract base class of an auto-generated User Interface.!

!UserInterface class methodsFor: 'testing' stamp: 'mt 10/30/2010 13:10:11.609'!
isTemplate
	"This is an indicator for the designer to use this user interface as design template."
	
	^ false! !

!UserInterface methodsFor: 'ui - private' stamp: 'mt 6/27/2010 09:48:47.906'!
initializePanel
	"Subclass will contain auto-generated code for this."
	
	self subclassResponsibility.! !

!UserInterface methodsFor: 'ui - private' stamp: 'mt 6/27/2010 09:45:06.967'!
panel
	"Returns the root panel for this UI."
	
	^ panel! !

!UserInterface methodsFor: 'ui - general' stamp: 'mt 4/11/2011 14:20:53.767'!
setupUi: aMorph
	"Configures this UI to be used in the given morph."
	
	self panel ifNil: [self initializePanel].
	
	aMorph extent < self panel extent
		ifTrue: [aMorph extent: self panel extent].
	
	self panel
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	
	aMorph
		changeTableLayout;
		color: self panel color;
		addMorphBack: self panel.! !

!Form methodsFor: '*widgets-support' stamp: 'mt 8/16/2012 14:34:32.226'!
adjustBrightness: aFloat

	^ self collectColors: [:color | color adjustBrightness: aFloat]! !

!Form methodsFor: '*widgets-support' stamp: 'mt 9/2/2010 11:52:12.191'!
asGrayScale2

	^ self collectColors: [:c | c isTransparent
		ifTrue: [c]
		ifFalse: [Color gray: c luminance]]! !

!Form methodsFor: '*widgets-support' stamp: 'mt 8/16/2012 16:28:27.096'!
asWireframe

	^ self collectColors: [:c | Color black alpha: c alpha]! !

!Form methodsFor: '*widgets-support' stamp: 'mt 8/21/2012 09:30:25.257'!
asWireframeSunken

	^ self asWireframeSunken: 0.4! !

!Form methodsFor: '*widgets-support' stamp: 'mt 8/22/2012 10:42:36.583'!
asWireframeSunken2

	| bw hi t |
	bw := self collectColors: [:c | (Color gray: 0.8) alpha: c alpha].
	hi := self collectColors: [:c | (Color gray: 0.1) alpha: c alpha].

	t := (Form extent: bw extent depth: bw depth)
		fillColor: Color transparent.
	t
		copy: (0@0 extent: hi extent) from: hi to:  0 @  -1 rule: Form paint;
		copy: (0@0 extent: bw extent) from: bw to:  0 @  0 rule: Form paint.	

	^ t! !

!Form methodsFor: '*widgets-support' stamp: 'mt 8/21/2012 09:24:31.943'!
asWireframeSunken: grayValue

	| bw hi t |
	bw := self collectColors: [:c | (Color gray: grayValue) alpha: c alpha].
	hi := self collectColors: [:c | Color white alpha: c alpha].

	t := (Form extent: bw extent depth: bw depth)
		fillColor: Color transparent.
	t
		copy: (0@0 extent: hi extent) from: hi to:  0 @  1 rule: Form paint;
		copy: (0@0 extent: bw extent) from: bw to:  0 @  0 rule: Form paint.	

	^ t! !

!Form methodsFor: '*widgets-support' stamp: 'mt 7/16/2010 13:08:02.826'!
clippedTo: aPoint
 	"Return a copy of this form that is clipped to the given size around the center."
	
	| newForm |
	newForm := self class extent: aPoint depth: depth.
	^ newForm copyBits: ((0@0 corner: aPoint) translateBy: (self extent - aPoint) // 2) from: self at: 0@0
		clippingBox: newForm boundingBox rule: Form over fillColor: nil! !

!Form methodsFor: '*widgets-support' stamp: 'mt 8/16/2012 14:35:03.067'!
muchLighter

	^ self collectColors: [:color | color adjustBrightness: 0.4]! !
UiButtonIcons initialize!
UiPlatform initialize!
